@@ -1,20 +1,23 @@
 #define __SP_COLOR_C__
 
 /*
- * SPColor, SPColorSpace
+ * Colors and colorspaces
  *
  * Author:
- *   Lauris Kaplinski <lauris@ximian.com>
+ *   Lauris Kaplinski <lauris@kaplinski.com>
  *
+ * Copyright (C) 2001-2002 Lauris Kaplinski
  * Copyright (C) 2001 Ximian, Inc.
  *
+ * Released under GNU GPL, read the file 'COPYING' for more information
  */
 
 #include <math.h>
+#include <glib.h>
 #include "color.h"
 
 struct _SPColorSpace {
-	guchar *name;
+	unsigned char *name;
 };
 
 static const SPColorSpace RGB = {"RGB"};
@@ -51,7 +54,7 @@ sp_color_copy (SPColor *dst, const SPColor *src)
 	*dst = *src;
 }
 
-gboolean
+unsigned int
 sp_color_is_equal (SPColor *c0, SPColor *c1)
 {
 	g_return_val_if_fail (c0 != NULL, TRUE);
@@ -67,7 +70,7 @@ sp_color_is_equal (SPColor *c0, SPColor *c1)
 }
 
 void
-sp_color_set_rgb_float (SPColor *color, gfloat r, gfloat g, gfloat b)
+sp_color_set_rgb_float (SPColor *color, float r, float g, float b)
 {
 	g_return_if_fail (color != NULL);
 	g_return_if_fail (r >= 0.0);
@@ -84,18 +87,18 @@ sp_color_set_rgb_float (SPColor *color, gfloat r, gfloat g, gfloat b)
 }
 
 void
-sp_color_set_rgb_rgba32 (SPColor *color, guint32 value)
+sp_color_set_rgb_rgba32 (SPColor *color, NRULong value)
 {
 	g_return_if_fail (color != NULL);
 
 	color->colorspace = &RGB;
-	color->v.c[0] = (value >> 24) / 255.0;
-	color->v.c[1] = ((value >> 16) & 0xff) / 255.0;
-	color->v.c[2] = ((value >> 8) & 0xff) / 255.0;
+	color->v.c[0] = (value >> 24) / 255.0F;
+	color->v.c[1] = ((value >> 16) & 0xff) / 255.0F;
+	color->v.c[2] = ((value >> 8) & 0xff) / 255.0F;
 }
 
 void
-sp_color_set_cmyk_float (SPColor *color, gfloat c, gfloat m, gfloat y, gfloat k)
+sp_color_set_cmyk_float (SPColor *color, float c, float m, float y, float k)
 {
 	g_return_if_fail (color != NULL);
 	g_return_if_fail (c >= 0.0);
@@ -114,8 +117,8 @@ sp_color_set_cmyk_float (SPColor *color, gfloat c, gfloat m, gfloat y, gfloat k)
 	color->v.c[3] = k;
 }
 
-guint32
-sp_color_get_rgba32_ualpha (const SPColor *color, guint alpha)
+NRULong
+sp_color_get_rgba32_ualpha (const SPColor *color, NRULong alpha)
 {
 	guint32 rgba;
 
@@ -125,7 +128,7 @@ sp_color_get_rgba32_ualpha (const SPColor *color, guint alpha)
 	if (color->colorspace == &RGB) {
 		rgba = SP_RGBA32_U_COMPOSE (SP_COLOR_F_TO_U (color->v.c[0]), SP_COLOR_F_TO_U (color->v.c[1]), SP_COLOR_F_TO_U (color->v.c[2]), alpha);
 	} else {
-		gfloat rgb[3];
+		float rgb[3];
 		sp_color_get_rgb_floatv (color, rgb);
 		rgba = SP_RGBA32_U_COMPOSE (SP_COLOR_F_TO_U (rgb[0]), SP_COLOR_F_TO_U (rgb[1]), SP_COLOR_F_TO_U (rgb[2]), alpha);
 	}
@@ -133,8 +136,8 @@ sp_color_get_rgba32_ualpha (const SPColor *color, guint alpha)
 	return rgba;
 }
 
-guint32
-sp_color_get_rgba32_falpha (const SPColor *color, gfloat alpha)
+NRULong
+sp_color_get_rgba32_falpha (const SPColor *color, float alpha)
 {
 	g_return_val_if_fail (color != NULL, 0x0);
 	g_return_val_if_fail (alpha >= 0.0, 0x0);
@@ -144,7 +147,7 @@ sp_color_get_rgba32_falpha (const SPColor *color, gfloat alpha)
 }
 
 void
-sp_color_get_rgb_floatv (const SPColor *color, gfloat *rgb)
+sp_color_get_rgb_floatv (const SPColor *color, float *rgb)
 {
 	g_return_if_fail (color != NULL);
 	g_return_if_fail (rgb != NULL);
@@ -159,7 +162,7 @@ sp_color_get_rgb_floatv (const SPColor *color, gfloat *rgb)
 }
 
 void
-sp_color_get_cmyk_floatv (const SPColor *color, gfloat *cmyk)
+sp_color_get_cmyk_floatv (const SPColor *color, float *cmyk)
 {
 	g_return_if_fail (color != NULL);
 	g_return_if_fail (cmyk != NULL);
@@ -177,9 +180,9 @@ sp_color_get_cmyk_floatv (const SPColor *color, gfloat *cmyk)
 /* Plain mode helpers */
 
 void
-sp_color_rgb_to_hsv_floatv (gfloat *hsv, gfloat r, gfloat g, gfloat b)
+sp_color_rgb_to_hsv_floatv (float *hsv, float r, float g, float b)
 {
-	gdouble max, min, delta;
+	float max, min, delta;
 
 	max = MAX (MAX (r, g), b);
 	min = MIN (MIN (r, g), b);
@@ -209,7 +212,7 @@ sp_color_rgb_to_hsv_floatv (gfloat *hsv, gfloat r, gfloat g, gfloat b)
 }
 
 void
-sp_color_hsv_to_rgb_floatv (gfloat *rgb, gfloat h, gfloat s, gfloat v)
+sp_color_hsv_to_rgb_floatv (float *rgb, float h, float s, float v)
 {
 	gdouble f, w, q, t, d;
 
@@ -247,9 +250,9 @@ sp_color_hsv_to_rgb_floatv (gfloat *rgb, gfloat h, gfloat s, gfloat v)
 }
 
 void
-sp_color_rgb_to_cmyk_floatv (gfloat *cmyk, gfloat r, gfloat g, gfloat b)
+sp_color_rgb_to_cmyk_floatv (float *cmyk, float r, float g, float b)
 {
-	gfloat c, m, y, k, kd;
+	float c, m, y, k, kd;
 
 	c = 1.0 - r;
 	m = 1.0 - g;
@@ -275,9 +278,9 @@ sp_color_rgb_to_cmyk_floatv (gfloat *cmyk, gfloat r, gfloat g, gfloat b)
 }
 
 void
-sp_color_cmyk_to_rgb_floatv (gfloat *rgb, gfloat c, gfloat m, gfloat y, gfloat k)
+sp_color_cmyk_to_rgb_floatv (float *rgb, float c, float m, float y, float k)
 {
-	gfloat kd;
+	float kd;
 
 	kd = 1.0 - k;
 
