@@ -232,7 +232,7 @@ sp_desktop_document_resized (SPView *view, SPDocument *doc, gdouble width, gdoub
 
 	desktop->doc2dt[5] = height;
 
-	sp_canvas_item_affine_absolute (SP_CANVAS_ITEM (desktop->drawing), desktop->doc2dt);
+	sp_canvas_item_affine_absolute (SP_CANVAS_ITEM (desktop->drawing), NR_MATRIX_D_FROM_DOUBLE (desktop->doc2dt));
 
 	sp_ctrlrect_set_area (SP_CTRLRECT (desktop->page), 0.0, 0.0, width, height);
 }
@@ -328,13 +328,13 @@ sp_desktop_new (SPNamedView *namedview, SPCanvas *canvas)
 
 	/* Connect event for page resize */
 	desktop->doc2dt[5] = sp_document_height (document);
-	sp_canvas_item_affine_absolute (SP_CANVAS_ITEM (desktop->drawing), desktop->doc2dt);
+	sp_canvas_item_affine_absolute (SP_CANVAS_ITEM (desktop->drawing), NR_MATRIX_D_FROM_DOUBLE (desktop->doc2dt));
 
 	/* Fixme: Setup initial zooming */
 	nr_matrix_d_set_scale (NR_MATRIX_D_FROM_DOUBLE (desktop->d2w), 1.0, -1.0);
 	desktop->d2w[5] = dh;
 	nr_matrix_d_invert (NR_MATRIX_D_FROM_DOUBLE (desktop->w2d), NR_MATRIX_D_FROM_DOUBLE (desktop->d2w));
-	sp_canvas_item_affine_absolute ((SPCanvasItem *) desktop->main, desktop->d2w);
+	sp_canvas_item_affine_absolute ((SPCanvasItem *) desktop->main, NR_MATRIX_D_FROM_DOUBLE (desktop->d2w));
 
 	g_signal_connect (G_OBJECT (desktop->selection), "modified", G_CALLBACK (sp_desktop_selection_modified), desktop);
 
@@ -363,11 +363,6 @@ sp_desktop_new (SPNamedView *namedview, SPCanvas *canvas)
 static void
 sp_desktop_prepare_shutdown (SPDesktop *dt)
 {
-	if (dt->sodipodi) {
-		sodipodi_remove_desktop (dt);
-		dt->sodipodi = NULL;
-	}
-
 	while (dt->event_context) {
 		SPEventContext *ec = dt->event_context;
 		dt->event_context = ec->next;
@@ -390,6 +385,11 @@ sp_desktop_prepare_shutdown (SPDesktop *dt)
 		sp_item_invoke_hide (SP_ITEM (sp_document_root (SP_VIEW_DOCUMENT (dt))), dt->dkey);
 		dt->drawing = NULL;
 	}
+
+	if (dt->sodipodi) {
+		sodipodi_remove_desktop (dt);
+		dt->sodipodi = NULL;
+	}
 }
 
 static void
@@ -520,7 +520,7 @@ sp_desktop_set_event_context (SPDesktop *dt, GtkType type, const unsigned char *
 		g_object_unref (G_OBJECT (ec));
 	}
 
-	repr = (config) ? sodipodi_get_repr (SODIPODI, config) : NULL;
+	repr = sp_config_node_get (config, TRUE);
 	ec = sp_event_context_new (type, dt, repr, SP_EVENT_CONTEXT_STATIC);
 	ec->next = dt->event_context;
 	dt->event_context = ec;
@@ -734,7 +734,7 @@ sp_desktop_widget_init (SPDesktopWidget *dtw)
 
 	/* Horizontal ruler */
 	eventbox = gtk_event_box_new ();
-	dtw->hruler = sp_hruler_new ();
+	dtw->hruler = sp_ruler_new (FALSE);
 	sp_ruler_set_metric (GTK_RULER (dtw->hruler), SP_PT);
 	gtk_container_add (GTK_CONTAINER (eventbox), dtw->hruler);
 	gtk_table_attach (GTK_TABLE (tbl), eventbox, 1, 2, 0, 1, GTK_FILL, GTK_FILL, widget->style->xthickness, 0);
@@ -743,7 +743,7 @@ sp_desktop_widget_init (SPDesktopWidget *dtw)
 	g_signal_connect (G_OBJECT (eventbox), "motion_notify_event", G_CALLBACK (sp_dt_hruler_event), dtw);
 	/* Vertical ruler */
 	eventbox = gtk_event_box_new ();
-	dtw->vruler = sp_vruler_new ();
+	dtw->vruler = sp_ruler_new (TRUE);
 	sp_ruler_set_metric (GTK_RULER (dtw->vruler), SP_PT);
 	gtk_container_add (GTK_CONTAINER (eventbox), GTK_WIDGET (dtw->vruler));
 	gtk_table_attach (GTK_TABLE (tbl), eventbox, 0, 1, 1, 2, GTK_FILL, GTK_FILL, 0, widget->style->ythickness);
@@ -808,8 +808,8 @@ sp_desktop_widget_init (SPDesktopWidget *dtw)
 	gtk_spin_button_set_update_policy (GTK_SPIN_BUTTON (dtw->zoom_status), GTK_UPDATE_ALWAYS);
 	g_signal_connect (G_OBJECT (dtw->zoom_status), "input", G_CALLBACK (sp_dtw_zoom_input), dtw);
 	g_signal_connect (G_OBJECT (dtw->zoom_status), "output", G_CALLBACK (sp_dtw_zoom_output), dtw);
-	dtw->zoom_update = g_signal_connect (G_OBJECT (dtw->zoom_status), "value_changed", G_CALLBACK (sp_dtw_zoom_value_changed), dtw);
-	dtw->zoom_update = g_signal_connect (G_OBJECT (dtw->zoom_status), "populate_popup", G_CALLBACK (sp_dtw_zoom_populate_popup), dtw);
+	dtw->zoom_update_id = g_signal_connect (G_OBJECT (dtw->zoom_status), "value_changed", G_CALLBACK (sp_dtw_zoom_value_changed), dtw);
+	dtw->zoom_update_id = g_signal_connect (G_OBJECT (dtw->zoom_status), "populate_popup", G_CALLBACK (sp_dtw_zoom_populate_popup), dtw);
 	gtk_box_pack_end (GTK_BOX (hbox), dtw->zoom_status, FALSE, FALSE, 0);
 
 	/* connecting canvas, scrollbars, rulers, statusbar */
@@ -818,6 +818,8 @@ sp_desktop_widget_init (SPDesktopWidget *dtw)
 
 	/* Be cautious about decorations (Lauris) */
 	gtk_widget_show_all (tbl);
+
+        gtk_object_sink(GTK_OBJECT(tt));
 }
 
 static void
@@ -980,7 +982,7 @@ sp_dtw_desktop_shutdown (SPView *view, SPDesktopWidget *dtw)
 	doc = SP_VIEW_DOCUMENT (view);
 
 	if (doc && (((GObject *) doc)->ref_count == 1)) {
-		if (sp_repr_attr (sp_document_repr_root (doc), "sodipodi:modified") != NULL) {
+		if (sp_repr_get_attr (sp_document_repr_root (doc), "sodipodi:modified") != NULL) {
 			GtkWidget *dlg;
 			gint b;
 			dlg = gtk_message_dialog_new (NULL, GTK_DIALOG_MODAL, GTK_MESSAGE_WARNING, GTK_BUTTONS_NONE,
@@ -1107,14 +1109,14 @@ sp_desktop_widget_namedview_modified (SPNamedView *nv, guint flags, SPDesktopWid
 static void
 sp_desktop_widget_adjustment_value_changed (GtkAdjustment *adj, SPDesktopWidget *dtw)
 {
-	if (dtw->update) return;
+	if (dtw->scroll_update) return;
 
-	dtw->update = 1;
+	dtw->scroll_update = 1;
 
 	sp_canvas_scroll_to (dtw->canvas, dtw->hadj->value, dtw->vadj->value, FALSE);
 	sp_desktop_widget_update_rulers (dtw);
 
-	dtw->update = 0;
+	dtw->scroll_update = 0;
 }
 
 /* we make the desktop window with focus active, signal is connected in interface.c */
@@ -1217,7 +1219,7 @@ sp_desktop_set_display_area (SPDesktop *dt, float x0, float y0, float x1, float
 		/* Set zoom factors */
 		nr_matrix_d_set_scale (NR_MATRIX_D_FROM_DOUBLE (dt->d2w), newscale, -newscale);
 		nr_matrix_d_invert (NR_MATRIX_D_FROM_DOUBLE (dt->w2d), NR_MATRIX_D_FROM_DOUBLE (dt->d2w));
-		sp_canvas_item_affine_absolute (SP_CANVAS_ITEM (dt->main), dt->d2w);
+		sp_canvas_item_affine_absolute (SP_CANVAS_ITEM (dt->main), NR_MATRIX_D_FROM_DOUBLE (dt->d2w));
 		clear = TRUE;
 	} else {
 		clear = FALSE;
@@ -1420,8 +1422,8 @@ sp_desktop_update_scrollbars (SPDesktop *dt)
 	dtw = g_object_get_data (G_OBJECT (dt), "widget");
 	if (!dtw) return;
 
-	if (dtw->update) return;
-	dtw->update = 1;
+	if (dtw->scroll_update) return;
+	dtw->scroll_update = 1;
 
 	doc = SP_VIEW_DOCUMENT (dt);
 	scale = SP_DESKTOP_ZOOM (dt);
@@ -1456,7 +1458,7 @@ sp_desktop_update_scrollbars (SPDesktop *dt)
 			(viewbox.y1 - viewbox.y0));
 	gtk_adjustment_set_value (dtw->vadj, viewbox.y0);
 
-	dtw->update = 0;
+	dtw->scroll_update = 0;
 }
 
 static void
@@ -1485,19 +1487,23 @@ void
 sp_dtw_zoom_value_changed (GtkSpinButton *spin, gpointer data)
 {
 	NRRectF d;
-	float zoom_factor;
+	double zoom, newzoom, delta;
 	SPDesktop *desktop;
 	SPDesktopWidget *dtw;
 
-	zoom_factor = pow (2, gtk_spin_button_get_value (spin));
-
 	dtw = SP_DESKTOP_WIDGET (data);
 	desktop = dtw->desktop;
 
-	sp_desktop_get_display_area (desktop, &d);
-	g_signal_handler_block (spin, dtw->zoom_update);
-	sp_desktop_zoom_absolute (desktop, (d.x0 + d.x1) / 2, (d.y0 + d.y1) / 2, zoom_factor);
-	g_signal_handler_unblock (spin, dtw->zoom_update);
+	zoom = SP_DESKTOP_ZOOM (desktop);
+	newzoom = pow (2, gtk_spin_button_get_value (spin));
+	delta = zoom / newzoom - 1.0;
+
+	if (fabs (delta) > 0.001) {
+		sp_desktop_get_display_area (desktop, &d);
+		g_signal_handler_block (spin, dtw->zoom_update_id);
+		sp_desktop_zoom_absolute (desktop, (d.x0 + d.x1) / 2, (d.y0 + d.y1) / 2, newzoom);
+		g_signal_handler_unblock (spin, dtw->zoom_update_id);
+	}
 }
 
 void
