@@ -24,6 +24,7 @@
 #include <gtk/gtkmenu.h>
 #include <gtk/gtkmenuitem.h>
 #include <gtk/gtkspinbutton.h>
+#include <gtk/gtkcheckbutton.h>
 
 #include "helper/sp-intl.h"
 #include "helper/window.h"
@@ -54,7 +55,7 @@ sp_display_dialog (void)
 				    GTK_SIGNAL_FUNC (sp_display_dialog_destroy), NULL);
 	}
 
-	gtk_widget_show (dialog);
+	gtk_window_present ((GtkWindow *) dialog);
 }
 
 static void
@@ -79,62 +80,62 @@ sp_display_dialog_cursor_tolerance_changed (GtkAdjustment *adj, gpointer data)
 	nr_arena_global_delta = adj->value;
 }
 
+static void
+sp_display_dialog_decoration_toggled (GtkToggleButton *tb, gpointer data)
+{
+	sp_config_value_set_boolean ("windows.dialogs.behaviour", "hintDialog", tb->active, TRUE);
+}
+
+static void
+sp_display_dialog_taskbar_toggled (GtkToggleButton *tb, gpointer data)
+{
+	sp_config_value_set_boolean ("windows.dialogs.behaviour", "skipTaskbar", tb->active, TRUE);
+}
+
 static GtkWidget *
 sp_display_dialog_new (void)
 {
-	GtkWidget *dialog, *nb, *l, *vb, *hb, *om, *m, *i, *sb;
+	GtkWidget *dialog, *nb, *l, *vb, *hb, *om, *m, *i, *sb, *cb;
 	GtkObject *a;
 
-	dialog = sp_window_new (_("Display settings"), FALSE);
+	dialog = sp_window_new (_("Display settings"), FALSE, TRUE);
 
 	nb = gtk_notebook_new ();
-	gtk_widget_show (nb);
 	gtk_container_add (GTK_CONTAINER (dialog), nb);
 
 	/* Rendering settings */
 	/* Notebook tab */
 	l = gtk_label_new (_("Rendering"));
-	gtk_widget_show (l);
 	vb = gtk_vbox_new (FALSE, 4);
-	gtk_widget_show (vb);
 	gtk_container_set_border_width (GTK_CONTAINER (vb), 4);
 	gtk_notebook_append_page (GTK_NOTEBOOK (nb), vb, l);
 
 	/* Oversampling menu */
 	hb = gtk_hbox_new (FALSE, 4);
-	gtk_widget_show (hb);
 	gtk_box_pack_start (GTK_BOX (vb), hb, FALSE, FALSE, 0);
 
 	l = gtk_label_new (_("Oversample bitmaps:"));
 	gtk_misc_set_alignment (GTK_MISC (l), 1.0, 0.5);
-	gtk_widget_show (l);
 	gtk_box_pack_start (GTK_BOX (hb), l, FALSE, FALSE, 0);
 	om = gtk_option_menu_new ();
-	gtk_widget_show (om);
 	gtk_box_pack_start (GTK_BOX (hb), om, TRUE, TRUE, 0);
 
 	m = gtk_menu_new ();
-	gtk_widget_show (m);
 
 	i = gtk_menu_item_new_with_label (_("None"));
 	gtk_signal_connect (GTK_OBJECT (i), "activate", GTK_SIGNAL_FUNC (sp_display_dialog_set_oversample), GINT_TO_POINTER (0));
-	gtk_widget_show (i);
 	gtk_menu_append (GTK_MENU (m), i);
 	i = gtk_menu_item_new_with_label (_("2x2"));
 	gtk_signal_connect (GTK_OBJECT (i), "activate", GTK_SIGNAL_FUNC (sp_display_dialog_set_oversample), GINT_TO_POINTER (1));
-	gtk_widget_show (i);
 	gtk_menu_append (GTK_MENU (m), i);
 	i = gtk_menu_item_new_with_label (_("4x4"));
 	gtk_signal_connect (GTK_OBJECT (i), "activate", GTK_SIGNAL_FUNC (sp_display_dialog_set_oversample), GINT_TO_POINTER (2));
-	gtk_widget_show (i);
 	gtk_menu_append (GTK_MENU (m), i);
 	i = gtk_menu_item_new_with_label (_("8x8"));
 	gtk_signal_connect (GTK_OBJECT (i), "activate", GTK_SIGNAL_FUNC (sp_display_dialog_set_oversample), GINT_TO_POINTER (3));
-	gtk_widget_show (i);
 	gtk_menu_append (GTK_MENU (m), i);
 	i = gtk_menu_item_new_with_label (_("16x16"));
 	gtk_signal_connect (GTK_OBJECT (i), "activate", GTK_SIGNAL_FUNC (sp_display_dialog_set_oversample), GINT_TO_POINTER (4));
-	gtk_widget_show (i);
 	gtk_menu_append (GTK_MENU (m), i);
 
 	gtk_option_menu_set_menu (GTK_OPTION_MENU (om), m);
@@ -144,30 +145,44 @@ sp_display_dialog_new (void)
 	/* Input settings */
 	/* Notebook tab */
 	l = gtk_label_new (_("Input"));
-	gtk_widget_show (l);
 	vb = gtk_vbox_new (FALSE, 4);
-	gtk_widget_show (vb);
 	gtk_container_set_border_width (GTK_CONTAINER (vb), 4);
 	gtk_notebook_append_page (GTK_NOTEBOOK (nb), vb, l);
 
 	hb = gtk_hbox_new (FALSE, 4);
-	gtk_widget_show (hb);
 	gtk_box_pack_start (GTK_BOX (vb), hb, FALSE, FALSE, 0);
 
 	l = gtk_label_new (_("Default cursor tolerance:"));
 	gtk_misc_set_alignment (GTK_MISC (l), 1.0, 0.5);
-	gtk_widget_show (l);
 	gtk_box_pack_start (GTK_BOX (hb), l, FALSE, FALSE, 0);
 
 	a = gtk_adjustment_new (0.0, 0.0, 10.0, 0.1, 1.0, 1.0);
 	gtk_adjustment_set_value (GTK_ADJUSTMENT (a), nr_arena_global_delta);
 	sb = gtk_spin_button_new (GTK_ADJUSTMENT (a), 0.1, 1);
-	gtk_widget_show (sb);
 	gtk_box_pack_start (GTK_BOX (hb), sb, TRUE, TRUE, 0);
 
 	gtk_signal_connect (GTK_OBJECT (a), "value_changed",
 			    GTK_SIGNAL_FUNC (sp_display_dialog_cursor_tolerance_changed), NULL);
 
+	/* Windows settings */
+	/* Notebook tab */
+	l = gtk_label_new (_("Windows"));
+	vb = gtk_vbox_new (FALSE, 4);
+	gtk_container_set_border_width (GTK_CONTAINER (vb), 4);
+	gtk_notebook_append_page (GTK_NOTEBOOK (nb), vb, l);
+
+	cb = gtk_check_button_new_with_label (_("Set dialog hint for dialogs"));
+	gtk_toggle_button_set_active ((GtkToggleButton *) cb, sp_config_value_get_boolean ("windows.dialogs.behaviour", "hintDialog", 1));
+	gtk_box_pack_start (GTK_BOX (vb), cb, FALSE, FALSE, 0);
+	g_signal_connect ((GObject *) cb, "toggled", (GCallback) sp_display_dialog_decoration_toggled, NULL);
+
+	cb = gtk_check_button_new_with_label (_("Skip taskbar entry for dialogs"));
+	gtk_toggle_button_set_active ((GtkToggleButton *) cb, sp_config_value_get_boolean ("windows.dialogs.behaviour", "skipTaskbar", 1));
+	gtk_box_pack_start (GTK_BOX (vb), cb, FALSE, FALSE, 0);
+	g_signal_connect ((GObject *) cb, "toggled", (GCallback) sp_display_dialog_taskbar_toggled, NULL);
+
+	gtk_widget_show_all (nb);
+
 	return dialog;
 }
 
