@@ -69,7 +69,7 @@ sp_document_dialog (void)
 		g_signal_connect (G_OBJECT (dialog), "destroy", G_CALLBACK (sp_doc_dialog_destroy), NULL);
 	}
 
-	gtk_widget_show (dialog);
+	gtk_window_present ((GtkWindow *) dialog);
 }
 
 #if 0
@@ -156,7 +156,7 @@ sp_doc_dialog_new (void)
 	GtkWidget *dialog, *nb, *vb, *hb, *l, *om, *m, *i, *f, *t, *us, *sb;
 	GtkObject *a;
 
-	dialog = sp_window_new (_("Document settings"), FALSE);
+	dialog = sp_window_new (_("Document settings"), FALSE, TRUE);
 
 	nb = gtk_notebook_new ();
 	gtk_widget_show (nb);
