@@ -19,7 +19,10 @@
 #include <math.h>
 #include <string.h>
 #include <glib.h>
+#include <gtk/gtkmain.h>
 #include <gtk/gtksignal.h>
+#include <gtk/gtkstock.h>
+#include <gtk/gtkdialog.h>
 #include <gtk/gtkvbox.h>
 #include <gtk/gtkhbox.h>
 #include <gtk/gtkframe.h>
@@ -28,10 +31,13 @@
 #include <gtk/gtkspinbutton.h>
 #include <gtk/gtktogglebutton.h>
 #include <gtk/gtkhseparator.h>
+#include <gtk/gtkprogressbar.h>
 
 #include "helper/sp-intl.h"
 #include "helper/window.h"
 #include "helper/unit-menu.h"
+#include "widgets/sp-color-preview.h"
+#include "dir-util.h"
 #include "sodipodi.h"
 #include "document.h"
 #include "desktop-handles.h"
@@ -40,7 +46,7 @@
 #include "file.h"
 #include "export.h"
 
-#define SP_EXPORT_MIN_SIZE 16.0
+#define SP_EXPORT_MIN_SIZE 1.0
 
 static void sp_export_area_toggled (GtkToggleButton *tb, GtkObject *base);
 static void sp_export_export_clicked (GtkButton *button, GtkObject *base);
@@ -58,11 +64,28 @@ static void sp_export_value_set_pt (GtkObject *base, const unsigned char *key, f
 static float sp_export_value_get (GtkObject *base, const unsigned char *key);
 static float sp_export_value_get_pt (GtkObject *base, const unsigned char *key);
 
+typedef struct {
+  GtkToggleButton *tb;
+  GtkObject *base;
+  SPSelection* selection;
+  guint changedId;
+} ActiveSelection;
+
+static ActiveSelection activeSelection={0,0,0,0};
+
 static GtkWidget *dlg = NULL;
 
 static void
 sp_export_dialog_destroy (GtkObject *object, gpointer data)
 {
+  if (activeSelection.selection && (activeSelection.changedId>0))
+    {
+      if (activeSelection.selection)
+        g_signal_handler_disconnect( G_OBJECT(activeSelection.selection),
+                                     activeSelection.changedId);
+      activeSelection.changedId=0;
+    }
+
 	dlg = NULL;
 }
 
@@ -112,9 +135,9 @@ void
 sp_export_dialog (void)
 {
 	if (!dlg) {
-		GtkWidget *vb, *f, *t, *hb, *us, *l, *fe, *hs, *b;
+		GtkWidget *vb, *f, *t, *hb, *us, *l, *cpicker, *fe, *hs, *b;
 
-		dlg = sp_window_new (_("Export bitmap"), FALSE);
+		dlg = sp_window_new (_("Export bitmap"), FALSE, TRUE);
 		gtk_signal_connect (GTK_OBJECT (dlg), "destroy", GTK_SIGNAL_FUNC (sp_export_dialog_destroy), NULL);
 
 		vb = gtk_vbox_new (FALSE, 4);
@@ -185,24 +208,32 @@ sp_export_dialog (void)
 		gtk_container_set_border_width (GTK_CONTAINER (t), 4);
 		gtk_container_add (GTK_CONTAINER (f), t);
 
-		sp_export_spinbutton_new ("bmwidth", 16.0, 16.0, 1000000.0, 1.0, 10.0, NULL, t, 0, 0,
+		sp_export_spinbutton_new ("bmwidth", 16.0, SP_EXPORT_MIN_SIZE, 1000000.0, 1.0, 10.0, NULL, t, 0, 0,
 					  _("Width:"), _("pixels"), 0, 1,
 					  G_CALLBACK (sp_export_bitmap_width_value_changed), dlg);
 
-		sp_export_spinbutton_new ("xdpi", 72.0, 1.0, 9600.0, 0.1, 1.0, NULL, t, 3, 0,
+		sp_export_spinbutton_new ("xdpi", 72.0, 0.01, 9600.0, 0.1, 1.0, NULL, t, 3, 0,
 					  NULL, _("dpi"), 2, 1,
 					  G_CALLBACK (sp_export_xdpi_value_changed), dlg);
 
-		sp_export_spinbutton_new ("bmheight", 16.0, 16.0, 1000000.0, 1, 10.0, NULL, t, 0, 1,
+		sp_export_spinbutton_new ("bmheight", 16.0, SP_EXPORT_MIN_SIZE, 1000000.0, 1, 10.0, NULL, t, 0, 1,
 					  _("Height:"), _("pixels"), 0, 0,
 					  NULL, dlg);
 
-		sp_export_spinbutton_new ("ydpi", 72.0, 1.0, 9600.0, 0.1, 1.0, NULL, t, 3, 1,
+		sp_export_spinbutton_new ("ydpi", 72.0, 0.01, 9600.0, 0.1, 1.0, NULL, t, 3, 1,
 					  NULL, _("dpi"), 2, 0,
 					  NULL, dlg);
 
 		gtk_widget_show_all (f);
 
+		/* Background color */
+		hb = gtk_hbox_new (FALSE, 0);
+		gtk_box_pack_start ((GtkBox *) vb, hb, FALSE, FALSE, 0);
+		l = gtk_label_new (_("Background color"));
+		gtk_box_pack_start ((GtkBox *) hb, l, FALSE, FALSE, 4);
+		cpicker = sp_color_picker_new (0x00000000, _("Bitmap background:"));
+		gtk_box_pack_start ((GtkBox *) hb, cpicker, FALSE, FALSE, 4);
+		gtk_object_set_data (GTK_OBJECT (dlg), "bgcolor", cpicker);
 		/* File entry */
 #if 0
 		fe = gnome_file_entry_new ("export", _("Export png file"));
@@ -225,34 +256,48 @@ sp_export_dialog (void)
 
 		}
 #endif
-		gtk_widget_show (fe);
 		gtk_box_pack_start (GTK_BOX (vb), fe, FALSE, FALSE, 0);
 		gtk_object_set_data (GTK_OBJECT (dlg), "filename", fe);
 
 		/* Buttons */
 		hb = gtk_hbox_new (FALSE, 0);
-		gtk_widget_show (hb);
 		gtk_box_pack_end (GTK_BOX (vb), hb, FALSE, FALSE, 0);
 
 		b = gtk_button_new_with_label (_("Export"));
-		gtk_widget_show (b);
 		gtk_box_pack_end (GTK_BOX (hb), b, FALSE, FALSE, 0);
 		gtk_signal_connect (GTK_OBJECT (b), "clicked", GTK_SIGNAL_FUNC (sp_export_export_clicked), dlg);
 
 		hs = gtk_hseparator_new ();
-		gtk_widget_show (hs);
 		gtk_box_pack_end (GTK_BOX (vb), hs, FALSE, FALSE, 0);
+
+		gtk_widget_show_all (dlg);
 	}
 
-	gtk_widget_show (dlg);
+	gtk_window_present ((GtkWindow *) dlg);
+}
+
+static void
+selection_modified (GObject* sodipodiOrSelection, void* data)
+{
+  if (activeSelection.tb && activeSelection.base)
+    sp_export_area_toggled(activeSelection.tb,activeSelection.base);
 }
 
 static void
 sp_export_area_toggled (GtkToggleButton *tb, GtkObject *base)
 {
+  if (activeSelection.selection && (activeSelection.changedId>0))
+    {
+      if (activeSelection.selection)
+        g_signal_handler_disconnect( G_OBJECT(activeSelection.selection),
+                                     activeSelection.changedId);
+      activeSelection.changedId=0;
+    }
+
 	if (gtk_toggle_button_get_active (tb)) {
 		const unsigned char *key;
 		key = gtk_object_get_data (GTK_OBJECT (tb), "key");
+
 		if (strcmp (key, "page")) {
 			gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (gtk_object_get_data (base, "page")), FALSE);
 		}
@@ -273,22 +318,61 @@ sp_export_area_toggled (GtkToggleButton *tb, GtkObject *base)
 				bbox.x1 = sp_document_width (doc);
 				bbox.y1 = sp_document_height (doc);
 			} else if (!strcmp (key, "drawing")) {
-				sp_item_bbox_desktop (SP_ITEM (SP_DOCUMENT_ROOT (doc)), &bbox);
+				sp_item_bbox_desktop_full (SP_ITEM (SP_DOCUMENT_ROOT (doc)), &bbox, SP_ITEM_BBOX_VISUAL);
 			} else {
-				sp_selection_bbox (SP_DT_SELECTION (SP_ACTIVE_DESKTOP), &bbox);
+                                SPSelection* selection=SP_DT_SELECTION (SP_ACTIVE_DESKTOP);
+                                activeSelection.tb=tb;
+                                activeSelection.base=base;
+                                activeSelection.selection=selection;
+                                
+				sp_selection_bbox_full (selection, &bbox, SP_ITEM_BBOX_VISUAL);
+                                activeSelection.changedId=g_signal_connect (G_OBJECT (selection), "changed", GTK_SIGNAL_FUNC (selection_modified), 0);
 			}
 			sp_export_set_area (base, bbox.x0, bbox.y0, bbox.x1, bbox.y1);
 		}
+        }
+}
+
+static gint
+sp_export_progress_delete (GtkWidget *widget, GdkEvent *event, GObject *base)
+{
+	g_object_set_data (base, "cancel", (gpointer) 1);
+	return TRUE;
+}
+
+static void
+sp_export_progress_cancel (GtkWidget *widget, GObject *base)
+{
+	g_object_set_data (base, "cancel", (gpointer) 1);
+}
+
+static unsigned int
+sp_export_progress_callback (float value, void *data)
+{
+	GtkWidget *prg;
+	int evtcount;
+	if (g_object_get_data ((GObject *) data, "cancel")) return FALSE;
+	prg = (GtkWidget *) g_object_get_data ((GObject *) data, "progress");
+	gtk_progress_bar_set_fraction ((GtkProgressBar *) prg, value);
+	evtcount = 0;
+	while ((evtcount < 16) && gdk_events_pending ()) {
+		/* g_print ("Iteration %d\n", evtcount); */
+		gtk_main_iteration_do (FALSE);
+		evtcount += 1;
 	}
+	gtk_main_iteration_do (FALSE);
+	return TRUE;
 }
 
 static void
 sp_export_export_clicked (GtkButton *button, GtkObject *base)
 {
 	GtkWidget *fe;
+	SPColorPicker *cpicker;
 	const unsigned char *filename;
 	float x0, y0, x1, y1;
 	int width, height;
+	guint32 rgba;
 
 	if (!SP_ACTIVE_DESKTOP) return;
 
@@ -305,9 +389,36 @@ sp_export_export_clicked (GtkButton *button, GtkObject *base)
 	y1 = sp_export_value_get_pt (base, "y1");
 	width = (int) (sp_export_value_get (base, "bmwidth") + 0.5);
 	height = (int) (sp_export_value_get (base, "bmheight") + 0.5);
-
+	cpicker = g_object_get_data ((GObject *) base, "bgcolor");
+	rgba = (cpicker) ? sp_color_picker_get_rgba32 (cpicker) : 0x00000000;
 	if ((x1 > x0) && (y1 > y0) && (width > 0) && (height > 0)) {
-		sp_export_png_file (SP_DT_DOCUMENT (SP_ACTIVE_DESKTOP), filename, x0, y0, x1, y1, width, height, 0x00000000);
+		GtkWidget *dlg, *prg, *btn;
+		const char *fn;
+		gchar *text;
+		/* Everything seems OK */
+		/* Create progress dialog */
+		dlg = gtk_dialog_new ();
+		gtk_window_set_title ((GtkWindow *) dlg, _("Export in progress"));
+		prg = gtk_progress_bar_new ();
+		g_object_set_data ((GObject *) base, "progress", prg);
+		fn = sp_filename_from_path (filename);
+		text = g_strdup_printf (_("Exporting [%d x %d] %s"), width, height, fn);
+		gtk_progress_bar_set_text ((GtkProgressBar *) prg, text);
+		g_free (text);
+		gtk_progress_bar_set_orientation ((GtkProgressBar *) prg, GTK_PROGRESS_LEFT_TO_RIGHT);
+		gtk_box_pack_start ((GtkBox *) ((GtkDialog *) dlg)->vbox, prg, FALSE, FALSE, 4);
+		btn = gtk_dialog_add_button (GTK_DIALOG (dlg), GTK_STOCK_CANCEL, GTK_RESPONSE_CANCEL);
+		g_signal_connect ((GObject *) dlg, "delete_event", (GCallback) sp_export_progress_delete, base);
+		g_signal_connect ((GObject *) btn, "clicked", (GCallback) sp_export_progress_cancel, base);
+		gtk_window_set_modal ((GtkWindow *) dlg, TRUE);
+		gtk_widget_show_all (dlg);
+		/* Do export */
+		sp_export_png_file (SP_DT_DOCUMENT (SP_ACTIVE_DESKTOP), filename,
+				    x0, y0, x1, y1, width, height,
+				    rgba,
+				    sp_export_progress_callback, base);
+		gtk_widget_destroy (dlg);
+		g_object_set_data ((GObject *) base, "cancel", (gpointer) 0);
 	}
 }
 
