@@ -12,16 +12,16 @@
  * Released under GNU GPL, read the file 'COPYING' for more information
  */
 
-#include <libgnome/gnome-defs.h>
-
-BEGIN_GNOME_DECLS
+#include <glib.h>
 
 #include <gtk/gtkwidget.h>
 
+#include "forward.h"
+
 GtkWidget *sp_fill_style_widget_new (void);
 
 void sp_fill_style_dialog (void);
 
-END_GNOME_DECLS
+void sp_fill_style_widget_system_color_set (GtkWidget *widget, SPColor *color, float opacity);
 
 #endif
