@@ -36,6 +36,7 @@
 
 #include "helper/sp-intl.h"
 #include "helper/window.h"
+#include "xml/repr-private.h"
 #include "../svg/svg.h"
 #include "../widgets/sp-widget.h"
 #include "../sodipodi.h"
@@ -137,7 +138,7 @@ sp_item_widget_new (void)
 	gtk_misc_set_alignment (GTK_MISC (l), 1.0, 0.5);
 	gtk_box_pack_start (GTK_BOX (hb), l, FALSE, FALSE, 0);
 
-	a = gtk_adjustment_new (1.0, 0.0, 1.0, 0.01, 0.01, 0.01);
+	a = gtk_adjustment_new (1.0, 0.0, 1.0, 0.01, 0.1, 0.0);
 
 	s = gtk_hscale_new (GTK_ADJUSTMENT (a));
 	gtk_scale_set_draw_value (GTK_SCALE (s), FALSE);
@@ -432,14 +433,14 @@ sp_item_dialog (void)
 {
 	if (dlg == NULL) {
 		GtkWidget *itemw;
-		dlg = sp_window_new (_("Item properties"), TRUE);
+		dlg = sp_window_new (_("Item properties"), TRUE, TRUE);
 		gtk_signal_connect (GTK_OBJECT (dlg), "destroy", GTK_SIGNAL_FUNC (sp_item_dialog_destroy), NULL);
 		itemw = sp_item_widget_new ();
 		gtk_widget_show (itemw);
 		gtk_container_add (GTK_CONTAINER (dlg), itemw);
 	}
 
-	if (!GTK_WIDGET_VISIBLE (dlg)) gtk_widget_show (dlg);
+	gtk_window_present ((GtkWindow *) dlg);
 }
 
 static void
