@@ -11,17 +11,17 @@
  *
  */
 
-#include <libgnome/gnome-defs.h>
+#include <glib.h>
 
-BEGIN_GNOME_DECLS
+G_BEGIN_DECLS
 
 #include <gtk/gtkwidget.h>
 #include "../forward.h"
 
 GtkWidget *sp_item_widget_new (void);
 
-void sp_item_dialog (SPItem *item);
+void sp_item_dialog (void);
 
-END_GNOME_DECLS
+G_END_DECLS
 
 #endif
