@@ -39,6 +39,12 @@ sp_node_path_edit_join (void)
 	sp_node_selected_join ();
 }
 
+void
+sp_node_path_edit_join_segment (void)
+{
+	sp_node_selected_join_segment ();
+}
+
 void
 sp_node_path_edit_toline (void)
 {
