@@ -15,9 +15,9 @@
  * Licensed under GNU GPL, read the file 'COPYING' for more information
  */
 
-#include <libgnome/gnome-defs.h>
+#include <glib.h>
 
-BEGIN_GNOME_DECLS
+G_BEGIN_DECLS
 
 #define SP_TYPE_ATTRIBUTE_WIDGET (sp_attribute_widget_get_type ())
 #define SP_ATTRIBUTE_WIDGET(obj) (GTK_CHECK_CAST ((obj), SP_TYPE_ATTRIBUTE_WIDGET, SPAttributeWidget))
@@ -90,6 +90,6 @@ GtkWidget *sp_attribute_table_new_repr (SPRepr *repr, gint num_attr, const gucha
 void sp_attribute_table_set_object (SPAttributeTable *spw, SPObject *object, gint num_attr, const guchar **labels, const guchar **attrs);
 void sp_attribute_table_set_repr (SPAttributeTable *spw, SPRepr *repr, gint num_attr, const guchar **labels, const guchar **attrs);
 
-END_GNOME_DECLS
+G_END_DECLS
 
 #endif
