@@ -11,19 +11,15 @@
  *
  */
 
-#include <libgnome/gnome-defs.h>
-
-BEGIN_GNOME_DECLS
+#include <glib.h>
 
 #include <gtk/gtkwidget.h>
 
+#include "forward.h"
+
 GtkWidget *sp_stroke_style_paint_widget_new (void);
 GtkWidget *sp_stroke_style_line_widget_new (void);
 
-#if 0
-void sp_stroke_style_dialog (void);
-#endif
-
-END_GNOME_DECLS
+void sp_stroke_style_paint_system_color_set (GtkWidget *widget, SPColor *color, float opacity);
 
 #endif
