@@ -44,7 +44,7 @@ sp_tool_options_dialog (void)
 	ec = SP_DT_EVENTCONTEXT (SP_ACTIVE_DESKTOP);
 
 	if (!dlg) {
-		dlg = sp_window_new (_("Tool options"), TRUE);
+		dlg = sp_window_new (_("Tool options"), TRUE, TRUE);
 		g_signal_connect (G_OBJECT (dlg), "destroy", G_CALLBACK (sp_tool_options_dialog_destroy), NULL);
 
 		sp_tool_options_dialog_setup (ec);
@@ -53,6 +53,8 @@ sp_tool_options_dialog (void)
 
 		gtk_widget_show (dlg);
 	}
+
+	gtk_window_present ((GtkWindow *) dlg);
 }
 
 static void
