@@ -1,15 +1,16 @@
 #define __SP_XMLVIEW_TREE_C__
 
 /*
- * XML View
+ * XML tree editing dialog for Sodipodi
  *
  * Authors:
  *   Lauris Kaplinski <lauris@kaplinski.com>
  *   MenTaLguY <mental@rydia.net>
  *
- * Copyright (C) 1999-2002 Authors
+ * Copyright (C) 2000-2003 Authors
  *
  * Released under GNU GPL, read the file 'COPYING' for more information
+ *
  */
 
 #ifdef HAVE_CONFIG_H
@@ -35,14 +36,8 @@
 #include "helper/sp-intl.h"
 #include "helper/window.h"
 #include "macros.h"
-#include "system.h"
-#include "sodipodi.h"
-#include "document.h"
-#include "desktop-handles.h"
-#include "selection.h"
-#include "sp-item.h"
 
-#include "../xml/repr-private.h"
+#include "api.h"
 
 #include "widgets/icon.h"
 #include "widgets/sp-xmlview-tree.h"
@@ -54,7 +49,7 @@ typedef struct _EditableDest {
 	gchar * text;
 } EditableDest;
 
-static GtkWidget * dialog = NULL;
+static GtkWidget *dialog = NULL;
 static GtkTooltips * tooltips = NULL;
 static GtkEditable * attr_name = NULL;
 static GtkTextView *attr_value = NULL;
@@ -132,7 +127,7 @@ sp_xml_tree_dialog (void)
 {
 	SPDesktop * desktop;
 
-	desktop = SP_ACTIVE_DESKTOP;
+	desktop = sodipodi_get_active_desktop ();
 	if (!desktop) return;
 	g_assert (SP_IS_DESKTOP (desktop));
 
@@ -144,7 +139,7 @@ sp_xml_tree_dialog (void)
 		tooltips = gtk_tooltips_new ();
 		gtk_tooltips_enable (tooltips);
 
-		dialog = sp_window_new ("", TRUE);
+		dialog = sp_window_new ("", TRUE, FALSE);
 		gtk_container_set_border_width (GTK_CONTAINER (dialog), 0);
 		gtk_window_set_default_size (GTK_WINDOW (dialog), 640, 384);
 		g_signal_connect (G_OBJECT (dialog), "destroy", G_CALLBACK (on_destroy), NULL);
@@ -391,7 +386,7 @@ sp_xml_tree_dialog (void)
 						(GCallback) on_tree_unselect_row_hide, text_container, GTK_OBJECT (text_container));
 		gtk_widget_hide (text_container);
 	} else {
-		gdk_window_raise (GTK_WIDGET (dialog)->window);
+		gtk_window_present ((GtkWindow *) dialog);
 	}
 
 	g_assert (desktop != NULL);
@@ -403,16 +398,22 @@ set_tree_desktop (SPDesktop * desktop)
 {
 	if ( desktop == current_desktop ) return;
 	if (current_desktop) {
-		if (SP_DT_SELECTION (current_desktop)) {
-			sp_signal_disconnect_by_data (SP_DT_SELECTION (current_desktop), dialog);
+		SPSelection *sel;
+		sel = sp_desktop_get_selection (current_desktop);
+		if (sel) {
+			sp_signal_disconnect_by_data (sel, dialog);
 		}
 		sp_signal_disconnect_by_data (current_desktop, dialog);
 	}
 	current_desktop = desktop;
 	if (desktop) {
+		SPDocument *doc;
+		SPSelection *sel;
+		sel = sp_desktop_get_selection (desktop);
 		g_signal_connect (G_OBJECT (desktop), "shutdown", G_CALLBACK (on_desktop_shutdown), dialog);
-		g_signal_connect (G_OBJECT (SP_DT_SELECTION (desktop)), "changed", G_CALLBACK (on_desktop_selection_changed), dialog);
-		set_tree_document (SP_DT_DOCUMENT (desktop));
+		g_signal_connect ((GObject *) sel, "changed", G_CALLBACK (on_desktop_selection_changed), dialog);
+		doc = sp_desktop_get_document (desktop);
+		set_tree_document (doc);
 	} else {
 		set_tree_document (NULL);
 	}
@@ -428,8 +429,8 @@ set_tree_document (SPDocument * document)
 	current_document = document;
 	if (current_document) {
 		g_signal_connect (G_OBJECT (current_document), "uri_set", G_CALLBACK (on_document_uri_set), dialog);
-		on_document_uri_set (current_document, SP_DOCUMENT_URI (current_document), dialog);
-		set_tree_repr (sp_document_repr_root (current_document));
+		on_document_uri_set (current_document, sp_document_get_uri (current_document), dialog);
+		set_tree_repr (sp_document_get_repr_root (current_document));
 	} else {
 		set_tree_repr (NULL);
 	}
@@ -488,12 +489,12 @@ set_tree_select (SPRepr * repr)
 void
 propagate_tree_select (SPRepr * repr)
 {
-	if (repr && SP_REPR_TYPE (repr) == SP_XML_ELEMENT_NODE) {
+	if (repr && sp_repr_is_element (repr)) {
 		sp_xmlview_attr_list_set_repr (attributes, repr);
 	} else {
 		sp_xmlview_attr_list_set_repr (attributes, NULL);
 	}
-	if (repr && SP_REPR_TYPE (repr) == SP_XML_TEXT_NODE) {
+	if (repr && (sp_repr_is_text (repr) || sp_repr_is_cdata (repr))) {
 		sp_xmlview_content_set_repr (content, repr);
 	} else {
 		sp_xmlview_content_set_repr (content, NULL);
@@ -505,7 +506,7 @@ get_dt_select (void)
 {
 	if (!current_desktop) return NULL;
 
-	return sp_selection_repr (SP_DT_SELECTION (current_desktop));
+	return sp_selection_get_repr (sp_desktop_get_selection (current_desktop));
 }
 
 void
@@ -513,16 +514,14 @@ set_dt_select (SPRepr *repr)
 {
 	SPSelection *selection;
 	SPObject *object;
-	const gchar *id;
 	
 	if (!current_desktop) return;
-	selection = SP_DT_SELECTION (current_desktop);
+	selection = sp_desktop_get_selection (current_desktop);
 
 	if (repr) {
-		while ( ( SP_REPR_TYPE(repr) != SP_XML_ELEMENT_NODE ) && sp_repr_parent(repr) ) repr = sp_repr_parent(repr);
+		while ((!sp_repr_is_element (repr)) && sp_repr_get_parent (repr)) repr = sp_repr_get_parent (repr);
 
-		id = sp_repr_attr(repr, "id");
-		object = (id) ? sp_document_lookup_id (SP_DT_DOCUMENT (current_desktop), id) : NULL;
+		object = sp_document_get_object_from_repr (sp_desktop_get_document (current_desktop), repr);
 	} else {
 		object = NULL;
 	}
@@ -531,7 +530,7 @@ set_dt_select (SPRepr *repr)
 	if (object && SP_IS_ITEM (object)) {
 		sp_selection_set_item (selection, SP_ITEM (object));
 	} else {
-		sp_selection_empty (selection);
+		sp_selection_set_empty (selection);
 	}
 	blocked--;
 }
@@ -592,6 +591,7 @@ on_destroy (GtkObject * object, gpointer data)
 {
 	set_tree_desktop (NULL);
 	gtk_object_destroy (GTK_OBJECT (tooltips));
+        tooltips = NULL;
 	dialog = NULL;
 }
 
@@ -606,7 +606,7 @@ on_tree_select_row_enable_if_element (GtkCTree * tree, GtkCTreeNode * node, gint
 {
 	SPRepr * repr;
 	repr = sp_xmlview_tree_node_get_repr (SP_XMLVIEW_TREE (tree), node);
-	if (SP_REPR_TYPE (repr) == SP_XML_ELEMENT_NODE) {
+	if (sp_repr_is_element (repr)) {
 		gtk_widget_set_sensitive (GTK_WIDGET (data), TRUE);
 	} else {
 		gtk_widget_set_sensitive (GTK_WIDGET (data), FALSE);
@@ -618,7 +618,7 @@ on_tree_select_row_show_if_element (GtkCTree * tree, GtkCTreeNode * node, gint c
 {
 	SPRepr * repr;
 	repr = sp_xmlview_tree_node_get_repr (SP_XMLVIEW_TREE (tree), node);
-	if (SP_REPR_TYPE (repr) == SP_XML_ELEMENT_NODE) {
+	if (sp_repr_is_element (repr)) {
 		gtk_widget_show (GTK_WIDGET (data));
 	} else {
 		gtk_widget_hide (GTK_WIDGET (data));
@@ -630,7 +630,7 @@ on_tree_select_row_show_if_text (GtkCTree * tree, GtkCTreeNode * node, gint colu
 {
 	SPRepr * repr;
 	repr = sp_xmlview_tree_node_get_repr (SP_XMLVIEW_TREE (tree), node);
-	if (SP_REPR_TYPE (repr) == SP_XML_TEXT_NODE) {
+	if (sp_repr_is_text (repr) || sp_repr_is_cdata (repr)) {
 		gtk_widget_show (GTK_WIDGET (data));
 	} else {
 		gtk_widget_hide (GTK_WIDGET (data));
@@ -704,22 +704,22 @@ on_attr_select_row_set_value_content (GtkCList *list, gint row, gint column, Gdk
 	const guchar *name, *value;
 	tb = gtk_text_view_get_buffer (GTK_TEXT_VIEW (data));
 	name = g_quark_to_string (sp_xmlview_attr_list_get_row_key (list, row));
-	value = sp_repr_attr (selected_repr, name);
+	value = sp_repr_get_attr (selected_repr, name);
 	gtk_text_buffer_set_text (tb, value, strlen (value));
 }
 
 void
 on_tree_select_row_enable_if_indentable (GtkCTree * tree, GtkCTreeNode * node, gint column, gpointer data)
 {
-	SPRepr * repr, * prev;
+	SPRepr *repr, *parent, *prev;
 	gboolean indentable;
 	indentable = FALSE;
 	repr = sp_xmlview_tree_node_get_repr (SP_XMLVIEW_TREE (tree), node);
-	if ( repr->parent && repr != repr->parent->children ) {
-		g_assert (repr->parent->children);
-		for ( prev = repr->parent->children ; prev != repr ;
-		      prev = prev->next );
-		if (SP_REPR_TYPE (prev) == SP_XML_ELEMENT_NODE) {
+	parent = sp_repr_get_parent (repr);
+	if (parent && repr != sp_repr_get_children (parent)) {
+		g_assert (sp_repr_get_children (parent));
+		for (prev = sp_repr_get_children (parent) ; prev != repr; prev = sp_repr_get_next (prev));
+		if (sp_repr_is_element (prev)) {
 			indentable = TRUE;
 		}
 	}
@@ -729,9 +729,10 @@ on_tree_select_row_enable_if_indentable (GtkCTree * tree, GtkCTreeNode * node, g
 void
 on_tree_select_row_enable_if_not_first_child (GtkCTree * tree, GtkCTreeNode * node, gint column, gpointer data)
 {
-	SPRepr * repr;
+	SPRepr *repr, *parent;
 	repr = sp_xmlview_tree_node_get_repr (SP_XMLVIEW_TREE (tree), node);
-	if ( repr->parent && repr != repr->parent->children ) {
+	parent = sp_repr_get_parent (repr);
+	if (parent && repr != sp_repr_get_children (parent)) {
 		gtk_widget_set_sensitive (GTK_WIDGET (data), TRUE);
 	} else {
 		gtk_widget_set_sensitive (GTK_WIDGET (data), FALSE);
@@ -741,9 +742,10 @@ on_tree_select_row_enable_if_not_first_child (GtkCTree * tree, GtkCTreeNode * no
 void
 on_tree_select_row_enable_if_not_last_child (GtkCTree * tree, GtkCTreeNode * node, gint column, gpointer data)
 {
-	SPRepr * repr;
+	SPRepr *repr, *parent;
 	repr = sp_xmlview_tree_node_get_repr (SP_XMLVIEW_TREE (tree), node);
-	if ( repr->parent && repr->parent->parent && repr->next ) {
+	parent = sp_repr_get_parent (repr);
+	if (parent && sp_repr_get_parent (parent) && sp_repr_get_next (repr)) {
 		gtk_widget_set_sensitive (GTK_WIDGET (data), TRUE);
 	} else {
 		gtk_widget_set_sensitive (GTK_WIDGET (data), FALSE);
@@ -827,7 +829,7 @@ void
 on_document_uri_set (SPDocument * document, const guchar * uri, gpointer data)
 {
 	guchar * title;
-	title = g_strdup_printf (_("Sodipodi: %s : XML View"), SP_DOCUMENT_NAME (document));
+	title = g_strdup_printf (_("Sodipodi: %s : XML View"), sp_document_get_name (document));
 	gtk_window_set_title (GTK_WINDOW (dialog), title);
 	g_free (title);
 }
@@ -929,7 +931,7 @@ cmd_duplicate_node (GtkObject * object, gpointer data)
 
 	g_assert (selected_repr != NULL);
 
-	parent = sp_repr_parent (selected_repr);
+	parent = sp_repr_get_parent (selected_repr);
 	dup = sp_repr_duplicate (selected_repr);
 	sp_repr_add_child (parent, dup, selected_repr);
 
@@ -992,15 +994,15 @@ cmd_raise_node (GtkObject * object, gpointer data)
 	SPRepr * before, * parent, * ref;
 	g_assert (selected_repr != NULL);
 
-	parent = sp_repr_parent (selected_repr);
+	parent = sp_repr_get_parent (selected_repr);
 	g_return_if_fail (parent != NULL);
-	g_return_if_fail (parent->children != selected_repr);
+	g_return_if_fail (sp_repr_get_children (parent) != selected_repr);
 
 	ref = NULL;
-	before = parent->children;
-	while (before && before->next != selected_repr) {
+	before = sp_repr_get_children (parent);
+	while (before && sp_repr_get_next (before) != selected_repr) {
 		ref = before;
-		before = before->next;
+		before = sp_repr_get_next (before);
 	}
 
 	sp_repr_change_order (parent, selected_repr, ref);
@@ -1016,10 +1018,10 @@ cmd_lower_node (GtkObject * object, gpointer data)
 {
 	SPRepr * parent;
 	g_assert (selected_repr != NULL);
-	g_return_if_fail (selected_repr->next != NULL);
-	parent = sp_repr_parent (selected_repr);
+	g_return_if_fail (sp_repr_get_next (selected_repr) != NULL);
+	parent = sp_repr_get_parent (selected_repr);
 
-	sp_repr_change_order (parent, selected_repr, selected_repr->next);
+	sp_repr_change_order (parent, selected_repr, sp_repr_get_next (selected_repr));
 
 	sp_document_done (current_document);
 
@@ -1035,19 +1037,19 @@ cmd_indent_node (GtkObject * object, gpointer data)
 
 	repr = selected_repr;
 	g_assert (repr != NULL);
-	parent = sp_repr_parent (repr);
+	parent = sp_repr_get_parent (repr);
 	g_return_if_fail (parent != NULL);
-	g_return_if_fail (parent->children != repr);
+	g_return_if_fail (sp_repr_get_children (parent) != repr);
 
-	prev = parent->children;
-	while (prev && prev->next != repr) {
-	      prev = prev->next;
+	prev = sp_repr_get_children (parent);
+	while (prev && sp_repr_get_next (prev) != repr) {
+	      prev = sp_repr_get_next (prev);
 	}
 	g_return_if_fail (prev != NULL);
-	g_return_if_fail (SP_REPR_TYPE (prev) == SP_XML_ELEMENT_NODE);
+	g_return_if_fail (sp_repr_is_element (prev));
 
-	if (prev->children) {
-		for ( ref = prev->children ; ref->next ; ref = ref->next );
+	if (sp_repr_get_children (prev)) {
+		for (ref = sp_repr_get_children (prev) ; sp_repr_get_next (ref) ; ref = sp_repr_get_next (ref));
 	} else {
 		ref = NULL;
 	}
@@ -1072,9 +1074,9 @@ cmd_unindent_node (GtkObject * object, gpointer data)
 
 	repr = selected_repr;
 	g_assert (repr != NULL);
-	parent = sp_repr_parent (repr);
+	parent = sp_repr_get_parent (repr);
 	g_return_if_fail (parent);
-	grandparent = sp_repr_parent (parent);
+	grandparent = sp_repr_get_parent (parent);
 	g_return_if_fail (grandparent);
 	
 	ok = sp_repr_remove_child (parent, repr) &&
@@ -1089,3 +1091,126 @@ cmd_unindent_node (GtkObject * object, gpointer data)
 	}
 }
 
+/* Experimental */
+
+#include <module.h>
+
+#define SP_TYPE_MODULE_XML_EDITOR (sp_module_xml_editor_get_type())
+#define SP_MODULE_XML_EDITOR(o) (G_TYPE_CHECK_INSTANCE_CAST ((o), SP_TYPE_MODULE_XML_EDITOR, SPModuleXMLEditor))
+#define SP_IS_MODULE_XML_EDITOR(o) (G_TYPE_CHECK_INSTANCE_TYPE ((o), SP_TYPE_MODULE_XML_EDITOR))
+
+typedef struct _SPModuleXMLEditor SPModuleXMLEditor;
+typedef struct _SPModuleXMLEditorClass SPModuleXMLEditorClass;
+
+struct _SPModuleXMLEditor {
+	SPModule module;
+};
+
+struct _SPModuleXMLEditorClass {
+	SPModuleClass module_class;
+};
+
+static void sp_module_xml_editor_class_init (SPModuleXMLEditorClass *klass);
+static void sp_module_xml_editor_init (SPModuleXMLEditor *fmod);
+static void sp_module_xml_editor_finalize (GObject *object);
+
+static unsigned int sp_module_xml_editor_execute (SPModule *module, SPRepr *config);
+
+static SPModuleClass *xml_editor_parent_class = NULL;
+
+static SPModuleXMLEditor *xml_editor_module = NULL;
+
+static GType
+sp_module_xml_editor_get_type (void)
+{
+	static GType type = 0;
+	if (!type) {
+		GTypeInfo info = {
+			sizeof (SPModuleXMLEditorClass),
+			NULL, NULL,
+			(GClassInitFunc) sp_module_xml_editor_class_init,
+			NULL, NULL,
+			sizeof (SPModuleXMLEditor),
+			16,
+			(GInstanceInitFunc) sp_module_xml_editor_init,
+		};
+		type = g_type_register_static (SP_TYPE_MODULE, "SPModuleXMLEditor", &info, 0);
+	}
+	return type;
+}
+
+static void
+sp_module_xml_editor_class_init (SPModuleXMLEditorClass *klass)
+{
+	GObjectClass *g_object_class;
+	SPModuleClass *module_class;
+
+	g_object_class = (GObjectClass *) klass;
+	module_class = (SPModuleClass *) klass;
+
+	xml_editor_parent_class = g_type_class_peek_parent (klass);
+
+	g_object_class->finalize = sp_module_xml_editor_finalize;
+
+	module_class->execute = sp_module_xml_editor_execute;
+}
+
+static void
+sp_module_xml_editor_init (SPModuleXMLEditor *module)
+{
+	xml_editor_module = module;
+}
+
+static void
+sp_module_xml_editor_finalize (GObject *object)
+{
+	g_assert (xml_editor_module);
+	xml_editor_module = NULL;
+
+	G_OBJECT_CLASS (xml_editor_parent_class)->finalize (object);
+}
+
+static unsigned int
+sp_module_xml_editor_execute (SPModule *module, SPRepr *config)
+{
+	sp_xml_tree_dialog ();
+
+	return TRUE;
+}
+
+/* Get new reference */
+SPModule *
+sp_xml_module_load (void)
+{
+	if (!xml_editor_module) {
+		g_object_new (SP_TYPE_MODULE_XML_EDITOR, NULL);
+	} else {
+		g_object_ref ((GObject *) xml_editor_module);
+	}
+	return (SPModule *) xml_editor_module;
+}
+
+/* Unref module */
+void
+sp_xml_module_unload (SPModule *module)
+{
+	g_object_unref ((GObject *) xml_editor_module);
+}
+
+#ifdef STANDALONE_MODULE
+/* Standalone module */
+
+SPModule *
+sp_module_load (void)
+{
+	return sp_xml_module_load ();
+}
+
+void
+sp_module_unload (SPModule *module)
+{
+	sp_xml_module_unload (module);
+}
+
+#endif
+
