@@ -15,41 +15,36 @@
 #include <math.h>
 #include <libnr/nr-rect.h>
 #include <libnr/nr-matrix.h>
-#include <libart_lgpl/art_affine.h>
-#include "../helper/nr-plain-stuff.h"
-#include "../helper/nr-buffers.h"
+#include <libnr/nr-blit.h>
 #include "nr-arena-group.h"
 
 static void nr_arena_group_class_init (NRArenaGroupClass *klass);
 static void nr_arena_group_init (NRArenaGroup *group);
-static void nr_arena_group_destroy (GtkObject *object);
+static void nr_arena_group_finalize (NRObject *object);
 
 static NRArenaItem *nr_arena_group_children (NRArenaItem *item);
 static void nr_arena_group_add_child (NRArenaItem *item, NRArenaItem *child, NRArenaItem *ref);
 static void nr_arena_group_remove_child (NRArenaItem *item, NRArenaItem *child);
 static void nr_arena_group_set_child_position (NRArenaItem *item, NRArenaItem *child, NRArenaItem *ref);
 
-static guint nr_arena_group_update (NRArenaItem *item, NRRectL *area, NRGC *gc, guint state, guint reset);
-static guint nr_arena_group_render (NRArenaItem *item, NRRectL *area, NRBuffer *b);
-static guint nr_arena_group_clip (NRArenaItem *item, NRRectL *area, NRBuffer *b);
-static NRArenaItem *nr_arena_group_pick (NRArenaItem *item, gdouble x, gdouble y, gdouble delta, gboolean sticky);
+static unsigned int nr_arena_group_update (NRArenaItem *item, NRRectL *area, NRGC *gc, unsigned int state, unsigned int reset);
+static unsigned int nr_arena_group_render (NRArenaItem *item, NRRectL *area, NRPixBlock *pb, unsigned int flags);
+static unsigned int nr_arena_group_clip (NRArenaItem *item, NRRectL *area, NRPixBlock *pb);
+static NRArenaItem *nr_arena_group_pick (NRArenaItem *item, double x, double y, double delta, unsigned int sticky);
 
 static NRArenaItemClass *parent_class;
 
-GtkType
+unsigned int
 nr_arena_group_get_type (void)
 {
-	static GtkType type = 0;
+	static unsigned int type = 0;
 	if (!type) {
-		GtkTypeInfo info = {
-			"NRArenaGroup",
-			sizeof (NRArenaGroup),
-			sizeof (NRArenaGroupClass),
-			(GtkClassInitFunc) nr_arena_group_class_init,
-			(GtkObjectInitFunc) nr_arena_group_init,
-			NULL, NULL, NULL
-		};
-		type = gtk_type_unique (NR_TYPE_ARENA_ITEM, &info);
+		type = nr_object_register_type (NR_TYPE_ARENA_ITEM,
+						"NRArenaGroup",
+						sizeof (NRArenaGroupClass),
+						sizeof (NRArenaGroup),
+						(void (*) (NRObjectClass *)) nr_arena_group_class_init,
+						(void (*) (NRObject *)) nr_arena_group_init);
 	}
 	return type;
 }
@@ -57,15 +52,15 @@ nr_arena_group_get_type (void)
 static void
 nr_arena_group_class_init (NRArenaGroupClass *klass)
 {
-	GtkObjectClass *object_class;
+	NRObjectClass *object_class;
 	NRArenaItemClass *item_class;
 
-	object_class = (GtkObjectClass *) klass;
+	object_class = (NRObjectClass *) klass;
 	item_class = (NRArenaItemClass *) klass;
 
-	parent_class = gtk_type_class (NR_TYPE_ARENA_ITEM);
+	parent_class = (NRArenaItemClass *) ((NRObjectClass *) klass)->parent;
 
-	object_class->destroy = nr_arena_group_destroy;
+	object_class->finalize = nr_arena_group_finalize;
 
 	item_class->children = nr_arena_group_children;
 	item_class->add_child = nr_arena_group_add_child;
@@ -83,11 +78,11 @@ nr_arena_group_init (NRArenaGroup *group)
 	group->transparent = FALSE;
 	group->children = NULL;
 	group->last = NULL;
-	nr_matrix_d_set_identity (&group->child_transform);
+	nr_matrix_f_set_identity (&group->child_transform);
 }
 
 static void
-nr_arena_group_destroy (GtkObject *object)
+nr_arena_group_finalize (NRObject *object)
 {
 	NRArenaItem *item;
 	NRArenaGroup *group;
@@ -99,10 +94,7 @@ nr_arena_group_destroy (GtkObject *object)
 		group->children = nr_arena_item_detach_unref (item, group->children);
 	}
 
-	group->last = NULL;
-
-	if (((GtkObjectClass *) (parent_class))->destroy)
-		(* ((GtkObjectClass *) (parent_class))->destroy) (object);
+	((NRObjectClass *) (parent_class))->finalize (object);
 }
 
 static NRArenaItem *
@@ -181,12 +173,12 @@ nr_arena_group_set_child_position (NRArenaItem *item, NRArenaItem *child, NRAren
 	nr_arena_item_request_render (child);
 }
 
-static guint
-nr_arena_group_update (NRArenaItem *item, NRRectL *area, NRGC *gc, guint state, guint reset)
+static unsigned int
+nr_arena_group_update (NRArenaItem *item, NRRectL *area, NRGC *gc, unsigned int state, unsigned int reset)
 {
 	NRArenaGroup *group;
 	NRArenaItem *child;
-	guint newstate, beststate;
+	unsigned int newstate, beststate;
 
 	group = NR_ARENA_GROUP (item);
 
@@ -194,11 +186,8 @@ nr_arena_group_update (NRArenaItem *item, NRRectL *area, NRGC *gc, guint state,
 
 	for (child = group->children; child != NULL; child = child->next) {
 		NRGC cgc;
-		art_affine_multiply (cgc.affine, group->child_transform.c, gc->affine);
+		nr_matrix_multiply_dfd (&cgc.transform, &group->child_transform, &gc->transform);
 		newstate = nr_arena_item_invoke_update (child, area, &cgc, state, reset);
-#if 0
-		g_return_val_if_fail (!(~newstate & state), newstate);
-#endif
 		beststate = beststate & newstate;
 	}
 
@@ -212,60 +201,12 @@ nr_arena_group_update (NRArenaItem *item, NRRectL *area, NRGC *gc, guint state,
 	return beststate;
 }
 
-static guint
-nr_arena_group_render (NRArenaItem *item, NRRectL *area, NRBuffer *b)
+static unsigned int
+nr_arena_group_render (NRArenaItem *item, NRRectL *area, NRPixBlock *pb, unsigned int flags)
 {
 	NRArenaGroup *group;
 	NRArenaItem *child;
-	guint ret;
-
-	group = NR_ARENA_GROUP (item);
-
-	ret = item->state;
-
-	if (item->opacity == 1.0) {
-		/* Just compose children into parent buffer */
-		for (child = group->children; child != NULL; child = child->next) {
-			ret = nr_arena_item_invoke_render (child, area, b);
-#if 0
-			if (!(ret & NR_ARENA_ITEM_STATE_RENDER)) break;
-#endif
-		}
-	} else {
-		NRBuffer *nb;
-		/* Need intermediate composing */
-		/* fixme: */
-		nb = nr_buffer_get (NR_IMAGE_R8G8B8A8, area->x1 - area->x0, area->y1 - area->y0, TRUE, TRUE);
-		/* fixme: */
-		nb->empty = FALSE;
-		for (child = group->children; child != NULL; child = child->next) {
-			ret = nr_arena_item_invoke_render (child, area, nb);
-#if 0
-			if (!(ret & NR_ARENA_ITEM_STATE_RENDER)) break;
-#endif
-		}
-		if (1 && (ret & NR_ARENA_ITEM_STATE_RENDER)) {
-			/* Compose */
-			nr_render_buf_buf_alpha (b, 0, 0, area->x1 - area->x0, area->y1 - area->y0,
-						 nb, 0, 0,
-						 (guint) floor (item->opacity * 255.9999));
-		}
-		nr_buffer_free (nb);
-	}
-
-#if 0
-	g_return_val_if_fail (ret & NR_ARENA_ITEM_STATE_RENDER, ret);
-#endif
-
-	return ret;
-}
-
-static guint
-nr_arena_group_clip (NRArenaItem *item, NRRectL *area, NRBuffer *b)
-{
-	NRArenaGroup *group;
-	NRArenaItem *child;
-	guint ret;
+	unsigned int ret;
 
 	group = NR_ARENA_GROUP (item);
 
@@ -273,17 +214,35 @@ nr_arena_group_clip (NRArenaItem *item, NRRectL *area, NRBuffer *b)
 
 	/* Just compose children into parent buffer */
 	for (child = group->children; child != NULL; child = child->next) {
-		ret = nr_arena_item_invoke_clip (child, area, b);
-		if (!(ret & NR_ARENA_ITEM_STATE_CLIP)) break;
+		ret = nr_arena_item_invoke_render (child, area, pb, flags);
+		if (ret & NR_ARENA_ITEM_STATE_INVALID) break;
 	}
 
-	g_return_val_if_fail (ret & NR_ARENA_ITEM_STATE_CLIP, ret);
+	return ret;
+}
+
+static unsigned int
+nr_arena_group_clip (NRArenaItem *item, NRRectL *area, NRPixBlock *pb)
+{
+	NRArenaGroup *group;
+	NRArenaItem *child;
+	unsigned int ret;
+
+	group = NR_ARENA_GROUP (item);
+
+	ret = item->state;
+
+	/* Just compose children into parent buffer */
+	for (child = group->children; child != NULL; child = child->next) {
+		ret = nr_arena_item_invoke_clip (child, area, pb);
+		if (ret & NR_ARENA_ITEM_STATE_INVALID) break;
+	}
 
 	return ret;
 }
 
 static NRArenaItem *
-nr_arena_group_pick (NRArenaItem *item, gdouble x, gdouble y, gdouble delta, gboolean sticky)
+nr_arena_group_pick (NRArenaItem *item, double x, double y, double delta, unsigned int sticky)
 {
 	NRArenaGroup *group;
 	NRArenaItem *child, *picked;
@@ -299,20 +258,24 @@ nr_arena_group_pick (NRArenaItem *item, gdouble x, gdouble y, gdouble delta, gbo
 }
 
 void
-nr_arena_group_set_transparent (NRArenaGroup *group, gboolean transparent)
+nr_arena_group_set_transparent (NRArenaGroup *group, unsigned int transparent)
 {
-	g_return_if_fail (group != NULL);
-	g_return_if_fail (NR_IS_ARENA_GROUP (group));
+	nr_return_if_fail (group != NULL);
+	nr_return_if_fail (NR_IS_ARENA_GROUP (group));
 
 	group->transparent = transparent;
 }
 
 void
-nr_arena_group_set_child_transform (NRArenaGroup *group, NRMatrixD *t)
+nr_arena_group_set_child_transform (NRArenaGroup *group, NRMatrixF *t)
 {
-	group->child_transform = *t;
+	if (!t) t = &NR_MATRIX_F_IDENTITY;
 
-	nr_arena_item_request_update (NR_ARENA_ITEM (group), NR_ARENA_ITEM_STATE_ALL, TRUE);
+	if (!NR_MATRIX_DF_TEST_CLOSE (t, &group->child_transform, NR_EPSILON_F)) {
+		nr_arena_item_request_render (NR_ARENA_ITEM (group));
+		group->child_transform = *t;
+		nr_arena_item_request_update (NR_ARENA_ITEM (group), NR_ARENA_ITEM_STATE_ALL, TRUE);
+	}
 }
 
 
