@@ -24,6 +24,7 @@
 #include <libnr/nr-blit.h>
 #include <libnr/nr-stroke.h>
 #include <libnr/nr-svp-render.h>
+#include <libnr/nr-pixblock-line.h>
 
 #include <libnr/nr-svp-private.h>
 
@@ -31,8 +32,6 @@
 #include <libart_lgpl/art_bpath.h>
 #include <libart_lgpl/art_vpath_bpath.h>
 #include <libart_lgpl/art_vpath.h>
-#include <libart_lgpl/art_svp.h>
-#include <libart_lgpl/art_svp_vpath_stroke.h>
 
 #include "../style.h"
 #include "nr-arena.h"
@@ -120,6 +119,7 @@ nr_arena_shape_finalize (NRObject *object)
 		shape->markers = nr_arena_item_detach_unref (item, shape->markers);
 	}
 
+	if (shape->path) free (shape->path);
 	if (shape->fill_svp) nr_svp_free (shape->fill_svp);
 	if (shape->stroke_svp) nr_svp_free (shape->stroke_svp);
 	if (shape->fill_painter) sp_painter_free (shape->fill_painter);
@@ -214,6 +214,11 @@ nr_arena_shape_update (NRArenaItem *item, NRRectL *area, NRGC *gc, guint state,
 
 	beststate = NR_ARENA_ITEM_STATE_ALL;
 
+	if (shape->path) {
+		free (shape->path);
+		shape->path = NULL;
+	}
+
 	for (child = shape->markers; child != NULL; child = child->next) {
 		newstate = nr_arena_item_invoke_update (child, area, gc, state, reset);
 		beststate = beststate & newstate;
@@ -280,6 +285,9 @@ nr_arena_shape_update (NRArenaItem *item, NRRectL *area, NRGC *gc, guint state,
 	if ((shape->style->fill.type == SP_PAINT_TYPE_NONE) && (shape->style->stroke.type == SP_PAINT_TYPE_NONE)) return NR_ARENA_ITEM_STATE_ALL;
 
 	/* Build state data */
+	shape->ctm = gc->transform;
+	shape->path = nr_path_new_from_art_bpath (shape->curve->bpath);
+
 	if (shape->style->fill.type != SP_PAINT_TYPE_NONE) {
 		if ((shape->curve->end > 2) || (shape->curve->bpath[1].code == ART_CURVETO)) {
 			if (TRUE || !shape->fill_svp) {
@@ -316,7 +324,6 @@ nr_arena_shape_update (NRArenaItem *item, NRRectL *area, NRGC *gc, guint state,
 				shape->fill_svp = svpa;
 #endif
 			}
-			shape->ctm = gc->transform;
 		}
 	}
 
@@ -338,7 +345,7 @@ nr_arena_shape_update (NRArenaItem *item, NRRectL *area, NRGC *gc, guint state,
 			double dlen;
 			int i;
 			ArtVpath *vp, *pvp;
-			ArtSVP *asvp;
+			/* ArtSVP *asvp; */
 			vp = art_bez_path_to_vec (bp.path, 0.25);
 			pvp = art_vpath_perturb (vp);
 			art_free (vp);
@@ -358,6 +365,7 @@ nr_arena_shape_update (NRArenaItem *item, NRRectL *area, NRGC *gc, guint state,
 				pvp = vp;
 				g_free (dash.dash);
 			}
+#if 0
 			asvp = art_svp_vpath_stroke (pvp,
 						     shape->style->stroke_linejoin.value,
 						     shape->style->stroke_linecap.value,
@@ -366,6 +374,14 @@ nr_arena_shape_update (NRArenaItem *item, NRRectL *area, NRGC *gc, guint state,
 			art_free (pvp);
 			svl = nr_svl_from_art_svp (asvp);
 			art_svp_free (asvp);
+#else
+			svl = nr_vpath_stroke (pvp, NULL, width,
+					       shape->style->stroke_linecap.value,
+					       shape->style->stroke_linejoin.value,
+					       shape->style->stroke_miterlimit.value * M_PI / 180.0,
+					       0.25);
+			art_free (pvp);
+#endif
 		}
 		shape->stroke_svp = nr_svp_from_svl (svl, NULL);
 		nr_svl_free_list (svl);
@@ -422,6 +438,16 @@ nr_arena_shape_render (NRArenaItem *item, NRRectL *area, NRPixBlock *pb, unsigne
 
 	style = shape->style;
 
+	if (0 || (flags & NR_ARENA_ITEM_RENDER_WIREFRAME)) {
+		if (shape->path) {
+			NRMatrixF ctmf;
+			nr_matrix_f_from_d (&ctmf, &shape->ctm);
+			nr_pixblock_draw_path_rgba32 (pb, shape->path, &ctmf, 0x000000ff);
+			pb->empty = FALSE;
+		}
+		return item->state;
+	}
+
 	if (shape->fill_svp) {
 		NRPixBlock m;
 		guint32 rgba;
@@ -514,24 +540,7 @@ nr_arena_shape_clip (NRArenaItem *item, NRRectL *area, NRPixBlock *pb)
 	if (!shape->curve) return item->state;
 
 	if (shape->fill_svp) {
-		NRPixBlock m;
-		int x, y;
-
-		/* fixme: We can OR in one step (Lauris) */
-		nr_pixblock_setup_fast (&m, NR_PIXBLOCK_MODE_A8, area->x0, area->y0, area->x1, area->y1, TRUE);
-		nr_pixblock_render_svp_mask_or (&m, shape->fill_svp);
-
-		for (y = area->y0; y < area->y1; y++) {
-			unsigned char *s, *d;
-			s = NR_PIXBLOCK_PX (&m) + (y - area->y0) * m.rs;
-			d = NR_PIXBLOCK_PX (pb) + (y - area->y0) * pb->rs;
-			for (x = area->x0; x < area->x1; x++) {
-				*d = (NR_A7 (*s, *d) + 127) / 255;
-				d += 1;
-				s += 1;
-			}
-		}
-		nr_pixblock_release (&m);
+		nr_pixblock_render_svp_mask_or (pb, shape->fill_svp);
 		pb->empty = FALSE;
 	}
 
