@@ -20,11 +20,14 @@
 #endif
 
 #include <string.h>
-#include <glib.h>
+
 #include <gtk/gtkmain.h>
 #include <xml/repr.h>
+
 #include "helper/sp-marshal.h"
 #include "helper/sp-intl.h"
+
+#include "api.h"
 #include "sodipodi-private.h"
 #include "sp-object-repr.h"
 #include "sp-root.h"
@@ -42,9 +45,12 @@
 #define SP_DOCUMENT_UPDATE_PRIORITY (G_PRIORITY_HIGH_IDLE - 1)
 
 enum {
+	DESTROY,
 	MODIFIED,
 	URI_SET,
 	RESIZED,
+	BEGIN,
+	END,
 	LAST_SIGNAL
 };
 
@@ -60,6 +66,9 @@ gboolean sp_document_resource_list_free (gpointer key, gpointer value, gpointer
 static gboolean sp_document_warn_undo_stack (SPDocument *doc);
 #endif
 
+/* Schedule either idle or timeout */
+static void sp_document_schedule_timers (SPDocument *doc);
+
 static GObjectClass * parent_class;
 static guint signals[LAST_SIGNAL] = {0};
 static gint doc_count = 0;
@@ -92,8 +101,15 @@ sp_document_class_init (SPDocumentClass * klass)
 
 	parent_class = g_type_class_peek_parent (klass);
 
+	signals[DESTROY] =   g_signal_new ("destroy",
+					    G_TYPE_FROM_CLASS (klass),
+					    G_SIGNAL_RUN_FIRST,
+					    G_STRUCT_OFFSET (SPDocumentClass, destroy),
+					    NULL, NULL,
+					    sp_marshal_NONE__NONE,
+					    G_TYPE_NONE, 0);
 	signals[MODIFIED] = g_signal_new ("modified",
-					    G_TYPE_FROM_CLASS(klass),
+					    G_TYPE_FROM_CLASS (klass),
 					    G_SIGNAL_RUN_FIRST,
 					    G_STRUCT_OFFSET (SPDocumentClass, modified),
 					    NULL, NULL,
@@ -101,7 +117,7 @@ sp_document_class_init (SPDocumentClass * klass)
 					    G_TYPE_NONE, 1,
 					    G_TYPE_UINT);
 	signals[URI_SET] =    g_signal_new ("uri_set",
-					    G_TYPE_FROM_CLASS(klass),
+					    G_TYPE_FROM_CLASS (klass),
 					    G_SIGNAL_RUN_FIRST,
 					    G_STRUCT_OFFSET (SPDocumentClass, uri_set),
 					    NULL, NULL,
@@ -109,47 +125,40 @@ sp_document_class_init (SPDocumentClass * klass)
 					    G_TYPE_NONE, 1,
 					    G_TYPE_POINTER);
 	signals[RESIZED] =    g_signal_new ("resized",
-					    G_TYPE_FROM_CLASS(klass),
+					    G_TYPE_FROM_CLASS (klass),
 					    G_SIGNAL_RUN_FIRST,
 					    G_STRUCT_OFFSET (SPDocumentClass, uri_set),
 					    NULL, NULL,
 					    sp_marshal_NONE__DOUBLE_DOUBLE,
 					    G_TYPE_NONE, 2,
 					    G_TYPE_DOUBLE, G_TYPE_DOUBLE);
+	signals[BEGIN] =      g_signal_new ("begin",
+					    G_TYPE_FROM_CLASS (klass),
+					    G_SIGNAL_RUN_FIRST,
+					    G_STRUCT_OFFSET (SPDocumentClass, begin),
+					    NULL, NULL,
+					    sp_marshal_NONE__DOUBLE,
+					    G_TYPE_NONE, 1,
+					    G_TYPE_DOUBLE);
+	signals[BEGIN] =      g_signal_new ("end",
+					    G_TYPE_FROM_CLASS (klass),
+					    G_SIGNAL_RUN_FIRST,
+					    G_STRUCT_OFFSET (SPDocumentClass, end),
+					    NULL, NULL,
+					    sp_marshal_NONE__DOUBLE,
+					    G_TYPE_NONE, 1,
+					    G_TYPE_DOUBLE);
 	object_class->dispose = sp_document_dispose;
 }
 
 static void
 sp_document_init (SPDocument *doc)
 {
-	SPDocumentPrivate *p;
+	doc->priv = g_new0 (SPDocumentPrivate, 1);
 
-	doc->advertize = FALSE;
-	doc->keepalive = FALSE;
+	doc->priv->iddef = g_hash_table_new (g_str_hash, g_str_equal);
 
-	doc->modified_id = 0;
-
-	doc->rdoc = NULL;
-	doc->rroot = NULL;
-	doc->root = NULL;
-
-	doc->uri = NULL;
-	doc->base = NULL;
-	doc->name = NULL;
-
-	p = g_new (SPDocumentPrivate, 1);
-
-	p->iddef = g_hash_table_new (g_str_hash, g_str_equal);
-
-	p->resources = g_hash_table_new (g_str_hash, g_str_equal);
-
-	p->sensitive = FALSE;
-	p->partial = NULL;
-	p->undo_size = 0;
-	p->undo = NULL;
-	p->redo = NULL;
-
-	doc->priv = p;
+	doc->priv->resources = g_hash_table_new (g_str_hash, g_str_equal);
 }
 
 static void
@@ -161,8 +170,28 @@ sp_document_dispose (GObject *object)
 	doc = (SPDocument *) object;
 	priv = doc->priv;
 
+	g_signal_emit (G_OBJECT (doc), signals [DESTROY], 0);
+
 	if (priv) {
-		sodipodi_remove_document (doc);
+		if (priv->idle) {
+			/* Remove idle handler */
+			gtk_idle_remove (priv->idle);
+			priv->idle = 0;
+		}
+		if (priv->timeout) {
+			/* Remove timer */
+			gtk_timeout_remove (priv->timeout);
+			priv->timeout = 0;
+		}
+		while (priv->itqueue) {
+			/* free (priv->itqueue->data); */
+			priv->itqueue = g_slist_remove (priv->itqueue, priv->itqueue->data);
+		}
+		while (priv->etqueue) {
+			/* free (priv->etqueue->data); */
+			priv->etqueue = g_slist_remove (priv->etqueue, priv->etqueue->data);
+		}
+		if (priv->timers) free (priv->timers);
 
 		if (priv->partial) {
 			sp_repr_free_log (priv->partial);
@@ -180,7 +209,7 @@ sp_document_dispose (GObject *object)
 
 		if (priv->iddef) g_hash_table_destroy (priv->iddef);
 
-		if (doc->rdoc) sp_repr_document_unref (doc->rdoc);
+		if (doc->rdoc) sp_repr_doc_unref (doc->rdoc);
 
 		/* Free resources */
 		g_hash_table_foreach_remove (priv->resources, sp_document_resource_list_free, doc);
@@ -203,16 +232,6 @@ sp_document_dispose (GObject *object)
 		doc->uri = NULL;
 	}
 
-	if (doc->modified_id) {
-		gtk_idle_remove (doc->modified_id);
-		doc->modified_id = 0;
-	}
-
-	if (doc->keepalive) {
-		sodipodi_unref ();
-		doc->keepalive = FALSE;
-	}
-
 	G_OBJECT_CLASS (parent_class)->dispose (object);
 }
 
@@ -220,21 +239,16 @@ static SPDocument *
 sp_document_create (SPReprDoc *rdoc,
 		    const unsigned char *uri,
 		    const unsigned char *base,
-		    const unsigned char *name,
-		    unsigned int advertize,
-		    unsigned int keepalive)
+		    const unsigned char *name)
 {
 	SPDocument *document;
 	SPRepr *rroot;
 	guint version;
 
-	rroot = sp_repr_document_root (rdoc);
+	rroot = sp_repr_doc_get_root (rdoc);
 
 	document = g_object_new (SP_TYPE_DOCUMENT, NULL);
 
-	document->advertize = advertize;
-	document->keepalive = keepalive;
-
 	document->rdoc = rdoc;
 	document->rroot = rroot;
 
@@ -260,8 +274,8 @@ sp_document_create (SPReprDoc *rdoc,
 	sp_repr_set_attr (rroot, "xmlns:xlink", SP_NAMESPACE_XLINK);
 	/* End of quick hack */
 	/* Quick hack 2 - get default image size into document */
-	if (!sp_repr_attr (rroot, "width")) sp_repr_set_attr (rroot, "width", A4_WIDTH_STR);
-	if (!sp_repr_attr (rroot, "height")) sp_repr_set_attr (rroot, "height", A4_HEIGHT_STR);
+	if (!sp_repr_get_attr (rroot, "width")) sp_repr_set_attr (rroot, "width", A4_WIDTH_STR);
+	if (!sp_repr_get_attr (rroot, "height")) sp_repr_set_attr (rroot, "height", A4_HEIGHT_STR);
 	/* End of quick hack 2 */
 	/* Quick hack 3 - Set uri attributes */
 	if (uri) {
@@ -281,8 +295,12 @@ sp_document_create (SPReprDoc *rdoc,
 	/* Namedviews */
 	if (!sp_item_group_get_child_by_name ((SPGroup *) document->root, NULL, "sodipodi:namedview")) {
 		SPRepr *r;
-		r = sodipodi_get_repr (SODIPODI, "template.sodipodi:namedview");
-		if (!r) r = sp_repr_new ("sodipodi:namedview");
+		r = sp_config_node_get ("template.sodipodi:namedview", FALSE);
+		if (!r || strcmp (sp_repr_get_name (r), "sodipodi:namedview")) {
+			r = sp_repr_new ("sodipodi:namedview");
+		} else {
+			r = sp_repr_duplicate (r);
+		}
 		sp_repr_set_attr (r, "id", "base");
 		sp_repr_add_child (rroot, r, NULL);
 		sp_repr_unref (r);
@@ -297,19 +315,11 @@ sp_document_create (SPReprDoc *rdoc,
 		g_assert (SP_ROOT (document->root)->defs);
 	}
 
-	if (keepalive) {
-		sodipodi_ref ();
-	}
-
-	sp_document_set_undo_sensitive (document, TRUE);
-
-	sodipodi_add_document (document);
-
 	return document;
 }
 
 SPDocument *
-sp_document_new (const gchar *uri, unsigned int advertize, unsigned int keepalive)
+sp_document_new (const gchar *uri)
 {
 	SPDocument *doc;
 	SPReprDoc *rdoc;
@@ -319,13 +329,13 @@ sp_document_new (const gchar *uri, unsigned int advertize, unsigned int keepaliv
 		SPRepr *rroot;
 		unsigned char *s, *p;
 		/* Try to fetch repr from file */
-		rdoc = sp_repr_read_file (uri, SP_SVG_NS_URI);
+		rdoc = sp_repr_doc_new_from_file (uri, SP_SVG_NS_URI);
 		/* If file cannot be loaded, return NULL without warning */
 		if (rdoc == NULL) return NULL;
-		rroot = sp_repr_document_root (rdoc);
+		rroot = sp_repr_doc_get_root (rdoc);
 		/* If xml file is not svg, return NULL without warning */
 		/* fixme: destroy document */
-		if (strcmp (sp_repr_name (rroot), "svg") != 0) return NULL;
+		if (strcmp (sp_repr_get_name (rroot), "svg") != 0) return NULL;
 		s = g_strdup (uri);
 		p = strrchr (s, '/');
 		if (p) {
@@ -338,12 +348,19 @@ sp_document_new (const gchar *uri, unsigned int advertize, unsigned int keepaliv
 		}
 		g_free (s);
 	} else {
-		rdoc = sp_repr_document_new ("svg");
+		rdoc = sp_repr_doc_new ("svg");
 		base = NULL;
 		name = g_strdup_printf (_("New document %d"), ++doc_count);
 	}
 
-	doc = sp_document_create (rdoc, uri, base, name, advertize, keepalive);
+	doc = sp_document_create (rdoc, uri, base, name);
+
+
+	if (!uri) {
+		/* Quick hack - set xml:space */
+		sp_repr_set_attr (doc->rroot, "xml:space", "preserve");
+		/* End of quick hack 3 */
+	}
 
 	if (base) g_free (base);
 	if (name) g_free (name);
@@ -352,26 +369,26 @@ sp_document_new (const gchar *uri, unsigned int advertize, unsigned int keepaliv
 }
 
 SPDocument *
-sp_document_new_from_mem (const gchar *buffer, gint length, unsigned int advertize, unsigned int keepalive)
+sp_document_new_from_mem (const gchar *buffer, gint length)
 {
 	SPDocument *doc;
 	SPReprDoc *rdoc;
 	SPRepr *rroot;
 	unsigned char *name;
 
-	rdoc = sp_repr_read_mem (buffer, length, SP_SVG_NS_URI);
+	rdoc = sp_repr_doc_new_from_mem (buffer, length, SP_SVG_NS_URI);
 
 	/* If it cannot be loaded, return NULL without warning */
 	if (rdoc == NULL) return NULL;
 
-	rroot = sp_repr_document_root (rdoc);
+	rroot = sp_repr_doc_get_root (rdoc);
 	/* If xml file is not svg, return NULL without warning */
 	/* fixme: destroy document */
-	if (strcmp (sp_repr_name (rroot), "svg") != 0) return NULL;
+	if (strcmp (sp_repr_get_name (rroot), "svg") != 0) return NULL;
 
 	name = g_strdup_printf (_("Memory document %d"), ++doc_count);
 
-	doc = sp_document_create (rdoc, NULL, NULL, name, advertize, keepalive);
+	doc = sp_document_create (rdoc, NULL, NULL, name);
 
 	return doc;
 }
@@ -420,6 +437,30 @@ sp_document_height (SPDocument * document)
 	return SP_ROOT (document->root)->height.computed / 1.25;
 }
 
+const unsigned char *
+sp_document_get_uri (SPDocument *doc)
+{
+	return doc->uri;
+}
+
+const unsigned char *
+sp_document_get_name (SPDocument *doc)
+{
+	return doc->name;
+}
+
+SPReprDoc *
+sp_document_get_repr_doc (SPDocument *doc)
+{
+	return doc->rdoc;
+}
+
+SPRepr *
+sp_document_get_repr_root (SPDocument *doc)
+{
+	return doc->rroot;
+}
+
 void
 sp_document_set_uri (SPDocument *doc, const guchar *uri)
 {
@@ -474,40 +515,6 @@ sp_document_set_size_px (SPDocument *doc, gdouble width, gdouble height)
 	g_signal_emit (G_OBJECT (doc), signals [RESIZED], 0, width / 1.25, height / 1.25);
 }
 
-#if 0
-/* named views */
-
-const GSList *
-sp_document_namedview_list (SPDocument * document)
-{
-	g_return_val_if_fail (document != NULL, NULL);
-	g_return_val_if_fail (SP_IS_DOCUMENT (document), NULL);
-
-	return SP_ROOT (document->root)->namedviews;
-}
-
-SPNamedView *
-sp_document_namedview (SPDocument * document, const gchar * id)
-{
-	const GSList * nvl, * l;
-
-	g_return_val_if_fail (document != NULL, NULL);
-	g_return_val_if_fail (SP_IS_DOCUMENT (document), NULL);
-
-	nvl = SP_ROOT (document->root)->namedviews;
-	g_assert (nvl != NULL);
-
-	if (id == NULL) return SP_NAMEDVIEW (nvl->data);
-
-	for (l = nvl; l != NULL; l = l->next) {
-		if (strcmp (id, SP_OBJECT (l->data)->id) == 0)
-			return SP_NAMEDVIEW (l->data);
-	}
-
-	return NULL;
-}
-#endif
-
 void
 sp_document_def_id (SPDocument * document, const gchar * id, SPObject * object)
 {
@@ -525,7 +532,7 @@ sp_document_undef_id (SPDocument * document, const gchar * id)
 }
 
 SPObject *
-sp_document_lookup_id (SPDocument *doc, const gchar *id)
+sp_document_get_object_from_id (SPDocument *doc, const unsigned char *id)
 {
 	g_return_val_if_fail (doc != NULL, NULL);
 	g_return_val_if_fail (SP_IS_DOCUMENT (doc), NULL);
@@ -534,13 +541,25 @@ sp_document_lookup_id (SPDocument *doc, const gchar *id)
 	return g_hash_table_lookup (doc->priv->iddef, id);
 }
 
+SPObject *
+sp_document_get_object_from_repr (SPDocument *doc, SPRepr *repr)
+{
+	const unsigned char *id;
+	g_return_val_if_fail (doc != NULL, NULL);
+	g_return_val_if_fail (SP_IS_DOCUMENT (doc), NULL);
+	g_return_val_if_fail (repr != NULL, NULL);
+
+	id = sp_repr_get_attr (repr, "id");
+	return sp_document_get_object_from_id (doc, id);
+}
+
 /* Object modification root handler */
 
 void
 sp_document_request_modified (SPDocument *doc)
 {
-	if (!doc->modified_id) {
-		doc->modified_id = gtk_idle_add_priority (SP_DOCUMENT_UPDATE_PRIORITY, sp_document_idle_handler, doc);
+	if (!doc->priv->idle) {
+		sp_document_schedule_timers (doc);
 	}
 }
 
@@ -553,11 +572,9 @@ sp_document_ensure_up_to_date (SPDocument *doc)
 		lc -= 1;
 		if (lc < 0) {
 			g_warning ("More than 16 iterations while updating document '%s'", doc->uri);
-			if (doc->modified_id) {
-				/* Remove handler */
-				gtk_idle_remove (doc->modified_id);
-				doc->modified_id = 0;
-			}
+			/* doc->priv->need_update = 0; */
+			/* Idle will be scheduled, but rien a faire */
+			sp_document_schedule_timers (doc);
 			return FALSE;
 		}
 		/* Process updates */
@@ -579,61 +596,10 @@ sp_document_ensure_up_to_date (SPDocument *doc)
 		g_signal_emit (G_OBJECT (doc), signals [MODIFIED], 0,
 			       SP_OBJECT_MODIFIED_FLAG | SP_OBJECT_CHILD_MODIFIED_FLAG | SP_OBJECT_PARENT_MODIFIED_FLAG);
 	}
-	if (doc->modified_id) {
-		/* Remove handler */
-		gtk_idle_remove (doc->modified_id);
-		doc->modified_id = 0;
-	}
+	sp_document_schedule_timers (doc);
 	return TRUE;
 }
 
-static gint
-sp_document_idle_handler (gpointer data)
-{
-	SPDocument *doc;
-	int repeat;
-
-	doc = SP_DOCUMENT (data);
-
-#ifdef SP_DOCUMENT_DEBUG_IDLE
-	g_print ("->\n");
-#endif
-
-	/* Process updates */
-	if (doc->root->uflags) {
-		SPItemCtx ctx;
-		ctx.ctx.flags = 0;
-		nr_matrix_d_set_identity (&ctx.i2doc);
-		/* Set up viewport in case svg has it defined as percentages */
-		ctx.vp.x0 = 0.0;
-		ctx.vp.y0 = 0.0;
-		ctx.vp.x1 = 21.0 / 2.54 * 72.0 * 1.25;
-		ctx.vp.y1 = 29.7 / 2.54 * 72.0 * 1.25;
-		nr_matrix_d_set_identity (&ctx.i2vp);
-		sp_object_invoke_update (doc->root, (SPCtx *) &ctx, 0);
-		/* if (doc->root->uflags & SP_OBJECT_MODIFIED_FLAG) return TRUE; */
-	}
-
-	/* Emit "modified" signal on objects */
-	sp_object_invoke_modified (doc->root, 0);
-
-#ifdef SP_DOCUMENT_DEBUG_IDLE
-	g_print ("\n->");
-#endif
-
-	/* Emit our own "modified" signal */
-	g_signal_emit (G_OBJECT (doc), signals [MODIFIED], 0,
-			 SP_OBJECT_MODIFIED_FLAG | SP_OBJECT_CHILD_MODIFIED_FLAG | SP_OBJECT_PARENT_MODIFIED_FLAG);
-
-#ifdef SP_DOCUMENT_DEBUG_IDLE
-	g_print (" S ->\n");
-#endif
-
-	repeat = (doc->root->uflags || doc->root->mflags);
-	if (!repeat) doc->modified_id = 0;
-	return repeat;
-}
-
 SPObject *
 sp_document_add_repr (SPDocument *document, SPRepr *repr)
 {
@@ -651,7 +617,7 @@ sp_document_add_repr (SPDocument *document, SPRepr *repr)
 		sp_repr_append_child (SP_OBJECT_REPR (SP_DOCUMENT_DEFS(document)), repr);
 	}
 
-	return sp_document_lookup_id (document, sp_repr_attr (repr, "id"));
+	return sp_document_lookup_id (document, sp_repr_get_attr (repr, "id"));
 }
 
 /* Returns the sequence number of object */
@@ -837,3 +803,307 @@ sp_document_get_version (SPDocument *document, guint version_type)
 
 	return 0;
 }
+
+/*
+ * Timer
+ *
+ * Objects are required to use time services only from their parent document
+ */
+
+#ifdef WIN32
+#include <sys/timeb.h>
+#include <time.h>
+#else
+#include <sys/time.h>
+#include <unistd.h>
+#endif
+
+static double
+sp_document_get_abstime (void)
+{
+#ifdef WIN32
+        struct _timeb t;
+        _ftime (&t);
+        return (t.time + t.millitm / 1000.0);
+#else
+        struct timeval tv;
+        struct timezone tz;
+
+        gettimeofday (&tv, &tz);
+
+        return (tv.tv_sec + tv.tv_usec / 1000000.0);
+#endif
+}
+
+/* Begin document and start timer */
+double
+sp_document_begin (SPDocument *doc)
+{
+	g_return_val_if_fail (doc->begintime == 0.0, 0.0);
+	doc->begintime = sp_document_get_abstime ();
+	g_signal_emit (G_OBJECT (doc), signals [BEGIN], 0, doc->begintime);
+	return doc->begintime;
+}
+
+/* End document and stop timer */
+double
+sp_document_end (SPDocument *doc)
+{
+	double endtime;
+	g_return_val_if_fail (doc->begintime != 0.0, 0.0);
+	endtime = sp_document_get_abstime ();
+	g_signal_emit (G_OBJECT (doc), signals [BEGIN], 0, endtime);
+	doc->begintime = 0.0;
+	return endtime;
+}
+
+/* Get time in seconds from document begin */
+double
+sp_document_get_time (SPDocument *document)
+{
+	return sp_document_get_abstime () - document->begintime;
+}
+
+struct _SPDocTimer {
+	double time;
+	unsigned int flags : 1;
+	unsigned int awake : 1;
+	unsigned int (* callback) (double, void *);
+	void *data;
+};
+
+gint
+sp_doc_timer_compare (gconstpointer a, gconstpointer b)
+{
+	SPDocTimer *ta, *tb;
+	ta = (SPDocTimer *) a;
+	tb = (SPDocTimer *) b;
+	if (ta->time < tb->time) return -1;
+	if (ta->time > tb->time) return 1;
+	return 0;
+}
+
+/* Create and set up timer */
+unsigned int
+sp_document_add_timer (SPDocument *doc, double time, unsigned int flags,
+		       unsigned int (* callback) (double, void *),
+		       void *data)
+{
+	SPDocumentPrivate *priv;
+	SPDocTimer *timer;
+	unsigned int id;
+	priv = doc->priv;
+	for (id = 0; id < priv->timers_size; id++) {
+		if (!priv->timers[id].callback) break;
+	}
+	if (id >= priv->timers_size) {
+		priv->timers_size = MIN ((priv->timers_size << 1), 4);
+		priv->timers = realloc (priv->timers, priv->timers_size * sizeof (SPDocTimer));
+		memset (priv->timers + id, 0, (priv->timers_size - id) * sizeof (SPDocTimer));
+	}
+	timer = &priv->timers[id];
+	timer->time = time;
+	timer->flags = flags;
+	timer->awake = 1;
+	timer->callback = callback;
+	timer->data = data;
+	if (flags == SP_TIMER_EXACT) {
+		priv->etqueue = g_slist_insert_sorted (priv->etqueue, timer, sp_doc_timer_compare);
+	} else {
+		priv->itqueue = g_slist_prepend (priv->itqueue, timer);
+	}
+	return id;
+}
+
+/* Remove timer */
+void
+sp_document_remove_timer (SPDocument *doc, unsigned int id)
+{
+	SPDocumentPrivate *priv;
+	SPDocTimer *timer;
+	priv = doc->priv;
+	timer = &priv->timers[id];
+	if (timer->awake) {
+		if (timer->flags == SP_TIMER_EXACT) {
+			priv->etqueue = g_slist_remove (priv->etqueue, timer);
+		} else {
+			priv->itqueue = g_slist_remove (priv->itqueue, timer);
+		}
+	}
+	timer->callback = NULL;
+	timer->data = NULL;
+}
+
+/* Set up timer */
+void
+sp_document_set_timer (SPDocument *doc, unsigned int id, double time, unsigned int flags)
+{
+	SPDocumentPrivate *priv;
+	SPDocTimer *timer;
+	priv = doc->priv;
+	timer = &priv->timers[id];
+	/* Remove from queue */
+	if (timer->awake) {
+		if (timer->flags == SP_TIMER_EXACT) {
+			priv->etqueue = g_slist_remove (priv->etqueue, timer);
+		} else {
+			priv->itqueue = g_slist_remove (priv->itqueue, timer);
+		}
+	}
+	timer->time = time;
+	timer->flags = flags;
+	timer->awake = 1;
+	if (flags == SP_TIMER_EXACT) {
+		priv->etqueue = g_slist_insert_sorted (priv->etqueue, timer, sp_doc_timer_compare);
+	} else {
+		priv->itqueue = g_slist_prepend (priv->itqueue, timer);
+	}
+}
+
+static void
+sp_document_process_updates (SPDocument *doc)
+{
+	if (doc->root->uflags || doc->root->mflags) {
+		SPItemCtx ctx;
+		ctx.ctx.flags = 0;
+		nr_matrix_d_set_identity (&ctx.i2doc);
+		/* Set up viewport in case svg has it defined as percentages */
+		ctx.vp.x0 = 0.0;
+		ctx.vp.y0 = 0.0;
+		ctx.vp.x1 = 21.0 / 2.54 * 72.0 * 1.25;
+		ctx.vp.y1 = 29.7 / 2.54 * 72.0 * 1.25;
+		nr_matrix_d_set_identity (&ctx.i2vp);
+		sp_object_invoke_update (doc->root, (SPCtx *) &ctx, 0);
+		/* if (doc->root->uflags & SP_OBJECT_MODIFIED_FLAG) return TRUE; */
+
+		/* Emit "modified" signal on objects */
+		sp_object_invoke_modified (doc->root, 0);
+
+		/* Emit our own "modified" signal */
+		g_signal_emit (G_OBJECT (doc), signals [MODIFIED], 0,
+			       SP_OBJECT_MODIFIED_FLAG | SP_OBJECT_CHILD_MODIFIED_FLAG | SP_OBJECT_PARENT_MODIFIED_FLAG);
+	}
+}
+
+#define deltat 0.0
+
+static void
+sp_document_process_timers (SPDocument *doc)
+{
+	static int tsize = 0;
+	static int tlen = 0;
+	static SPDocTimer *timers = NULL;
+	SPDocumentPrivate *priv;
+	double ntime;
+	GSList *l;
+	int i;
+
+	priv = doc->priv;
+
+	ntime = sp_document_get_time (doc) + deltat;
+
+	tlen = 0;
+	while (priv->etqueue && (((SPDocTimer *) priv->etqueue->data)->time <= ntime)) {
+		if (tlen >= tsize) {
+			tsize = MAX (4, (tsize << 1));
+			timers = (SPDocTimer *) realloc (timers, tsize * sizeof (SPDocTimer));
+		}
+		timers[tlen++] = *((SPDocTimer *) priv->etqueue->data);
+		((SPDocTimer *) priv->etqueue->data)->awake = FALSE;
+		priv->etqueue = g_slist_remove (priv->etqueue, priv->etqueue->data);
+	}
+	for (l = priv->itqueue; l != NULL; l = l->next) {
+		if (tlen >= tsize) {
+			tsize = MAX (4, (tsize << 1));
+			timers = (SPDocTimer *) realloc (timers, tsize * sizeof (SPDocTimer));
+		}
+		timers[tlen++] = *((SPDocTimer *) l->data);
+	}
+	for (i = 0; i < tlen; i++) {
+		timers[i].callback (ntime, timers[i].data);
+	}
+}
+
+static int
+sp_document_timeout_handler (void *data)
+{
+	SPDocument *doc;
+
+	doc = SP_DOCUMENT (data);
+
+	/* Process updates if needed */
+	sp_document_process_updates (doc);
+	/* Process timers */
+	sp_document_process_timers (doc);
+
+	sp_document_schedule_timers (doc);
+
+	return FALSE;
+}
+
+static int
+sp_document_idle_handler (void *data)
+{
+	SPDocument *doc;
+
+	doc = SP_DOCUMENT (data);
+
+	/* Process updates if needed */
+	sp_document_process_updates (doc);
+	/* Process timers */
+	sp_document_process_timers (doc);
+
+	sp_document_schedule_timers (doc);
+
+	/* If idle not cleared we have to repeat */
+	return doc->priv->idle != 0;
+}
+
+static void
+sp_document_schedule_timers (SPDocument *doc)
+{
+	SPDocumentPrivate *priv;
+	double ctime, dtime;
+	unsigned int idle;
+	priv = doc->priv;
+	idle = FALSE;
+	dtime = 1e18;
+	if (doc->root && (doc->root->uflags || doc->root->mflags)) idle = TRUE;
+	if (priv->itqueue) idle = TRUE;
+	if (!idle && priv->etqueue) {
+		SPDocTimer *timer;
+		timer = (SPDocTimer *) priv->itqueue->data;
+		/* Get current time */
+		ctime = sp_document_get_time (doc);
+		dtime = timer->time - ctime;
+		if (dtime <= deltat) idle = TRUE;
+	}
+	/* We have to remove old timer always */
+	if (priv->timeout) {
+		/* Remove timer */
+		gtk_timeout_remove (priv->timeout);
+		priv->timeout = 0;
+	}
+	if (idle) {
+		if (!priv->idle) {
+			/* Register idle */
+			priv->idle = gtk_idle_add_priority (SP_DOCUMENT_UPDATE_PRIORITY, sp_document_idle_handler, doc);
+		}
+	} else if (dtime < 1000 * 365.25 * 86400) {
+		if (priv->idle) {
+			/* Remove idle handler */
+			gtk_idle_remove (priv->idle);
+			priv->idle = 0;
+		}
+		/* Register timeout */
+		priv->timeout = gtk_timeout_add ((int) dtime * 1000, sp_document_timeout_handler, doc);
+	} else {
+		if (priv->idle) {
+			/* Remove idle handler */
+			gtk_idle_remove (priv->idle);
+			priv->idle = 0;
+		}
+	}
+}
+
+
