@@ -18,7 +18,6 @@
 #include "xml/repr.h"
 #include "forward.h"
 
-
 /* Version type. See SPRoot for more detail */
 enum {
 	SP_VERSION_SVG,
@@ -26,14 +25,12 @@ enum {
 	SP_VERSION_ORIGINAL,
 };
 
-
 typedef struct _SPDocumentPrivate SPDocumentPrivate;
 
 struct _SPDocument {
 	GObject object;
 
 	unsigned int advertize : 1;
-	unsigned int keepalive : 1;
 
 	SPReprDoc *rdoc; /* Our SPReprDoc */
 	SPRepr *rroot; /* Root element of SPReprDoc */
@@ -48,16 +45,25 @@ struct _SPDocument {
 
 	/* Last action key */
 	const guchar *actionkey;
+
 	/* Handler ID */
-	guint modified_id;
+	/* guint modified_id; */
+
+	/* Begin time */
+	double begintime;
 };
 
 struct _SPDocumentClass {
 	GObjectClass parent_class;
 
+	void (* destroy) (SPDocument *document);
+
 	void (* modified) (SPDocument *document, guint flags);
 	void (* uri_set) (SPDocument *document, const guchar *uri);
 	void (* resized) (SPDocument *document, gdouble width, gdouble height);
+	/* Timer events */
+	void (* begin) (SPDocument *doc, double dtime);
+	void (* end) (SPDocument *doc, double dtime);
 };
 
 /*
@@ -65,8 +71,8 @@ struct _SPDocumentClass {
  * Public document appear in document list
  */
 
-SPDocument *sp_document_new (const gchar *uri, unsigned int advertize, unsigned int keepalive);
-SPDocument *sp_document_new_from_mem (const gchar *buffer, gint length, unsigned int advertize, unsigned int keepalive);
+SPDocument *sp_document_new (const gchar *uri);
+SPDocument *sp_document_new_from_mem (const gchar *buffer, gint length);
 
 SPDocument *sp_document_ref (SPDocument *doc);
 SPDocument *sp_document_unref (SPDocument *doc);
@@ -75,6 +81,8 @@ SPDocument *sp_document_unref (SPDocument *doc);
  * Access methods
  */
 
+SPReprDoc *sp_document_get_repr_doc (SPDocument *doc);
+SPRepr *sp_document_get_repr_root (SPDocument *doc);
 #define sp_document_repr_doc(d) (SP_DOCUMENT (d)->rdoc)
 #define sp_document_repr_root(d) (SP_DOCUMENT (d)->rroot)
 #define sp_document_root(d) (SP_DOCUMENT (d)->root)
@@ -82,14 +90,13 @@ SPDocument *sp_document_unref (SPDocument *doc);
 
 gdouble sp_document_width (SPDocument * document);
 gdouble sp_document_height (SPDocument * document);
+const unsigned char *sp_document_get_uri (SPDocument *doc);
+const unsigned char *sp_document_get_name (SPDocument *doc);
 #define SP_DOCUMENT_URI(d) (SP_DOCUMENT (d)->uri)
 #define SP_DOCUMENT_NAME(d) (SP_DOCUMENT (d)->name)
 #define SP_DOCUMENT_BASE(d) (SP_DOCUMENT (d)->base)
 
-#if 0
-const GSList * sp_document_namedview_list (SPDocument * document);
-SPNamedView *sp_document_namedview (SPDocument * document, const gchar * name);
-#endif
+#define sp_document_set_advertize(d,v) (((SPDocument *)(d))->advertize = ((v) != 0))
 
 /*
  * Dictionary
@@ -97,7 +104,9 @@ SPNamedView *sp_document_namedview (SPDocument * document, const gchar * name);
 
 void sp_document_def_id (SPDocument * document, const gchar * id, SPObject * object);
 void sp_document_undef_id (SPDocument * document, const gchar * id);
-SPObject * sp_document_lookup_id (SPDocument * document, const gchar * id);
+SPObject *sp_document_get_object_from_id (SPDocument *doc, const unsigned char *id);
+SPObject *sp_document_get_object_from_repr (SPDocument *doc, SPRepr *repr);
+#define sp_document_lookup_id sp_document_get_object_from_id
 
 /*
  * Undo & redo
@@ -120,8 +129,7 @@ gint sp_document_ensure_up_to_date (SPDocument *doc);
 
 /* Save all previous actions to stack, as one undo step */
 void sp_document_done (SPDocument *document);
-void sp_document_maybe_done (SPDocument *document, const guchar *key);
-
+void sp_document_maybe_done (SPDocument *document, const unsigned char *key);
 /* Cancel (and revert) current unsaved actions */
 void sp_document_cancel (SPDocument *document);
 
@@ -168,4 +176,31 @@ void sp_document_set_size_px (SPDocument *doc, gdouble width, gdouble height);
 
 guint sp_document_get_version (SPDocument *document, guint version_type);
 
+/*
+ * Timer
+ *
+ * Objects are required to use time services only from their parent document
+ */
+
+enum {
+	SP_TIMER_IDLE,
+	SP_TIMER_EXACT
+};
+
+/* Begin document and start timer */
+double sp_document_begin (SPDocument *doc);
+/* End document and stop timer */
+double sp_document_end (SPDocument *doc);
+/* Get time in seconds from document begin */
+double sp_document_get_time (SPDocument *doc);
+
+/* Create and set up timer */
+unsigned int sp_document_add_timer (SPDocument *doc, double time, unsigned int klass,
+				    unsigned int (* callback) (double, void *),
+				    void *data);
+/* Remove timer */
+void sp_document_remove_timer (SPDocument *doc, unsigned int id);
+/* Set up timer */
+void sp_document_set_timer (SPDocument *doc, unsigned int id, double time, unsigned int klass);
+
 #endif
