@@ -6,24 +6,29 @@
  * Author:
  *   Lauris Kaplinski <lauris@kaplinski.com>
  *
- * Copyright (C) 2000 Lauris Kaplinski
+ * Copyright (C) 2000-2004 Lauris Kaplinski
  * Copyright (C) 2000-2001 Ximian, Inc.
- * Copyright (C) 2002 Lauris Kaplinski
  *
  * Released under GNU GPL, read the file 'COPYING' for more information
  */
 
 #define DRAW_VERBOSE
 
+#define noDRAW_GRAB
+
 #ifdef HAVE_CONFIG_H
 #include <config.h>
 #endif
 
 #include <math.h>
 #include <string.h>
+#include <assert.h>
+
 #include <glib.h>
 #include <gdk/gdkkeysyms.h>
 #include <gtk/gtksignal.h>
+#include <gtk/gtkmain.h>
+#include "xml/repr-private.h"
 #include "macros.h"
 #include "xml/repr.h"
 #include "svg/svg.h"
@@ -35,6 +40,7 @@
 #include "helper/canvas-bpath.h"
 
 #include "enums.h"
+#include "verbs.h"
 #include "sodipodi.h"
 #include "document.h"
 #include "sp-path.h"
@@ -90,7 +96,7 @@ static void spdc_free_colors (SPDrawContext *dc);
 static SPDrawAnchor *test_inside (SPDrawContext * dc, gdouble wx, gdouble wy);
 static SPDrawAnchor *sp_draw_anchor_test (SPDrawAnchor *anchor, gdouble wx, gdouble wy, gboolean activate);
 
-static void fit_and_split (SPDrawContext * dc);
+/* static void fit_and_split (SPDrawContext * dc); */
 
 static SPDrawAnchor *sp_draw_anchor_new (SPDrawContext *dc, SPCurve *curve, gboolean start, gdouble x, gdouble y);
 static SPDrawAnchor *sp_draw_anchor_destroy (SPDrawAnchor *anchor);
@@ -138,7 +144,8 @@ sp_draw_context_class_init (SPDrawContextClass *klass)
 static void
 sp_draw_context_init (SPDrawContext *dc)
 {
-	dc->attach = FALSE;
+	dc->attach = 0;
+	dc->max_points = 32;
 
 	dc->red_color = 0xff00007f;
 	dc->blue_color = 0x0000ff7f;
@@ -190,12 +197,14 @@ sp_draw_context_setup (SPEventContext *ec)
 	/* Create red bpath */
 	dc->red_bpath = sp_canvas_bpath_new (SP_DT_SKETCH (ec->desktop), NULL);
 	sp_canvas_bpath_set_stroke (SP_CANVAS_BPATH (dc->red_bpath), dc->red_color, 1.0, SP_STROKE_LINEJOIN_MITER, SP_STROKE_LINECAP_BUTT);
+	sp_canvas_bpath_set_fill ((SPCanvasBPath *) dc->red_bpath, 0x00000000, 0);
 	/* Create red curve */
 	dc->red_curve = sp_curve_new_sized (4);
 
 	/* Create blue bpath */
 	dc->blue_bpath = sp_canvas_bpath_new (SP_DT_SKETCH (ec->desktop), NULL);
 	sp_canvas_bpath_set_stroke (SP_CANVAS_BPATH (dc->blue_bpath), dc->blue_color, 1.0, SP_STROKE_LINEJOIN_MITER, SP_STROKE_LINECAP_BUTT);
+	sp_canvas_bpath_set_fill ((SPCanvasBPath *) dc->blue_bpath, 0x00000000, 0);
 	/* Create blue curve */
 	dc->blue_curve = sp_curve_new_sized (8);
 
@@ -205,6 +214,8 @@ sp_draw_context_setup (SPEventContext *ec)
 	dc->green_anchor = NULL;
 
 	spdc_set_attach (dc, FALSE);
+
+	dc->npoints = 0;
 }
 
 static void
@@ -216,6 +227,7 @@ sp_draw_context_finish (SPEventContext *ec)
 
 	if (dc->grab) {
 		sp_canvas_item_ungrab (dc->grab, GDK_CURRENT_TIME);
+		dc->grab = NULL;
 	}
 
 	if (dc->selection) {
@@ -265,9 +277,8 @@ sp_draw_context_root_handler (SPEventContext *ec, GdkEvent *event)
 		break;
 	}
 
-	if (!ret) {
-		if (((SPEventContextClass *) draw_parent_class)->root_handler)
-			ret = ((SPEventContextClass *) draw_parent_class)->root_handler (ec, event);
+	if (!ret && ((SPEventContextClass *) draw_parent_class)->root_handler) {
+		ret = ((SPEventContextClass *) draw_parent_class)->root_handler (ec, event);
 	}
 
 	return ret;
@@ -627,19 +638,31 @@ test_inside (SPDrawContext *dc, gdouble wx, gdouble wy)
 	return active;
 }
 
+#if 0
 static void
 fit_and_split (SPDrawContext * dc)
 {
-	NRPointF b[4];
-	gdouble tolerance;
+	NRPointF b[8];
+	int nsegs;
+	double tolerance;
 
 	g_assert (dc->npoints > 1);
 
+	if (dc->npoints == 2) {
+		/* Update existing red curve */
+		sp_curve_reset (dc->red_curve);
+		sp_curve_moveto (dc->red_curve, dc->p[0].x, dc->p[0].y);
+		sp_curve_curveto (dc->red_curve, dc->p[0].x, dc->p[0].y, dc->p[1].x, dc->p[1].y, dc->p[1].x, dc->p[1].y);
+		sp_canvas_bpath_set_bpath (SP_CANVAS_BPATH (dc->red_bpath), dc->red_curve);
+		return;
+	}
+
 	tolerance = SP_EVENT_CONTEXT (dc)->desktop->w2d[0] * TOLERANCE;
 	tolerance = tolerance * tolerance;
 
-	if (sp_bezier_fit_cubic (b, dc->p, dc->npoints, tolerance) > 0 && dc->npoints < SP_DRAW_POINTS_MAX) {
-		/* Fit and draw and reset state */
+	nsegs = sp_bezier_fit_cubic (b, dc->p, dc->npoints, tolerance);
+	if ((nsegs == 1) && (dc->npoints <= dc->max_points)) {
+		/* Update existing red curve */
 		sp_curve_reset (dc->red_curve);
 		sp_curve_moveto (dc->red_curve, b[0].x, b[0].y);
 		sp_curve_curveto (dc->red_curve, b[1].x, b[1].y, b[2].x, b[2].y, b[3].x, b[3].y);
@@ -647,23 +670,27 @@ fit_and_split (SPDrawContext * dc)
 	} else {
 		SPCurve *curve;
 		SPCanvasItem *cshape;
-		/* Fit and draw and copy last point */
+		/* Append curve to green path and list */
 		g_assert (!sp_curve_empty (dc->red_curve));
 		sp_curve_append_continuous (dc->green_curve, dc->red_curve, 0.0625);
 		curve = sp_curve_copy (dc->red_curve);
-
-		/* fixme: */
 		cshape = sp_canvas_bpath_new (SP_DT_SKETCH (SP_EVENT_CONTEXT (dc)->desktop), curve);
 		sp_curve_unref (curve);
-		sp_canvas_bpath_set_stroke (SP_CANVAS_BPATH (cshape), dc->green_color, 1.0, SP_STROKE_LINEJOIN_MITER, SP_STROKE_LINECAP_BUTT);
-
+		sp_canvas_bpath_set_stroke (SP_CANVAS_BPATH (cshape),
+					    dc->green_color, 1.0,
+					    SP_STROKE_LINEJOIN_MITER, SP_STROKE_LINECAP_BUTT);
 		dc->green_bpaths = g_slist_prepend (dc->green_bpaths, cshape);
-
-		dc->p[0] = dc->p[dc->npoints - 2];
-		dc->p[1] = dc->p[dc->npoints - 1];
-		dc->npoints = 2;
+		/* Reset existing red curve */
+		sp_curve_reset (dc->red_curve);
+		sp_canvas_bpath_set_bpath (SP_CANVAS_BPATH (dc->red_bpath), dc->red_curve);
+		/* Copy last point as first */
+		dc->p[0] = dc->p[dc->npoints - 1];
+		dc->npoints = 1;
+		/* dc->p[1] = dc->p[dc->npoints - 1]; */
+		/* dc->npoints = 2; */
 	}
 }
+#endif
 
 static void
 spdc_reset_colors (SPDrawContext *dc)
@@ -824,7 +851,6 @@ static gint sp_pencil_context_root_handler (SPEventContext * event_context, GdkE
 static void spdc_set_startpoint (SPPencilContext *dc, NRPointF *p, guint state);
 static void spdc_set_endpoint (SPPencilContext *dc, NRPointF *p, guint state);
 static void spdc_finish_endpoint (SPPencilContext *dc, NRPointF *p, gboolean snap, guint state);
-static void spdc_add_freehand_point (SPPencilContext *dc, NRPointF *p, guint state);
 
 static SPDrawContextClass *pencil_parent_class;
 
@@ -866,7 +892,12 @@ sp_pencil_context_class_init (SPPencilContextClass *klass)
 static void
 sp_pencil_context_init (SPPencilContext *pc)
 {
+	SPEventContext *ec;
+	ec = (SPEventContext *) pc;
+	ec->verb = SP_VERB_CONTEXT_PENCIL;
 	pc->state = SP_PENCIL_CONTEXT_IDLE;
+
+	nr_synthesizer_setup (&pc->sz, 64, 1.0);
 }
 
 static void
@@ -876,9 +907,229 @@ sp_pencil_context_dispose (GObject *object)
 
 	pc = SP_PENCIL_CONTEXT (object);
 
+	if (pc->timeout) {
+		gtk_timeout_remove (pc->timeout);
+		pc->timeout = 0;
+	}
+
+	nr_synthesizer_release (&pc->sz);
+
 	G_OBJECT_CLASS (pencil_parent_class)->dispose (object);
 }
 
+#ifdef __USE_ISOC99
+#define ASSERT_VAL(v) assert (isfinite (v) && ((v) > -NR_HUGE_F) && ((v) < NR_HUGE_F))
+#else
+#define ASSERT_VAL(v)
+#endif
+
+static void
+sp_pencil_context_update_curves (SPPencilContext *pc)
+{
+	SPDrawContext *dc;
+	unsigned int blen, i;
+
+	dc = (SPDrawContext *) pc;
+
+	blen = pc->sz.numcpts - pc->bluestart;
+	if (blen > 31) {
+		SPCurve *curve;
+		SPCanvasItem *cshape;
+		unsigned int pos, i;
+		/* Append new segment to green list */
+		curve = sp_curve_new ();
+		pos = pc->bluestart;
+		ASSERT_VAL (pc->sz.cpts[pos].x);
+		ASSERT_VAL (pc->sz.cpts[pos].y);
+		sp_curve_moveto (curve, pc->sz.cpts[pos].x, pc->sz.cpts[pos].y);
+		for (i = 0; i < 10; i++) {
+			pos = pc->bluestart + 1 + 3 * i;
+			ASSERT_VAL (pc->sz.cpts[pos].x);
+			ASSERT_VAL (pc->sz.cpts[pos].y);
+			ASSERT_VAL (pc->sz.cpts[pos + 1].x);
+			ASSERT_VAL (pc->sz.cpts[pos + 1].y);
+			ASSERT_VAL (pc->sz.cpts[pos + 2].x);
+			ASSERT_VAL (pc->sz.cpts[pos + 2].y);
+			sp_curve_curveto (curve,
+					  pc->sz.cpts[pos].x, pc->sz.cpts[pos].y,
+					  pc->sz.cpts[pos + 1].x, pc->sz.cpts[pos + 1].y,
+					  pc->sz.cpts[pos + 2].x, pc->sz.cpts[pos + 2].y);
+		}
+		/* fixme: Something sounds fishy to me (Lauris) */
+		sp_curve_append_continuous (dc->green_curve, curve, 0.25);
+		cshape = sp_canvas_bpath_new (SP_DT_SKETCH (SP_EVENT_CONTEXT (dc)->desktop), curve);
+		sp_curve_unref (curve);
+		sp_canvas_bpath_set_stroke (SP_CANVAS_BPATH (cshape),
+					    dc->green_color, 1.0,
+					    SP_STROKE_LINEJOIN_MITER, SP_STROKE_LINECAP_BUTT);
+		sp_canvas_bpath_set_fill ((SPCanvasBPath *) cshape, 0x00000000, 0);
+		dc->green_bpaths = g_slist_prepend (dc->green_bpaths, cshape);
+		pc->bluestart += 30;
+	}
+
+	/* Blue */
+	sp_curve_reset (dc->blue_curve);
+	sp_curve_moveto (dc->blue_curve, pc->sz.cpts[pc->bluestart].x, pc->sz.cpts[pc->bluestart].y);
+	for (i = pc->bluestart + 1; i < pc->sz.numcpts; i += 3) {
+		ASSERT_VAL (pc->sz.cpts[i].x);
+		ASSERT_VAL (pc->sz.cpts[i].y);
+		ASSERT_VAL (pc->sz.cpts[i + 1].x);
+		ASSERT_VAL (pc->sz.cpts[i + 1].y);
+		ASSERT_VAL (pc->sz.cpts[i + 2].x);
+		ASSERT_VAL (pc->sz.cpts[i + 2].y);
+		sp_curve_curveto (dc->blue_curve,
+				  pc->sz.cpts[i].x, pc->sz.cpts[i].y,
+				  pc->sz.cpts[i + 1].x, pc->sz.cpts[i + 1].y,
+				  pc->sz.cpts[i + 2].x, pc->sz.cpts[i + 2].y);
+	}
+	sp_canvas_bpath_set_bpath ((SPCanvasBPath *) dc->blue_bpath, dc->blue_curve);
+	/* Red */
+	sp_curve_reset (dc->red_curve);
+	if (pc->sz.fsegs > 0) {
+		ASSERT_VAL (pc->sz.fpts[0].x);
+		ASSERT_VAL (pc->sz.fpts[0].y);
+		sp_curve_moveto (dc->red_curve, pc->sz.fpts[0].x, pc->sz.fpts[0].y);
+		ASSERT_VAL (pc->sz.fpts[1].x);
+		ASSERT_VAL (pc->sz.fpts[1].y);
+		ASSERT_VAL (pc->sz.fpts[2].x);
+		ASSERT_VAL (pc->sz.fpts[2].y);
+		ASSERT_VAL (pc->sz.fpts[3].x);
+		ASSERT_VAL (pc->sz.fpts[3].y);
+		sp_curve_curveto (dc->red_curve,
+				  pc->sz.fpts[1].x, pc->sz.fpts[1].y,
+				  pc->sz.fpts[2].x, pc->sz.fpts[2].y,
+				  pc->sz.fpts[3].x, pc->sz.fpts[3].y);
+	}
+	if (pc->sz.fsegs > 1) {
+		ASSERT_VAL (pc->sz.fpts[5].x);
+		ASSERT_VAL (pc->sz.fpts[5].y);
+		ASSERT_VAL (pc->sz.fpts[6].x);
+		ASSERT_VAL (pc->sz.fpts[6].y);
+		ASSERT_VAL (pc->sz.fpts[7].x);
+		ASSERT_VAL (pc->sz.fpts[7].y);
+		sp_curve_curveto (dc->red_curve,
+				  pc->sz.fpts[5].x, pc->sz.fpts[5].y,
+				  pc->sz.fpts[6].x, pc->sz.fpts[6].y,
+				  pc->sz.fpts[7].x, pc->sz.fpts[7].y);
+	}
+	sp_canvas_bpath_set_bpath ((SPCanvasBPath *) dc->red_bpath, dc->red_curve);
+}
+
+#define FLERP(f0,f1,p) ((f0) + ((f1) - (f0)) * (p))
+
+#define MASSVAL 0.0
+#define SP_PENCIL_MASS FLERP (0.025, 160.0, MASSVAL)
+#define DRAGVAL 0.85
+#define SP_PENCIL_DRAG FLERP (0.0, 1.0, DRAGVAL * DRAGVAL)
+
+#define SAMPLE_TIMEOUT 50
+
+static void
+sp_pencil_draw (SPPencilContext *pc, double xd, double yd, double time)
+{
+	static int distance = 0;
+	SPDesktop *dt;
+	NRPointD fval;
+	NRPointF p;
+	double len;
+	double dtime;
+	double dx, dy;
+
+	dt = ((SPEventContext *) pc)->desktop;
+
+	dtime = (time - pc->tval);
+	if (dtime <= 0.0) return;
+	if (dtime > 1.0) dtime = 1.0;
+
+	dx = xd - pc->ppos.x;
+	dy = yd - pc->ppos.y;
+	len = hypot (dx, dy);
+
+	/* Skip if changing direction with 2 pixel force */
+	if (((dx * pc->fval.x + dy * pc->fval.y) < 0.0) && (len <= 5.0)) return;
+
+	if ((len > 4.0) || (dtime > 0.02)) {
+		double x0, y0, t0, l, s;
+		x0 = pc->ppos.x;
+		y0 = pc->ppos.y;
+		t0 = pc->tval;
+		s = MIN (4.0, 0.1 * len / dtime);
+		for (l = s; l < len; l += s) {
+			double lf, x, y, t;
+			lf = MIN (l, len) / len;
+			x = x0 + lf * dx;
+			y = y0 + lf * dy;
+			t = t0 + lf * dtime;
+
+			/* Calculate force and acceleration */
+			fval.x = x - pc->ppos.x;
+			fval.y = y - pc->ppos.y;
+			/* if (len < 4.0) return; */
+			pc->fval = fval;
+
+			/* Apply acceleration */
+			pc->pvel.x += fval.x / SP_PENCIL_MASS;
+			pc->pvel.y += fval.y / SP_PENCIL_MASS;
+			/* Apply drag */
+			pc->pvel.x *= (1.0 - SP_PENCIL_DRAG);
+			pc->pvel.y *= (1.0 - SP_PENCIL_DRAG);
+
+			/* Advance position */
+			pc->ppos.x += (t - pc->tval) * pc->pvel.x;
+			pc->ppos.y += (t - pc->tval) * pc->pvel.y;
+
+			pc->tval = t;
+		}
+
+		dtime = (time - pc->tval);
+	}
+
+	if (dtime > 0.0) {
+		/* Calculate force and acceleration */
+		fval.x = xd - pc->ppos.x;
+		fval.y = yd - pc->ppos.y;
+		/* if (len < 4.0) return; */
+		/* Skip if changing direction with 2 pixel force */
+		pc->fval = fval;
+
+		/* Apply acceleration */
+		pc->pvel.x += fval.x / SP_PENCIL_MASS;
+		pc->pvel.y += fval.y / SP_PENCIL_MASS;
+		/* Apply drag */
+		pc->pvel.x *= (1.0 - SP_PENCIL_DRAG);
+		pc->pvel.y *= (1.0 - SP_PENCIL_DRAG);
+
+		/* Advance position */
+		pc->ppos.x += (time - pc->tval) * pc->pvel.x;
+		pc->ppos.y += (time - pc->tval) * pc->pvel.y;
+
+		pc->tval = time;
+	}
+
+	sp_desktop_w2d_xy_point (dt, &p, pc->ppos.x, pc->ppos.y);
+	nr_synthesizer_add_point (&pc->sz, p.x, p.y, distance++);
+	sp_pencil_context_update_curves (pc);
+}
+
+static int
+sp_pencil_timeout (gpointer data)
+{
+	SPPencilContext *pc;
+	SPDesktop *dt;
+	double xd, yd;
+	int x, y;
+
+	pc = (SPPencilContext *) data;
+	dt = ((SPEventContext *) pc)->desktop;
+
+	gtk_widget_get_pointer ((GtkWidget *) SP_DT_CANVAS (dt), &x, &y);
+	sp_canvas_window_to_world (SP_DT_CANVAS (dt), x, y, &xd, &yd);
+
+	sp_pencil_draw (pc, xd, yd, sp_document_get_time (SP_DT_DOCUMENT (dt)));
+
+	return TRUE;
+}
+
 gint
 sp_pencil_context_root_handler (SPEventContext *ec, GdkEvent *event)
 {
@@ -886,9 +1137,11 @@ sp_pencil_context_root_handler (SPEventContext *ec, GdkEvent *event)
 	SPPencilContext *pc;
 	SPDesktop *dt;
 	NRPointF p;
-	gint ret;
+	int ret;
 	SPDrawAnchor *anchor;
-	NRPointF fp;
+	double dx, dy;
+
+	static double distance = 0.0;
 
 	dc = SP_DRAW_CONTEXT (ec);
 	pc = SP_PENCIL_CONTEXT (ec);
@@ -899,16 +1152,21 @@ sp_pencil_context_root_handler (SPEventContext *ec, GdkEvent *event)
 	switch (event->type) {
 	case GDK_BUTTON_PRESS:
 		if (event->button.button == 1) {
-			NRPointF fp;
+			NRMatrixD w2dt;
+			double tolerance;
 #if 0
 			/* Grab mouse, so release will not pass unnoticed */
 			dc->grab = SP_CANVAS_ITEM (dt->acetate);
 			sp_canvas_item_grab (dc->grab, SPDC_EVENT_MASK, NULL, event->button.time);
 #endif
+			pc->tval = sp_document_get_time (SP_DT_DOCUMENT (dt));
+			pc->cpos.x = pc->ppos.x = event->button.x;
+			pc->cpos.y = pc->ppos.y = event->button.y;
+			pc->pvel.x = pc->pvel.y = 0.0;
+			pc->fval.x = pc->fval.y = 0.0;
 			/* Find desktop coordinates */
-			sp_desktop_w2d_xy_point (dt, &fp, event->button.x, event->button.y);
-			p.x = fp.x;
-			p.y = fp.y;
+			sp_desktop_w2d_xy_point (dt, &p, event->button.x, event->button.y);
+
 			/* Test, whether we hit any anchor */
 			anchor = test_inside (dc, event->button.x, event->button.y);
 
@@ -923,14 +1181,30 @@ sp_pencil_context_root_handler (SPEventContext *ec, GdkEvent *event)
 					p = anchor->dp;
 				}
 				dc->sa = anchor;
+
+#if 1
+				sp_desktop_w2dt_affine (dt, &w2dt);
+				tolerance = 1.0 * NR_MATRIX_DF_EXPANSION (&w2dt);
+				pc->sz.tolerance2 = tolerance * tolerance;
+				nr_synthesizer_begin (&pc->sz, p.x, p.y, distance++);
+				pc->bluestart = 0;
+#endif
+
 				spdc_set_startpoint (pc, &p, event->button.state);
+
 				ret = TRUE;
 				break;
 			}
 		}
 		break;
 	case GDK_MOTION_NOTIFY:
-#if 1
+		dx = event->motion.x - pc->cpos.x;
+		dy = event->motion.y - pc->cpos.y;
+		/* Skip one-pixel noise */
+		if ((dx * dx + dy * dy) <= 2) break;
+		pc->cpos.x = event->motion.x;
+		pc->cpos.y = event->motion.y;
+#ifdef DRAW_GRAB
 		if ((event->motion.state & GDK_BUTTON1_MASK) && !dc->grab) {
 			/* Grab mouse, so release will not pass unnoticed */
 			dc->grab = SP_CANVAS_ITEM (dt->acetate);
@@ -938,9 +1212,7 @@ sp_pencil_context_root_handler (SPEventContext *ec, GdkEvent *event)
 		}
 #endif
 		/* Find desktop coordinates */
-		sp_desktop_w2d_xy_point (dt, &fp, event->motion.x, event->motion.y);
-		p.x = fp.x;
-		p.y = fp.y;
+		sp_desktop_w2d_xy_point (dt, &p, event->motion.x, event->motion.y);
 		/* Test, whether we hit any anchor */
 		anchor = test_inside (dc, event->button.x, event->button.y);
 
@@ -959,15 +1231,25 @@ sp_pencil_context_root_handler (SPEventContext *ec, GdkEvent *event)
 				pc->state = SP_PENCIL_CONTEXT_FREEHAND;
 				if (!dc->sa && !dc->green_anchor) {
 					/* Create green anchor */
-					dc->green_anchor = sp_draw_anchor_new (dc, dc->green_curve, TRUE, dc->p[0].x, dc->p[0].y);
+					dc->green_anchor = sp_draw_anchor_new (dc, dc->green_curve, TRUE,
+									       dc->p[0].x, dc->p[0].y);
 				}
+#if 0
 				/* fixme: I am not sure, whether we want to snap to anchors in middle of freehand (Lauris) */
 				if (anchor) {
 					p = anchor->dp;
 				} else {
 					sp_desktop_free_snap (dt, &p);
 				}
+#endif
+				if (!pc->timeout) {
+					pc->timeout = gtk_timeout_add (SAMPLE_TIMEOUT, sp_pencil_timeout, pc);
+				}
+				sp_pencil_draw (pc, event->motion.x, event->motion.y, sp_document_get_time (SP_DT_DOCUMENT (dt)));
+#if 0
 				spdc_add_freehand_point (pc, &p, event->motion.state);
+#endif
+
 				ret = TRUE;
 			}
 			break;
@@ -1001,6 +1283,11 @@ sp_pencil_context_root_handler (SPEventContext *ec, GdkEvent *event)
 				ret = TRUE;
 				break;
 			case SP_PENCIL_CONTEXT_FREEHAND:
+				/* Remove timeout */
+				if (pc->timeout) {
+					gtk_timeout_remove (pc->timeout);
+					pc->timeout = 0;
+				}
 				/* Finish segment now */
 				/* fixme: Clean up what follows (Lauris) */
 				if (anchor) {
@@ -1021,13 +1308,11 @@ sp_pencil_context_root_handler (SPEventContext *ec, GdkEvent *event)
 			default:
 				break;
 			}
-#if 1
 			if (dc->grab) {
 				/* Release grab now */
 				sp_canvas_item_ungrab (dc->grab, event->button.time);
 				dc->grab = NULL;
 			}
-#endif
 			dc->grab = NULL;
 			ret = TRUE;
 		}
@@ -1138,6 +1423,7 @@ spdc_finish_endpoint (SPPencilContext *pc, NRPointF *p, gboolean snap, guint sta
 	}
 }
 
+#if 0
 static void
 spdc_add_freehand_point (SPPencilContext *pc, NRPointF *p, guint state)
 {
@@ -1152,6 +1438,7 @@ spdc_add_freehand_point (SPPencilContext *pc, NRPointF *p, guint state)
 		fit_and_split (dc);
 	}
 }
+#endif
 
 /* Pen context */
 
@@ -1213,14 +1500,12 @@ sp_pen_context_class_init (SPPenContextClass *klass)
 static void
 sp_pen_context_init (SPPenContext *pc)
 {
+	SPEventContext *ec;
+	ec = (SPEventContext *) pc;
+	ec->verb = SP_VERB_CONTEXT_PEN;
+
 	pc->mode = SP_PEN_CONTEXT_MODE_CLICK;
-
 	pc->state = SP_PEN_CONTEXT_POINT;
-
-	pc->c0 = NULL;
-	pc->c1 = NULL;
-	pc->cl0 = NULL;
-	pc->cl1 = NULL;
 }
 
 static void
@@ -1323,11 +1608,6 @@ sp_pen_context_root_handler (SPEventContext *ec, GdkEvent *event)
 	switch (event->type) {
 	case GDK_BUTTON_PRESS:
 		if (event->button.button == 1) {
-#if 0
-			/* Grab mouse, so release will not pass unnoticed */
-			dc->grab = SP_CANVAS_ITEM (dt->acetate);
-			sp_canvas_item_grab (dc->grab, SPDC_EVENT_MASK, NULL, event->button.time);
-#endif
 			/* Find desktop coordinates */
 			sp_desktop_w2d_xy_point (dt, &fp, event->button.x, event->button.y);
 			p.x = fp.x;
@@ -1396,7 +1676,7 @@ sp_pen_context_root_handler (SPEventContext *ec, GdkEvent *event)
 		}
 		break;
 	case GDK_MOTION_NOTIFY:
-#if 1
+#ifdef DRAW_GRAB
 		if ((event->motion.state & GDK_BUTTON1_MASK) && !dc->grab) {
 			/* Grab mouse, so release will not pass unnoticed */
 			dc->grab = SP_CANVAS_ITEM (dt->acetate);
@@ -1538,13 +1818,11 @@ sp_pen_context_root_handler (SPEventContext *ec, GdkEvent *event)
 			default:
 				break;
 			}
-#if 1
 			if (dc->grab) {
 				/* Release grab now */
 				sp_canvas_item_ungrab (dc->grab, event->button.time);
 				dc->grab = NULL;
 			}
-#endif
 			ret = TRUE;
 		}
 		break;
