@@ -237,18 +237,14 @@ sp_event_context_private_root_handler (SPEventContext *event_context, GdkEvent *
 				ret = TRUE;
 			}
 			break;
-		case GDK_i: // Ctrl i - import file
-			if (event->key.state & GDK_CONTROL_MASK) {
-				sp_file_import (NULL);
-				ret = TRUE;
-			}
-			break;
+#if 0
 		case GDK_q: // Ctrl q - quit
 			if (event->key.state & GDK_CONTROL_MASK) {
 				sp_file_exit ();
 				ret = TRUE;
 			}
 			break;
+#endif
 		case GDK_Left: // Ctrl Left 
 			if (event->key.state & GDK_CONTROL_MASK) {
 				sp_desktop_scroll_relative_canvas (event_context->desktop, 10, 0);
@@ -534,6 +530,13 @@ set_event_location (SPDesktop * desktop, GdkEvent * event)
 	sp_desktop_set_coordinate_status (desktop, p.x, p.y, 0);
 }
 
+static void
+menu_position_handler (GtkMenu *menu, gint *x, gint *y, gboolean *push_in, gpointer user_data)
+{
+	*x += 1;
+	*y += 1;
+}
+
 void
 sp_event_root_menu_popup (SPDesktop *desktop, SPItem *item, GdkEvent *event)
 {
@@ -548,7 +551,7 @@ sp_event_root_menu_popup (SPDesktop *desktop, SPItem *item, GdkEvent *event)
 
 	switch (event->type) {
 	case GDK_BUTTON_PRESS:
-		gtk_menu_popup (GTK_MENU (menu), NULL, NULL, 0, NULL, event->button.button, event->button.time);
+		gtk_menu_popup (GTK_MENU (menu), NULL, NULL, menu_position_handler, NULL, event->button.button, event->button.time);
 		break;
 	case GDK_KEY_PRESS:
 		gtk_menu_popup (GTK_MENU (menu), NULL, NULL, 0, NULL, 0, event->key.time);
