@@ -51,11 +51,7 @@
 /* fixme: (Lauris) */
 #include "sp-namedview.h"
 
-#ifdef WITH_MODULES
 #include "module.h"
-#include "modules/sp-module-sys.h"
-#include "modules/sp-module-doc.h"
-#endif /* WITH_MODULES */
 
 #ifdef WITH_KDE
 #include "modules/kde.h"
@@ -74,7 +70,7 @@ void sp_file_new (void)
 	SPDocument * doc;
 	SPViewWidget *dtw;
 
-	doc = sp_document_new (NULL, TRUE, TRUE);
+	doc = sodipodi_document_new (NULL, TRUE, TRUE);
 	g_return_if_fail (doc != NULL);
 
 	dtw = sp_desktop_widget_new (sp_document_namedview (doc, NULL));
@@ -91,8 +87,10 @@ sp_file_open (const unsigned char *uri, const unsigned char *key)
 	SPModule *mod;
 
 	doc = NULL;
+	/* Get preferred module key */
 	if (!key) key = SP_MODULE_KEY_INPUT_DEFAULT;
-	mod = sp_module_system_get (key);
+	mod = sp_module_find (key);
+	/* Open file with given module */
 	if (mod) doc = sp_module_input_document_open (SP_MODULE_INPUT (mod), uri, TRUE, TRUE);
 	sp_module_unref (mod);
 	if (doc) {
@@ -103,7 +101,7 @@ sp_file_open (const unsigned char *uri, const unsigned char *key)
 	}
 }
 
-#ifndef WITH_KDE
+#if 0
 static void
 file_open_ok (GtkWidget *widget, GtkFileSelection *fs)
 {
@@ -141,51 +139,58 @@ file_open_cancel (GtkButton *b, GtkFileSelection *fs)
 {
 	gtk_widget_destroy (GTK_WIDGET (fs));
 }
+#endif
 
 static void
 sp_file_open_dialog_type_selected (SPMenu *menu, gpointer itemdata, GObject *fsel)
 {
 	g_object_set_data (fsel, "type-key", itemdata);
 }
-#endif
 
 void sp_file_open_dialog (gpointer object, gpointer data)
 {
-#ifdef WITH_KDE
-	char *filename;
-	if (!open_path) open_path = g_strconcat (SODIPODI_DOCDIR, G_DIR_SEPARATOR_S, NULL);
-	filename = sp_kde_get_open_filename (open_path, "*.svg *.svgz|SVG files\n*.*|All files", _("Select file to open"));
-	if (filename) {
-		if (open_path) g_free (open_path);
-		open_path = g_dirname (filename);
-		if (open_path) open_path = g_strconcat (open_path, G_DIR_SEPARATOR_S, NULL);
-		sp_file_open (filename, NULL);
-		g_free (filename);
-	}
-#else
-#ifdef WIN32
-	char *filename;
-	if (!open_path) open_path = g_strconcat (SODIPODI_DOCDIR, G_DIR_SEPARATOR_S, NULL);
-	filename = sp_win32_get_open_filename (open_path, "SVG files|*.svg;*.svgz|All files|*|", _("Select file to open"));
-	if (filename) {
-		if (open_path) g_free (open_path);
-		open_path = g_dirname (filename);
-		if (open_path) open_path = g_strconcat (open_path, G_DIR_SEPARATOR_S, NULL);
-		sp_file_open (filename, NULL);
-		free (filename);
-	}
-#else
 	GtkFileSelection *fsel;
 	GtkWidget *hb, *l, *om, *m;
+	int res;
 
+	/* Initialize default open location */
 	if (!open_path) open_path = g_strconcat (SODIPODI_DOCDIR, G_DIR_SEPARATOR_S, NULL);
 
+#ifdef WITH_KDE
+	if (!use_gtk_dialogs) {
+		char *filename;
+		filename = sp_kde_get_open_filename (open_path, "*.svg *.svgz|SVG files\n*.*|All files", _("Select file to open"));
+		if (filename) {
+			if (open_path) g_free (open_path);
+			open_path = g_dirname (filename);
+			if (open_path) open_path = g_strconcat (open_path, G_DIR_SEPARATOR_S, NULL);
+			sp_file_open (filename, NULL);
+			g_free (filename);
+		}
+		return;
+	}
+#endif
+#ifdef WIN32
+	if (!use_gtk_dialogs) {
+		char *filename;
+		filename = sp_win32_get_open_filename (open_path, "SVG files|*.svg;*.svgz|All files|*|", _("Select file to open"));
+		if (filename) {
+			if (open_path) g_free (open_path);
+			open_path = g_dirname (filename);
+			if (open_path) open_path = g_strconcat (open_path, G_DIR_SEPARATOR_S, NULL);
+			sp_file_open (filename, NULL);
+			free (filename);
+		}
+		return;
+	}
+#endif
+
 	fsel = (GtkFileSelection *) gtk_file_selection_new (_("Select file to open"));
 	gtk_file_selection_hide_fileop_buttons (fsel);
 
-	g_signal_connect (G_OBJECT (fsel->ok_button), "clicked", G_CALLBACK (file_open_ok), fsel);
-	g_signal_connect (G_OBJECT (fsel->cancel_button), "clicked", G_CALLBACK (file_open_cancel), fsel);
-
+	/* g_signal_connect (G_OBJECT (fsel->ok_button), "clicked", G_CALLBACK (file_open_ok), fsel); */
+	/* g_signal_connect (G_OBJECT (fsel->cancel_button), "clicked", G_CALLBACK (file_open_cancel), fsel); */
+	/* open_path = NULL; */
 	if (open_path) gtk_file_selection_set_filename (fsel, open_path);
 
 	/* Create file type box */
@@ -194,12 +199,8 @@ void sp_file_open_dialog (gpointer object, gpointer data)
 	om = gtk_option_menu_new ();
 	gtk_box_pack_end (GTK_BOX (hb), om, FALSE, FALSE, 0);
 	m = sp_menu_new ();
-#ifdef WITH_MODULES
 	sp_module_system_menu_open (SP_MENU (m));
-#else
-	sp_menu_append (SP_MENU (m), _("Scalable Vector Graphic (SVG)"), _("Sodipodi native file format and W3C standard"), NULL);
 	gtk_widget_set_sensitive (m, FALSE);
-#endif
 	g_object_set_data (G_OBJECT (fsel), "type-key", ((SPMenu *) m)->activedata);
 	g_signal_connect (G_OBJECT (m), "selected", G_CALLBACK (sp_file_open_dialog_type_selected), fsel);
 	gtk_option_menu_set_menu (GTK_OPTION_MENU (om), m);
@@ -207,9 +208,33 @@ void sp_file_open_dialog (gpointer object, gpointer data)
 	gtk_box_pack_end (GTK_BOX (hb), l, FALSE, FALSE, 0);
 	gtk_widget_show_all (hb);
 
-	gtk_widget_show ((GtkWidget *) fsel);
-#endif
-#endif
+	res = GTK_RESPONSE_OK;
+	while (res != GTK_RESPONSE_CANCEL) {
+		unsigned char *filename;
+		res = gtk_dialog_run ((GtkDialog *) fsel);
+		filename = g_strdup (gtk_file_selection_get_filename (fsel));
+		if (filename && g_file_test (filename, G_FILE_TEST_IS_DIR)) {
+			if (open_path) g_free (open_path);
+			if (filename[strlen(filename) - 1] != G_DIR_SEPARATOR) {
+				open_path = g_strconcat (filename, G_DIR_SEPARATOR_S, NULL);
+				g_free (filename);
+			} else {
+				open_path = filename;
+			}
+			gtk_file_selection_set_filename (fsel, open_path);
+		} else if (filename && *filename) {
+			gpointer key;
+			if (open_path) g_free (open_path);
+			open_path = g_dirname (filename);
+			if (open_path) open_path = g_strconcat (open_path, G_DIR_SEPARATOR_S, NULL);
+			key = g_object_get_data (G_OBJECT (fsel), "type-key");
+			sp_file_open (filename, key);
+			g_free (filename);
+			break;
+		}
+	}
+
+	gtk_widget_destroy ((GtkWidget *) fsel);
 }
 
 /* Save */
@@ -223,7 +248,7 @@ sp_file_do_save (SPDocument *doc, const unsigned char *uri, const unsigned char
 	if (!uri) return;
 
 	if (!key) key = SP_MODULE_KEY_OUTPUT_DEFAULT;
-	mod = sp_module_system_get (key);
+	mod = sp_module_find (key);
 	if (mod) sp_module_output_document_save (SP_MODULE_OUTPUT (mod), doc, uri);
 	sp_module_unref (mod);
 }
@@ -240,7 +265,7 @@ sp_file_save_dialog (SPDocument *doc)
 		sp_file_do_save (doc, filename, (spns) ? SP_MODULE_KEY_OUTPUT_SVG_SODIPODI : SP_MODULE_KEY_OUTPUT_SVG);
 		if (save_path) g_free (save_path);
 		save_path = g_dirname (filename);
-		save_path = g_strdup (save_path);
+		if (save_path) save_path = g_strconcat (save_path, G_DIR_SEPARATOR_S, NULL);
 		g_free (filename);
 	}
 #else
@@ -253,7 +278,7 @@ sp_file_save_dialog (SPDocument *doc)
 		sp_file_do_save (doc, filename, (spns) ? SP_MODULE_KEY_OUTPUT_SVG_SODIPODI : SP_MODULE_KEY_OUTPUT_SVG);
 		if (save_path) g_free (save_path);
 		save_path = g_dirname (filename);
-		save_path = g_strdup (save_path);
+		if (save_path) save_path = g_strconcat (save_path, G_DIR_SEPARATOR_S, NULL);
 		free (filename);
 	}
 #else
@@ -291,7 +316,7 @@ sp_file_save_dialog (SPDocument *doc)
 		sp_file_do_save (doc, filename, ((SPMenu *) menu)->activedata);
 		if (save_path) g_free (save_path);
 		save_path = g_dirname (filename);
-		save_path = g_strdup (save_path);
+		if (save_path) save_path = g_strconcat (save_path, G_DIR_SEPARATOR_S, NULL);
 	}
 
 	gtk_widget_destroy (dlg);
@@ -315,7 +340,7 @@ sp_file_save_document (SPDocument *doc)
 		sp_document_set_undo_sensitive (doc, FALSE);
 		sp_repr_set_attr (repr, "sodipodi:modified", NULL);
 		sp_document_set_undo_sensitive (doc, TRUE);
-		sp_repr_save_file (sp_document_repr_doc (doc), fn);
+		sp_repr_doc_write_file (sp_document_repr_doc (doc), fn);
 	}
 }
 
@@ -338,11 +363,6 @@ sp_file_save_as (gpointer object, gpointer data)
 static void
 sp_file_do_import (SPDocument *doc, const unsigned char *filename)
 {
-	SPRepr *rdoc;
-	const gchar *e, *docbase, *relname;
-	SPRepr * repr;
-	SPReprDoc * rnewdoc;
-
 	if (filename && g_file_test (filename, G_FILE_TEST_IS_DIR)) {
 		if (import_path) g_free (import_path);
 		if (filename[strlen(filename) - 1] != G_DIR_SEPARATOR) {
@@ -358,6 +378,24 @@ sp_file_do_import (SPDocument *doc, const unsigned char *filename)
 	import_path = g_dirname (filename);
 	if (import_path) import_path = g_strconcat (import_path, G_DIR_SEPARATOR_S, NULL);
 
+	sp_file_import (doc, filename);
+}
+
+#ifdef WIN32
+#define strcasecmp _stricmp
+
+#endif
+
+unsigned int
+sp_file_import (SPDocument *doc, const unsigned char *filename)
+{
+	static const char *imgext[] = {"png", "jpg", "jpeg", "gif", "tif", "tiff", "bmp", "xpm", NULL};
+	const unsigned char *e, *docbase, *relname;
+	unsigned int i;
+	SPReprDoc *rnewdoc;
+	SPRepr *repr;
+	SPRepr *rdoc;
+
 	rdoc = sp_document_repr_root (doc);
 
 	docbase = sp_repr_attr (rdoc, "sodipodi:docbase");
@@ -365,14 +403,14 @@ sp_file_do_import (SPDocument *doc, const unsigned char *filename)
 	/* fixme: this should be implemented with mime types */
 	e = sp_extension_from_path (filename);
 
-	if ((e == NULL) || (strcmp (e, "svg") == 0) || (strcmp (e, "xml") == 0)) {
+	if (!strcasecmp (e, "svg") || !strcasecmp (e, "svgz") || !strcasecmp (e, "xml")) {
 		SPRepr * newgroup;
 		const gchar * style;
 		SPRepr * child;
 
-		rnewdoc = sp_repr_read_file (filename, SP_SVG_NS_URI);
-		if (rnewdoc == NULL) return;
-		repr = sp_repr_document_root (rnewdoc);
+		rnewdoc = sp_repr_doc_new_from_file (filename, SP_SVG_NS_URI);
+		if (rnewdoc == NULL) return FALSE;
+		repr = sp_repr_doc_get_root (rnewdoc);
 		style = sp_repr_attr (repr, "style");
 
 		newgroup = sp_repr_new ("g");
@@ -384,40 +422,44 @@ sp_file_do_import (SPDocument *doc, const unsigned char *filename)
 			sp_repr_append_child (newgroup, newchild);
 		}
 
-		sp_repr_document_unref (rnewdoc);
+		sp_repr_doc_unref (rnewdoc);
 
 		sp_document_add_repr (doc, newgroup);
 		sp_repr_unref (newgroup);
 		sp_document_done (doc);
-		return;
+		return TRUE;
 	}
 
-	if ((strcmp (e, "png") == 0) ||
-	    (strcmp (e, "jpg") == 0) ||
-	    (strcmp (e, "jpeg") == 0) ||
-	    (strcmp (e, "bmp") == 0) ||
-	    (strcmp (e, "gif") == 0) ||
-	    (strcmp (e, "tiff") == 0) ||
-	    (strcmp (e, "xpm") == 0)) {
+	for (i = 0; imgext[i]; i++) {
+		if (!strcasecmp (e, imgext[i])) break;
+	}
+
+	if (imgext[i]) {
 		/* Try pixbuf */
+		unsigned char *osfn;
+		gsize bytesin, bytesout;
 		GdkPixbuf *pb;
-		pb = gdk_pixbuf_new_from_file (filename, NULL);
+		osfn = g_filename_from_utf8 (filename, strlen (filename), &bytesin, &bytesout, NULL);
+		pb = gdk_pixbuf_new_from_file (osfn, NULL);
+		g_free (osfn);
 		if (pb) {
 			/* We are readable */
 			repr = sp_repr_new ("image");
 			sp_repr_set_attr (repr, "xlink:href", relname);
 			sp_repr_set_attr (repr, "sodipodi:absref", filename);
-			sp_repr_set_double_attribute (repr, "width", gdk_pixbuf_get_width (pb));
-			sp_repr_set_double_attribute (repr, "height", gdk_pixbuf_get_height (pb));
+			sp_repr_set_double (repr, "width", gdk_pixbuf_get_width (pb));
+			sp_repr_set_double (repr, "height", gdk_pixbuf_get_height (pb));
 			sp_document_add_repr (doc, repr);
 			sp_repr_unref (repr);
 			sp_document_done (doc);
 			gdk_pixbuf_unref (pb);
+			return TRUE;
 		}
 	}
+	return FALSE;
 }
 
-void sp_file_import (GtkWidget * widget)
+void sp_file_import_dialog (GtkWidget * widget)
 {
         SPDocument *doc;
 #ifdef WITH_KDE
@@ -479,7 +521,7 @@ sp_file_print (void)
 {
 	SPDocument *doc;
 	doc = SP_ACTIVE_DOCUMENT;
-	if (doc) sp_print_document (doc, FALSE);
+	if (doc) sp_print_document (doc, FALSE, NULL);
 }
 
 void
@@ -487,7 +529,7 @@ sp_file_print_direct (void)
 {
 	SPDocument *doc;
 	doc = SP_ACTIVE_DOCUMENT;
-	if (doc) sp_print_document (doc, TRUE);
+	if (doc) sp_print_document (doc, TRUE, NULL);
 }
 
 void
@@ -502,12 +544,14 @@ sp_file_print_preview (gpointer object, gpointer data)
 	}
 }
 
+#if 0
 void sp_file_exit (void)
 {
 	if (sp_ui_close_all ()) {
 		sodipodi_exit (SODIPODI);
 	}
 }
+#endif
 
 void
 sp_file_export_dialog (void *widget)
@@ -523,6 +567,8 @@ struct SPEBP {
 	unsigned char r, g, b, a;
 	NRArenaItem *root;
 	unsigned char *px;
+	unsigned int (*status) (float, void *);
+	void *data;
 };
 
 static int
@@ -536,6 +582,10 @@ sp_export_get_rows (const unsigned char **rows, int row, int num_rows, void *dat
 
 	ebp = (struct SPEBP *) data;
 
+	if (ebp->status) {
+		if (!ebp->status ((float) row / ebp->height, ebp->data)) return 0;
+	}
+
 	num_rows = MIN (num_rows, ebp->sheight);
 	num_rows = MIN (num_rows, ebp->height - row);
 
@@ -579,7 +629,8 @@ void
 sp_export_png_file (SPDocument *doc, const unsigned char *filename,
 		    double x0, double y0, double x1, double y1,
 		    unsigned int width, unsigned int height,
-		    unsigned long bgcolor)
+		    unsigned long bgcolor,
+		    unsigned int (*status) (float, void *), void *data)
 {
 	NRMatrixF affine;
 	gdouble t;
@@ -590,8 +641,8 @@ sp_export_png_file (SPDocument *doc, const unsigned char *filename,
 	g_return_if_fail (doc != NULL);
 	g_return_if_fail (SP_IS_DOCUMENT (doc));
 	g_return_if_fail (filename != NULL);
-	g_return_if_fail (width >= 16);
-	g_return_if_fail (height >= 16);
+	g_return_if_fail (width >= 1);
+	g_return_if_fail (height >= 1);
 
 	sp_document_ensure_up_to_date (doc);
 
@@ -639,6 +690,9 @@ sp_export_png_file (SPDocument *doc, const unsigned char *filename,
 	ebp.root = sp_item_invoke_show (SP_ITEM (sp_document_root (doc)), arena, dkey, SP_ITEM_SHOW_PRINT);
 	nr_arena_item_set_transform (ebp.root, &affine);
 
+	ebp.status = status;
+	ebp.data = data;
+
 	if ((width < 256) || ((width * height) < 32768)) {
 		ebp.px = nr_pixelstore_64K_new (FALSE, 0);
 		ebp.sheight = 65536 / (4 * width);
