@@ -27,7 +27,8 @@ void sp_file_save_as (gpointer object, gpointer data);
 
 void sp_file_save_document (SPDocument *document);
 
-void sp_file_import (GtkWidget * widget);
+unsigned int sp_file_import (SPDocument *doc, const unsigned char *filename);
+void sp_file_import_dialog (GtkWidget *widget);
 
 void sp_file_print (void);
 void sp_file_print_direct (void);
@@ -37,13 +38,15 @@ void sp_file_print_preview (gpointer object, gpointer data);
 void sp_do_file_print (SPDocument *doc);
 void sp_do_file_print_to_file (SPDocument * doc, gchar *filename);
 #endif
-
+#if 0
 void sp_file_exit (void);
+#endif
 
 void sp_file_export_dialog (void *widget);
 void sp_export_png_file (SPDocument *doc, const unsigned char *filename,
 			 double x0, double y0, double x1, double y1,
 			 unsigned int width, unsigned int height,
-			 unsigned long bgcolor);
+			 unsigned long bgcolor,
+			 unsigned int (*status) (float, void *), void *data);
 
 #endif
