@@ -1,19 +1,18 @@
-#ifndef SP_CANVAS_GRID_H
-#define SP_CANVAS_GRID_H
+#ifndef __SP_CANVAS_GRID_H__
+#define __SP_CANVAS_GRID_H__
 
 /*
- * SPCGrid
+ * Infinite visual grid
  *
- * Generic (and quite unintelligent) grid item for gnome canvas
- *
- * Copyright (C) Lauris Kaplinski 2000
+ * Author:
+ *   Lauris Kaplinski <lauris@kaplinski.com>
  *
+ * This code is in public domain
  */
 
-#include <libart_lgpl/art_point.h>
 #include "sp-canvas.h"
 
-BEGIN_GNOME_DECLS
+G_BEGIN_DECLS
 
 #define SP_TYPE_CGRID            (sp_cgrid_get_type ())
 #define SP_CGRID(obj)            (GTK_CHECK_CAST ((obj), SP_TYPE_CGRID, SPCGrid))
@@ -28,11 +27,11 @@ typedef struct _SPCGridClass SPCGridClass;
 struct _SPCGrid {
 	SPCanvasItem item;
 
-	ArtPoint origin;
-	ArtPoint spacing;
+	NRPointF origin;
+	NRPointF spacing;
 	guint32 color;
 
-	ArtPoint ow, sw;
+	NRPointF ow, sw;
 };
 
 struct _SPCGridClass {
@@ -43,6 +42,6 @@ struct _SPCGridClass {
 /* Standard Gtk function */
 GtkType sp_cgrid_get_type (void);
 
-END_GNOME_DECLS
+G_END_DECLS
 
 #endif
