@@ -1,10 +1,17 @@
-#ifndef SP_CURVE_H
-#define SP_CURVE_H
+#ifndef __SP_CURVE_H__
+#define __SP_CURVE_H__
 
 /*
- * This is mostly like GnomeCanvasBpathDef, but with added functionality
- * and shorter name
+ * Wrapper around ArtBpath
  *
+ * Author:
+ *   Lauris Kaplinski <lauris@kaplinski.com>
+ *
+ * Copyright (C) 2000 Lauris Kaplinski
+ * Copyright (C) 2000-2001 Ximian, Inc.
+ * Copyright (C) 2002 Lauris Kaplinski
+ *
+ * Released under GNU GPL
  */
 
 #include <glib.h>
@@ -26,6 +33,9 @@ struct _SPCurve {
 	guint closed : 1;	/* All subpaths are closed */
 };
 
+#define SP_CURVE_LENGTH(c) (((SPCurve *)(c))->end)
+#define SP_CURVE_BPATH(c) (((SPCurve *)(c))->bpath)
+#define SP_CURVE_SEGMENT(c,i) (((SPCurve *)(c))->bpath + (i))
 /* Constructors */
 
 SPCurve * sp_curve_new (void);
@@ -42,6 +52,7 @@ void sp_curve_ensure_space (SPCurve * curve, gint space);
 SPCurve * sp_curve_copy (SPCurve * curve);
 SPCurve * sp_curve_concat (const GSList * list);
 GSList * sp_curve_split (SPCurve * curve);
+SPCurve *sp_curve_transform (SPCurve *curve, const gdouble transform[]);
 
 /* Methods */
 
@@ -54,12 +65,15 @@ void sp_curve_curveto (SPCurve * curve, gdouble x0, gdouble y0, gdouble x1, gdou
 void sp_curve_closepath (SPCurve * curve);
 void sp_curve_closepath_current (SPCurve * curve);
 
+SPCurve *sp_curve_append_continuous (SPCurve *c0, SPCurve *c1, gdouble tolerance);
+
+#define sp_curve_is_empty sp_curve_empty
 gboolean sp_curve_empty (SPCurve * curve);
-ArtBpath * sp_curve_last_bpath (SPCurve * curve);
-ArtBpath * sp_curve_first_bpath (SPCurve * curve);
+ArtBpath *sp_curve_last_bpath (SPCurve *curve);
+ArtBpath *sp_curve_first_bpath (SPCurve *curve);
 
 void sp_curve_append (SPCurve *curve, SPCurve *curve2, gboolean use_lineto);
 SPCurve *sp_curve_reverse (SPCurve *curve);
-
+void sp_curve_backspace (SPCurve *curve);
 
 #endif
