@@ -20,7 +20,7 @@ static void sp_canvas_acetate_class_init (SPCanvasAcetateClass *class);
 static void sp_canvas_acetate_init (SPCanvasAcetate *acetate);
 static void sp_canvas_acetate_destroy (GtkObject *object);
 
-static void sp_canvas_acetate_update (SPCanvasItem *item, double *affine, unsigned int flags);
+static void sp_canvas_acetate_update (SPCanvasItem *item, const NRMatrixD *ctm, unsigned int flags);
 static double sp_canvas_acetate_point (SPCanvasItem *item, double x, double y, SPCanvasItem **actual_item);
 
 static SPCanvasItemClass *parent_class;
@@ -81,7 +81,7 @@ sp_canvas_acetate_destroy (GtkObject *object)
 }
 
 static void
-sp_canvas_acetate_update (SPCanvasItem *item, double *affine, unsigned int flags)
+sp_canvas_acetate_update (SPCanvasItem *item, const NRMatrixD *ctm, unsigned int flags)
 {
 	item->x1 = -G_MAXINT;
 	item->y1 = -G_MAXINT;
