@@ -1,23 +1,21 @@
-#define SODIPODI_CTRL_C
+#define __SODIPODI_CTRL_C__
 
 /*
- * SPCtrl
+ * Visual handle for various things
  *
- * We render it by hand to reduce allocing/freeing svps & to get clean
- *    (non-aa) images
+ * Author:
+ *   Lauris Kaplinski <lauris@kaplinski.com>
  *
+ * This code is in public domain
  */
 
 #include <math.h>
+
+#include <libnr/nr-matrix.h>
+
 #include "sp-canvas.h"
 #include "sp-canvas-util.h"
 
-#include <libart_lgpl/art_affine.h>
-#include <libart_lgpl/art_vpath.h>
-#include <libart_lgpl/art_svp.h>
-#include <libart_lgpl/art_svp_vpath.h>
-#include <libart_lgpl/art_svp_vpath_stroke.h>
-
 #include "sodipodi-ctrl.h"
 
 enum {
@@ -39,7 +37,7 @@ static void sp_ctrl_init (SPCtrl *ctrl);
 static void sp_ctrl_destroy (GtkObject *object);
 static void sp_ctrl_set_arg (GtkObject *object, GtkArg *arg, guint arg_id);
 
-static void sp_ctrl_update (SPCanvasItem *item, double *affine, unsigned int flags);
+static void sp_ctrl_update (SPCanvasItem *item, const NRMatrixD *ctm, unsigned int flags);
 static void sp_ctrl_render (SPCanvasItem *item, SPCanvasBuf *buf);
 
 static double sp_ctrl_point (SPCanvasItem *item, double x, double y, SPCanvasItem **actual_item);
@@ -127,6 +125,11 @@ sp_ctrl_destroy (GtkObject *object)
 
 	ctrl = SP_CTRL (object);
 
+        if (ctrl->cache) {
+          g_free (ctrl->cache);
+          ctrl->cache = NULL;
+        }
+
 	if (GTK_OBJECT_CLASS (parent_class)->destroy)
 		(* GTK_OBJECT_CLASS (parent_class)->destroy) (object);
 }
@@ -199,7 +202,7 @@ sp_ctrl_set_arg (GtkObject *object, GtkArg *arg, guint arg_id)
 }
 
 static void
-sp_ctrl_update (SPCanvasItem *item, double *affine, unsigned int flags)
+sp_ctrl_update (SPCanvasItem *item, const NRMatrixD *ctm, unsigned int flags)
 {
 	SPCtrl *ctrl;
 	gint x, y;
@@ -207,7 +210,7 @@ sp_ctrl_update (SPCanvasItem *item, double *affine, unsigned int flags)
 	ctrl = SP_CTRL (item);
 
 	if (((SPCanvasItemClass *) parent_class)->update)
-		(* ((SPCanvasItemClass *) parent_class)->update) (item, affine, flags);
+		(* ((SPCanvasItemClass *) parent_class)->update) (item, ctm, flags);
 
 	sp_canvas_item_reset_bounds (item);
 
@@ -217,8 +220,8 @@ sp_ctrl_update (SPCanvasItem *item, double *affine, unsigned int flags)
 
 	if (!ctrl->defined) return;
 
-	x = (gint) (affine[4] + 0.5) - ctrl->span;
-	y = (gint) (affine[5] + 0.5) - ctrl->span;
+	x = (gint) (ctm->c[4] + 0.5) - ctrl->span;
+	y = (gint) (ctm->c[5] + 0.5) - ctrl->span;
 
 	switch (ctrl->anchor) {
 	case GTK_ANCHOR_N:
@@ -493,9 +496,9 @@ sp_ctrl_render (SPCanvasItem *item, SPCanvasBuf *buf)
 void
 sp_ctrl_moveto (SPCtrl * ctrl, double x, double y)
 {
-	double affine[6];
+	NRMatrixD transform;
 
-	art_affine_translate (affine, x, y);
+	nr_matrix_d_set_translate (&transform, x, y);
 
-	sp_canvas_item_affine_absolute (SP_CANVAS_ITEM (ctrl), affine);
+	sp_canvas_item_affine_absolute (SP_CANVAS_ITEM (ctrl), &transform);
 }
