@@ -2,14 +2,12 @@
 #define __SODIPODI_CTRLLINE_H__
 
 /*
- * Simple straight line
+ * Simple visual line for various things
  *
  * Author:
  *   Lauris Kaplinski <lauris@kaplinski.com>
  *
- * Copyright (C) 1999-2002 Lauris Kaplinski
- *
- * Released under GNU GPL
+ * This code is in public domain
  */
 
 #include "sp-canvas.h"
@@ -26,7 +24,7 @@ typedef struct _SPCtrlLineClass SPCtrlLineClass;
 GtkType sp_ctrlline_get_type (void);
 
 void sp_ctrlline_set_rgba32 (SPCtrlLine *cl, guint32 rgba);
-void sp_ctrlline_set_coords (SPCtrlLine *cl, gdouble x0, gdouble y0, gdouble x1, gdouble y1);
+void sp_ctrlline_set_coords (SPCtrlLine *cl, float x0, float y0, float x1, float y1);
 
 G_END_DECLS
 
