@@ -12,7 +12,6 @@
 
 #include <gtk/gtkwidget.h>
 
-GtkWidget *sp_window_new (const unsigned char *title, unsigned int resizeable);
-
+GtkWidget *sp_window_new (const unsigned char *title, unsigned int resizeable, unsigned int dialog);
 
 #endif
