@@ -66,7 +66,7 @@ static GtkTargetEntry ui_drop_target_entries [] = {
 #define ENTRIES_SIZE(n) sizeof(n)/sizeof(n[0]) 
 static guint nui_drop_target_entries = ENTRIES_SIZE(ui_drop_target_entries);
 static void sp_ui_import_files(gchar * buffer);
-static void sp_ui_import_one_file(gchar * filename);
+/* static void sp_ui_import_one_file(gchar * filename); */
 static void sp_ui_import_one_file_with_check(gpointer filename, gpointer unused);
 static void sp_ui_drag_data_received (GtkWidget * widget,
 				      GdkDragContext * drag_context,
@@ -83,7 +83,7 @@ sp_create_window (SPViewWidget *vw, gboolean editable)
 	g_return_if_fail (vw != NULL);
 	g_return_if_fail (SP_IS_VIEW_WIDGET (vw));
 
-	w = sp_window_new ("", TRUE);
+	w = sp_window_new ("", TRUE, FALSE);
 	g_object_set_data (G_OBJECT (vw), "window", w);
 	g_object_set_data (G_OBJECT (SP_VIEW_WIDGET_VIEW (vw)), "window", w);
 
@@ -169,18 +169,15 @@ sp_ui_close_view (GtkWidget * widget)
 	gtk_widget_destroy (w);
 }
 
-unsigned int
+#if 0
+static void
 sp_ui_close_all (void)
 {
-	while (SP_ACTIVE_DESKTOP) {
-		GtkWidget *w;
-		w = g_object_get_data (G_OBJECT (SP_ACTIVE_DESKTOP), "window");
-		if (sp_view_shutdown (SP_VIEW (SP_ACTIVE_DESKTOP))) return FALSE;
-		gtk_widget_destroy (w);
-	}
-
-	return TRUE;
+	if (!sodipodi_shutdown_all_views ()) return;
+	/* All views successfully closed */
+	sodipodi_exit ();
 }
+#endif
 
 static gint
 sp_ui_delete (GtkWidget *widget, GdkEvent *event, SPView *view)
@@ -210,7 +207,13 @@ sp_ui_menu_append_item (GtkMenu *menu, const guchar *stock, const guchar *label,
 }
 
 static void
-sp_ui_menu_activate (void *object, SPAction *action)
+sp_ui_menu_activate (void *object, unsigned int verb)
+{
+	sodipodi_verb_perform (verb, NULL);
+}
+
+static void
+sp_ui_menu_activate_direct (void *object, SPAction *action)
 {
 	sp_action_perform (action, NULL);
 }
@@ -230,7 +233,7 @@ sp_ui_menu_key_press (GtkMenuItem *item, GdkEventKey *event, void *data)
 }
 
 static GtkWidget *
-sp_ui_menu_append_item_from_verb (GtkMenu *menu, unsigned int verb)
+sp_ui_menu_append_item_from_verb (GtkMenu *menu, unsigned int verb, unsigned int repeatable)
 {
 	SPAction *action;
 	GtkWidget *item, *icon;
@@ -267,7 +270,11 @@ sp_ui_menu_append_item_from_verb (GtkMenu *menu, unsigned int verb)
 			gtk_image_menu_item_set_image ((GtkImageMenuItem *) item, icon);
 		}
 		gtk_widget_set_events (item, GDK_KEY_PRESS_MASK);
-		g_signal_connect (G_OBJECT (item), "activate", G_CALLBACK (sp_ui_menu_activate), action);
+		if (repeatable) {
+			g_signal_connect ((GObject *) item, "activate", (GCallback) sp_ui_menu_activate, (void *) verb);
+		} else {
+			g_signal_connect ((GObject *) item, "activate", (GCallback) sp_ui_menu_activate_direct, action);
+		}
 		g_signal_connect (G_OBJECT (item), "key_press_event", G_CALLBACK (sp_ui_menu_key_press), (void *) verb);
 	}
 	gtk_widget_show (item);
@@ -277,19 +284,18 @@ sp_ui_menu_append_item_from_verb (GtkMenu *menu, unsigned int verb)
 }
 
 static void
-sp_ui_menu_append (GtkMenu *menu, const unsigned int *verbs)
+sp_ui_menu_append (GtkMenu *menu, const unsigned int *verbs, unsigned int repeatable)
 {
 	int i;
 	for (i = 0; verbs[i] < SP_VERB_LAST; i++) {
-		sp_ui_menu_append_item_from_verb (menu, verbs[i]);
+		sp_ui_menu_append_item_from_verb (menu, verbs[i], repeatable);
 	}
 }
 
 static void
 sp_ui_file_menu (GtkMenu *fm, SPDocument *doc)
 {
- 	GtkWidget *item_recent;
- 	GtkWidget *menu_recent;
+ 	GtkWidget *item, *menu;
 
 	static const unsigned int file_verbs_one[] = {
 		SP_VERB_FILE_NEW, SP_VERB_FILE_OPEN, SP_VERB_LAST
@@ -304,37 +310,40 @@ sp_ui_file_menu (GtkMenu *fm, SPDocument *doc)
 		SP_VERB_LAST
 	};
 
-	sp_ui_menu_append (fm, file_verbs_one);
+	sp_ui_menu_append (fm, file_verbs_one, FALSE);
 
- 	item_recent = sp_ui_menu_append_item (fm, NULL, _("Open Recent"), NULL, NULL);
- 	menu_recent = gtk_menu_new ();
- 	sp_menu_append_recent_documents (GTK_WIDGET (menu_recent));
-	gtk_menu_item_set_submenu (GTK_MENU_ITEM (item_recent), menu_recent);
+ 	item = sp_ui_menu_append_item (fm, NULL, _("Open Recent"), NULL, NULL);
+ 	menu = gtk_menu_new ();
+ 	sp_menu_append_recent_documents (GTK_WIDGET (menu));
+	gtk_menu_item_set_submenu ((GtkMenuItem *) item, menu);
 
-	sp_ui_menu_append (fm, file_verbs_two);
+	sp_ui_menu_append (fm, file_verbs_two, FALSE);
 
 #ifdef WIN32
-	sp_ui_menu_append_item_from_verb (fm, SP_VERB_FILE_PRINT_DIRECT);
+	sp_ui_menu_append_item_from_verb (fm, SP_VERB_FILE_PRINT_DIRECT, FALSE);
 #endif
 #ifdef WITH_KDE
-	sp_ui_menu_append_item_from_verb (fm, SP_VERB_FILE_PRINT_DIRECT);
+	sp_ui_menu_append_item_from_verb (fm, SP_VERB_FILE_PRINT_DIRECT, FALSE);
 #endif
 #ifdef WITH_GNOME_PRINT
-	sp_ui_menu_append_item_from_verb (fm, SP_VERB_FILE_PRINT_DIRECT);
+	sp_ui_menu_append_item_from_verb (fm, SP_VERB_FILE_PRINT_DIRECT, FALSE);
 #endif
-	sp_ui_menu_append_item_from_verb (fm, SP_VERB_FILE_PRINT_PREVIEW);
+	sp_ui_menu_append_item_from_verb (fm, SP_VERB_FILE_PRINT_PREVIEW, FALSE);
 
-	sp_ui_menu_append_item_from_verb (fm, SP_VERB_NONE);
+	sp_ui_menu_append_item_from_verb (fm, SP_VERB_NONE, FALSE);
 
 	sp_ui_menu_append_item (fm, GTK_STOCK_CLOSE, _("Close View"), G_CALLBACK (sp_ui_close_view), NULL);
-	sp_ui_menu_append_item (fm, GTK_STOCK_QUIT, _("Exit Program"), G_CALLBACK (sp_file_exit), NULL);
+	sp_ui_menu_append_item_from_verb (GTK_MENU (fm), SP_VERB_FILE_EXIT, FALSE);
 	sp_ui_menu_append_item (fm, NULL, NULL, NULL, NULL);
 	sp_ui_menu_append_item (fm, NULL, _("About Sodipodi"), G_CALLBACK(sp_help_about), NULL);
-#ifdef WITH_MODULES
-	/* Modules need abouts too */
-	gtk_menu_item_set_submenu (GTK_MENU_ITEM(sp_ui_menu_append_item (GTK_MENU (fm), NULL, _("About Modules"), NULL, NULL)),
-			                   GTK_WIDGET(sp_modulesys_menu_about()));
-#endif /* WITH_MODULES */
+
+	/* Modules about menu */
+	menu = sp_modules_menu_about_new ();
+	if (menu) {
+		/* Have modules about menu */
+		item = sp_ui_menu_append_item (GTK_MENU (fm), NULL, _("About Modules"), NULL, NULL);
+		gtk_menu_item_set_submenu ((GtkMenuItem *) item, menu);
+	}
 }
 
 static void
@@ -351,7 +360,7 @@ sp_ui_edit_menu (GtkMenu *menu, SPDocument *doc)
 		SP_VERB_EDIT_SELECT_ALL,
 		SP_VERB_LAST
 	};
-	sp_ui_menu_append (menu, edit_verbs);
+	sp_ui_menu_append (menu, edit_verbs, FALSE);
 	sp_ui_menu_append_item (menu, NULL, _("Cleanup"), G_CALLBACK (sp_edit_cleanup), NULL);
 }
 
@@ -363,13 +372,18 @@ sp_ui_selection_menu (GtkMenu *menu, SPDocument *doc)
 		SP_VERB_NONE,
 		SP_VERB_SELECTION_COMBINE, SP_VERB_SELECTION_BREAK_APART,
 		SP_VERB_NONE,
+		SP_VERB_SELECTION_UNCROSS,
+		SP_VERB_SELECTION_UNION,
+		SP_VERB_SELECTION_INTERSECTION,
+		SP_VERB_SELECTION_SUBTRACTION,
+		SP_VERB_NONE,
 		SP_VERB_SELECTION_TO_FRONT,
 		SP_VERB_SELECTION_TO_BACK,
 		SP_VERB_SELECTION_RAISE,
 		SP_VERB_SELECTION_LOWER,
 		SP_VERB_LAST
 	};
-	sp_ui_menu_append (menu, select_verbs);
+	sp_ui_menu_append (menu, select_verbs, TRUE);
 }
 
 static void
@@ -426,13 +440,13 @@ sp_ui_view_menu (GtkMenu *menu, SPDocument *doc)
 	zm = gtk_menu_new ();
 	gtk_widget_show (zm);
 	gtk_menu_item_set_submenu (GTK_MENU_ITEM (zi), zm);
-	sp_ui_menu_append_item_from_verb (GTK_MENU (zm), SP_VERB_ZOOM_SELECTION);
-	sp_ui_menu_append_item_from_verb (GTK_MENU (zm), SP_VERB_ZOOM_DRAWING);
-	sp_ui_menu_append_item_from_verb (GTK_MENU (zm), SP_VERB_ZOOM_PAGE);
+	sp_ui_menu_append_item_from_verb (GTK_MENU (zm), SP_VERB_ZOOM_SELECTION, FALSE);
+	sp_ui_menu_append_item_from_verb (GTK_MENU (zm), SP_VERB_ZOOM_DRAWING, FALSE);
+	sp_ui_menu_append_item_from_verb (GTK_MENU (zm), SP_VERB_ZOOM_PAGE, FALSE);
 	sp_ui_menu_append_item (GTK_MENU (zm), NULL, NULL, NULL, NULL);
-	sp_ui_menu_append_item_from_verb (GTK_MENU (zm), SP_VERB_ZOOM_1_2);
-	sp_ui_menu_append_item_from_verb (GTK_MENU (zm), SP_VERB_ZOOM_1_1);
-	sp_ui_menu_append_item_from_verb (GTK_MENU (zm), SP_VERB_ZOOM_2_1);
+	sp_ui_menu_append_item_from_verb (GTK_MENU (zm), SP_VERB_ZOOM_1_2, FALSE);
+	sp_ui_menu_append_item_from_verb (GTK_MENU (zm), SP_VERB_ZOOM_1_1, FALSE);
+	sp_ui_menu_append_item_from_verb (GTK_MENU (zm), SP_VERB_ZOOM_2_1, FALSE);
 	/* View:New View*/
 	sp_ui_menu_append_item (menu, NULL, _("New View"), G_CALLBACK(sp_ui_new_view), NULL);
 	/* View:New Preview*/
@@ -453,7 +467,7 @@ sp_ui_event_context_menu (GtkMenu *menu, SPDocument *doc)
 		SP_VERB_CONTEXT_TEXT, SP_VERB_CONTEXT_ZOOM,
 		SP_VERB_LAST
 	};
-	sp_ui_menu_append (menu, context_verbs);
+	sp_ui_menu_append (menu, context_verbs, FALSE);
 }
 
 static void
@@ -474,36 +488,47 @@ sp_ui_dialog_menu (GtkMenu *menu, SPDocument *doc)
 		SP_VERB_DIALOG_DISPLAY,
 		SP_VERB_LAST
 	};
-	sp_ui_menu_append (menu, verbs);
+	sp_ui_menu_append (menu, verbs, FALSE);
 }
 
 /* Menus */
 
 static void
-sp_ui_populate_main_menu(GtkWidget *m)
+sp_ui_exit (GtkWidget *widget)
 {
-	GtkWidget *item_recent;
-	GtkWidget *menu_recent;
+	SPAction *action;
+	/* This HAS to go through verb */
+	action = sp_verb_get_action (SP_VERB_FILE_EXIT);
+	if (action) sp_action_perform (action, NULL);
+}
+
+static void
+sp_ui_populate_main_menu (GtkWidget *m)
+{
+	GtkWidget *i, *sm;
 
 	sp_ui_menu_append_item (GTK_MENU (m), GTK_STOCK_NEW, _("New"), G_CALLBACK(sp_file_new), NULL);
 	sp_ui_menu_append_item (GTK_MENU (m), GTK_STOCK_OPEN, _("Open"), G_CALLBACK(sp_file_open_dialog), NULL);
         
-	item_recent = sp_ui_menu_append_item (GTK_MENU (m), NULL, _("Open Recent"), NULL, NULL);
-	menu_recent = gtk_menu_new ();
-
-	sp_menu_append_recent_documents (GTK_WIDGET (menu_recent));
-	gtk_menu_item_set_submenu (GTK_MENU_ITEM (item_recent), menu_recent); 
+	i = sp_ui_menu_append_item (GTK_MENU (m), NULL, _("Open Recent"), NULL, NULL);
+	sm = gtk_menu_new ();
+	sp_menu_append_recent_documents (sm);
+	gtk_menu_item_set_submenu (GTK_MENU_ITEM (i), sm); 
 
 	sp_ui_menu_append_item (GTK_MENU (m), NULL, NULL, NULL, NULL);
 	sp_ui_menu_append_item (GTK_MENU (m), NULL, _("About Sodipodi"), G_CALLBACK(sp_help_about), NULL);
-#ifdef lalaWITH_MODULES
-	/* Modules need abouts too */
-	gtk_menu_item_set_submenu (GTK_MENU_ITEM(sp_ui_menu_append_item (GTK_MENU (m), NULL, _("About Modules"), NULL, NULL)),
-			                   GTK_WIDGET(sp_modulesys_menu_about()));
-#endif /* WITH_MODULES */
+
+	/* Modules about menu */
+	sm = sp_modules_menu_about_new ();
+	if (sm) {
+		/* Have modules about menu */
+		i = sp_ui_menu_append_item (GTK_MENU (m), NULL, _("About Modules"), NULL, NULL);
+		gtk_menu_item_set_submenu ((GtkMenuItem *) i, sm);
+	}
 
 	sp_ui_menu_append_item (GTK_MENU (m), NULL, NULL, NULL, NULL);
-	sp_ui_menu_append_item (GTK_MENU (m), GTK_STOCK_QUIT, _("Exit Program"), G_CALLBACK(sp_file_exit), NULL);
+	/* For some reason action menu does not work on menubar :-( */
+	sp_ui_menu_append_item ((GtkMenu *) m, GTK_STOCK_QUIT, _("Exit program"), G_CALLBACK (sp_ui_exit), NULL);
 }
 
 static void
@@ -542,8 +567,8 @@ sp_ui_generic_menu (SPView *v, SPItem *item)
 	m = gtk_menu_new ();
 
 	/* Undo and Redo */
-	sp_ui_menu_append_item_from_verb (GTK_MENU (m), SP_VERB_EDIT_UNDO);
-	sp_ui_menu_append_item_from_verb (GTK_MENU (m), SP_VERB_EDIT_REDO);
+	sp_ui_menu_append_item_from_verb (GTK_MENU (m), SP_VERB_EDIT_UNDO, FALSE);
+	sp_ui_menu_append_item_from_verb (GTK_MENU (m), SP_VERB_EDIT_REDO, FALSE);
 	/* Separator */
 	sp_ui_menu_append_item (GTK_MENU (m), NULL, NULL, NULL, NULL);
 
@@ -601,11 +626,14 @@ sp_ui_generic_menu (SPView *v, SPItem *item)
 	sp_ui_dialog_menu (GTK_MENU (sm), NULL);
 	gtk_widget_show (sm);
 	gtk_menu_item_set_submenu (GTK_MENU_ITEM (i), sm);
-	/* Filters submenu */
-#ifdef lalaWITH_MODULES
-	gtk_menu_item_set_submenu (GTK_MENU_ITEM(sp_ui_menu_append_item (GTK_MENU (m), NULL, _("Filters"), NULL, NULL)),
-			                   GTK_WIDGET(sp_modulesys_menu_filter()));
-#endif /* WITH_MODULES */
+	/* Modules submenu */
+	sm = sp_modules_menu_new ();
+	if (sm) {
+		/* Have modules menu */
+		i = sp_ui_menu_append_item (GTK_MENU (m), NULL, _("Modules"), NULL, NULL);
+		gtk_widget_show (sm);
+		gtk_menu_item_set_submenu (GTK_MENU_ITEM (i), sm);
+	}
 
 	return m;
 }
@@ -670,82 +698,8 @@ sp_ui_import_files(gchar * buffer)
 static void
 sp_ui_import_one_file_with_check(gpointer filename, gpointer unused)
 {
-	if (filename) {
-		if (strlen(filename) > 2)
-			sp_ui_import_one_file(filename);
+	if (filename && strlen (filename) > 2) {
+		sp_file_import (SP_ACTIVE_DOCUMENT, filename);
 	}
 }
 
-/* Cut&Paste'ed from file.c:file_import_ok */
-static void
-sp_ui_import_one_file(gchar * filename)
-{
-	SPDocument * doc;
-	SPRepr * rdoc;
-	const gchar * e, * docbase, * relname;
-	SPRepr * repr;
-	SPReprDoc * rnewdoc;
-
-	doc = SP_ACTIVE_DOCUMENT;
-	if (!SP_IS_DOCUMENT(doc)) return;
-	
-	if (filename == NULL) return;  
-
-	rdoc = sp_document_repr_root (doc);
-
-	docbase = sp_repr_attr (rdoc, "sodipodi:docbase");
-	relname = sp_relative_path_from_path (filename, docbase);
-	/* fixme: this should be implemented with mime types */
-	e = sp_extension_from_path (filename);
-
-	if ((e == NULL) || (strcmp (e, "svg") == 0) || (strcmp (e, "xml") == 0)) {
-		SPRepr * newgroup;
-		const gchar * style;
-		SPRepr * child;
-
-		rnewdoc = sp_repr_read_file (filename, SP_SVG_NS_URI);
-		if (rnewdoc == NULL) return;
-		repr = sp_repr_document_root (rnewdoc);
-		style = sp_repr_attr (repr, "style");
-
-		newgroup = sp_repr_new ("g");
-		sp_repr_set_attr (newgroup, "style", style);
-
-		for (child = repr->children; child != NULL; child = child->next) {
-			SPRepr * newchild;
-			newchild = sp_repr_duplicate (child);
-			sp_repr_append_child (newgroup, newchild);
-		}
-
-		sp_repr_document_unref (rnewdoc);
-
-		sp_document_add_repr (doc, newgroup);
-		sp_repr_unref (newgroup);
-		sp_document_done (doc);
-		return;
-	}
-
-	if ((strcmp (e, "png") == 0) ||
-	    (strcmp (e, "jpg") == 0) ||
-	    (strcmp (e, "jpeg") == 0) ||
-	    (strcmp (e, "bmp") == 0) ||
-	    (strcmp (e, "gif") == 0) ||
-	    (strcmp (e, "tiff") == 0) ||
-	    (strcmp (e, "xpm") == 0)) {
-		/* Try pixbuf */
-		GdkPixbuf *pb;
-		pb = gdk_pixbuf_new_from_file (filename, NULL);
-		if (pb) {
-			/* We are readable */
-			repr = sp_repr_new ("image");
-			sp_repr_set_attr (repr, "xlink:href", relname);
-			sp_repr_set_attr (repr, "sodipodi:absref", filename);
-			sp_repr_set_double_attribute (repr, "width", gdk_pixbuf_get_width (pb));
-			sp_repr_set_double_attribute (repr, "height", gdk_pixbuf_get_height (pb));
-			sp_document_add_repr (doc, repr);
-			sp_repr_unref (repr);
-			sp_document_done (doc);
-			gdk_pixbuf_unref (pb);
-		}
-	}
-}
