@@ -4,7 +4,7 @@
  * Container for SPKnot visual handles
  *
  * Authors:
- *   Mitsuru Oka
+ *   Mitsuru Oka <oka326@parkcity.ne.jp>
  *
  * Copyright (C) 2001-2002 authors
  *
@@ -14,7 +14,6 @@
 #define noKNOT_HOLDER_DEBUG
 
 #include <glib.h>
-#include <libart_lgpl/art_affine.h>
 #include <gtk/gtksignal.h>
 #include "document.h"
 #include "desktop.h"
@@ -22,7 +21,7 @@
 #include "sp-shape.h"
 #include "knotholder.h"
 
-static void knot_moved_handler (SPKnot *knot, ArtPoint *p, guint state, gpointer data);
+static void knot_moved_handler (SPKnot *knot, NRPointF *p, guint state, gpointer data);
 static void knot_ungrabbed_handler (SPKnot *knot, unsigned int state, SPKnotHolder *kh);
 
 #ifdef KNOT_HOLDER_DEBUG
@@ -48,14 +47,13 @@ sp_knot_holder_new (SPDesktop *desktop, SPItem *item, SPKnotHolderReleasedFunc r
 	knot_holder = g_new (SPKnotHolder, 1);
 	knot_holder->desktop = desktop;
 	knot_holder->item = item;
-	gtk_object_ref (GTK_OBJECT (item));
+	g_object_ref (G_OBJECT (item));
 	knot_holder->entity = NULL;
 
 	knot_holder->released = relhandler;
 
 #ifdef KNOT_HOLDER_DEBUG
-	gtk_signal_connect (GTK_OBJECT(desktop), "destroy",
-			    sp_knot_holder_debug, (gpointer)"SPKnotHolder::item");
+	g_signal_connect (G_OBJECT (desktop), "destroy", sp_knot_holder_debug, (gpointer)"SPKnotHolder::item");
 #endif
 
 	return knot_holder;
@@ -65,12 +63,12 @@ void
 sp_knot_holder_destroy	(SPKnotHolder *kh)
 {
 	if (kh) {
-		gtk_object_unref (GTK_OBJECT (kh->item));
+		g_object_unref (G_OBJECT (kh->item));
 		while (kh->entity) {
 			SPKnotHolderEntity *e;
 			e = (SPKnotHolderEntity *) kh->entity->data;
 			/* unref should call destroy */
-			gtk_object_unref (GTK_OBJECT (e->knot));
+			g_object_unref (G_OBJECT (e->knot));
 			g_free (e);
 			kh->entity = g_slist_remove (kh->entity, e);
 		}
@@ -94,9 +92,8 @@ sp_knot_holder_add_full	(SPKnotHolder       *knot_holder,
 {
 	SPKnotHolderEntity *e;
 	SPItem        *item;
-	ArtPoint       p;
-	gdouble        affine[6];
-	GtkObject     *ob;
+	NRPointF sp, dp;
+	NRMatrixF i2d;
 
 	g_return_if_fail (knot_holder != NULL);
 	g_return_if_fail (knot_set != NULL);
@@ -107,7 +104,7 @@ sp_knot_holder_add_full	(SPKnotHolder       *knot_holder,
 #define KH_EPSILON 1e-6
 	/* Precondition for knot_set and knot_get */
 	{
-		ArtPoint p1, p2;
+		NRPointF p1, p2;
 		knot_get (item, &p1);
 		knot_set (item, &p1, 0);
 		knot_get (item, &p2);
@@ -120,36 +117,35 @@ sp_knot_holder_add_full	(SPKnotHolder       *knot_holder,
 	e->knot = sp_knot_new (knot_holder->desktop);
 	e->knot_set = knot_set;
 	e->knot_get = knot_get;
-	ob = GTK_OBJECT (e->knot->item);
-	gtk_object_set (ob, "shape", shape, NULL);
-	gtk_object_set (ob, "mode", mode, NULL);
+
+	g_object_set (G_OBJECT (e->knot->item), "shape", shape, NULL);
+	g_object_set (G_OBJECT (e->knot->item), "mode", mode, NULL);
 	knot_holder->entity = g_slist_append (knot_holder->entity, e);
 
-	ob = GTK_OBJECT (e->knot);
 	/* move to current point */
-	e->knot_get (item, &p);
-	sp_item_i2d_affine(item, affine);
-	art_affine_point (&p, &p, affine);
-	sp_knot_set_position (e->knot, &p, SP_KNOT_STATE_NORMAL);
+	e->knot_get (item, &sp);
+	sp_item_i2d_affine(item, &i2d);
+	dp.x = NR_MATRIX_DF_TRANSFORM_X (&i2d, sp.x, sp.y);
+	dp.y = NR_MATRIX_DF_TRANSFORM_Y (&i2d, sp.x, sp.y);
+	sp_knot_set_position (e->knot, &dp, SP_KNOT_STATE_NORMAL);
 
-	e->handler_id = gtk_signal_connect (ob, "moved", knot_moved_handler, knot_holder);
+	e->handler_id = g_signal_connect (G_OBJECT (e->knot), "moved", G_CALLBACK (knot_moved_handler), knot_holder);
 
-	gtk_signal_connect (ob, "ungrabbed", GTK_SIGNAL_FUNC (knot_ungrabbed_handler), knot_holder);
+	g_signal_connect (G_OBJECT (e->knot), "ungrabbed", G_CALLBACK (knot_ungrabbed_handler), knot_holder);
 
 #ifdef KNOT_HOLDER_DEBUG
-	gtk_signal_connect (ob, "destroy",
-			    sp_knot_holder_debug, "SPKnotHolder::knot");
+	g_signal_connect (ob, "destroy", sp_knot_holder_debug, "SPKnotHolder::knot");
 #endif
 	sp_knot_show (e->knot);
 }
 
 static void
-knot_moved_handler (SPKnot *knot, ArtPoint *p, guint state, gpointer data)
+knot_moved_handler (SPKnot *knot, NRPointF *p, guint state, gpointer data)
 {
 	SPKnotHolder *knot_holder;
 	SPItem *item;
 	SPObject *object;
-	gdouble affine[6];
+	NRMatrixF i2d;
 	GSList *el;
 
 	knot_holder = (SPKnotHolder *) data;
@@ -159,12 +155,13 @@ knot_moved_handler (SPKnot *knot, ArtPoint *p, guint state, gpointer data)
 	for (el = knot_holder->entity; el; el = el->next) {
 		SPKnotHolderEntity *e = (SPKnotHolderEntity *)el->data;
 		if (e->knot == knot) {
-			ArtPoint q;
-			double d2i[6];
+			NRMatrixF d2i;
+			NRPointF q;
 
-			sp_item_i2d_affine(item, affine);
-			art_affine_invert (d2i, affine);
-			art_affine_point (&q, p, d2i);
+			sp_item_i2d_affine(item, &i2d);
+			nr_matrix_f_invert (&d2i, &i2d);
+			q.x = NR_MATRIX_DF_TRANSFORM_X (&d2i, p->x, p->y);
+			q.y = NR_MATRIX_DF_TRANSFORM_Y (&d2i, p->x, p->y);
 
 			e->knot_set (item, &q, state);
 
@@ -174,25 +171,23 @@ knot_moved_handler (SPKnot *knot, ArtPoint *p, guint state, gpointer data)
 	
 	sp_shape_set_shape (SP_SHAPE (item));
 
-	sp_object_invoke_write (object, object->repr, SP_OBJECT_WRITE_SODIPODI);
-	
-	sp_item_i2d_affine(item, affine);
+	sp_item_i2d_affine(item, &i2d);
 
 	for (el = knot_holder->entity; el; el = el->next) {
 		SPKnotHolderEntity *e = (SPKnotHolderEntity *)el->data;
-		ArtPoint p1;
-		GtkObject *kob;
+		NRPointF sp, dp;
+		GObject *kob;
 		
-		kob = GTK_OBJECT (e->knot);
+		kob = G_OBJECT (e->knot);
 
-		e->knot_get (item, &p1);
-		
-		art_affine_point (&p1, &p1, affine);
+		e->knot_get (item, &sp);
 
-		gtk_signal_handler_block (kob, e->handler_id);
-		sp_knot_set_position (e->knot, &p1,
-				      SP_KNOT_STATE_NORMAL);
-		gtk_signal_handler_unblock (kob, e->handler_id);
+		dp.x = NR_MATRIX_DF_TRANSFORM_X (&i2d, sp.x, sp.y);
+		dp.y = NR_MATRIX_DF_TRANSFORM_Y (&i2d, sp.x, sp.y);
+
+		g_signal_handler_block (kob, e->handler_id);
+		sp_knot_set_position (e->knot, &dp, SP_KNOT_STATE_NORMAL);
+		g_signal_handler_unblock (kob, e->handler_id);
 	}
 }
 
@@ -202,6 +197,9 @@ knot_ungrabbed_handler (SPKnot *knot, unsigned int state, SPKnotHolder *kh)
 	if (kh->released) {
 		kh->released (kh->item);
 	} else {
+		SPObject *object;
+		object = (SPObject *) kh->item;
+		sp_object_invoke_write (object, object->repr, SP_OBJECT_WRITE_SODIPODI);
 		sp_document_done (SP_OBJECT_DOCUMENT (kh->item));
 	}
 }
