@@ -34,6 +34,7 @@ struct _NRSVLStrokeBuild {
 	NRSVL *svl;
 	NRFlat *flats;
 	NRMatrixF transform;
+	unsigned int bboxonly : 1;
 	unsigned int closed : 1;
 	unsigned int cap : 2;
 	unsigned int join : 2;
@@ -93,17 +94,21 @@ nr_svl_stroke_build_lineto (NRSVLStrokeBuild *svlb, float x, float y)
 			svlb->x[1] = x;
 			svlb->y[1] = y;
 			len = (float) hypot (svlb->x[1] - svlb->x[0], svlb->y[1] - svlb->y[0]);
+			/* Extra check for WinME */
+			if (fabs (len) < NR_EPSILON_F) return;
 			dx = (float) (svlb->x[1] - svlb->x[0]) / len;
 			dy = (float) (svlb->y[1] - svlb->y[0]) / len;
+			dx *= svlb->width_2;
+			dy *= svlb->width_2;
 			if (!svlb->closed) {
 				/* Draw cap[0,1] if open */
-				nr_svl_build_moveto (&svlb->left, svlb->x[0] + dy * svlb->width_2, svlb->y[0] - dx * svlb->width_2);
-				nr_svl_build_moveto (&svlb->right, svlb->x[0] + dy * svlb->width_2, svlb->y[0] - dx * svlb->width_2);
+				nr_svl_build_moveto (&svlb->left, svlb->x[0] + dy, svlb->y[0] - dx);
+				nr_svl_build_moveto (&svlb->right, svlb->x[0] + dy, svlb->y[0] - dx);
 				nr_svl_stroke_build_draw_cap (svlb, svlb->x[0], svlb->y[0], svlb->x[1], svlb->y[1], FALSE);
 			} else {
 				/* Set starting point */
-				nr_svl_build_moveto (&svlb->left, svlb->x[0] - dy * svlb->width_2, svlb->y[0] + dx * svlb->width_2);
-				nr_svl_build_moveto (&svlb->right, svlb->x[0] + dy * svlb->width_2, svlb->y[0] - dx * svlb->width_2);
+				nr_svl_build_moveto (&svlb->left, svlb->x[0] - dy, svlb->y[0] + dx);
+				nr_svl_build_moveto (&svlb->right, svlb->x[0] + dy, svlb->y[0] - dx);
 			}
 		} else {
 			/* Draw 2->3 + join 2->3->CP */
@@ -130,6 +135,8 @@ nr_svl_stroke_build_finish_subpath (NRSVLStrokeBuild *svlb)
 		nr_svl_stroke_build_draw_join (svlb, svlb->x[2], svlb->y[2], svlb->x[3], svlb->y[3], svlb->x[1], svlb->y[1]);
 		/* And finsih possibly open paths */
 		len = (float) hypot (svlb->x[1] - svlb->x[0], svlb->y[1] - svlb->y[0]);
+		/* Extra check for WinME */
+		if (fabs (len) < NR_EPSILON_F) return;
 		dx = (svlb->x[1] - svlb->x[0]) / len;
 		dy = (svlb->y[1] - svlb->y[0]) / len;
 		nr_svl_build_lineto (&svlb->left, svlb->x[0] - dy * svlb->width_2, svlb->y[0] + dx * svlb->width_2);
@@ -162,6 +169,7 @@ nr_svl_stroke_build_curveto (NRSVLStrokeBuild *svlb,
 
 	if (level >= MAX_SUBDIVIDE_DEPTH) goto nosubdivide;
 
+#if 0
 	t_x0 = NR_MATRIX_DF_TRANSFORM_X (&svlb->transform, x0, y0);
 	t_y0 = NR_MATRIX_DF_TRANSFORM_Y (&svlb->transform, x0, y0);
 	t_x1 = NR_MATRIX_DF_TRANSFORM_X (&svlb->transform, x1, y1);
@@ -170,6 +178,16 @@ nr_svl_stroke_build_curveto (NRSVLStrokeBuild *svlb,
 	t_y2 = NR_MATRIX_DF_TRANSFORM_Y (&svlb->transform, x2, y2);
 	t_x3 = NR_MATRIX_DF_TRANSFORM_X (&svlb->transform, x3, y3);
 	t_y3 = NR_MATRIX_DF_TRANSFORM_Y (&svlb->transform, x3, y3);
+#else
+	t_x0 = x0;
+	t_y0 = y0;
+	t_x1 = x1;
+	t_y1 = y1;
+	t_x2 = x2;
+	t_y2 = y2;
+	t_x3 = x3;
+	t_y3 = y3;
+#endif
 
 	dx1_0 = t_x1 - t_x0;
 	dy1_0 = t_y1 - t_y0;
@@ -243,6 +261,7 @@ nr_bpath_stroke (const NRBPath *path, NRMatrixF *transform,
 	} else {
 		nr_matrix_f_set_identity (&svlb.transform);
 	}
+	svlb.bboxonly = FALSE;
 	svlb.closed = FALSE;
 	svlb.width_2 = width / 2.0;
 	svlb.cap = cap;
@@ -256,6 +275,7 @@ nr_bpath_stroke (const NRBPath *path, NRMatrixF *transform,
 	svlb.left.refvx = NULL;
 	svlb.left.dir = 0;
 	svlb.left.reverse = FALSE;
+	svlb.left.bboxonly = FALSE;
 	svlb.left.sx = svlb.left.sy = 0.0;
 	nr_rect_f_set_empty (&svlb.left.bbox);
 
@@ -264,6 +284,7 @@ nr_bpath_stroke (const NRBPath *path, NRMatrixF *transform,
 	svlb.right.refvx = NULL;
 	svlb.right.dir = 0;
 	svlb.right.reverse = TRUE;
+	svlb.right.bboxonly = FALSE;
 	svlb.right.sx = svlb.right.sy = 0.0;
 	nr_rect_f_set_empty (&svlb.right.bbox);
 
@@ -274,25 +295,55 @@ nr_bpath_stroke (const NRBPath *path, NRMatrixF *transform,
 		switch (bp->code) {
 		case ART_MOVETO:
 			nr_svl_stroke_build_finish_subpath (&svlb);
-			sx = x = bp->x3;
-			sy = y = bp->y3;
+			if (transform) {
+				sx = x = NR_MATRIX_DF_TRANSFORM_X (transform, bp->x3, bp->y3);
+				sy = y = NR_MATRIX_DF_TRANSFORM_Y (transform, bp->x3, bp->y3);
+			} else {
+				sx = x = bp->x3;
+				sy = y = bp->y3;
+			}
 			nr_svl_stroke_build_start_closed_subpath (&svlb, (float) x, (float) y);
 			break;
 		case ART_MOVETO_OPEN:
 			nr_svl_stroke_build_finish_subpath (&svlb);
-			sx = x = bp->x3;
-			sy = y = bp->y3;
+			if (transform) {
+				sx = x = NR_MATRIX_DF_TRANSFORM_X (transform, bp->x3, bp->y3);
+				sy = y = NR_MATRIX_DF_TRANSFORM_Y (transform, bp->x3, bp->y3);
+			} else {
+				sx = x = bp->x3;
+				sy = y = bp->y3;
+			}
 			nr_svl_stroke_build_start_open_subpath (&svlb, (float) x, (float) y);
 			break;
 		case ART_LINETO:
-			sx = x = bp->x3;
-			sy = y = bp->y3;
+			if (transform) {
+				sx = x = NR_MATRIX_DF_TRANSFORM_X (transform, bp->x3, bp->y3);
+				sy = y = NR_MATRIX_DF_TRANSFORM_Y (transform, bp->x3, bp->y3);
+			} else {
+				sx = x = bp->x3;
+				sy = y = bp->y3;
+			}
 			nr_svl_stroke_build_lineto (&svlb, (float) x, (float) y);
 			break;
 		case ART_CURVETO:
-			x = bp->x3;
-			y = bp->y3;
-			nr_svl_stroke_build_curveto (&svlb, sx, sy, bp->x1, bp->y1, bp->x2, bp->y2, x, y, flatness, 0);
+			if (transform) {
+				x = NR_MATRIX_DF_TRANSFORM_X (transform, bp->x3, bp->y3);
+				y = NR_MATRIX_DF_TRANSFORM_Y (transform, bp->x3, bp->y3);
+				nr_svl_stroke_build_curveto (&svlb,
+							     sx, sy,
+							     NR_MATRIX_DF_TRANSFORM_X (transform, bp->x1, bp->y1),
+							     NR_MATRIX_DF_TRANSFORM_Y (transform, bp->x1, bp->y1),
+							     NR_MATRIX_DF_TRANSFORM_X (transform, bp->x2, bp->y2),
+							     NR_MATRIX_DF_TRANSFORM_Y (transform, bp->x2, bp->y2),
+							     x, y,
+							     flatness, 0);
+			} else {
+				x = bp->x3;
+				y = bp->y3;
+				nr_svl_stroke_build_curveto (&svlb,
+							     sx, sy, bp->x1, bp->y1, bp->x2, bp->y2, x, y,
+							     flatness, 0);
+			}
 			/* Restore original join type */
 			svlb.curve = FALSE;
 			sx = x;
@@ -318,6 +369,128 @@ nr_bpath_stroke (const NRBPath *path, NRMatrixF *transform,
 	return svlb.svl;
 }
 
+NRRectF *
+nr_bpath_stroke_bbox_union (const NRBPath *path, NRMatrixF *transform,
+			    NRRectF *bbox,
+			    float width,
+			    unsigned int cap, unsigned int join, float miterlimit,
+			    float flatness)
+{
+	NRSVLStrokeBuild svlb;
+	ArtBpath *bp;
+	double x, y, sx, sy;
+
+	/* Initialize NRSVLBuilds */
+	svlb.svl = NULL;
+	svlb.flats = NULL;
+	if (transform) {
+		svlb.transform = *transform;
+	} else {
+		nr_matrix_f_set_identity (&svlb.transform);
+	}
+	svlb.bboxonly = FALSE;
+	svlb.closed = FALSE;
+	svlb.width_2 = width / 2.0;
+	svlb.cap = cap;
+	svlb.join = join;
+	svlb.curve = FALSE;
+	svlb.cosml = MIN (cos (miterlimit), 0.9998477);
+	svlb.npoints = 0;
+
+	svlb.left.svl = &svlb.svl;
+	svlb.left.flats = &svlb.flats;
+	svlb.left.refvx = NULL;
+	svlb.left.dir = 0;
+	svlb.left.reverse = FALSE;
+	svlb.left.bboxonly = TRUE;
+	svlb.left.sx = svlb.left.sy = 0.0;
+	nr_rect_f_set_empty (&svlb.left.bbox);
+
+	svlb.right.svl = &svlb.svl;
+	svlb.right.flats = &svlb.flats;
+	svlb.right.refvx = NULL;
+	svlb.right.dir = 0;
+	svlb.right.reverse = TRUE;
+	svlb.right.bboxonly = TRUE;
+	svlb.right.sx = svlb.right.sy = 0.0;
+	nr_rect_f_set_empty (&svlb.right.bbox);
+
+	x = y = 0.0;
+	sx = sy = 0.0;
+
+	for (bp = path->path; bp->code != ART_END; bp++) {
+		switch (bp->code) {
+		case ART_MOVETO:
+			nr_svl_stroke_build_finish_subpath (&svlb);
+			if (transform) {
+				sx = x = NR_MATRIX_DF_TRANSFORM_X (transform, bp->x3, bp->y3);
+				sy = y = NR_MATRIX_DF_TRANSFORM_Y (transform, bp->x3, bp->y3);
+			} else {
+				sx = x = bp->x3;
+				sy = y = bp->y3;
+			}
+			nr_svl_stroke_build_start_closed_subpath (&svlb, (float) x, (float) y);
+			break;
+		case ART_MOVETO_OPEN:
+			nr_svl_stroke_build_finish_subpath (&svlb);
+			if (transform) {
+				sx = x = NR_MATRIX_DF_TRANSFORM_X (transform, bp->x3, bp->y3);
+				sy = y = NR_MATRIX_DF_TRANSFORM_Y (transform, bp->x3, bp->y3);
+			} else {
+				sx = x = bp->x3;
+				sy = y = bp->y3;
+			}
+			nr_svl_stroke_build_start_open_subpath (&svlb, (float) x, (float) y);
+			break;
+		case ART_LINETO:
+			if (transform) {
+				sx = x = NR_MATRIX_DF_TRANSFORM_X (transform, bp->x3, bp->y3);
+				sy = y = NR_MATRIX_DF_TRANSFORM_Y (transform, bp->x3, bp->y3);
+			} else {
+				sx = x = bp->x3;
+				sy = y = bp->y3;
+			}
+			nr_svl_stroke_build_lineto (&svlb, (float) x, (float) y);
+			break;
+		case ART_CURVETO:
+			if (transform) {
+				x = NR_MATRIX_DF_TRANSFORM_X (transform, bp->x3, bp->y3);
+				y = NR_MATRIX_DF_TRANSFORM_Y (transform, bp->x3, bp->y3);
+				nr_svl_stroke_build_curveto (&svlb,
+							     sx, sy,
+							     NR_MATRIX_DF_TRANSFORM_X (transform, bp->x1, bp->y1),
+							     NR_MATRIX_DF_TRANSFORM_Y (transform, bp->x1, bp->y1),
+							     NR_MATRIX_DF_TRANSFORM_X (transform, bp->x2, bp->y2),
+							     NR_MATRIX_DF_TRANSFORM_Y (transform, bp->x2, bp->y2),
+							     x, y,
+							     flatness, 0);
+			} else {
+				x = bp->x3;
+				y = bp->y3;
+				nr_svl_stroke_build_curveto (&svlb,
+							     sx, sy, bp->x1, bp->y1, bp->x2, bp->y2, x, y,
+							     flatness, 0);
+			}
+			/* Restore original join type */
+			svlb.curve = FALSE;
+			sx = x;
+			sy = y;
+			break;
+		default:
+			/* fixme: free lists */
+			return NULL;
+			break;
+		}
+	}
+	nr_svl_stroke_build_finish_subpath (&svlb);
+	nr_svl_stroke_build_finish_segment (&svlb);
+
+	nr_rect_f_union (bbox, bbox, &svlb.left.bbox);
+	nr_rect_f_union (bbox, bbox, &svlb.right.bbox);
+
+	return bbox;
+}
+
 NRSVL *
 nr_vpath_stroke (const ArtVpath *path, NRMatrixF *transform,
 		 float width,
@@ -349,6 +522,7 @@ nr_vpath_stroke (const ArtVpath *path, NRMatrixF *transform,
 	svlb.left.refvx = NULL;
 	svlb.left.dir = 0;
 	svlb.left.reverse = FALSE;
+	svlb.left.bboxonly = FALSE;
 	svlb.left.sx = svlb.left.sy = 0.0;
 	nr_rect_f_set_empty (&svlb.left.bbox);
 
@@ -357,6 +531,7 @@ nr_vpath_stroke (const ArtVpath *path, NRMatrixF *transform,
 	svlb.right.refvx = NULL;
 	svlb.right.dir = 0;
 	svlb.right.reverse = TRUE;
+	svlb.right.bboxonly = FALSE;
 	svlb.right.sx = svlb.right.sy = 0.0;
 	nr_rect_f_set_empty (&svlb.right.bbox);
 
@@ -407,6 +582,8 @@ nr_svl_stroke_build_draw_cap (NRSVLStrokeBuild *svlb, float x0, float y0, float
 {
 	float len, dx, dy;
 	len = hypot (x1 - x0, y1 - y0);
+	/* Extra check for WinME */
+	if (fabs (len) < NR_EPSILON_F) return;
 	dx = (x1 - x0) / len;
 	dy = (y1 - y0) / len;
 	if (svlb->cap == NR_STROKE_CAP_BUTT) {
@@ -471,11 +648,15 @@ nr_svl_stroke_build_draw_join (NRSVLStrokeBuild *svlb, float x0, float y0, float
 	float len1, dx1, dy1, px1, py1;
 	double costheta;
 	len0 = hypot (x1 - x0, y1 - y0);
+	/* Extra check for WinME */
+	if (fabs (len0) < NR_EPSILON_F) return;
 	dx0 = (x1 - x0) / len0;
 	dy0 = (y1 - y0) / len0;
 	px0 = dy0 * svlb->width_2;
 	py0 = -dx0 * svlb->width_2;
 	len1 = hypot (x2 - x1, y2 - y1);
+	/* Extra check for WinME */
+	if (fabs (len1) < NR_EPSILON_F) return;
 	dx1 = (x2 - x1) / len1;
 	dy1 = (y2 - y1) / len1;
 	px1 = dy1 * svlb->width_2;
@@ -491,6 +672,8 @@ nr_svl_stroke_build_draw_join (NRSVLStrokeBuild *svlb, float x0, float y0, float
 		dx = (px0 + px1) / 2.0;
 		dy = (py0 + py1) / 2.0;
 		d2 = dx * dx + dy * dy;
+		/* Extra check for WinME */
+		if (fabs (d2) < NR_EPSILON_F) return;
 		D_d = svlb->width_2 * svlb->width_2 / d2;
 		dx *= D_d;
 		dy *= D_d;
