@@ -7,11 +7,32 @@
  * Authors:
  *   Lauris Kaplinski <lauris@kaplinski.com>
  *
- * Copyright (C) 2002 Lauris Kaplinski
- *
- * Released under GNU GPL, read the file 'COPYING' for more information
+ * This code is in public domain
  */
 
+#ifdef HAVE_INTTYPES_H
+/* ISO C99 headers */
+#include <inttypes.h>
+typedef int8_t NRByte;
+typedef uint8_t NRUByte;
+typedef int16_t NRShort;
+typedef uint16_t NRUShort;
+typedef int32_t NRLong;
+typedef uint32_t NRULong;
+#else
+#ifdef HAVE_NR_CONFIG_H
+/* Automake based systems */
+#include <libnr/nr_config.h>
+#else
+/* Currently only Win32 */
+typedef signed char NRByte;
+typedef unsigned char NRUByte;
+typedef signed short NRShort;
+typedef unsigned short NRUShort;
+typedef signed int NRLong;
+typedef unsigned int NRULong;
+#endif
+#endif
 
 typedef struct _NRMatrixD NRMatrixD;
 typedef struct _NRMatrixF NRMatrixF;
@@ -41,11 +62,11 @@ struct _NRPointF {
 };
 
 struct _NRPointL {
-	long x, y;
+	NRLong x, y;
 };
 
 struct _NRPointS {
-	short x, y;
+	NRShort x, y;
 };
 
 struct _NRRectD {
@@ -57,11 +78,11 @@ struct _NRRectF {
 };
 
 struct _NRRectL {
-	long x0, y0, x1, y1;
+	NRLong x0, y0, x1, y1;
 };
 
 struct _NRRectS {
-	short x0, y0, x1, y1;
+	NRShort x0, y0, x1, y1;
 };
 
 #endif
