@@ -10,6 +10,7 @@
  * This code is in public domain
  */
 
+typedef struct _NRPosGlyph NRPosGlyph;
 typedef struct _NRPGL NRPGL;
 
 #include <libnr/nr-types.h>
@@ -23,6 +24,7 @@ struct _NRPosGlyph {
 
 struct _NRPGL {
 	NRRasterFont *rfont;
+	NRPointF origin;
 	NRRectF area;
 	NRPointF advance;
 	unsigned int length;
@@ -33,5 +35,7 @@ struct _NRPGL {
 NRPGL *nr_pgl_new_from_string (NRUShort *text, unsigned int length, NRFont *font, NRMatrixF *transform);
 NRPGL *nr_pgl_free (NRPGL *pgl);
 
+void nr_pgl_set_origin (NRPGL *pgl, float x, float y);
+
 #endif
 
