@@ -1,5 +1,14 @@
-#ifndef NODEPATH_H
-#define NODEPATH_H
+#ifndef __SP_NODEPATH_H__
+#define __SP_NODEPATH_H__
+
+/*
+ * Path handler in node edit mode
+ *
+ * Authors:
+ *   Lauris Kaplinski <lauris@kaplinski.com>
+ *
+ * This code is in public domain
+ */
 
 #include "xml/repr.h"
 #include "knot.h"
@@ -22,7 +31,7 @@ struct _SPNodePath {
 	SPPath * path;
 	GSList * subpaths;
 	GSList * selected;
-	double i2d[6], d2i[6];
+	NRMatrixD i2d, d2i;
 };
 
 struct _SPNodeSubPath {
@@ -35,7 +44,7 @@ struct _SPNodeSubPath {
 
 typedef struct {
 	SPPathNode * other;
-	ArtPoint pos;
+	NRPointF pos;
 	SPKnot * knot;
 	SPCanvasItem * line;
 } SPPathNodeSide;
@@ -45,7 +54,7 @@ struct _SPPathNode {
 	guint type : 4;
 	guint code : 4;
 	guint selected : 1;
-	ArtPoint pos;
+	NRPointF pos;
 	SPKnot * knot;
 	SPPathNodeSide n;
 	SPPathNodeSide p;
@@ -54,7 +63,7 @@ struct _SPPathNode {
 SPNodePath * sp_nodepath_new (SPDesktop * desktop, SPItem * item);
 void sp_nodepath_destroy (SPNodePath * nodepath);
 
-void sp_nodepath_select_rect (SPNodePath * nodepath, ArtDRect * b, gboolean incremental);
+void sp_nodepath_select_rect (SPNodePath * nodepath, NRRectD * b, gboolean incremental);
 gboolean node_key (GdkEvent * event);
 
 /* possibly private functions */
@@ -63,6 +72,7 @@ void sp_node_selected_add_node (void);
 void sp_node_selected_delete (void);
 void sp_node_selected_break (void);
 void sp_node_selected_join (void);
+void sp_node_selected_join_segment (void);
 void sp_node_selected_set_type (SPPathNodeType type);
 void sp_node_selected_set_line_type (ArtPathcode code);
 
