@@ -29,26 +29,6 @@
 #include "module.h"
 #include "print.h"
 
-#include <modules/ps.h>
-
-#ifdef WIN32
-#include <modules/win32.h>
-#endif
-
-#ifdef WITH_GNOME_PRINT
-#include <modules/gnome.h>
-#endif
-
-#ifdef WITH_KDE
-#include <modules/kde.h>
-#endif
-
-/* Identity typedef */
-
-struct _SPPrintContext {
-	SPModulePrint module;
-};
-
 unsigned int
 sp_print_bind (SPPrintContext *ctx, const NRMatrixF *transform, float opacity)
 {
@@ -98,6 +78,19 @@ sp_print_image_R8G8B8A8_N (SPPrintContext *ctx,
 	return 0;
 }
 
+#ifndef WITHOUT_PRINT_UI
+
+#ifdef WIN32
+#include <modules/win32.h>
+#endif
+
+#ifdef WITH_GNOME_PRINT
+#include <modules/gnome.h>
+#endif
+
+#ifdef WITH_KDE
+#include <modules/kde.h>
+#endif
 
 #include "display/nr-arena.h"
 #include "display/nr-arena-item.h"
@@ -112,20 +105,18 @@ sp_print_preview_document (SPDocument *doc)
 
 	sp_document_ensure_up_to_date (doc);
 
+	mod = NULL;
 #ifdef WIN32
-	mod = g_object_new (SP_TYPE_MODULE_PRINT_WIN32, NULL);
-#else
-// Some unix probably
+	if (!mod) mod = (SPModulePrint *) sp_module_find (SP_MODULE_KEY_PRINT_WIN32);
+#endif
 #ifdef WITH_KDE
-	mod = sp_kde_get_module_print ();
-#else
+	if (!mod) mod = (SPModulePrint *) sp_module_find (SP_MODULE_KEY_PRINT_KDE);
+#endif
 #ifdef WITH_GNOME_PRINT
-	mod = (SPModulePrint *) sp_module_new (SP_TYPE_MODULE_PRINT_GNOME, NULL);
-#else
-	mod = (SPModulePrint *) sp_module_new_from_path (SP_TYPE_MODULE_PRINT_PLAIN, "printing.ps");
-#endif
-#endif
+	if (!mod) mod = (SPModulePrint *) sp_module_find (SP_MODULE_KEY_PRINT_GNOME);
 #endif
+	/* Still nothing, fall back to plain */
+	if (!mod) mod = (SPModulePrint *) sp_module_find (SP_MODULE_KEY_PRINT_PLAIN);
 
 	ret = FALSE;
 	if (((SPModulePrintClass *) G_OBJECT_GET_CLASS (mod))->set_preview)
@@ -157,7 +148,7 @@ sp_print_preview_document (SPDocument *doc)
 }
 
 void
-sp_print_document (SPDocument *doc, unsigned int direct)
+sp_print_document (SPDocument *doc, unsigned int direct, const unsigned char *filename)
 {
 	SPModulePrint *mod;
 	unsigned int ret;
@@ -165,21 +156,26 @@ sp_print_document (SPDocument *doc, unsigned int direct)
 	sp_document_ensure_up_to_date (doc);
 
 	mod = NULL;
-	if (direct) mod = (SPModulePrint *) sp_module_new_from_path (SP_TYPE_MODULE_PRINT_PLAIN, "printing.ps");
+	if (direct) mod = (SPModulePrint *) sp_module_find (SP_MODULE_KEY_PRINT_PLAIN);
 #ifdef WIN32
-	if (!direct) mod = g_object_new (SP_TYPE_MODULE_PRINT_WIN32, NULL);
+	if (!mod) mod = (SPModulePrint *) sp_module_find (SP_MODULE_KEY_PRINT_WIN32);
 #endif
 #ifdef WITH_KDE
-	if (!direct) mod = sp_kde_get_module_print ();
+	if (!mod) mod = (SPModulePrint *) sp_module_find (SP_MODULE_KEY_PRINT_KDE);
 #endif
 #ifdef WITH_GNOME_PRINT
-	if (!direct) mod = g_object_new (SP_TYPE_MODULE_PRINT_GNOME, NULL);
+	if (!mod) mod = (SPModulePrint *) sp_module_find (SP_MODULE_KEY_PRINT_GNOME);
 #endif
-	if (!mod) mod = (SPModulePrint *) sp_module_new_from_path (SP_TYPE_MODULE_PRINT_PLAIN, "printing.ps");
+	if (!mod) mod = (SPModulePrint *) sp_module_find (SP_MODULE_KEY_PRINT_PLAIN);
 
 	ret = FALSE;
-	if (((SPModulePrintClass *) G_OBJECT_GET_CLASS (mod))->setup)
-		ret = ((SPModulePrintClass *) G_OBJECT_GET_CLASS (mod))->setup (mod);
+        if (filename != NULL) {
+		if (((SPModulePrintClass *) G_OBJECT_GET_CLASS (mod))->setup_file)
+			ret = ((SPModulePrintClass *) G_OBJECT_GET_CLASS (mod))->setup_file (mod, filename);
+        } else {
+		if (((SPModulePrintClass *) G_OBJECT_GET_CLASS (mod))->setup)
+			ret = ((SPModulePrintClass *) G_OBJECT_GET_CLASS (mod))->setup (mod);
+        }
 
 	if (ret) {
 		/* fixme: This has to go into module constructor somehow */
@@ -206,48 +202,5 @@ sp_print_document (SPDocument *doc, unsigned int direct)
 	g_object_unref (G_OBJECT (mod));
 }
 
-void
-sp_print_document_to_file (SPDocument *doc, const unsigned char *filename)
-{
-#ifdef lalala
-#ifdef WITH_GNOME_PRINT
-        GnomePrintConfig *config;
-	SPPrintContext ctx;
-        GnomePrintContext *gpc;
 
-	config = gnome_print_config_default ();
-        if (!gnome_print_config_set (config, "Settings.Engine.Backend.Driver", "gnome-print-ps")) return;
-        if (!gnome_print_config_set (config, "Settings.Transport.Backend", "file")) return;
-        if (!gnome_print_config_set (config, GNOME_PRINT_KEY_OUTPUT_FILENAME, filename)) return;
-
-	sp_document_ensure_up_to_date (doc);
-
-	gpc = gnome_print_context_new (config);
-	ctx.gpc = gpc;
-
-	g_return_if_fail (gpc != NULL);
-
-	/* Print document */
-	gnome_print_beginpage (gpc, SP_DOCUMENT_NAME (doc));
-	gnome_print_translate (gpc, 0.0, sp_document_height (doc));
-	/* From desktop points to document pixels */
-	gnome_print_scale (gpc, 0.8, -0.8);
-	sp_item_invoke_print (SP_ITEM (sp_document_root (doc)), &ctx);
-        gnome_print_showpage (gpc);
-        gnome_print_context_close (gpc);
-#else
-	SPPrintContext ctx;
-
-	ctx.stream = fopen (filename, "w");
-	if (ctx.stream) {
-		sp_document_ensure_up_to_date (doc);
-		fprintf (ctx.stream, "%g %g translate\n", 0.0, sp_document_height (doc));
-		fprintf (ctx.stream, "0.8 -0.8 scale\n");
-		sp_item_invoke_print (SP_ITEM (sp_document_root (doc)), &ctx);
-		fprintf (ctx.stream, "showpage\n");
-		fclose (ctx.stream);
-	}
 #endif
-#endif
-}
-
