@@ -89,6 +89,18 @@ sp_selection_action_perform (SPAction *action, void *config, void *data)
 	case SP_VERB_SELECTION_BREAK_APART:
 		sp_selected_path_break_apart ();
 		break;
+	case SP_VERB_SELECTION_UNCROSS:
+		sp_selected_path_uncross (SP_PATH_UNCROSS);
+		break;
+	case SP_VERB_SELECTION_UNION:
+		sp_selected_path_uncross (SP_PATH_UNION);
+		break;
+	case SP_VERB_SELECTION_INTERSECTION:
+		sp_selected_path_uncross (SP_PATH_INTERSECTION);
+		break;
+	case SP_VERB_SELECTION_SUBTRACTION:
+		sp_selected_path_uncross (SP_PATH_SUBTRACTION);
+		break;
 	case SP_VERB_SELECTION_TRANSFORM:
 		sp_selection_perform_transform (dt, (SPRepr *) config);
 		break;
@@ -393,7 +405,9 @@ sp_selection_delete (gpointer object, gpointer data)
 	sp_selection_empty (selection);
 
 	while (selected) {
-		sp_repr_unparent ((SPRepr *) selected->data);
+		SPRepr *repr;
+		repr = (SPRepr *) selected->data;
+		if (sp_repr_parent (repr)) sp_repr_unparent (repr);
 		selected = g_slist_remove (selected, selected->data);
 	}
 
@@ -799,22 +813,22 @@ sp_selection_move_relative (SPSelection * selection, double dx, double dy, unsig
 void
 sp_selection_rotate_90 (void)
 {
-	SPDesktop * desktop;
-	SPSelection * selection;
-	SPItem * item;
-	GSList * l, * l2;
+	SPDesktop *desktop;
+	SPSelection *selection;
+	NRRectF bbox;
+	NRPointF center;
 
 	desktop = SP_ACTIVE_DESKTOP;
 	if (!SP_IS_DESKTOP(desktop)) return;
 	selection = SP_DT_SELECTION(desktop);
-	if sp_selection_is_empty(selection) return;
-	l = selection->items;  
-	for (l2 = l; l2 != NULL; l2 = l2-> next) {
-		item = SP_ITEM (l2->data);
-		sp_item_rotate_rel (item,-90);
-	}
+	if sp_selection_is_empty (selection) return;
+	sp_selection_bbox (selection, &bbox);
+	center.x = (bbox.x0 + bbox.x1) / 2.0;
+	center.y = (bbox.y0 + bbox.y1) / 2.0;
 
-	sp_selection_changed (selection);
+	sp_selection_rotate_relative (selection, &center, 90.0, FALSE);
+
+	/* sp_selection_changed (selection); */
 	sp_document_done (SP_DT_DOCUMENT (desktop));
 }
 
