@@ -18,12 +18,11 @@
 #include <string.h>
 #include <stdlib.h>
 #include <sys/stat.h>
-#include <time.h>
 #include <fcntl.h>
-#include <signal.h>
 #include <ctype.h>
+#include <assert.h>
 
-#include "monostd.h"
+#include <libarikkei/arikkei-token.h>
 
 #include <glib.h>
 #include <gdk/gdkkeysyms.h>
@@ -31,11 +30,14 @@
 #include <gtk/gtksignal.h>
 #include <gtk/gtkmessagedialog.h>
 
+#include "monostd.h"
+
 #include "helper/sp-intl.h"
 #include "helper/sp-marshal.h"
 #include "helper/action.h"
 #include "xml/repr-private.h"
 
+#include "api.h"
 #include "system.h"
 #include "verbs.h"
 #include "shortcuts.h"
@@ -66,10 +68,15 @@ enum {
 	LAST_SIGNAL
 };
 
-#define DESKTOP_IS_ACTIVE(d) ((d) == sodipodi->desktops->data)
+#define SP_TYPE_SODIPODI (sodipodi_get_type ())
+#define SP_SODIPODI(obj) (GTK_CHECK_CAST ((obj), SP_TYPE_SODIPODI, Sodipodi))
+#define SP_IS_SODIPODI(obj) (GTK_CHECK_TYPE ((obj), SP_TYPE_SODIPODI))
+
+#define DESKTOP_IS_ACTIVE(d) ((d) && (sodipodi->desktops) && (d) == sodipodi->desktops->data)
 
 static void sodipodi_class_init (SodipodiClass *klass);
 static void sodipodi_init (SPObject *object);
+static void sodipodi_finalize (GObject *object);
 static void sodipodi_dispose (GObject *object);
 
 static void sodipodi_activate_desktop_private (Sodipodi *sodipodi, SPDesktop *desktop);
@@ -83,7 +90,7 @@ static void sodipodi_init_config (SPReprDoc *doc, const gchar *config_name, cons
 static void sodipodi_init_preferences (Sodipodi *sodipodi);
 static void sodipodi_init_extensions (Sodipodi *sodipodi);
 
-static void sodipodi_init_preferences (Sodipodi * sodipodi);
+void sodipodi_save_extensions (Sodipodi * sodipodi);
 
 struct _Sodipodi {
 	GObject object;
@@ -94,6 +101,8 @@ struct _Sodipodi {
 	/* Last action */
 	unsigned int action_verb;
 	SPRepr *action_config;
+	/* Current tool verb */
+	unsigned int eventverb;
 };
 
 struct _SodipodiClass {
@@ -119,9 +128,7 @@ static guint sodipodi_signals[LAST_SIGNAL] = {0};
 
 Sodipodi *sodipodi = NULL;
 
-static void (* segv_handler) (int) = NULL;
-
-GType
+static GType
 sodipodi_get_type (void)
 {
 	static GType type = 0;
@@ -238,6 +245,7 @@ sodipodi_class_init (SodipodiClass * klass)
 							   G_TYPE_NONE, 2,
 							   G_TYPE_POINTER, G_TYPE_DOUBLE);
 
+	object_class->finalize = sodipodi_finalize;
 	object_class->dispose = sodipodi_dispose;
 
 	klass->activate_desktop = sodipodi_activate_desktop_private;
@@ -247,67 +255,66 @@ sodipodi_class_init (SodipodiClass * klass)
 static void
 sodipodi_init (SPObject * object)
 {
-	if (!sodipodi) {
-		sodipodi = (Sodipodi *) object;
-	} else {
-		g_assert_not_reached ();
-	}
+	g_assert (!sodipodi);
 
-	sodipodi->preferences = sp_repr_read_mem (preferences_skeleton, PREFERENCES_SKELETON_SIZE, NULL);
+	/* Set static object */
+	sodipodi = (Sodipodi *) object;
+	sodipodi->eventverb = SP_VERB_NONE;
+	/* Initialize preferences to default tree */
+	sodipodi->preferences = sp_repr_doc_new_from_mem (preferences_skeleton, PREFERENCES_SKELETON_SIZE, NULL);
+	/* Initialize extensions to default tree */
+	sodipodi->extensions = sp_repr_doc_new_from_mem (extensions_skeleton, EXTENSIONS_SKELETON_SIZE, NULL);
 
-	sodipodi->extensions = sp_repr_read_mem (extensions_skeleton, EXTENSIONS_SKELETON_SIZE, NULL);
-
-	/* Initialize shortcuts */
+	/* Initialize shortcut table */
 	sp_shortcut_table_load (NULL);
-
-	sodipodi->documents = NULL;
-	sodipodi->desktops = NULL;
 }
 
+static void
+sodipodi_finalize (GObject *object)
+{
+	g_assert (object == (GObject *) sodipodi);
+
+	sodipodi = NULL;
+
+	G_OBJECT_CLASS (parent_class)->finalize (object);
+
+	gtk_main_quit ();
+}
+
+/* fixme: We need to use ::dispose because document want to ref sodipodi */
+
 static void
 sodipodi_dispose (GObject *object)
 {
-	Sodipodi *sodipodi;
+	g_assert (object == (GObject *) sodipodi);
+	g_assert (!sodipodi->desktops);
 
-	sodipodi = (Sodipodi *) object;
+	/* fixme: This is uttermost crap (Lauris) */
+	while (sodipodi->documents) {
+		g_object_unref ((GObject *) sodipodi->documents->data);
+		/* Only private documents can exist at this step */
+		g_assert (sodipodi);
+	}
 
 	if (sodipodi->action_config) {
 		sodipodi->action_config = sp_repr_unref (sodipodi->action_config);
 	}
 
-	while (sodipodi->documents) {
-		g_object_unref (G_OBJECT (sodipodi->documents->data));
-	}
-
-	g_assert (!sodipodi->desktops);
-
 	if (sodipodi->extensions) {
-		sp_repr_document_unref (sodipodi->extensions);
+		/* fixme: This is not the best place */
+		sodipodi_save_extensions (sodipodi);
+		sp_repr_doc_unref (sodipodi->extensions);
 		sodipodi->extensions = NULL;
 	}
 
 	if (sodipodi->preferences) {
 		/* fixme: This is not the best place */
 		sodipodi_save_preferences (sodipodi);
-		sp_repr_document_unref (sodipodi->preferences);
+		sp_repr_doc_unref (sodipodi->preferences);
 		sodipodi->preferences = NULL;
 	}
 
 	G_OBJECT_CLASS (parent_class)->dispose (object);
-
-	gtk_main_quit ();
-}
-
-void
-sodipodi_ref (void)
-{
-	g_object_ref (G_OBJECT (sodipodi));
-}
-
-void
-sodipodi_unref (void)
-{
-	g_object_unref (G_OBJECT (sodipodi));
 }
 
 static void
@@ -322,166 +329,6 @@ sodipodi_desactivate_desktop_private (Sodipodi *sodipodi, SPDesktop *desktop)
 	sp_desktop_set_active (desktop, FALSE);
 }
 
-/* fixme: This is EVIL, and belongs to main after all */
-
-#define SP_INDENT 8
-
-static void
-sodipodi_segv_handler (int signum)
-{
-	static gint recursion = FALSE;
-	GSList *savednames, *failednames, *l;
-	const gchar *home;
-	gchar *istr, *sstr, *fstr, *b;
-	gint count, nllen, len, pos;
-	time_t sptime;
-	struct tm *sptm;
-	char sptstr[256];
-	GtkWidget *msgbox;
-
-	/* Kill loops */
-	if (recursion) abort ();
-	recursion = TRUE;
-
-	g_warning ("Emergency save activated");
-
-	home = g_get_home_dir ();
-	sptime = time (NULL);
-	sptm = localtime (&sptime);
-	strftime (sptstr, 256, "%Y_%m_%d_%H_%M_%S", sptm);
-
-	count = 0;
-	savednames = NULL;
-	failednames = NULL;
-	for (l = sodipodi->documents; l != NULL; l = l->next) {
-		SPDocument *doc;
-		SPRepr *repr;
-		doc = (SPDocument *) l->data;
-		repr = sp_document_repr_root (doc);
-		if (sp_repr_attr (repr, "sodipodi:modified")) {
-			const guchar *docname, *d0, *d;
-			gchar n[64], c[1024];
-			FILE *file;
-#if 0
-			docname = sp_repr_attr (repr, "sodipodi:docname");
-			if (docname) {
-				docname = g_basename (docname);
-			}
-#else
-			docname = doc->name;
-			if (docname) {
-				/* fixme: Quick hack to remove emergency file suffix */
-				d0 = strrchr (docname, '.');
-				if (d0 && (d0 > docname)) {
-					d0 = strrchr (d0 - 1, '.');
-					if (d0 && (d0 > docname)) {
-						d = d0;
-						while (isdigit (*d) || (*d == '.') || (*d == '_')) d += 1;
-						if (*d) {
-							memcpy (n, docname, MIN (d0 - docname - 1, 64));
-							n[64] = '\0';
-							docname = n;
-						}
-					}
-				}
-			}
-#endif
-			if (!docname || !*docname) docname = "emergency";
-			g_snprintf (c, 1024, "%s/.sodipodi/%.256s.%s.%d", home, docname, sptstr, count);
-			file = fopen (c, "w");
-			if (!file) {
-				g_snprintf (c, 1024, "%s/sodipodi-%.256s.%s.%d", home, docname, sptstr, count);
-				file = fopen (c, "w");
-			}
-			if (!file) {
-				g_snprintf (c, 1024, "/tmp/sodipodi-%.256s.%s.%d", docname, sptstr, count);
-				file = fopen (c, "w");
-			}
-			if (file) {
-				sp_repr_save_stream (sp_repr_document (repr), file);
-				savednames = g_slist_prepend (savednames, g_strdup (c));
-				fclose (file);
-			} else {
-				docname = sp_repr_attr (repr, "sodipodi:docname");
-				failednames = g_slist_prepend (failednames, (docname) ? g_strdup (docname) : g_strdup (_("Untitled document")));
-			}
-			count++;
-		}
-	}
-
-	savednames = g_slist_reverse (savednames);
-	failednames = g_slist_reverse (failednames);
-	if (savednames) {
-		fprintf (stderr, "\nEmergency save locations:\n");
-		for (l = savednames; l != NULL; l = l->next) {
-			fprintf (stderr, "  %s\n", (gchar *) l->data);
-		}
-	}
-	if (failednames) {
-		fprintf (stderr, "Failed to do emergency save for:\n");
-		for (l = failednames; l != NULL; l = l->next) {
-			fprintf (stderr, "  %s\n", (gchar *) l->data);
-		}
-	}
-
-	g_warning ("Emergency save completed, now crashing...");
-
-	/* Show nice dialog box */
-
-	istr = N_("Sodipodi encountered an internal error and will close now.\n");
-	sstr = N_("Automatic backups of unsaved documents were done to following locations:\n");
-	fstr = N_("Automatic backup of following documents failed:\n");
-	nllen = strlen ("\n");
-	len = strlen (istr) + strlen (sstr) + strlen (fstr);
-	for (l = savednames; l != NULL; l = l->next) {
-		len = len + SP_INDENT + strlen ((gchar *) l->data) + nllen;
-	}
-	for (l = failednames; l != NULL; l = l->next) {
-		len = len + SP_INDENT + strlen ((gchar *) l->data) + nllen;
-	}
-	len += 1;
-	b = g_new (gchar, len);
-	pos = 0;
-	len = strlen (istr);
-	memcpy (b + pos, istr, len);
-	pos += len;
-	if (savednames) {
-		len = strlen (sstr);
-		memcpy (b + pos, sstr, len);
-		pos += len;
-		for (l = savednames; l != NULL; l = l->next) {
-			memset (b + pos, ' ', SP_INDENT);
-			pos += SP_INDENT;
-			len = strlen ((gchar *) l->data);
-			memcpy (b + pos, l->data, len);
-			pos += len;
-			memcpy (b + pos, "\n", nllen);
-			pos += nllen;
-		}
-	}
-	if (failednames) {
-		len = strlen (fstr);
-		memcpy (b + pos, fstr, len);
-		pos += len;
-		for (l = failednames; l != NULL; l = l->next) {
-			memset (b + pos, ' ', SP_INDENT);
-			pos += SP_INDENT;
-			len = strlen ((gchar *) l->data);
-			memcpy (b + pos, l->data, len);
-			pos += len;
-			memcpy (b + pos, "\n", nllen);
-			pos += nllen;
-		}
-	}
-	*(b + pos) = '\0';
-	msgbox = gtk_message_dialog_new (NULL, GTK_DIALOG_MODAL, GTK_MESSAGE_ERROR, GTK_BUTTONS_CLOSE, "%s", b);
-	gtk_dialog_run (GTK_DIALOG (msgbox));
-	gtk_widget_destroy (msgbox);
-	g_free (b);
-
-	(* segv_handler) (signum);
-}
-
 Sodipodi *
 sodipodi_application_new (void)
 {
@@ -490,15 +337,52 @@ sodipodi_application_new (void)
 	sp = g_object_new (SP_TYPE_SODIPODI, NULL);
 	/* fixme: load application defaults */
 
-#ifndef WIN32
-	segv_handler = signal (SIGSEGV, sodipodi_segv_handler);
-	signal (SIGFPE, sodipodi_segv_handler);
-	signal (SIGILL, sodipodi_segv_handler);
-#endif
-
 	return sp;
 }
 
+static void
+sodipodi_document_destroy (SPDocument *doc, void *data)
+{
+	sodipodi_remove_document (doc);
+	sodipodi_unref ();
+}
+
+SPDocument *
+sodipodi_document_new (const unsigned char *uri, unsigned int advertize, unsigned int keepalive)
+{
+	SPDocument *doc;
+	doc = sp_document_new (uri);
+	if (doc) {
+		sp_document_set_advertize (doc, TRUE);
+		if (keepalive) {
+			g_signal_connect ((GObject *) doc, "destroy", (GCallback) sodipodi_document_destroy, NULL);
+			sodipodi_ref ();
+		}
+		sp_document_set_undo_sensitive (doc, TRUE);
+		sodipodi_add_document (doc);
+	}
+	return doc;
+}
+
+SPDocument *
+sodipodi_document_new_from_mem (const unsigned char *cdata, unsigned int length,
+				unsigned int advertize, unsigned int keepalive)
+{
+	SPDocument *doc;
+	doc = sp_document_new_from_mem (cdata, length);
+	if (doc) {
+		sp_document_set_advertize (doc, TRUE);
+		if (keepalive) {
+			g_signal_connect ((GObject *) doc, "destroy", (GCallback) sodipodi_document_destroy, NULL);
+			sodipodi_ref ();
+		}
+		sp_document_set_undo_sensitive (doc, TRUE);
+		sodipodi_add_document (doc);
+	}
+	return doc;
+}
+
+
 /* Preference management */
 /* We use '.' as separator */
 
@@ -506,18 +390,21 @@ static void
 sodipodi_load_config (const unsigned char *filename, SPReprDoc *config, const unsigned char *skeleton, unsigned int skel_size,
 		      const unsigned char *e_notreg, const unsigned char *e_notxml, const unsigned char *e_notsp)
 {
-	gchar *fn;
+	unsigned char *fnutf8, *fnos;
 	struct stat s;
 	GtkWidget * w;
 	SPReprDoc * doc;
 	SPRepr * root;
+	gsize bytesin, bytesout;
 
 #ifdef WIN32
-	fn = g_build_filename (SODIPODI_APPDATADIR, filename, NULL);
+	fnutf8 = g_build_filename (SODIPODI_APPDATADIR, filename, NULL);
+	fnos = g_filename_from_utf8 (fnutf8, strlen (fnutf8), &bytesin, &bytesout, NULL);
 #else
-	fn = g_build_filename (g_get_home_dir (), ".sodipodi", filename, NULL);
+	fnutf8 = g_build_filename (g_get_home_dir (), ".sodipodi", filename, NULL);
+	fnos = g_filename_from_utf8 (fnutf8, strlen (fnutf8), &bytesin, &bytesout, NULL);
 #endif
-	if (stat (fn, &s)) {
+	if (stat (fnos, &s)) {
 		/* No such file */
 		/* fixme: Think out something (Lauris) */
 		if (!strcmp (filename, "extensions")) {
@@ -525,42 +412,51 @@ sodipodi_load_config (const unsigned char *filename, SPReprDoc *config, const un
 		} else {
 			sodipodi_init_preferences (SODIPODI);
 		}
-		g_free (fn);
+		g_free (fnos);
+		g_free (fnutf8);
 		return;
 	}
 
 	if (!S_ISREG (s.st_mode)) {
 		/* Not a regular file */
-		w = gtk_message_dialog_new (NULL, GTK_DIALOG_MODAL, GTK_MESSAGE_WARNING, GTK_BUTTONS_OK, e_notreg, fn);
+		w = gtk_message_dialog_new (NULL, GTK_DIALOG_MODAL, GTK_MESSAGE_WARNING, GTK_BUTTONS_OK,
+					    e_notreg, fnutf8);
 		gtk_dialog_run (GTK_DIALOG (w));
 		gtk_widget_destroy (w);
-		g_free (fn);
+		g_free (fnos);
+		g_free (fnutf8);
 		return;
 	}
 
-	doc = sp_repr_read_file (fn, NULL);
+	doc = sp_repr_doc_new_from_file (fnutf8, NULL);
 	if (doc == NULL) {
 		/* Not an valid xml file */
-		w = gtk_message_dialog_new (NULL, GTK_DIALOG_MODAL, GTK_MESSAGE_WARNING, GTK_BUTTONS_OK, e_notxml, fn);
+		w = gtk_message_dialog_new (NULL, GTK_DIALOG_MODAL, GTK_MESSAGE_WARNING, GTK_BUTTONS_OK,
+					    e_notxml, fnutf8);
 		gtk_dialog_run (GTK_DIALOG (w));
 		gtk_widget_destroy (w);
-		g_free (fn);
+		g_free (fnos);
+		g_free (fnutf8);
 		return;
 	}
 
-	root = sp_repr_document_root (doc);
+	root = sp_repr_doc_get_root (doc);
 	if (strcmp (sp_repr_name (root), "sodipodi")) {
-		w = gtk_message_dialog_new (NULL, GTK_DIALOG_MODAL, GTK_MESSAGE_WARNING, GTK_BUTTONS_OK, e_notsp, fn);
+		w = gtk_message_dialog_new (NULL, GTK_DIALOG_MODAL, GTK_MESSAGE_WARNING, GTK_BUTTONS_OK,
+					    e_notsp, fnutf8);
 		gtk_dialog_run (GTK_DIALOG (w));
 		gtk_widget_destroy (w);
-		sp_repr_document_unref (doc);
-		g_free (fn);
+		sp_repr_doc_unref (doc);
+		g_free (fnos);
+		g_free (fnutf8);
 		return;
 	}
 
-	sp_repr_document_merge (config, doc, "id");
-	sp_repr_document_unref (doc);
-	g_free (fn);
+	g_free (fnos);
+	g_free (fnutf8);
+
+	sp_repr_doc_merge (config, doc, "id");
+	sp_repr_doc_unref (doc);
 }
 
 /* Preferences management */
@@ -612,47 +508,25 @@ sodipodi_save_preferences (Sodipodi * sodipodi)
 	fn = g_build_filename (g_get_home_dir (), ".sodipodi/preferences", NULL);
 #endif
 
-	sp_repr_save_file (sodipodi->preferences, fn);
+	sp_repr_doc_write_file (sodipodi->preferences, fn);
 
 	g_free (fn);
 }
 
-/* We use '.' as separator */
-SPRepr *
-sodipodi_get_repr (Sodipodi *sodipodi, const unsigned char *key)
+void
+sodipodi_save_extensions (Sodipodi * sodipodi)
 {
-	SPRepr * repr;
-	const gchar * id, * s, * e;
-	gint len;
+	gchar * fn;
 
-	if (key == NULL) return NULL;
+#ifdef WIN32
+	fn = g_build_filename (SODIPODI_APPDATADIR, "extensions", NULL);
+#else
+	fn = g_build_filename (g_get_home_dir (), ".sodipodi/extensions", NULL);
+#endif
 
-	if (!strncmp (key, "extensions", 10) && (!key[10] || (key[10] == '.'))) {
-		repr = sp_repr_document_root (sodipodi->extensions);
-	} else {
-		repr = sp_repr_document_root (sodipodi->preferences);
-	}
-	g_assert (!(strcmp (sp_repr_name (repr), "sodipodi")));
+	sp_repr_doc_write_file (sodipodi->extensions, fn);
 
-	s = key;
-	while ((s) && (*s)) {
-		SPRepr * child;
-		/* Find next name */
-		if ((e = strchr (s, '.'))) {
-			len = e++ - s;
-		} else {
-			len = strlen (s);
-		}
-		for (child = repr->children; child != NULL; child = child->next) {
-			id = sp_repr_attr (child, "id");
-			if ((id) && (strlen (id) == len) && (!strncmp (id, s, len))) break;
-		}
-		if (child == NULL) return NULL;
-
-		repr = child;
-		s = e;
-	}
-	return repr;
+	g_free (fn);
 }
 
 void
@@ -693,13 +567,19 @@ sodipodi_selection_set (SPSelection * selection)
 }
 
 void
-sodipodi_eventcontext_set (SPEventContext * eventcontext)
+sodipodi_eventcontext_set (SPEventContext *eventcontext)
 {
-	g_return_if_fail (sodipodi != NULL);
-	g_return_if_fail (eventcontext != NULL);
-	g_return_if_fail (SP_IS_EVENT_CONTEXT (eventcontext));
+	SPAction *action;
 
-	if (DESKTOP_IS_ACTIVE (eventcontext->desktop)) {
+	g_return_if_fail (sodipodi != NULL);
+
+	action = sp_verb_get_action (sodipodi->eventverb);
+	if (action) sp_action_set_active (action, FALSE);
+	sodipodi->eventverb = (eventcontext) ? eventcontext->verb : SP_VERB_NONE;
+	action = sp_verb_get_action (sodipodi->eventverb);
+	if (action) sp_action_set_active (action, TRUE);
+
+	if (eventcontext && DESKTOP_IS_ACTIVE (eventcontext->desktop)) {
 		g_signal_emit (G_OBJECT (sodipodi), sodipodi_signals[SET_EVENTCONTEXT], 0, eventcontext);
 	}
 }
@@ -719,7 +599,9 @@ sodipodi_add_desktop (SPDesktop * desktop)
 
 	if (DESKTOP_IS_ACTIVE (desktop)) {
 		g_signal_emit (G_OBJECT (sodipodi), sodipodi_signals[ACTIVATE_DESKTOP], 0, desktop);
-		g_signal_emit (G_OBJECT (sodipodi), sodipodi_signals[SET_EVENTCONTEXT], 0, SP_DT_EVENTCONTEXT (desktop));
+		/* Update actions and emit signal */
+		sodipodi_eventcontext_set (SP_DT_EVENTCONTEXT (desktop));
+		/* g_signal_emit (G_OBJECT (sodipodi), sodipodi_signals[SET_EVENTCONTEXT], 0, SP_DT_EVENTCONTEXT (desktop)); */
 		g_signal_emit (G_OBJECT (sodipodi), sodipodi_signals[SET_SELECTION], 0, SP_DT_SELECTION (desktop));
 		g_signal_emit (G_OBJECT (sodipodi), sodipodi_signals[CHANGE_SELECTION], 0, SP_DT_SELECTION (desktop));
 	}
@@ -742,11 +624,15 @@ sodipodi_remove_desktop (SPDesktop * desktop)
 			sodipodi->desktops = g_slist_remove (sodipodi->desktops, new);
 			sodipodi->desktops = g_slist_prepend (sodipodi->desktops, new);
 			g_signal_emit (G_OBJECT (sodipodi), sodipodi_signals[ACTIVATE_DESKTOP], 0, new);
-			g_signal_emit (G_OBJECT (sodipodi), sodipodi_signals[SET_EVENTCONTEXT], 0, SP_DT_EVENTCONTEXT (new));
+			/* Update actions and emit signal */
+			sodipodi_eventcontext_set (SP_DT_EVENTCONTEXT (new));
+			/* g_signal_emit (G_OBJECT (sodipodi), sodipodi_signals[SET_EVENTCONTEXT], 0, SP_DT_EVENTCONTEXT (new)); */
 			g_signal_emit (G_OBJECT (sodipodi), sodipodi_signals[SET_SELECTION], 0, SP_DT_SELECTION (new));
 			g_signal_emit (G_OBJECT (sodipodi), sodipodi_signals[CHANGE_SELECTION], 0, SP_DT_SELECTION (new));
 		} else {
-			g_signal_emit (G_OBJECT (sodipodi), sodipodi_signals[SET_EVENTCONTEXT], 0, NULL);
+			/* Update actions and emit signal */
+			sodipodi_eventcontext_set (NULL);
+			/* g_signal_emit (G_OBJECT (sodipodi), sodipodi_signals[SET_EVENTCONTEXT], 0, NULL); */
 			g_signal_emit (G_OBJECT (sodipodi), sodipodi_signals[SET_SELECTION], 0, NULL);
 			g_signal_emit (G_OBJECT (sodipodi), sodipodi_signals[CHANGE_SELECTION], 0, NULL);
 		}
@@ -778,7 +664,9 @@ sodipodi_activate_desktop (SPDesktop * desktop)
 	sodipodi->desktops = g_slist_prepend (sodipodi->desktops, desktop);
 
 	g_signal_emit (G_OBJECT (sodipodi), sodipodi_signals[ACTIVATE_DESKTOP], 0, desktop);
-	g_signal_emit (G_OBJECT (sodipodi), sodipodi_signals[SET_EVENTCONTEXT], 0, SP_DT_EVENTCONTEXT (desktop));
+	/* Update actions and emit signal */
+	sodipodi_eventcontext_set (SP_DT_EVENTCONTEXT (desktop));
+	/* g_signal_emit (G_OBJECT (sodipodi), sodipodi_signals[SET_EVENTCONTEXT], 0, SP_DT_EVENTCONTEXT (desktop)); */
 	g_signal_emit (G_OBJECT (sodipodi), sodipodi_signals[SET_SELECTION], 0, SP_DT_SELECTION (desktop));
 	g_signal_emit (G_OBJECT (sodipodi), sodipodi_signals[CHANGE_SELECTION], 0, SP_DT_SELECTION (desktop));
 }
@@ -840,14 +728,6 @@ sodipodi_set_color (SPColor *color, float opacity)
 	g_signal_emit (G_OBJECT (sodipodi), sodipodi_signals[COLOR_SET], 0, color, (double) opacity);
 }
 
-SPDesktop *
-sodipodi_active_desktop (void)
-{
-	if (sodipodi->desktops == NULL) return NULL;
-
-	return (SPDesktop *) sodipodi->desktops->data;
-}
-
 SPDocument *
 sodipodi_active_document (void)
 {
@@ -870,63 +750,74 @@ static void
 sodipodi_init_config (SPReprDoc *doc, const gchar *config_name, const gchar *skeleton, int skel_size,
 		      const unsigned char *e_mkdir, const unsigned char *e_notdir, const unsigned char *e_ccf, const unsigned char *e_cwf)
 {
-	gchar * dn, *fn;
+	unsigned char *dnutf8, *dnos;
+	unsigned char *fnutf8, *fnos;
 	struct stat s;
+	GtkWidget *mdlg;
 	int fh;
-	GtkWidget * w;
+	gsize bytesin, bytesout;
+	unsigned int valid;
+
+	valid = TRUE;
+	mdlg = NULL;
 
 #ifdef WIN32
-	dn = g_strdup (SODIPODI_APPDATADIR);
+	dnutf8 = g_strdup (SODIPODI_APPDATADIR);
 #else
-	dn = g_build_filename (g_get_home_dir (), ".sodipodi", NULL);
+	dnutf8 = g_build_filename (g_get_home_dir (), ".sodipodi", NULL);
 #endif
-	if (stat (dn, &s)) {
-		if (mkdir (dn, S_IRWXU | S_IRGRP | S_IXGRP))
-		{
+	dnos = g_filename_from_utf8 (dnutf8, strlen (dnutf8), &bytesin, &bytesout, NULL);
+	if (stat (dnos, &s)) {
+		if (mkdir (dnos, S_IRWXU | S_IRGRP | S_IXGRP)) {
 			/* Cannot create directory */
-			w = gtk_message_dialog_new (NULL, GTK_DIALOG_MODAL, GTK_MESSAGE_WARNING, GTK_BUTTONS_OK, e_mkdir, dn);
-			gtk_dialog_run (GTK_DIALOG (w));
-			gtk_widget_destroy (w);
-			g_free (dn);
-			return;
+			mdlg = gtk_message_dialog_new (NULL, GTK_DIALOG_MODAL, GTK_MESSAGE_WARNING, GTK_BUTTONS_OK,
+						       e_mkdir, dnutf8);
+			valid = FALSE;
 		}
 	} else if (!S_ISDIR (s.st_mode)) {
 		/* Not a directory */
-		w = gtk_message_dialog_new (NULL, GTK_DIALOG_MODAL, GTK_MESSAGE_WARNING, GTK_BUTTONS_OK, e_notdir, dn);
-		gtk_dialog_run (GTK_DIALOG (w));
-		gtk_widget_destroy (w);
-		g_free (dn);
-		return;
+		mdlg = gtk_message_dialog_new (NULL, GTK_DIALOG_MODAL, GTK_MESSAGE_WARNING, GTK_BUTTONS_OK,
+					       e_notdir, dnutf8);
+		valid = FALSE;
 	}
-	g_free (dn);
+	g_free (dnos);
+	g_free (dnutf8);
+	/* Return id error */
+	if (mdlg) {
+		gtk_dialog_run (GTK_DIALOG (mdlg));
+		gtk_widget_destroy (mdlg);
+		mdlg = NULL;
+	}
+	if (!valid) return;
 
 #ifdef WIN32
-	fn = g_build_filename (SODIPODI_APPDATADIR, config_name, NULL);
-	fh = creat (fn, S_IREAD | S_IWRITE);
+	fnutf8 = g_build_filename (SODIPODI_APPDATADIR, config_name, NULL);
+	fnos = g_filename_from_utf8 (fnutf8, strlen (fnutf8), &bytesin, &bytesout, NULL);
+	fh = creat (fnos, S_IREAD | S_IWRITE);
 #else
-	fn = g_build_filename (g_get_home_dir (), ".sodipodi", config_name, NULL);
-	fh = creat (fn, S_IRUSR | S_IWUSR | S_IRGRP);
+	fnutf8 = g_build_filename (g_get_home_dir (), ".sodipodi", config_name, NULL);
+	fnos = g_filename_from_utf8 (fnutf8, strlen (fnutf8), &bytesin, &bytesout, NULL);
+	fh = creat (fnos, S_IRUSR | S_IWUSR | S_IRGRP);
 #endif
 	if (fh < 0) {
 		/* Cannot create file */
-		w = gtk_message_dialog_new (NULL, GTK_DIALOG_MODAL, GTK_MESSAGE_WARNING, GTK_BUTTONS_OK, e_ccf, fn);
-		gtk_dialog_run (GTK_DIALOG (w));
-		gtk_widget_destroy (w);
-		g_free (fn);
-		return;
+		mdlg = gtk_message_dialog_new (NULL, GTK_DIALOG_MODAL, GTK_MESSAGE_WARNING, GTK_BUTTONS_OK,
+					       e_ccf, fnutf8);
+		valid = FALSE;
 	}
 	if (write (fh, skeleton, skel_size) != skel_size) {
 		/* Cannot create file */
-		w = gtk_message_dialog_new (NULL, GTK_DIALOG_MODAL, GTK_MESSAGE_WARNING, GTK_BUTTONS_OK, e_cwf, fn);
-		gtk_dialog_run (GTK_DIALOG (w));
-		gtk_widget_destroy (w);
-		g_free (fn);
-		close (fh);
-		return;
+		mdlg = gtk_message_dialog_new (NULL, GTK_DIALOG_MODAL, GTK_MESSAGE_WARNING, GTK_BUTTONS_OK,
+					       e_cwf, fnutf8);
+		valid = FALSE;
+	}
+	if (fh >= 0) close (fh);
+	g_free (fnos);
+	g_free (fnutf8);
+	if (mdlg) {
+		gtk_dialog_run (GTK_DIALOG (mdlg));
+		gtk_widget_destroy (mdlg);
 	}
-
-	g_free (fn);
-	close (fh);
 }
 
 /* This routine should be obsoleted in favor of the generic version */
@@ -1003,9 +894,205 @@ sodipodi_refresh_display (Sodipodi *sodipodi)
 	}
 }
 
-void
-sodipodi_exit (Sodipodi *sodipodi)
+unsigned int
+sodipodi_shutdown_all_views (void)
 {
-	gtk_main_quit ();
+	GSList *views;
+	views = g_slist_copy (sodipodi->desktops);
+	while (views) {
+		if (sp_view_shutdown ((SPView *) views->data)) {
+			g_slist_free (views);
+			return FALSE;
+		}
+		views = g_slist_remove (views, views->data);
+	}
+	return TRUE;
+}
+
+void
+sodipodi_exit (void)
+{
+	while (sodipodi && sodipodi->documents) g_object_unref ((GObject *) sodipodi->documents->data);
+	while (sodipodi) sodipodi_unref ();
+}
+
+const GSList *
+sodipodi_get_document_list (void)
+{
+	return sodipodi->documents;
+}
+
+SPDesktop *
+sodipodi_get_active_desktop (void)
+{
+	if (sodipodi->desktops == NULL) return NULL;
+
+	return (SPDesktop *) sodipodi->desktops->data;
+}
+
+/* Configuration management */
+
+#if 0
+/* We use '.' as separator */
+SPRepr *
+sodipodi_get_repr (Sodipodi *sodipodi, const unsigned char *key)
+{
+	SPRepr * repr;
+	const gchar * id, * s, * e;
+	gint len;
+
+	if (key == NULL) return NULL;
+
+	if (!strncmp (key, "extensions", 10) && (!key[10] || (key[10] == '.'))) {
+		repr = sp_repr_doc_get_root (sodipodi->extensions);
+	} else {
+		repr = sp_repr_doc_get_root (sodipodi->preferences);
+	}
+	g_assert (!(strcmp (sp_repr_get_name (repr), "sodipodi")));
+
+	s = key;
+	while ((s) && (*s)) {
+		SPRepr * child;
+		/* Find next name */
+		if ((e = strchr (s, '.'))) {
+			len = e++ - s;
+		} else {
+			len = strlen (s);
+		}
+		for (child = repr->children; child != NULL; child = child->next) {
+			id = sp_repr_attr (child, "id");
+			if ((id) && (strlen (id) == len) && (!strncmp (id, s, len))) break;
+		}
+		if (child == NULL) return NULL;
+
+		repr = child;
+		s = e;
+	}
+	return repr;
+}
+#endif
+
+#define SP_MAX_CONFIG_DEPTH 64
+#define SP_MAX_CONFIG_ID_LENGTH 256
+
+SPRepr *
+sp_config_node_get (const unsigned char *path, unsigned int create)
+{
+	ArikkeiToken tokens[SP_MAX_CONFIG_DEPTH];
+	ArikkeiToken ptoken;
+	unsigned int ntokens;
+	SPRepr *repr;
+	unsigned int pos;
+
+	if (!path) return NULL;
+	if (!*path) return NULL;
+
+	/* Tokenize path */
+	arikkei_token_set_from_string (&ptoken, path);
+	ntokens = arikkei_token_tokenize_ws (&ptoken, tokens, SP_MAX_CONFIG_DEPTH, ".", 0);
+	assert (ntokens > 0);
+	if (!arikkei_token_strcmp (&tokens[0], "extensions")) {
+		repr = sp_repr_doc_get_root (sodipodi->extensions);
+	} else {
+		repr = sp_repr_doc_get_root (sodipodi->preferences);
+	}
+	assert (!(strcmp (sp_repr_get_name (repr), "sodipodi")));
+
+	/* Parse all tokens */
+	pos = 0;
+	while (pos < ntokens) {
+		unsigned char c[SP_MAX_CONFIG_ID_LENGTH];
+		SPRepr *child;
+
+		arikkei_token_strncpy (&tokens[pos], c, SP_MAX_CONFIG_ID_LENGTH);
+		if (!c[0]) return NULL;
+		child = sp_repr_lookup_child (repr, "id", c);
+		if (!child && create) {
+			child = sp_repr_new ("node");
+			sp_repr_set_attr (child, "id", c);
+			sp_repr_append_child (repr, child);
+		}
+		if (!child) return NULL;
+		repr = child;
+		pos += 1;
+	}
+	return repr;
+}
+
+const unsigned char *
+sp_config_value_get (const unsigned char *path, const unsigned char *key, const unsigned char *defval)
+{
+	SPRepr *repr;
+	const unsigned char *val;
+	repr = sp_config_node_get (path, FALSE);
+	if (!repr) return defval;
+	val = sp_repr_get_attr (repr, key);
+	return (val) ? val : defval;
+}
+
+unsigned int
+sp_config_value_get_boolean (const unsigned char *path, const unsigned char *key, unsigned int defval)
+{
+	SPRepr *repr;
+	repr = sp_config_node_get (path, FALSE);
+	if (!repr) return defval;
+	sp_repr_get_boolean (repr, key, &defval);
+	return defval;
+}
+
+int
+sp_config_value_get_int (const unsigned char *path, const unsigned char *key, int defval)
+{
+	SPRepr *repr;
+	repr = sp_config_node_get (path, FALSE);
+	if (!repr) return defval;
+	sp_repr_get_int (repr, key, &defval);
+	return defval;
+}
+
+double
+sp_config_value_get_double (const unsigned char *path, const unsigned char *key, double defval)
+{
+	SPRepr *repr;
+	repr = sp_config_node_get (path, FALSE);
+	if (!repr) return defval;
+	sp_repr_get_double (repr, key, &defval);
+	return defval;
+}
+
+unsigned int
+sp_config_value_set (const unsigned char *path, const unsigned char *key, const unsigned char *val, unsigned int create)
+{
+	SPRepr *repr;
+	repr = sp_config_node_get (path, create);
+	if (!repr) return FALSE;
+	return sp_repr_set_attr (repr, key, val);
+}
+
+unsigned int
+sp_config_value_set_boolean (const unsigned char *path, const unsigned char *key, unsigned int val, unsigned int create)
+{
+	SPRepr *repr;
+	repr = sp_config_node_get (path, create);
+	if (!repr) return FALSE;
+	return sp_repr_set_boolean (repr, key, val);
+}
+
+unsigned int
+sp_config_value_set_int (const unsigned char *path, const unsigned char *key, int val, unsigned int create)
+{
+	SPRepr *repr;
+	repr = sp_config_node_get (path, create);
+	if (!repr) return FALSE;
+	return sp_repr_set_int (repr, key, val);
+}
+
+unsigned int
+sp_config_value_set_double (const unsigned char *path, const unsigned char *key, double val, unsigned int create)
+{
+	SPRepr *repr;
+	repr = sp_config_node_get (path, create);
+	if (!repr) return FALSE;
+	return sp_repr_set_double (repr, key, val);
 }
 
