@@ -26,7 +26,7 @@ static void sp_defs_init (SPDefs * defs);
 static void sp_defs_build (SPObject *object, SPDocument * document, SPRepr * repr);
 static void sp_defs_release (SPObject *object);
 static void sp_defs_child_added (SPObject * object, SPRepr * child, SPRepr * ref);
-static void sp_defs_remove_child (SPObject * object, SPRepr * child);
+static unsigned int sp_defs_remove_child (SPObject * object, SPRepr * child);
 static void sp_defs_order_changed (SPObject * object, SPRepr * child, SPRepr * old, SPRepr * new);
 static void sp_defs_update (SPObject *object, SPCtx *ctx, guint flags);
 static void sp_defs_modified (SPObject *object, guint flags);
@@ -157,32 +157,37 @@ sp_defs_child_added (SPObject * object, SPRepr * child, SPRepr * ref)
 	sp_object_invoke_build (ochild, object->document, child, SP_OBJECT_IS_CLONED (object));
 }
 
-static void
+static unsigned int
 sp_defs_remove_child (SPObject * object, SPRepr * child)
 {
-	SPDefs * defs;
-	SPObject * prev, * ochild;
+	SPDefs *defs;
+	SPObject *ref, *oc;
+	unsigned int ret;
 
 	defs = SP_DEFS (object);
 
-	if (((SPObjectClass *) (parent_class))->remove_child)
-		(* ((SPObjectClass *) (parent_class))->remove_child) (object, child);
-
-	prev = NULL;
-	ochild = object->children;
-	while (ochild && ochild->repr != child) {
-		prev = ochild;
-		ochild = ochild->next;
+	if (((SPObjectClass *) (parent_class))->remove_child) {
+		ret = ((SPObjectClass *) (parent_class))->remove_child (object, child);
+		if (!ret) return ret;
 	}
 
-	if (prev) {
-		prev->next = ochild->next;
-	} else {
-		object->children = ochild->next;
+	ref = NULL;
+	oc = object->children;
+	while (oc && (SP_OBJECT_REPR (oc) != child)) {
+		ref = oc;
+		oc = oc->next;
 	}
-	ochild->parent = NULL;
-	ochild->next = NULL;
-	g_object_unref (G_OBJECT (ochild));
+
+	if (oc) {
+		if (ref) {
+			ref->next = sp_object_detach_unref (object, oc);
+		} else {
+			object->children = sp_object_detach_unref (object, oc);
+		}
+		sp_object_request_modified (object, SP_OBJECT_MODIFIED_FLAG);
+	}
+
+	return TRUE;
 }
 
 static void
@@ -302,7 +307,7 @@ sp_defs_write (SPObject *object, SPRepr *repr, guint flags)
 		l = NULL;
 		for (child = object->children; child != NULL; child = child->next) {
 			crepr = sp_object_invoke_write (child, NULL, flags);
-			l = g_slist_prepend (l, crepr);
+			if (crepr) l = g_slist_prepend (l, crepr);
 		}
 		while (l) {
 			sp_repr_add_child (repr, (SPRepr *) l->data, NULL);
