@@ -381,10 +381,10 @@ sp_ellipse_write (SPObject *object, SPRepr *repr, guint flags)
 		repr = sp_repr_new ("ellipse");
 	}
 
-	sp_repr_set_double_attribute (repr, "cx", ellipse->cx.computed);
-	sp_repr_set_double_attribute (repr, "cy", ellipse->cy.computed);
-	sp_repr_set_double_attribute (repr, "rx", ellipse->rx.computed);
-	sp_repr_set_double_attribute (repr, "ry", ellipse->ry.computed);
+	sp_repr_set_double (repr, "cx", ellipse->cx.computed);
+	sp_repr_set_double (repr, "cy", ellipse->cy.computed);
+	sp_repr_set_double (repr, "rx", ellipse->rx.computed);
+	sp_repr_set_double (repr, "ry", ellipse->ry.computed);
 	
 	if (((SPObjectClass *) ellipse_parent_class)->write)
 		(* ((SPObjectClass *) ellipse_parent_class)->write) (object, repr, flags);
@@ -537,9 +537,9 @@ sp_circle_write (SPObject *object, SPRepr *repr, guint flags)
 		repr = sp_repr_new ("circle");
 	}
 
-	sp_repr_set_double_attribute (repr, "cx", ellipse->cx.computed);
-	sp_repr_set_double_attribute (repr, "cy", ellipse->cy.computed);
-	sp_repr_set_double_attribute (repr, "r", ellipse->rx.computed);
+	sp_repr_set_double (repr, "cx", ellipse->cx.computed);
+	sp_repr_set_double (repr, "cy", ellipse->cy.computed);
+	sp_repr_set_double (repr, "r", ellipse->rx.computed);
 	
 	if (((SPObjectClass *) circle_parent_class)->write)
 		((SPObjectClass *) circle_parent_class)->write (object, repr, flags);
@@ -729,9 +729,11 @@ sp_arc_set_elliptical_path_attribute (SPArc *arc, SPRepr *repr)
 	sp_arc_get_xy (arc, ge->end, &p2);
 
 	dt = fmod (ge->end - ge->start, SP_2PI);
-	if (fabs (dt) < 1e-6) {
+	if (fabs (dt) < 1e-3) {
 		NRPointF ph;
-		int pos = 0;
+		int pos;
+		/* Full circle */
+		pos = 0;
 		sp_arc_get_xy (arc, (ge->start + ge->end) / 2.0, &ph);
 		c[pos++] = 'M';
 		c[pos++] = ' ';
@@ -749,7 +751,7 @@ sp_arc_set_elliptical_path_attribute (SPArc *arc, SPRepr *repr)
 		c[pos++] = ' ';
 		c[pos++] = '1';
 		c[pos++] = ' ';
-		c[pos++] = (dt > 0) ? '1' : 0;
+		c[pos++] = (dt > 0) ? '1' : '0';
 		c[pos++] = ' ';
 		pos += sp_svg_number_write_d (c + pos, ph.x, 6, 0, FALSE);
 		c[pos++] = ' ';
@@ -765,7 +767,7 @@ sp_arc_set_elliptical_path_attribute (SPArc *arc, SPRepr *repr)
 		c[pos++] = ' ';
 		c[pos++] = '1';
 		c[pos++] = ' ';
-		c[pos++] = (dt > 0) ? '1' : 0;
+		c[pos++] = (dt > 0) ? '1' : '0';
 		c[pos++] = ' ';
 		c[pos++] = ' ';
 		pos += sp_svg_number_write_d (c + pos, p2.x, 6, 0, FALSE);
@@ -806,9 +808,9 @@ sp_arc_set_elliptical_path_attribute (SPArc *arc, SPRepr *repr)
 			c[pos++] = ' ';
 			c[pos++] = '0';
 			c[pos++] = ' ';
-			c[pos++] = (fabs (dt) > M_PI) ? 1 : 0;
+			c[pos++] = (fabs (dt) > M_PI) ? '1' : '0';
 			c[pos++] = ' ';
-			c[pos++] = (dt > 0) ? '1' : 0;
+			c[pos++] = (dt > 0) ? '1' : '0';
 			c[pos++] = ' ';
 			pos += sp_svg_number_write_d (c + pos, p2.x, 6, 0, FALSE);
 			c[pos++] = ' ';
@@ -839,9 +841,9 @@ sp_arc_set_elliptical_path_attribute (SPArc *arc, SPRepr *repr)
 			c[pos++] = ' ';
 			c[pos++] = '0';
 			c[pos++] = ' ';
-			c[pos++] = (fabs (dt) > M_PI) ? 1 : 0;
+			c[pos++] = (fabs (dt) > M_PI) ? '1' : '0';
 			c[pos++] = ' ';
-			c[pos++] = (dt > 0) ? '1' : 0;
+			c[pos++] = (dt > 0) ? '1' : '0';
 			c[pos++] = ' ';
 			pos += sp_svg_number_write_d (c + pos, p2.x, 6, 0, FALSE);
 			c[pos++] = ' ';
@@ -868,12 +870,12 @@ sp_arc_write (SPObject *object, SPRepr *repr, guint flags)
 		}
 
 		sp_repr_set_attr (repr, "sodipodi:type", "arc");
-		sp_repr_set_double_attribute (repr, "sodipodi:cx", ge->cx.computed);
-		sp_repr_set_double_attribute (repr, "sodipodi:cy", ge->cy.computed);
-		sp_repr_set_double_attribute (repr, "sodipodi:rx", ge->rx.computed);
-		sp_repr_set_double_attribute (repr, "sodipodi:ry", ge->ry.computed);
-		sp_repr_set_double_attribute (repr, "sodipodi:start", ge->start);
-		sp_repr_set_double_attribute (repr, "sodipodi:end", ge->end);
+		sp_repr_set_double (repr, "sodipodi:cx", ge->cx.computed);
+		sp_repr_set_double (repr, "sodipodi:cy", ge->cy.computed);
+		sp_repr_set_double (repr, "sodipodi:rx", ge->rx.computed);
+		sp_repr_set_double (repr, "sodipodi:ry", ge->ry.computed);
+		sp_repr_set_double (repr, "sodipodi:start", ge->start);
+		sp_repr_set_double (repr, "sodipodi:end", ge->end);
 		sp_repr_set_attr (repr, "sodipodi:open", (!ge->closed) ? "true" : NULL);
 
 		sp_arc_set_elliptical_path_attribute (arc, repr);
@@ -884,10 +886,10 @@ sp_arc_write (SPObject *object, SPRepr *repr, guint flags)
 			if ((flags & SP_OBJECT_WRITE_BUILD) && !repr) {
 				repr = sp_repr_new ("ellipse");
 			}
-			sp_repr_set_double_attribute (repr, "cx", ge->cx.computed);
-			sp_repr_set_double_attribute (repr, "cy", ge->cy.computed);
-			sp_repr_set_double_attribute (repr, "rx", ge->rx.computed);
-			sp_repr_set_double_attribute (repr, "ry", ge->ry.computed);
+			sp_repr_set_double (repr, "cx", ge->cx.computed);
+			sp_repr_set_double (repr, "cy", ge->cy.computed);
+			sp_repr_set_double (repr, "rx", ge->rx.computed);
+			sp_repr_set_double (repr, "ry", ge->ry.computed);
 		} else {
 			if ((flags & SP_OBJECT_WRITE_BUILD) && !repr) {
 				repr = sp_repr_new ("path");
