@@ -154,7 +154,7 @@ sp_stop_write (SPObject *object, SPRepr *repr, guint flags)
 
 	stop = SP_STOP (object);
 
-	if ((flags & SP_OBJECT_WRITE_BUILD) & !repr) {
+	if ((flags & SP_OBJECT_WRITE_BUILD) && !repr) {
 		repr = sp_repr_new ("stop");
 	}
 
@@ -183,7 +183,7 @@ static void sp_gradient_build (SPObject *object, SPDocument *document, SPRepr *r
 static void sp_gradient_release (SPObject *object);
 static void sp_gradient_set (SPObject *object, unsigned int key, const unsigned char *value);
 static void sp_gradient_child_added (SPObject *object, SPRepr *child, SPRepr *ref);
-static void sp_gradient_remove_child (SPObject *object, SPRepr *child);
+static unsigned int sp_gradient_remove_child (SPObject *object, SPRepr *child);
 static void sp_gradient_modified (SPObject *object, guint flags);
 static unsigned int sp_gradient_sequence (SPObject *object, SPObject *target, unsigned int *seq);
 static SPRepr *sp_gradient_write (SPObject *object, SPRepr *repr, guint flags);
@@ -452,16 +452,19 @@ sp_gradient_child_added (SPObject *object, SPRepr *child, SPRepr *ref)
 	/* fixme: should we schedule "modified" here? */
 }
 
-static void
+static unsigned int
 sp_gradient_remove_child (SPObject *object, SPRepr *child)
 {
 	SPGradient *gr;
 	SPObject *prev, *ochild;
+	unsigned int ret;
 
 	gr = SP_GRADIENT (object);
 
-	if (((SPObjectClass *) gradient_parent_class)->remove_child)
-		(* ((SPObjectClass *) gradient_parent_class)->remove_child) (object, child);
+	if (((SPObjectClass *) gradient_parent_class)->remove_child) {
+		ret = ((SPObjectClass *) gradient_parent_class)->remove_child (object, child);
+		if (!ret) return ret;
+	}
 
 	sp_gradient_invalidate_vector (gr);
 
@@ -492,6 +495,7 @@ sp_gradient_remove_child (SPObject *object, SPRepr *child)
 	}
 
 	/* fixme: should we schedule "modified" here? */
+	return TRUE;
 }
 
 static void
@@ -657,6 +661,7 @@ sp_gradient_set_units (SPGradient *gr, unsigned int units)
 {
 	if (units != gr->units) {
 		gr->units = units;
+		gr->units_set = TRUE;
 		sp_object_request_modified (SP_OBJECT (gr), SP_OBJECT_MODIFIED_FLAG);
 	}
 }
@@ -689,7 +694,7 @@ sp_gradient_repr_set_vector (SPGradient *gr, SPRepr *repr, SPGradientVector *vec
 			SPRepr *child;
 			guchar c0[64], c1[32], s[256];
 			child = sp_repr_new ("stop");
-			sp_repr_set_double_attribute (child, "offset",
+			sp_repr_set_double (child, "offset",
 						      vector->stops[i].offset * (vector->end - vector->start) + vector->start);
 			sp_svg_write_color (c0, 64, sp_color_get_rgba32_ualpha (&vector->stops[i].color, 0x00));
 			sp_svg_number_write_d (c1, vector->stops[i].opacity, 4, 2, FALSE);
@@ -1371,10 +1376,10 @@ sp_lineargradient_write (SPObject *object, SPRepr *repr, guint flags)
 		repr = sp_repr_new ("linearGradient");
 	}
 
-	if ((flags & SP_OBJECT_WRITE_ALL) || lg->x1.set) sp_repr_set_double_attribute (repr, "x1", lg->x1.computed);
-	if ((flags & SP_OBJECT_WRITE_ALL) || lg->y1.set) sp_repr_set_double_attribute (repr, "y1", lg->y1.computed);
-	if ((flags & SP_OBJECT_WRITE_ALL) || lg->x2.set) sp_repr_set_double_attribute (repr, "x2", lg->x2.computed);
-	if ((flags & SP_OBJECT_WRITE_ALL) || lg->y2.set) sp_repr_set_double_attribute (repr, "y2", lg->y2.computed);
+	if ((flags & SP_OBJECT_WRITE_ALL) || lg->x1.set) sp_repr_set_double (repr, "x1", lg->x1.computed);
+	if ((flags & SP_OBJECT_WRITE_ALL) || lg->y1.set) sp_repr_set_double (repr, "y1", lg->y1.computed);
+	if ((flags & SP_OBJECT_WRITE_ALL) || lg->x2.set) sp_repr_set_double (repr, "x2", lg->x2.computed);
+	if ((flags & SP_OBJECT_WRITE_ALL) || lg->y2.set) sp_repr_set_double (repr, "y2", lg->y2.computed);
 
 	if (((SPObjectClass *) lg_parent_class)->write)
 		(* ((SPObjectClass *) lg_parent_class)->write) (object, repr, flags);
@@ -1729,11 +1734,11 @@ sp_radialgradient_write (SPObject *object, SPRepr *repr, guint flags)
 		repr = sp_repr_new ("radialGradient");
 	}
 
-	if ((flags & SP_OBJECT_WRITE_ALL) || rg->cx.set) sp_repr_set_double_attribute (repr, "cx", rg->cx.computed);
-	if ((flags & SP_OBJECT_WRITE_ALL) || rg->cy.set) sp_repr_set_double_attribute (repr, "cy", rg->cy.computed);
-	if ((flags & SP_OBJECT_WRITE_ALL) || rg->r.set) sp_repr_set_double_attribute (repr, "r", rg->r.computed);
-	if ((flags & SP_OBJECT_WRITE_ALL) || rg->fx.set) sp_repr_set_double_attribute (repr, "fx", rg->fx.computed);
-	if ((flags & SP_OBJECT_WRITE_ALL) || rg->fy.set) sp_repr_set_double_attribute (repr, "fy", rg->fy.computed);
+	if ((flags & SP_OBJECT_WRITE_ALL) || rg->cx.set) sp_repr_set_double (repr, "cx", rg->cx.computed);
+	if ((flags & SP_OBJECT_WRITE_ALL) || rg->cy.set) sp_repr_set_double (repr, "cy", rg->cy.computed);
+	if ((flags & SP_OBJECT_WRITE_ALL) || rg->r.set) sp_repr_set_double (repr, "r", rg->r.computed);
+	if ((flags & SP_OBJECT_WRITE_ALL) || rg->fx.set) sp_repr_set_double (repr, "fx", rg->fx.computed);
+	if ((flags & SP_OBJECT_WRITE_ALL) || rg->fy.set) sp_repr_set_double (repr, "fy", rg->fy.computed);
 
 	if (((SPObjectClass *) rg_parent_class)->write)
 		(* ((SPObjectClass *) rg_parent_class)->write) (object, repr, flags);
