@@ -242,7 +242,7 @@ sp_marker_set (SPObject *object, unsigned int key, const unsigned char *value)
 	case SP_ATTR_PRESERVEASPECTRATIO:
 		/* Do setup before, so we can use break to escape */
 		marker->aspect_set = FALSE;
-		marker->aspect_align = SP_ASPECT_NONE;
+		marker->aspect_align = SP_ASPECT_XMID_YMID;
 		marker->aspect_clip = SP_ASPECT_MEET;
 		sp_object_request_update (object, SP_OBJECT_MODIFIED_FLAG | SP_OBJECT_VIEWPORT_MODIFIED_FLAG);
 		if (value) {
@@ -500,8 +500,8 @@ sp_marker_write (SPObject *object, SPRepr *repr, guint flags)
 		sp_repr_set_attr (repr, "orient", NULL);
 	}
 	/* fixme: */
-	sp_repr_set_attr (repr, "viewBox", sp_repr_attr (object->repr, "viewBox"));
-	sp_repr_set_attr (repr, "preserveAspectRatio", sp_repr_attr (object->repr, "preserveAspectRatio"));
+	sp_repr_set_attr (repr, "viewBox", sp_repr_get_attr (object->repr, "viewBox"));
+	sp_repr_set_attr (repr, "preserveAspectRatio", sp_repr_get_attr (object->repr, "preserveAspectRatio"));
 
 	if (((SPObjectClass *) (parent_class))->write)
 		((SPObjectClass *) (parent_class))->write (object, repr, flags);
@@ -610,6 +610,50 @@ sp_marker_show_instance (SPMarker *marker, NRArenaItem *parent,
 	return NULL;
 }
 
+void
+sp_marker_union_bbox (SPMarker *marker,
+                      unsigned int key, unsigned int pos,
+                      const NRMatrixD *base, float linewidth,
+                      NRRectF *bbox)
+{
+	SPMarkerView *v;
+
+	printf("sp_marker_union_bbox(key: %u)\n",key);
+
+	for (v = marker->views; v != NULL; v = v->next) {
+		printf("\tv->key: %u\n",v->key);
+		if (v->key == key) {
+			NRMatrixD m;
+			float x, y;
+			if (pos >= v->size) return;
+			if (marker->orient_auto) {
+				m = *base;
+			} else {
+				/* fixme: Orient units (Lauris) */
+				nr_matrix_d_set_rotate (&m, marker->orient * M_PI / 180.0);
+				m.c[4] = base->c[4];
+				m.c[5] = base->c[5];
+			}
+			if (marker->markerUnits == SP_MARKER_UNITS_STROKEWIDTH) {
+				m.c[0] *= linewidth;
+				m.c[1] *= linewidth;
+				m.c[2] *= linewidth;
+				m.c[3] *= linewidth;
+			}
+
+			x = fabs (NR_MATRIX_DF_TRANSFORM_X(&m,1,1));
+			y = fabs (NR_MATRIX_DF_TRANSFORM_Y(&m,1,1));
+
+			bbox->x0=MIN(bbox->x0,-x);
+			bbox->y0=MIN(bbox->y0,-y);
+			bbox->x1=MAX(bbox->x1,x);
+			bbox->y1=MAX(bbox->y1,y);
+
+			break;
+		}
+	}
+}
+
 /* This replaces SPItem implementation because we have own views */
 
 void
