@@ -380,12 +380,17 @@ sp_object_repr_child_added (SPRepr *repr, SPRepr *child, SPRepr *ref, gpointer d
 static unsigned int
 sp_object_repr_remove_child (SPRepr *repr, SPRepr *child, SPRepr *ref, gpointer data)
 {
-	SPObject * object;
+	SPObject *object, *ochild;
+	const unsigned char *id;
 
 	object = SP_OBJECT (data);
 
-	if (((SPObjectClass *) G_OBJECT_GET_CLASS(object))->remove_child)
-		(* ((SPObjectClass *)G_OBJECT_GET_CLASS(object))->remove_child) (object, child);
+	id = sp_repr_attr (child, "id");
+	ochild = sp_document_lookup_id (object->document, id);
+	if (ochild->blocked) return FALSE;
+
+	if (((SPObjectClass *) G_OBJECT_GET_CLASS (object))->remove_child)
+		return (* ((SPObjectClass *)G_OBJECT_GET_CLASS (object))->remove_child) (object, child);
 
 	return TRUE;
 }
@@ -432,10 +437,12 @@ sp_object_private_set (SPObject *object, unsigned int key, const unsigned char *
 		} else if (value && !strcmp (value, "default")) {
 			object->xml_space.value = SP_XML_SPACE_DEFAULT;
 			object->xml_space.set = TRUE;
-		} else if (object->parent) {
-			SPObject *parent;
-			parent = object->parent;
-			object->xml_space.value = parent->xml_space.value;
+		} else {
+			/* unset value */
+			object->xml_space.value = object->parent ?
+				object->parent->xml_space.value :
+				SP_XML_SPACE_DEFAULT;
+			object->xml_space.set = FALSE;
 		}
 		sp_object_request_update (object, SP_OBJECT_MODIFIED_FLAG | SP_OBJECT_STYLE_MODIFIED_FLAG);
 		break;
