@@ -15,27 +15,9 @@
 
 /* SPObject flags */
 
-/* Async modification flags */
-#define SP_OBJECT_MODIFIED_FLAG (1 << 0)
-#define SP_OBJECT_CHILD_MODIFIED_FLAG (1 << 1)
-#define SP_OBJECT_PARENT_MODIFIED_FLAG (1 << 2)
-#define SP_OBJECT_STYLE_MODIFIED_FLAG (1 << 3)
-#define SP_OBJECT_VIEWPORT_MODIFIED_FLAG (1 << 4)
-#define SP_OBJECT_USER_MODIFIED_FLAG_A (1 << 5)
-#define SP_OBJECT_USER_MODIFIED_FLAG_B (1 << 6)
-#define SP_OBJECT_USER_MODIFIED_FLAG_C (1 << 7)
-
 /* Conveneience */
 #define SP_OBJECT_FLAGS_ALL 0xff
 
-/* Flags that mark object as modified */
-/* Object, Child, Style, Viewport, User */
-#define SP_OBJECT_MODIFIED_STATE (SP_OBJECT_FLAGS_ALL & ~(SP_OBJECT_PARENT_MODIFIED_FLAG))
-
-/* Flags that will propagate downstreams */
-/* Parent, Style, Viewport, User */
-#define SP_OBJECT_MODIFIED_CASCADE (SP_OBJECT_FLAGS_ALL & ~(SP_OBJECT_MODIFIED_FLAG | SP_OBJECT_CHILD_MODIFIED_FLAG))
-
 /* Generic */
 #define SP_OBJECT_IS_CLONED(o) (((SPObject *) (o))->cloned)
 
@@ -110,6 +92,7 @@ struct _SPIXmlSpace {
 struct _SPObject {
 	GObject object;
 	unsigned int cloned : 1;
+	unsigned int blocked : 1;
 	unsigned int uflags : 8;
 	unsigned int mflags : 8;
 	SPIXmlSpace xml_space;
@@ -131,7 +114,7 @@ struct _SPObjectClass {
 
 	/* Virtual handlers of repr signals */
 	void (* child_added) (SPObject *object, SPRepr *child, SPRepr *ref);
-	void (* remove_child) (SPObject *object, SPRepr *child);
+	unsigned int (* remove_child) (SPObject *object, SPRepr *child);
 
 	void (* order_changed) (SPObject *object, SPRepr *child, SPRepr *old, SPRepr *new);
 
@@ -175,6 +158,8 @@ SPObject *sp_object_attach_reref (SPObject *parent, SPObject *object, SPObject *
 SPObject *sp_object_detach (SPObject *parent, SPObject *object);
 SPObject *sp_object_detach_unref (SPObject *parent, SPObject *object);
 
+#define sp_object_set_blocked(o,v) (((SPObject *) (o))->blocked = ((v) != 0))
+
 void sp_object_invoke_build (SPObject * object, SPDocument * document, SPRepr * repr, unsigned int cloned);
 void sp_object_invoke_release (SPObject *object);
 
