@@ -34,9 +34,6 @@ enum {
 struct _SPPattern {
 	SPPaintServer paint_server;
 
-	/* Children list */
-	SPObject *children;
-
 	/* Reference (href) */
 	SPPattern *href;
 	/* patternUnits and patternContentUnits attribute */
@@ -61,6 +58,6 @@ struct _SPPatternClass {
 	SPPaintServerClass parent_class;
 };
 
-GtkType sp_pattern_get_type (void);
+GType sp_pattern_get_type (void);
 
 #endif
