@@ -16,13 +16,13 @@
 #include "svg/svg-types.h"
 #include "sp-shape.h"
 
-BEGIN_GNOME_DECLS
+G_BEGIN_DECLS
 
-#define SP_TYPE_RECT (sp_rect_get_type ())
-#define SP_RECT(o) (GTK_CHECK_CAST ((o), SP_TYPE_RECT, SPRect))
-#define SP_RECT_CLASS(k) (GTK_CHECK_CLASS_CAST ((k), SP_TYPE_RECT, SPRectClass))
-#define SP_IS_RECT(o) (GTK_CHECK_TYPE ((o), SP_TYPE_RECT))
-#define SP_IS_RECT_CLASS(k) (GTK_CHECK_CLASS_TYPE ((k), SP_TYPE_RECT))
+#define SP_TYPE_RECT            (sp_rect_get_type ())
+#define SP_RECT(obj)            (G_TYPE_CHECK_INSTANCE_CAST ((obj), SP_TYPE_RECT, SPRect))
+#define SP_RECT_CLASS(klass)    (G_TYPE_CHECK_CLASS_CAST ((klass), SP_TYPE_RECT, SPRectClass))
+#define SP_IS_RECT(obj)         (G_TYPE_CHECK_INSTANCE_TYPE ((obj), SP_TYPE_RECT))
+#define SP_IS_RECT_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), SP_TYPE_RECT))
 
 typedef struct _SPRect SPRect;
 typedef struct _SPRectClass SPRectClass;
@@ -43,15 +43,15 @@ struct _SPRectClass {
 };
 
 
-/* Standard Gtk function */
-GtkType sp_rect_get_type (void);
+/* Standard GType function */
+GType sp_rect_get_type (void);
 
-void sp_rect_set (SPRect * rect, gdouble x, gdouble y, gdouble width, gdouble height);
+void sp_rect_position_set (SPRect * rect, gdouble x, gdouble y, gdouble width, gdouble height);
 
 /* If SET if FALSE, VALUE is just ignored */
 void sp_rect_set_rx(SPRect * rect, gboolean set, gdouble value);
 void sp_rect_set_ry(SPRect * rect, gboolean set, gdouble value);
 
-END_GNOME_DECLS
+G_END_DECLS
 
 #endif
