@@ -13,33 +13,46 @@
  * Released under GNU GPL, read the file 'COPYING' for more information
  */
 
+#define SP_TYPE_ROOT (sp_root_get_type ())
+#define SP_ROOT(o) (G_TYPE_CHECK_INSTANCE_CAST ((o), SP_TYPE_ROOT, SPRoot))
+#define SP_ROOT_CLASS(k) (G_TYPE_CHECK_CLASS_CAST ((k), SP_TYPE_ROOT, SPRootClass))
+#define SP_IS_ROOT(o) (G_TYPE_CHECK_INSTANCE_TYPE ((o), SP_TYPE_ROOT))
+#define SP_IS_ROOT_CLASS(k) (G_TYPE_CHECK_CLASS_TYPE ((k), SP_TYPE_ROOT))
+
 #include <libnr/nr-matrix.h>
 #include "svg/svg-types.h"
+#include "enums.h"
 #include "sp-item-group.h"
 
-BEGIN_GNOME_DECLS
-
-#define SP_TYPE_ROOT (sp_root_get_type ())
-#define SP_ROOT(o) (GTK_CHECK_CAST ((o), SP_TYPE_ROOT, SPRoot))
-#define SP_ROOT_CLASS(k) (GTK_CHECK_CLASS_CAST ((k), SP_TYPE_ROOT, SPRootClass))
-#define SP_IS_ROOT(o) (GTK_CHECK_TYPE ((o), SP_TYPE_ROOT))
-#define SP_IS_ROOT_CLASS(k) (GTK_CHECK_CLASS_TYPE ((k), SP_TYPE_ROOT))
-
 struct _SPRoot {
 	SPGroup group;
 
+	float version;
+
 	guint svg : 8;
 	guint sodipodi : 8;
 	guint original : 8;
 
+	SPSVGLength x;
+	SPSVGLength y;
 	SPSVGLength width;
 	SPSVGLength height;
 
-	NRMatrixD viewbox;
+	/* viewBox; */
+	unsigned int viewBox_set : 1;
+	NRRectD viewBox;
+
+	/* preserveAspectRatio */
+	unsigned int aspect_set : 1;
+	unsigned int aspect_align : 4;
+	unsigned int aspect_clip : 1;
+
+	/* Child to parent additional transform */
+	NRMatrixD c2p;
 
 	/* List of namedviews */
 	/* fixme: use single container instead */
-	GSList *namedviews;
+	/* GSList *namedviews; */
 	/* Root-level <defs> node */
 	SPDefs *defs;
 };
@@ -48,8 +61,6 @@ struct _SPRootClass {
 	SPGroupClass parent_class;
 };
 
-GtkType sp_root_get_type (void);
-
-END_GNOME_DECLS
+GType sp_root_get_type (void);
 
 #endif
