@@ -150,13 +150,13 @@ sp_spiral_write (SPObject *object, SPRepr *repr, guint flags)
 		 * sodipodi:spiral="cx cy exp revo rad arg t0"
 		 */
 		sp_repr_set_attr (repr, "sodipodi:type", "spiral");
-		sp_repr_set_double_attribute (repr, "sodipodi:cx", spiral->cx);
-		sp_repr_set_double_attribute (repr, "sodipodi:cy", spiral->cy);
-		sp_repr_set_double_attribute (repr, "sodipodi:expansion", spiral->exp);
-		sp_repr_set_double_attribute (repr, "sodipodi:revolution", spiral->revo);
-		sp_repr_set_double_attribute (repr, "sodipodi:radius", spiral->rad);
-		sp_repr_set_double_attribute (repr, "sodipodi:argument", spiral->arg);
-		sp_repr_set_double_attribute (repr, "sodipodi:t0", spiral->t0);
+		sp_repr_set_double (repr, "sodipodi:cx", spiral->cx);
+		sp_repr_set_double (repr, "sodipodi:cy", spiral->cy);
+		sp_repr_set_double (repr, "sodipodi:expansion", spiral->exp);
+		sp_repr_set_double (repr, "sodipodi:revolution", spiral->revo);
+		sp_repr_set_double (repr, "sodipodi:radius", spiral->rad);
+		sp_repr_set_double (repr, "sodipodi:argument", spiral->arg);
+		sp_repr_set_double (repr, "sodipodi:t0", spiral->t0);
 	}
 
 	d = sp_svg_write_path (((SPShape *) spiral)->curve->bpath);
@@ -283,6 +283,7 @@ sp_spiral_fit_and_draw (SPSpiral *spiral,
 	NRPointF bezier[BEZIER_LENGTH];
 	double d;
 	int depth, i;
+	NRPointF dir2;
 	
 	for (d = t, i = 0; i <= SAMPLE_SIZE; d += dstep, i++) {
 		sp_spiral_get_xy (spiral, d, &darray[i]);
@@ -293,8 +294,10 @@ sp_spiral_fit_and_draw (SPSpiral *spiral,
 	/* Fixme:
 	   we should use better algorithm to specify maximum error.
 	*/
+	dir2.x = -hat2->x;
+	dir2.y = -hat2->y;
 	depth = sp_bezier_fit_cubic_full (bezier, darray, SAMPLE_SIZE,
-					  hat1, hat2,
+					  hat1, NULL, &dir2, NULL,
 					  SPIRAL_TOLERANCE*SPIRAL_TOLERANCE,
 					  FITTING_DEPTH);
 #ifdef SPIRAL_DEBUG
