@@ -16,13 +16,13 @@
 
 #include "sp-shape.h"
 
-BEGIN_GNOME_DECLS
+G_BEGIN_DECLS
 
 #define SP_TYPE_SPIRAL            (sp_spiral_get_type ())
-#define SP_SPIRAL(obj)            (GTK_CHECK_CAST ((obj), SP_TYPE_SPIRAL, SPSpiral))
-#define SP_SPIRAL_CLASS(klass)    (GTK_CHECK_CLASS_CAST ((klass), SP_TYPE_SPIRAL, SPSpiralClass))
-#define SP_IS_SPIRAL(obj)         (GTK_CHECK_TYPE ((obj), SP_TYPE_SPIRAL))
-#define SP_IS_SPIRAL_CLASS(klass) (GTK_CHECK_CLASS_TYPE ((klass), SP_TYPE_SPIRAL))
+#define SP_SPIRAL(obj)            (G_TYPE_CHECK_INSTANCE_CAST ((obj), SP_TYPE_SPIRAL, SPSpiral))
+#define SP_SPIRAL_CLASS(klass)    (G_TYPE_CHECK_CLASS_CAST ((klass), SP_TYPE_SPIRAL, SPSpiralClass))
+#define SP_IS_SPIRAL(obj)         (G_TYPE_CHECK_INSTANCE_TYPE ((obj), SP_TYPE_SPIRAL))
+#define SP_IS_SPIRAL_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), SP_TYPE_SPIRAL))
 
 typedef struct _SPSpiral SPSpiral;
 typedef struct _SPSpiralClass SPSpiralClass;
@@ -52,10 +52,10 @@ struct _SPSpiralClass {
 
 
 /* Standard Gtk function */
-GtkType sp_spiral_get_type  (void);
+GType sp_spiral_get_type  (void);
 
 /* Lowlevel interface */
-void    sp_spiral_set		(SPSpiral      *spiral,
+void    sp_spiral_position_set		(SPSpiral      *spiral,
 				 gdouble	cx,
 				 gdouble	cy,
 				 gdouble	exp,
@@ -66,7 +66,7 @@ void    sp_spiral_set		(SPSpiral      *spiral,
 
 void    sp_spiral_get_xy	(SPSpiral      *spiral,
 				 gdouble	t,
-				 ArtPoint      *p);
+				 NRPointF      *p);
 void    sp_spiral_get_polar	(SPSpiral      *spiral,
 				 gdouble	t,
 				 gdouble       *rad,
@@ -74,6 +74,6 @@ void    sp_spiral_get_polar	(SPSpiral      *spiral,
 gboolean sp_spiral_is_invalid   (SPSpiral      *spiral);
 
 
-END_GNOME_DECLS
+G_END_DECLS
 
 #endif
