@@ -16,13 +16,13 @@
 
 #include "sp-polygon.h"
 
-BEGIN_GNOME_DECLS
+G_BEGIN_DECLS
 
-#define SP_TYPE_STAR (sp_star_get_type ())
-#define SP_STAR(o) (GTK_CHECK_CAST ((o), SP_TYPE_STAR, SPStar))
-#define SP_STAR_CLASS(k) (GTK_CHECK_CLASS_CAST ((k), SP_TYPE_STAR, SPStarClass))
-#define SP_IS_STAR(o) (GTK_CHECK_TYPE ((o), SP_TYPE_STAR))
-#define SP_IS_STAR_CLASS(k) (GTK_CHECK_CLASS_TYPE ((k), SP_TYPE_STAR))
+#define SP_TYPE_STAR            (sp_star_get_type ())
+#define SP_STAR(obj)            (G_TYPE_CHECK_INSTANCE_CAST ((obj), SP_TYPE_STAR, SPStar))
+#define SP_STAR_CLASS(klass)    (G_TYPE_CHECK_CLASS_CAST ((klass), SP_TYPE_STAR, SPStarClass))
+#define SP_IS_STAR(obj)         (G_TYPE_CHECK_INSTANCE_TYPE ((obj), SP_TYPE_STAR))
+#define SP_IS_STAR_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), SP_TYPE_STAR))
 
 typedef struct _SPStar SPStar;
 typedef struct _SPStarClass SPStarClass;
@@ -46,12 +46,12 @@ struct _SPStarClass {
 	SPPolygonClass parent_class;
 };
 
-GtkType sp_star_get_type (void);
+GType sp_star_get_type (void);
 
-void sp_star_set (SPStar *star, gint sides, gdouble cx, gdouble cy, gdouble r1, gdouble r2, gdouble arg1, gdouble arg2);
+void sp_star_position_set (SPStar *star, gint sides, gdouble cx, gdouble cy, gdouble r1, gdouble r2, gdouble arg1, gdouble arg2);
 
-void sp_star_get_xy (SPStar *star, SPStarPoint point, gint index, ArtPoint *p);
+void sp_star_get_xy (SPStar *star, SPStarPoint point, gint index, NRPointF *p);
 
-END_GNOME_DECLS
+G_END_DECLS
 
 #endif
