@@ -33,6 +33,7 @@
 #include <libnrtype/nr-type-directory.h>
 #include <libnrtype/nr-font.h>
 #include <libnrtype/nr-glyphs.h>
+#include <libarikkei/arikkei-strlib.h>
 
 #include "helper/sp-intl.h"
 #include "xml/repr-private.h"
@@ -47,6 +48,22 @@
 
 #include "sp-text.h"
 
+/* fixme: */
+unsigned int
+sp_repr_set_content_ucs2 (SPRepr *repr, const NRUShort *newcontent)
+{
+	char *utf8;
+	unsigned int retval;
+
+	utf8 = g_utf16_to_utf8 (newcontent, -1, NULL, NULL, NULL);
+
+	retval = sp_repr_set_content (repr, utf8);
+
+	g_free (utf8);
+
+	return retval;
+}
+
 static guchar *sp_text_font_style_to_lookup (SPStyle *style);
 
 static void sp_text_update_length (SPSVGLength *length, gdouble em, gdouble ex, gdouble scale);
@@ -129,12 +146,8 @@ sp_string_release (SPObject *object)
 
 	string = SP_STRING (object);
 
-	if (string->p) g_free (string->p);
-	if (string->text) g_free (string->text);
-
-#ifdef SP_TEXT_NEW_CONTENT
 	if (string->uchars) nr_free (string->uchars);
-#endif
+	if (string->pgl) nr_pgl_free (string->pgl);
 
 	if (((SPObjectClass *) string_parent_class)->release)
 		((SPObjectClass *) string_parent_class)->release (object);
@@ -150,30 +163,26 @@ sp_string_read_content (SPObject *object)
 
 	string = SP_STRING (object);
 
-	if (string->p) g_free (string->p);
-	string->p = NULL;
-	if (string->text) g_free (string->text);
 	t = sp_repr_content (object->repr);
-	string->text = (t) ? g_strdup (t) : NULL;
 
-#ifdef SP_TEXT_NEW_CONTENT
 	/* New content stuff */
 	if (string->uchars) free (string->uchars);
 	string->uchars = NULL;
 	string->ulen = 0;
-	if (string->text) {
-		unsigned char *p;
+	if (t) {
+		const unsigned char *p;
 		unsigned int unival;
 		unsigned int preserve, inspace, intext;
 		int pos;
-		preserve = (((SPObject*)string)->xml_space.value == SP_XML_SPACE_PRESERVE);
+
+		preserve = SP_STRING_IS_SPACE_PRESERVE (string);
 		inspace = FALSE;
 		intext = FALSE;
-		for (p = string->text; p && *p; p = g_utf8_next_char (p)) {
+		for (p = t; p && *p; p = g_utf8_next_char (p)) {
 			unival = g_utf8_get_char (p);
 			if (g_unichar_isspace (unival)) {
 				if (preserve) string->ulen += 1;
-				inspace = TRUE;
+				if (unival == ' ' || unival == '\t') inspace = TRUE;
 			} else {
 				/* Append space if needed */
 				if (!preserve && inspace && intext) string->ulen += 1;
@@ -183,18 +192,20 @@ sp_string_read_content (SPObject *object)
 				intext = TRUE;
 			}
 		}
-		string->uchars = nr_new (NRUShort, string->ulen);
+		if (!preserve && inspace && intext) string->ulen += 1;
+
+		string->uchars = nr_new (NRUShort, string->ulen + 1);
 		string->spstart = FALSE;
 		string->spend = FALSE;
 		inspace = FALSE;
 		intext = FALSE;
 		pos = 0;
-		for (p = string->text; p && *p; p = g_utf8_next_char (p)) {
+		for (p = t; p && *p; p = g_utf8_next_char (p)) {
 			unival = g_utf8_get_char (p);
 			if (g_unichar_isspace (unival)) {
-				if (preserve) string->uchars[pos++] = unival;
+				if (preserve) string->uchars[pos++] = 32;
 				if (!intext) string->spstart = TRUE;
-				inspace = TRUE;
+				if (unival == ' ' || unival == '\t') inspace = TRUE;
 			} else {
 				/* Append space if needed */
 				if (!preserve && inspace && intext) string->uchars[pos++] = 32;
@@ -204,9 +215,11 @@ sp_string_read_content (SPObject *object)
 				intext = TRUE;
 			}
 		}
+		if (!preserve && inspace && intext) string->uchars[pos++] = 32;
+
+		string->uchars[pos] = '\0';
 		string->spend = inspace;
 	}
-#endif
 
 	/* Is this correct? I think so (Lauris) */
 	/* Virtual method will be invoked BEFORE signal, so we can update there */
@@ -238,13 +251,7 @@ sp_string_calculate_dimensions (SPString *string)
 	NRFont *font;
 	gdouble size;
 	unsigned int metrics;
-#ifdef SP_TEXT_NEW_CONTENT
-	NRPGL *pgl;
 	NRMatrixF gtr;
-#else
-	gint spglyph;
-	NRPointF spadv;
-#endif
 
 	string->bbox.x0 = string->bbox.y0 = 1e18;
 	string->bbox.x1 = string->bbox.y1 = -1e18;
@@ -262,73 +269,13 @@ sp_string_calculate_dimensions (SPString *string)
 	}
 	font = nr_font_new_default (face, metrics, size);
 
-#ifdef SP_TEXT_NEW_CONTENT
 	/* fixme: SPChars should do this upright instead */
 	nr_matrix_f_set_scale (&gtr, 1.0, -1.0);
-	pgl = nr_pgl_new_from_string (string->uchars, string->ulen, font, &gtr);
+	if (string->pgl) nr_pgl_free (string->pgl);
 
-	string->bbox = pgl->area;
-	string->advance = pgl->advance;
-	nr_pgl_free (pgl);
-
-#else
-	if (style->writing_mode.computed == SP_CSS_WRITING_MODE_TB) {
-		spadv.x = 0.0;
-		spadv.y = size;
-	} else {
-		spadv.x = size;
-		spadv.y = 0.0;
-	}
-	spglyph = nr_typeface_lookup_default (face, ' ');
-	nr_font_glyph_advance_get (font, spglyph, &spadv);
-
-	if (string->text) {
-		const guchar *p;
-		gboolean preserve, inspace, intext;
-
-		preserve = (((SPObject*)string)->xml_space.value == SP_XML_SPACE_PRESERVE);
-		inspace = FALSE;
-		intext = FALSE;
-
-		for (p = string->text; p && *p; p = g_utf8_next_char (p)) {
-			gunichar unival;
-			
-			unival = g_utf8_get_char (p);
-
-			if (g_unichar_isspace(unival)) {
-				if (preserve) {
-					string->advance.x += spadv.x;
-					string->advance.y -= spadv.y;
-				}
-				if (unival != '\n' && unival != '\r') inspace = TRUE;
-			} else {
-				NRRectF bbox;
-				NRPointF adv;
-				gint glyph;
-
-				glyph = nr_typeface_lookup_default (face, unival);
-
-				if (!preserve && inspace && intext) {
-					string->advance.x += spadv.x;
-					string->advance.y -= spadv.y;
-				}
-
-				if (nr_font_glyph_area_get (font, glyph, &bbox)) {
-					string->bbox.x0 = MIN (string->bbox.x0, string->advance.x + bbox.x0);
-					string->bbox.y0 = MIN (string->bbox.y0, string->advance.y - bbox.y1);
-					string->bbox.x1 = MAX (string->bbox.x1, string->advance.x + bbox.x1);
-					string->bbox.y1 = MAX (string->bbox.y1, string->advance.y - bbox.y0);
-				}
-				if (nr_font_glyph_advance_get (font, glyph, &adv)) {
-					string->advance.x += adv.x;
-					string->advance.y -= adv.y;
-				}
-				inspace = FALSE;
-				intext = TRUE;
-			}
-		}
-	}
-#endif
+	string->pgl = nr_pgl_new_from_string (string->uchars, string->ulen, font, &gtr);
+	string->bbox = string->pgl->area;
+	string->advance = string->pgl->advance;
 
 	nr_font_unref (font);
 	nr_typeface_unref (face);
@@ -347,51 +294,19 @@ sp_string_set_shape (SPString *string, SPLayoutData *ly, ArtPoint *cp, gboolean
 {
 	SPChars *chars;
 	SPStyle *style;
-	NRTypeFace *face;
-	NRFont *font;
-	gdouble size;
-	gdouble x, y;
+	gint len;
 	NRMatrixF a;
-	const guchar *p;
-	gboolean preserve;
-	gboolean inspace;
-	gboolean intext;
-	gint len, pos;
-	unsigned int metrics;
-#ifdef SP_TEXT_NEW_CONTENT
-	NRPGL *pgl;
-	NRMatrixF gtr;
+	gdouble x, y;
 	gint i;
-#else
-	gint spglyph;
-	NRPointF spadv;
-#endif
 
 	chars = SP_CHARS (string);
 	style = SP_OBJECT_STYLE (SP_OBJECT_PARENT (string));
 
 	sp_chars_clear (chars);
 
-	if (!string->text || !*string->text) return;
-	len = g_utf8_strlen (string->text, -1);
+	if (!string->uchars || !*string->uchars) return;
+	len = arikkei_ucs2_strlen (string->uchars);
 	if (!len) return;
-	if (string->p) g_free (string->p);
-	string->p = g_new (NRPointF, len + 1);
-
-	/* fixme: Adjusted value (Lauris) */
-	size = style->font_size.computed;
-	face = nr_type_directory_lookup_fuzzy (style->text->font_family.value, sp_text_font_style_to_lookup (style));
-	if (style->writing_mode.computed == SP_CSS_WRITING_MODE_TB) {
-		metrics = NR_TYPEFACE_METRICS_VERTICAL;
-	} else {
-		metrics = NR_TYPEFACE_METRICS_HORIZONTAL;
-	}
-	font = nr_font_new_default (face, metrics, size);
-
-#ifdef SP_TEXT_NEW_CONTENT
-	/* fixme: SPChars should do this upright instead */
-	nr_matrix_f_set_scale (&gtr, 1.0, -1.0);
-	pgl = nr_pgl_new_from_string (string->uchars, string->ulen, font, &gtr);
 
 	/* fixme: Find a way how to manipulate these */
 	x = cp->x;
@@ -400,93 +315,24 @@ sp_string_set_shape (SPString *string, SPLayoutData *ly, ArtPoint *cp, gboolean
 	/* fixme: SPChars should do this upright instead */
 	nr_matrix_f_set_scale (&a, 1.0, -1.0);
 
-	for (i = 0; i < string->ulen; i++) {
+	g_assert (string->pgl);
+	nr_pgl_set_origin (string->pgl, x, y);
+
+	for (i = 0; i < string->pgl->length; i++) {
+		NRFont *font;
+		NRPosGlyph *pglyph;
+
+		/* fixme: we should use rasterfont instead */
+		font = string->pgl->rfont->font;
+		pglyph = &string->pgl->glyphs[i];
+		a.c[4] = pglyph->x;
+		a.c[5] = pglyph->y;
+
+		sp_chars_add_element (chars, pglyph->glyph, font, &a);
 	}
 
-	nr_pgl_free (pgl);
-
-
-#else
-	if (style->writing_mode.computed == SP_CSS_WRITING_MODE_TB) {
-		spadv.x = 0.0;
-		spadv.y = size;
-	} else {
-		spadv.x = size;
-		spadv.y = 0.0;
-	}
-	spglyph = nr_typeface_lookup_default (face, ' ');
-	nr_font_glyph_advance_get (font, spglyph, &spadv);
-
-	/* fixme: Find a way how to manipulate these */
-	x = cp->x;
-	y = cp->y;
-
-	/* fixme: SPChars should do this upright instead */
-	nr_matrix_f_set_scale (&a, 1.0, -1.0);
-
-	intext = FALSE;
-	preserve = (((SPObject*)string)->xml_space.value == SP_XML_SPACE_PRESERVE);
-	inspace = pinspace ? *pinspace : FALSE;
-	pos = 0;
-	for (p = string->text; p && *p; p = g_utf8_next_char (p)) {
-		gunichar unival;
-		if (!preserve && inspace && intext) {
-			/* SP_XML_SPACE_DEFAULT */
-			string->p[pos].x = x + spadv.x;
-			string->p[pos].y = y - spadv.y;
-		} else {
-			string->p[pos].x = x;
-			string->p[pos].y = y;
-		}
-		unival = g_utf8_get_char (p);
-
-		if (g_unichar_isspace(unival)) {
-			if (preserve) {
-				x += spadv.x;
-				y -= spadv.y;
-			}
-			if (unival != '\n' && unival != '\r') inspace = TRUE;
-		} else {
-			NRPointF adv;
-			gint glyph;
-
-			glyph = nr_typeface_lookup_default (face, unival);
-
-			if (!preserve && inspace && intext) {
-				x += spadv.x;
-				y -= spadv.y;
-			}
-
-			a.c[4] = x;
-			a.c[5] = y;
-
-			sp_chars_add_element (chars, glyph, font, &a);
-			if (nr_font_glyph_advance_get (font, glyph, &adv)) {
-				x += adv.x;
-				y -= adv.y;
-			}
-			inspace = FALSE;
-			intext = TRUE;
-		}
-		pos += 1;
-	}
-
-	nr_font_unref (font);
-	nr_typeface_unref (face);
-
-	if (inspace) {
-		string->p[pos].x = x + spadv.x;
-		string->p[pos].y = y - spadv.y;
-	} else {
-		string->p[pos].x = x;
-		string->p[pos].y = y;
-	}
-
-	cp->x = x;
-	cp->y = y;
-
-	if (pinspace) *pinspace = inspace;
-#endif
+	cp->x += string->pgl->advance.x;
+	cp->y -= string->pgl->advance.y;
 }
 
 /* SPTSpan */
@@ -498,7 +344,7 @@ static void sp_tspan_build (SPObject * object, SPDocument * document, SPRepr * r
 static void sp_tspan_release (SPObject *object);
 static void sp_tspan_set (SPObject *object, unsigned int key, const unsigned char *value);
 static void sp_tspan_child_added (SPObject *object, SPRepr *rch, SPRepr *ref);
-static void sp_tspan_remove_child (SPObject *object, SPRepr *rch);
+static unsigned int sp_tspan_remove_child (SPObject *object, SPRepr *rch);
 static void sp_tspan_update (SPObject *object, SPCtx *ctx, guint flags);
 static void sp_tspan_modified (SPObject *object, unsigned int flags);
 static SPRepr *sp_tspan_write (SPObject *object, SPRepr *repr, guint flags);
@@ -589,11 +435,12 @@ sp_tspan_build (SPObject *object, SPDocument *doc, SPRepr *repr)
 	sp_object_read_attr (object, "sodipodi:role");
 
 	for (rch = repr->children; rch != NULL; rch = rch->next) {
-		if (rch->type == SP_XML_TEXT_NODE) break;
+		if (sp_repr_is_text (rch)) break;
+		if (sp_repr_is_cdata (rch)) break;
 	}
 
 	if (!rch) {
-		rch = sp_xml_document_createTextNode (sp_repr_document (repr), "");
+		rch = sp_xml_document_createTextNode (sp_repr_get_doc (repr), "");
 		sp_repr_add_child (repr, rch, NULL);
 	}
 
@@ -686,7 +533,7 @@ sp_tspan_child_added (SPObject *object, SPRepr *rch, SPRepr *ref)
 	if (((SPObjectClass *) tspan_parent_class)->child_added)
 		((SPObjectClass *) tspan_parent_class)->child_added (object, rch, ref);
 
-	if (!tspan->string && rch->type == SP_XML_TEXT_NODE) {
+	if (!tspan->string && (sp_repr_is_text (rch) || sp_repr_is_cdata (rch))) {
 		SPString *string;
 		/* fixme: We should really pick up first child always */
 		string = g_object_new (SP_TYPE_STRING, 0);
@@ -696,19 +543,24 @@ sp_tspan_child_added (SPObject *object, SPRepr *rch, SPRepr *ref)
 	}
 }
 
-static void
+static unsigned int
 sp_tspan_remove_child (SPObject *object, SPRepr *rch)
 {
 	SPTSpan *tspan;
+	unsigned int ret;
 
 	tspan = SP_TSPAN (object);
 
-	if (((SPObjectClass *) tspan_parent_class)->remove_child)
-		((SPObjectClass *) tspan_parent_class)->remove_child (object, rch);
+	if (((SPObjectClass *) tspan_parent_class)->remove_child) {
+		ret = ((SPObjectClass *) tspan_parent_class)->remove_child (object, rch);
+		if (!ret) return ret;
+	}
 
 	if (tspan->string && (SP_OBJECT_REPR (tspan->string) == rch)) {
 		tspan->string = sp_object_detach_unref (object, tspan->string);
 	}
+
+	return TRUE;
 }
 
 static void
@@ -776,23 +628,27 @@ sp_tspan_write (SPObject *object, SPRepr *repr, guint flags)
 		repr = sp_repr_new ("tspan");
 	}
 
-	if (tspan->ly.x.set) sp_repr_set_double_attribute (repr, "x", tspan->ly.x.computed);
-	if (tspan->ly.y.set) sp_repr_set_double_attribute (repr, "y", tspan->ly.y.computed);
-	if (tspan->ly.dx.set) sp_repr_set_double_attribute (repr, "dx", tspan->ly.dx.computed);
-	if (tspan->ly.dy.set) sp_repr_set_double_attribute (repr, "dy", tspan->ly.dy.computed);
-	if (tspan->ly.rotate_set) sp_repr_set_double_attribute (repr, "rotate", tspan->ly.rotate);
+	if (tspan->ly.x.set) sp_repr_set_double (repr, "x", tspan->ly.x.computed);
+	if (tspan->ly.y.set) sp_repr_set_double (repr, "y", tspan->ly.y.computed);
+	if (tspan->ly.dx.set) sp_repr_set_double (repr, "dx", tspan->ly.dx.computed);
+	if (tspan->ly.dy.set) sp_repr_set_double (repr, "dy", tspan->ly.dy.computed);
+	if (tspan->ly.rotate_set) sp_repr_set_double (repr, "rotate", tspan->ly.rotate);
 	if (flags & SP_OBJECT_WRITE_SODIPODI) {
 		sp_repr_set_attr (repr, "sodipodi:role", (tspan->role != SP_TSPAN_ROLE_UNSPECIFIED) ? "line" : NULL);
 	}
 
 	if (flags & SP_OBJECT_WRITE_BUILD) {
 		SPRepr *rstr;
+		char *text;
+		text = g_utf16_to_utf8 (SP_STRING_UCHARS (tspan->string), -1, NULL, NULL, NULL);
 		/* TEXT element */
-		rstr = sp_xml_document_createTextNode (sp_repr_document (repr), SP_STRING_TEXT (tspan->string));
+		rstr = sp_xml_document_createTextNode (sp_repr_get_doc (repr), text);
+		g_free (text);
+
 		sp_repr_append_child (repr, rstr);
 		sp_repr_unref (rstr);
 	} else {
-		sp_repr_set_content (SP_OBJECT_REPR (tspan->string), SP_STRING_TEXT (tspan->string));
+		sp_repr_set_content_ucs2 (SP_OBJECT_REPR (tspan->string), SP_STRING_UCHARS (tspan->string));
 	}
 
 	/* fixme: Strictly speaking, item class write 'transform' too */
@@ -869,7 +725,7 @@ static void sp_text_build (SPObject * object, SPDocument * document, SPRepr * re
 static void sp_text_release (SPObject *object);
 static void sp_text_set (SPObject *object, unsigned int key, const unsigned char *value);
 static void sp_text_child_added (SPObject *object, SPRepr *rch, SPRepr *ref);
-static void sp_text_remove_child (SPObject *object, SPRepr *rch);
+static unsigned int sp_text_remove_child (SPObject *object, SPRepr *rch);
 static void sp_text_update (SPObject *object, SPCtx *ctx, guint flags);
 static void sp_text_modified (SPObject *object, guint flags);
 static unsigned int sp_text_sequence (SPObject *object, SPObject *target, unsigned int *seq);
@@ -994,7 +850,7 @@ sp_text_build (SPObject *object, SPDocument *doc, SPRepr *repr)
 		const guchar *content;
 		/* Old sodipodi */
 		for (rch = repr->children; rch != NULL; rch = rch->next) {
-			if (rch->type == SP_XML_TEXT_NODE) {
+			if (sp_repr_is_text (rch) || sp_repr_is_cdata (rch)) {
 				content = sp_repr_content (rch);
 				sp_text_set_repr_text_multiline (text, content);
 				break;
@@ -1005,7 +861,7 @@ sp_text_build (SPObject *object, SPDocument *doc, SPRepr *repr)
 	ref = NULL;
 
 	for (rch = repr->children; rch != NULL; rch = rch->next) {
-		if (rch->type == SP_XML_TEXT_NODE) {
+		if (sp_repr_is_text (rch) || sp_repr_is_cdata (rch)) {
 			SPString *string;
 			string = g_object_new (SP_TYPE_STRING, 0);
 			if (ref) {
@@ -1124,7 +980,7 @@ sp_text_child_added (SPObject *object, SPRepr *rch, SPRepr *ref)
 		while (prev && (prev->repr != ref)) prev = prev->next;
 	}
 
-	if (rch->type == SP_XML_TEXT_NODE) {
+	if (sp_repr_is_text (rch) || sp_repr_is_cdata (rch)) {
 		SPString *string;
 		string = g_object_new (SP_TYPE_STRING, 0);
 		if (prev) {
@@ -1167,16 +1023,19 @@ sp_text_child_added (SPObject *object, SPRepr *rch, SPRepr *ref)
 	sp_text_update_immediate_state (text);
 }
 
-static void
+static unsigned int
 sp_text_remove_child (SPObject *object, SPRepr *rch)
 {
 	SPText *text;
 	SPObject *prev, *och;
+	unsigned int ret;
 
 	text = SP_TEXT (object);
 
-	if (((SPObjectClass *) text_parent_class)->remove_child)
-		((SPObjectClass *) text_parent_class)->remove_child (object, rch);
+	if (((SPObjectClass *) text_parent_class)->remove_child) {
+		ret = ((SPObjectClass *) text_parent_class)->remove_child (object, rch);
+		if (!ret) return ret;
+	}
 
 	prev = NULL;
 	och = object->children;
@@ -1193,6 +1052,8 @@ sp_text_remove_child (SPObject *object, SPRepr *rch)
 
 	sp_text_request_relayout (text, SP_OBJECT_MODIFIED_FLAG | SP_TEXT_CONTENT_MODIFIED_FLAG);
 	sp_text_update_immediate_state (text);
+
+	return TRUE;
 }
 
 /* fixme: This is wrong, as we schedule relayout every time something changes */
@@ -1320,7 +1181,10 @@ sp_text_write (SPObject *object, SPRepr *repr, guint flags)
 				crepr = sp_object_invoke_write (child, NULL, flags);
 				if (crepr) l = g_slist_prepend (l, crepr);
 			} else {
-				crepr = sp_xml_document_createTextNode (sp_repr_document (repr), SP_STRING_TEXT (child));
+				char *text;
+				text = g_utf16_to_utf8 (SP_STRING_UCHARS (child), -1, NULL, NULL, NULL);
+				crepr = sp_xml_document_createTextNode (sp_repr_get_doc (repr), text);
+				g_free (text);
 			}
 		}
 		while (l) {
@@ -1333,16 +1197,16 @@ sp_text_write (SPObject *object, SPRepr *repr, guint flags)
 			if (SP_IS_TSPAN (child)) {
 				sp_object_invoke_write (child, SP_OBJECT_REPR (child), flags);
 			} else {
-				sp_repr_set_content (SP_OBJECT_REPR (child), SP_STRING_TEXT (child));
+				sp_repr_set_content_ucs2 (SP_OBJECT_REPR (child), SP_STRING_UCHARS (child));
 			}
 		}
 	}
 
-	if (text->ly.x.set) sp_repr_set_double_attribute (repr, "x", text->ly.x.computed);
-	if (text->ly.y.set) sp_repr_set_double_attribute (repr, "y", text->ly.y.computed);
-	if (text->ly.dx.set) sp_repr_set_double_attribute (repr, "dx", text->ly.dx.computed);
-	if (text->ly.dy.set) sp_repr_set_double_attribute (repr, "dy", text->ly.dy.computed);
-	if (text->ly.rotate_set) sp_repr_set_double_attribute (repr, "rotate", text->ly.rotate);
+	if (text->ly.x.set) sp_repr_set_double (repr, "x", text->ly.x.computed);
+	if (text->ly.y.set) sp_repr_set_double (repr, "y", text->ly.y.computed);
+	if (text->ly.dx.set) sp_repr_set_double (repr, "dx", text->ly.dx.computed);
+	if (text->ly.dy.set) sp_repr_set_double (repr, "dy", text->ly.dy.computed);
+	if (text->ly.rotate_set) sp_repr_set_double (repr, "rotate", text->ly.rotate);
 
 	if (((SPObjectClass *) (text_parent_class))->write)
 		((SPObjectClass *) (text_parent_class))->write (object, repr, flags);
@@ -1519,6 +1383,8 @@ sp_text_set_shape (SPText *text)
 
 	cp.x = text->ly.x.computed;
 	cp.y = text->ly.y.computed;
+	if (text->ly.dx.set) cp.x += text->ly.dx.computed;
+	if (text->ly.dy.set) cp.y += text->ly.dy.computed;
 
 	isfirstline = TRUE;
 	inspace = FALSE;
@@ -1564,12 +1430,8 @@ sp_text_set_shape (SPText *text)
 				} else {
 					tspan->ly.y.computed = cp.y;
 				}
-				if (tspan->ly.dx.set) {
-					cp.x += tspan->ly.dx.computed;
-				}
-				if (tspan->ly.dy.set) {
-					cp.y += tspan->ly.dy.computed;
-				}
+				if (tspan->ly.dx.set) cp.x += tspan->ly.dx.computed;
+				if (tspan->ly.dy.set) cp.y += tspan->ly.dy.computed;
 				break;
 			default:
 				/* Error */
@@ -1764,11 +1626,11 @@ sp_text_is_empty (SPText *text)
 
 	for (ch = ((SPObject *) text)->children; ch != NULL; ch = ch->next) {
 		SPString *str;
-		unsigned char *p;
+		NRUShort *p;
 		str = SP_TEXT_CHILD_STRING (ch);
-		for (p = str->text; p && *p; p = g_utf8_next_char (p)) {
+		for (p = str->uchars; p && *p; p++) {
 			gunichar unival;
-			unival = g_utf8_get_char (p);
+			unival = (gunichar)*p;
 			if ((unival > 0xe000) && (unival <= 0xf8ff)) return FALSE;
 			if (g_unichar_isgraph (unival)) return FALSE;
 		}
@@ -1790,11 +1652,17 @@ sp_text_get_string_multiline (SPText *text)
 		if (SP_IS_TSPAN (ch)) {
 			SPTSpan *tspan;
 			tspan = SP_TSPAN (ch);
-			if (tspan->string && SP_STRING (tspan->string)->text) {
-				strs = g_slist_prepend (strs, SP_STRING (tspan->string)->text);
+
+			if (tspan->string && SP_STRING_UCHARS (tspan->string)) {
+				char *text;
+				/* Memory allocated, it should be freed later */
+				text = g_utf16_to_utf8 (SP_STRING_UCHARS (tspan->string), -1, NULL, NULL, NULL);
+				strs = g_slist_prepend (strs, text);
 			}
-		} else if (SP_IS_STRING (ch) && SP_STRING (ch)->text) {
-			strs = g_slist_prepend (strs, SP_STRING (ch)->text);
+		} else if (SP_IS_STRING (ch) && SP_STRING_UCHARS (ch)) {
+			char *text;
+			text = g_utf16_to_utf8 (SP_STRING_UCHARS (ch), -1, NULL, NULL, NULL);
+			strs = g_slist_prepend (strs, text);
 		} else {
 			continue;
 		}
@@ -1815,6 +1683,10 @@ sp_text_get_string_multiline (SPText *text)
 	while (strs) {
 		memcpy (p, strs->data, strlen (strs->data));
 		p += strlen (strs->data);
+
+		/* Free utf8 string */
+		g_free (strs->data);
+
 		strs = g_slist_remove (strs, strs->data);
 		if (strs) *p++ = '\n';
 	}
@@ -1866,7 +1738,7 @@ sp_text_set_repr_text_multiline (SPText *text, const guchar *str)
 			cp.y += style->font_size.computed;
 		}
 		sp_repr_set_attr (rtspan, "sodipodi:role", "line");
-		rstr = sp_xml_document_createTextNode (sp_repr_document (repr), p);
+		rstr = sp_xml_document_createTextNode (sp_repr_get_doc (repr), p);
 		sp_repr_add_child (rtspan, rstr, NULL);
 		sp_repr_append_child (repr, rtspan);
 		p = (e) ? e + 1 : NULL;
@@ -1877,33 +1749,34 @@ sp_text_set_repr_text_multiline (SPText *text, const guchar *str)
 	/* fixme: Calculate line positions (Lauris) */
 }
 
-SPCurve *
-sp_text_normalized_bpath (SPText *text)
+GSList *
+sp_text_normalized_bpath_list (SPText *text)
 {
 	SPObject *child;
 	GSList *cc;
-	SPCurve *curve;
+	/* SPCurve *curve; */
 
 	g_return_val_if_fail (text != NULL, NULL);
 	g_return_val_if_fail (SP_IS_TEXT (text), NULL);
 
 	cc = NULL;
 	for (child = ((SPObject *) text)->children; child != NULL; child = child->next) {
-		SPCurve *c;
+		GSList *c;
 		if (SP_IS_STRING (child)) {
-			c = sp_chars_normalized_bpath (SP_CHARS (child));
+			c = sp_chars_normalized_bpath_list (SP_CHARS (child));
 		} else if (SP_IS_TSPAN (child)) {
 			SPTSpan *tspan;
 			tspan = SP_TSPAN (child);
-			c = sp_chars_normalized_bpath (SP_CHARS (tspan->string));
+			c = sp_chars_normalized_bpath_list (SP_CHARS (tspan->string));
 		} else {
 			c = NULL;
 		}
-		if (c) cc = g_slist_prepend (cc, c);
+		if (c) cc = g_slist_concat (cc, c);
 	}
 
 	cc = g_slist_reverse (cc);
 
+#if 0
 	curve = sp_curve_concat (cc);
 
 	while (cc) {
@@ -1912,6 +1785,9 @@ sp_text_normalized_bpath (SPText *text)
 	}
 
 	return curve;
+#endif
+
+	return cc;
 }
 
 static void
@@ -1929,8 +1805,9 @@ sp_text_update_immediate_state (SPText *text)
 			string = SP_STRING (child);
 		}
 		string->start = start;
-		string->length = (string->text) ? g_utf8_strlen (string->text, -1) : 0;
-		start += string->length;
+		string->ulen = (string->uchars) ? arikkei_ucs2_strlen (string->uchars) : 0;
+		start += string->ulen;
+
 		/* Count newlines as well */
 		if (child->next) start += 1;
 	}
@@ -1966,7 +1843,7 @@ sp_text_get_length (SPText *text)
 		string = SP_TSPAN_STRING (child);
 	}
 
-	return string->start + string->length;
+	return string->start + string->ulen;
 }
 
 SPTSpan *
@@ -2010,7 +1887,7 @@ sp_text_append_line (SPText *text)
 	sp_repr_set_attr (rtspan, "sodipodi:role", "line");
 
 	/* Create TEXT */
-	rstring = sp_xml_document_createTextNode (sp_repr_document (rtspan), "");
+	rstring = sp_xml_document_createTextNode (sp_repr_get_doc (rtspan), "");
 	sp_repr_add_child (rtspan, rstring, NULL);
 	sp_repr_unref (rstring);
 	/* Append to text */
@@ -2026,7 +1903,6 @@ sp_text_insert_line (SPText *text, gint pos)
 	SPObject *child;
 	SPString *string;
 	SPRepr *rtspan, *rstring;
-	guchar *ip;
 
 	g_return_val_if_fail (text != NULL, NULL);
 	g_return_val_if_fail (SP_IS_TEXT (text), NULL);
@@ -2039,18 +1915,19 @@ sp_text_insert_line (SPText *text, gint pos)
 	rtspan = sp_repr_new ("tspan");
 	sp_repr_set_attr (rtspan, "sodipodi:role", "line");
 	/* Create TEXT */
-	rstring = sp_xml_document_createTextNode (sp_repr_document (rtspan), "");
+	rstring = sp_xml_document_createTextNode (sp_repr_get_doc (rtspan), "");
 	sp_repr_add_child (rtspan, rstring, NULL);
 	sp_repr_unref (rstring);
 
 	sp_repr_add_child (SP_OBJECT_REPR (text), rtspan, SP_OBJECT_REPR (child));
 	sp_repr_unref (rtspan);
 
-	if (string->text) {
-		ip = g_utf8_offset_to_pointer (string->text, pos - string->start);
-		sp_repr_set_content (rstring, ip);
+	if (string->uchars) {
+		NRUShort *ip;
+		ip = string->uchars + (pos - string->start);
+		sp_repr_set_content_ucs2 (rstring, ip);
 		*ip = '\0';
-		sp_repr_set_content (SP_OBJECT_REPR (string), string->text);
+		sp_repr_set_content_ucs2 (SP_OBJECT_REPR (string), string->uchars);
 	}
 
 	return (SPTSpan *) sp_document_lookup_id (SP_OBJECT_DOCUMENT (text), sp_repr_attr (rtspan, "id"));
@@ -2074,7 +1951,7 @@ sp_text_append (SPText *text, const guchar *utf8)
 		/* Create <tspan> */
 		rtspan = sp_repr_new ("tspan");
 		/* Create TEXT */
-		rstring = sp_xml_document_createTextNode (sp_repr_document (rtspan), "");
+		rstring = sp_xml_document_createTextNode (sp_repr_get_doc (rtspan), "");
 		sp_repr_add_child (rtspan, rstring, NULL);
 		sp_repr_unref (rstring);
 		/* Add string */
@@ -2118,15 +1995,87 @@ sp_text_append (SPText *text, const guchar *utf8)
 	return string->start + cchars + uchars;
 }
 
-/* Returns position after inserted */
+static NRUShort *
+ucs2_concat_ws (NRUShort *d, NRUShort *s, int len)
+{
+	NRUShort *dp, *sp, *spacep;
+	gboolean inspace, intext;
 
+	g_assert (d);
+
+	dp = d;
+	inspace = FALSE;
+	intext = FALSE;
+	while (*dp) {
+		if (g_unichar_isspace (*dp)) {
+			if (!inspace) spacep = dp;
+			inspace = TRUE;
+		} else {
+			inspace = FALSE;
+			intext = TRUE;
+		}
+		dp++;
+	}
+	if (inspace) dp = spacep;
+
+	if (!s || len == 0) {
+		if (inspace && intext) *dp++ = ' ';
+		*dp = '\0';
+		return dp;
+	}
+
+	sp = s;
+	if (len < 0) {
+		while (*sp) {
+			if (g_unichar_isspace (*sp)) {
+				inspace = TRUE;
+				sp++;
+			} else {
+				if (inspace && intext) *dp++ = ' ';
+				*dp++ = *sp++;
+				inspace = FALSE;
+				intext = TRUE;
+			}
+		}
+	} else {
+		int i;
+		for (i = 0; *sp && (i < len); i++) {
+			if (g_unichar_isspace (*sp)) {
+				inspace = TRUE;
+				sp++;
+			} else {
+				if (inspace) *dp++ = ' ';
+				*dp++ = *sp++;
+				inspace = FALSE;
+				intext = TRUE;
+			}
+		}
+	}
+
+	if (inspace && intext) *dp++ = ' ';
+	*dp = '\0';
+
+#if 0
+	guchar *utf8;
+	utf8 = arikkei_ucs2_utf8_strdup (d);
+	g_print ("d[%s] ", utf8);
+	g_free (utf8);
+	utf8 = arikkei_ucs2_utf8_strdup (s);
+	g_print ("s[%s]\n", utf8);
+	g_free (utf8);
+#endif
+
+	return dp;
+}
+
+/* Returns position after inserted */
 gint
 sp_text_insert (SPText *text, gint pos, const guchar *utf8, gboolean preservews)
 {
 	SPObject *child;
 	SPString *string;
-	guchar *new, *ip;
-	int slen, ulen, i;
+	NRUShort *ucs2, *new, *ip;
+	int slen, ulen, newpos, i;
 
 	g_return_val_if_fail (text != NULL, -1);
 	g_return_val_if_fail (SP_IS_TEXT (text), -1);
@@ -2139,28 +2088,67 @@ sp_text_insert (SPText *text, gint pos, const guchar *utf8, gboolean preservews)
 	if (!child) return sp_text_append (text, utf8);
 	string = SP_TEXT_CHILD_STRING (child);
 
-	ip = g_utf8_offset_to_pointer (string->text, pos - string->start);
+	ip = string->uchars + (pos - string->start);
 	/* fixme: Do it the right way (Lauris) */
-	if (!preservews && !strcmp (utf8, " ") && (ip > string->text) && *(ip - 1) == ' ') {
+	if (!preservews && !strcmp (utf8, " ") && (ip > string->uchars) && *(ip - 1) == (NRUShort)' ') {
 		return pos;
 	}
 
-	slen = ip - string->text;
-	ulen = strlen (utf8);
-	new = g_new (guchar, strlen (string->text) + ulen + 1);
+	slen = ip - string->uchars;
+	ulen = g_utf8_strlen (utf8, -1);
+	ucs2 = g_utf8_to_utf16 (utf8, -1, NULL, NULL, NULL);
+	new = g_new (NRUShort, arikkei_ucs2_strlen (string->uchars) + ulen + 1);
+
+#if 1
+	if (SP_STRING_IS_SPACE_PRESERVE (string)) {
+		/* Copy start */
+		arikkei_ucs2_strncpy (string->uchars, new, slen);
+		/* Copy string */
+		arikkei_ucs2_strncpy (ucs2, new + slen, ulen);
+		for (i = slen; i < slen + ulen; i++) {
+			if ((new[i] < 32) && ((new[i] != 9) && (new[i] != 10) && (new[i] != 13))) new[i] = 32;
+		}
+		/* Copy end */
+		arikkei_ucs2_strncpy (ip, new + slen + ulen, -1);
+
+		newpos = pos + ulen;
+	} else {
+		NRUShort *p1, *p2;
+		new[0] = '\0';
+
+		/* Copy start */
+		p1 = ucs2_concat_ws (new, string->uchars, slen);
+		/* Copy string */
+		p2 = ucs2_concat_ws (new, ucs2, -1); /* ulen */
+		/* Copy end */
+		ucs2_concat_ws (new, ip, -1);
+
+		newpos = pos + (int)(p2 - p1);
+	}
+	
+	sp_repr_set_content_ucs2 (SP_OBJECT_REPR (string), new);
+	g_free (new);
+	g_free (ucs2);
+
+	return newpos;
+#else
 	/* Copy start */
-	memcpy (new, string->text, slen);
+	arikkei_ucs2_strncpy (string->uchars, new, slen);
 	/* Copy string */
-	memcpy (new + slen, utf8, ulen);
+	arikkei_ucs2_strncpy (ucs2, new + slen, ulen);
 	for (i = slen; i < slen + ulen; i++) {
 		if ((new[i] < 32) && ((new[i] != 9) && (new[i] != 10) && (new[i] != 13))) new[i] = 32;
 	}
 	/* Copy end */
-	strcpy (new + slen + ulen, ip);
-	sp_repr_set_content (SP_OBJECT_REPR (string), new);
-	g_free (new);
+	arikkei_ucs2_strncpy (ip, new + slen + ulen, -1);
 
-	return pos + g_utf8_strlen (utf8, -1);
+	sp_repr_set_content_ucs2 (SP_OBJECT_REPR (string), new);
+	g_free (new);
+	g_free (ucs2);
+
+	return pos + ulen;
+/* 	return CLAMP (pos + ulen, string->start, string->start + string->ulen); */
+#endif
 }
 
 /* Returns start position */
@@ -2184,18 +2172,21 @@ sp_text_delete (SPText *text, gint start, gint end)
 	if (schild != echild) {
 		SPString *sstring, *estring;
 		SPObject *child;
-		guchar *utf8, *sp, *ep;
+		NRUShort *ucs2, *sp, *ep;
 		GSList *cl;
+
 		/* Easy case */
 		sstring = SP_TEXT_CHILD_STRING (schild);
 		estring = SP_TEXT_CHILD_STRING (echild);
-		sp = g_utf8_offset_to_pointer (sstring->text, start - sstring->start);
-		ep = g_utf8_offset_to_pointer (estring->text, end - estring->start);
-		utf8 = g_new (guchar, (sp - sstring->text) + strlen (ep) + 1);
-		if (sp > sstring->text) memcpy (utf8, sstring->text, sp - sstring->text);
-		memcpy (utf8 + (sp - sstring->text), ep, strlen (ep) + 1);
-		sp_repr_set_content (SP_OBJECT_REPR (sstring), utf8);
-		g_free (utf8);
+
+		sp = sstring->uchars + (start - sstring->start);
+		ep = estring->uchars + (end - estring->start);
+		ucs2 = g_new (NRUShort, (sp - sstring->uchars) + arikkei_ucs2_strlen (ep) + 1);
+		if (sp > sstring->uchars) arikkei_ucs2_strncpy (sstring->uchars, ucs2, sp - sstring->uchars);
+		arikkei_ucs2_strncpy (ep, ucs2 + (sp - sstring->uchars), arikkei_ucs2_strlen (ep) + 1);
+		sp_repr_set_content_ucs2 (SP_OBJECT_REPR (sstring), ucs2);
+		g_free (ucs2);
+
 		/* Delete nodes */
 		cl = NULL;
 		for (child = schild->next; child != echild; child = child->next) {
@@ -2208,13 +2199,14 @@ sp_text_delete (SPText *text, gint start, gint end)
 		}
 	} else {
 		SPString *string;
-		gchar *sp, *ep;
+		NRUShort *sp, *ep;
+
 		/* Easy case */
 		string = SP_TEXT_CHILD_STRING (schild);
-		sp = g_utf8_offset_to_pointer (string->text, start - string->start);
-		ep = g_utf8_offset_to_pointer (string->text, end - string->start);
-		memmove (sp, ep, strlen (ep) + 1);
-		sp_repr_set_content (SP_OBJECT_REPR (string), string->text);
+		sp = string->uchars + (start - string->start);
+		ep = string->uchars + (end - string->start);
+		memmove (sp, ep, sizeof(NRUShort) * (arikkei_ucs2_strlen (ep) + 1));
+		sp_repr_set_content_ucs2 (SP_OBJECT_REPR (string), string->uchars);
 	}
 
 	return start;
@@ -2237,7 +2229,8 @@ sp_text_up (SPText *text, gint pos)
 	up = ((SPObject *) text)->children;
 	while (up->next != child) up = up->next;
 	string = SP_TEXT_CHILD_STRING (up);
-	col = MIN (col, string->length);
+
+	col = MIN (col, string->ulen);
 
 	return string->start + col;
 }
@@ -2256,7 +2249,8 @@ sp_text_down (SPText *text, gint pos)
 
 	child = child->next;
 	string = SP_TEXT_CHILD_STRING (child);
-	col = MIN (col, string->length);
+
+	col = MIN (col, string->ulen);
 
 	return string->start + col;
 }
@@ -2284,7 +2278,7 @@ sp_text_end_of_line (SPText *text, gint pos)
 	if (!child) return sp_text_get_length (text);
 	string = SP_TEXT_CHILD_STRING (child);
 
-	return string->start + string->length;
+	return string->start + string->ulen;
 }
 
 void
@@ -2293,16 +2287,20 @@ sp_text_get_cursor_coords (SPText *text, gint position, ArtPoint *p0, ArtPoint *
 	SPObject *child;
 	SPString *string;
 	gfloat x, y;
+	NRPosGlyph *pglyph;
+	/* NRRectF area; */
+	/* NRPointF adv; */
 
 	child = sp_text_get_child_by_position (text, position);
 	string = SP_TEXT_CHILD_STRING (child);
 
-	if (!string->p) {
+	if (string->ulen > 0 && string->pgl) {
+		pglyph = &string->pgl->glyphs[position - string->start];
+		x = pglyph->x;
+		y = pglyph->y;
+	} else {
 		x = string->ly->x.computed;
 		y = string->ly->y.computed;
-	} else {
-		x = string->p[position - string->start].x;
-		y = string->p[position - string->start].y;
 	}
 
 	if (child->style->writing_mode.computed == SP_CSS_WRITING_MODE_TB) {
@@ -2330,7 +2328,8 @@ sp_text_get_child_by_position (SPText *text, gint pos)
 		} else {
 			string = SP_TSPAN_STRING (child);
 		}
-		if (pos <= (string->start + string->length)) return child;
+
+		if (pos <= (string->start + string->ulen)) return child;
 	}
 
 	return child;
@@ -2347,4 +2346,3 @@ sp_text_update_length (SPSVGLength *length, gdouble em, gdouble ex, gdouble scal
 		length->computed = length->value * scale;
 	}
 }
-
