@@ -2,7 +2,7 @@
 #define __SP_USE_H__
 
 /*
- * SVG <svg> implementation
+ * SVG <use> implementation
  *
  * Authors:
  *   Lauris Kaplinski <lauris@kaplinski.com>
@@ -13,15 +13,16 @@
  * Released under GNU GPL, read the file 'COPYING' for more information
  */
 
+#include "svg/svg-types.h"
 #include "sp-item.h"
 
-BEGIN_GNOME_DECLS
+G_BEGIN_DECLS
 
 #define SP_TYPE_USE            (sp_use_get_type ())
-#define SP_USE(obj)            (GTK_CHECK_CAST ((obj), SP_TYPE_USE, SPUse))
-#define SP_USE_CLASS(klass)    (GTK_CHECK_CLASS_CAST ((klass), SP_TYPE_USE, SPUseClass))
-#define SP_IS_USE(obj)         (GTK_CHECK_TYPE ((obj), SP_TYPE_USE))
-#define SP_IS_USE_CLASS(klass) (GTK_CHECK_CLASS_TYPE ((klass), SP_TYPE_USE))
+#define SP_USE(obj)            (G_TYPE_CHECK_INSTANCE_CAST ((obj), SP_TYPE_USE, SPUse))
+#define SP_USE_CLASS(klass)    (G_TYPE_CHECK_CLASS_CAST ((klass), SP_TYPE_USE, SPUseClass))
+#define SP_IS_USE(obj)         (G_TYPE_CHECK_INSTANCE_TYPE ((obj), SP_TYPE_USE))
+#define SP_IS_USE_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), SP_TYPE_USE))
 
 typedef struct _SPUse SPUse;
 typedef struct _SPUseClass SPUseClass;
@@ -29,7 +30,10 @@ typedef struct _SPUseClass SPUseClass;
 struct _SPUse {
 	SPItem item;
 	SPObject *child;
-	gdouble x, y, width, height;
+	SPSVGLength x;
+	SPSVGLength y;
+	SPSVGLength width;
+	SPSVGLength height;
 	guchar *href;
 };
 
@@ -37,8 +41,8 @@ struct _SPUseClass {
 	SPItemClass parent_class;
 };
 
-GtkType sp_use_get_type (void);
+GType sp_use_get_type (void);
 
-END_GNOME_DECLS
+G_END_DECLS
 
 #endif
