@@ -26,6 +26,8 @@
 #include <gtk/gtkspinbutton.h>
 #include <gtk/gtklabel.h>
 #include "helper/sp-canvas.h"
+#include "xml/repr-private.h"
+#include "verbs.h"
 #include "sp-star.h"
 #include "sodipodi.h"
 #include "document.h"
@@ -92,18 +94,14 @@ sp_star_context_class_init (SPStarContextClass * klass)
 }
 
 static void
-sp_star_context_init (SPStarContext * star_context)
+sp_star_context_init (SPStarContext *star_context)
 {
-	SPEventContext * event_context;
-	
-	event_context = SP_EVENT_CONTEXT (star_context);
-
-	event_context->cursor_shape = cursor_star_xpm;
-	event_context->hot_x = 4;
-	event_context->hot_y = 4;
-
-	star_context->item = NULL;
-
+	SPEventContext *ec;
+	ec = (SPEventContext *) star_context;
+	ec->verb = SP_VERB_CONTEXT_STAR;
+	ec->cursor_shape = cursor_star_xpm;
+	ec->hot_x = 4;
+	ec->hot_y = 4;
 	star_context->magnitude = 5;
 	star_context->proportion = 0.5;
 }
