@@ -56,6 +56,7 @@ static void sp_style_read_iscale24 (SPIScale24 *val, const guchar *str);
 static void sp_style_read_ienum (SPIEnum *val, const guchar *str, const SPStyleEnum *dict, unsigned int inherit);
 static void sp_style_read_istring (SPIString *val, const guchar *str);
 static void sp_style_read_ilength (SPILength *val, const guchar *str);
+static void sp_style_read_icolor (SPIPaint *paint, const guchar *str, SPStyle *style, SPDocument *document);
 static void sp_style_read_ipaint (SPIPaint *paint, const guchar *str, SPStyle *style, SPDocument *document);
 static void sp_style_read_ifontsize (SPIFontSize *val, const guchar *str);
 
@@ -275,7 +276,7 @@ sp_style_read (SPStyle *style, SPObject *object, SPRepr *repr)
 	sp_style_clear (style);
 
 	/* 1. Style itself */
-	val = sp_repr_attr (repr, "style");
+	val = sp_repr_get_attr (repr, "style");
 	if (val != NULL) {
 		sp_style_merge_from_style_string (style, val);
 	}
@@ -284,12 +285,12 @@ sp_style_read (SPStyle *style, SPObject *object, SPRepr *repr)
 	/* CMYK has precedence and can only be presentation attribute */
 	if (!style->fill.set || (style->fill.type == SP_PAINT_TYPE_COLOR)) {
 		/* backwards-compatability */
-		val = sp_repr_attr (repr, "fill-cmyk");
+		val = sp_repr_get_attr (repr, "fill-cmyk");
 		if (val) {
 			sp_repr_set_attr (repr, "sodipodi:fill-cmyk", val);
 			sp_repr_set_attr (repr, "fill-cmyk", NULL);
 		}
-		val = sp_repr_attr (repr, "sodipodi:fill-cmyk");
+		val = sp_repr_get_attr (repr, "sodipodi:fill-cmyk");
 		if (val && sp_style_read_color_cmyk (&style->fill.value.color, val)) {
 			style->fill.set = TRUE;
 			style->fill.inherit = FALSE;
@@ -297,12 +298,12 @@ sp_style_read (SPStyle *style, SPObject *object, SPRepr *repr)
 	}
 	if (!style->stroke.set || (style->stroke.type == SP_PAINT_TYPE_COLOR)) {
 		/* backwards-compatability */
-		val = sp_repr_attr (repr, "stroke-cmyk");
+		val = sp_repr_get_attr (repr, "stroke-cmyk");
 		if (val) {
 			sp_repr_set_attr (repr, "sodipodi:stroke-cmyk", val);
 			sp_repr_set_attr (repr, "stroke-cmyk", NULL);
 		}
-		val = sp_repr_attr (repr, "sodipodi:stroke-cmyk");
+		val = sp_repr_get_attr (repr, "sodipodi:stroke-cmyk");
 		if (val && sp_style_read_color_cmyk (&style->stroke.value.color, val)) {
 			style->stroke.set = TRUE;
 			style->stroke.inherit = FALSE;
@@ -322,21 +323,29 @@ sp_style_read (SPStyle *style, SPObject *object, SPRepr *repr)
 	SPS_READ_PENUM_IF_UNSET (&style->font_stretch, repr, "font-stretch", enum_font_stretch, TRUE);
 	/* opacity */
 	if (!style->opacity.set) {
-		val = sp_repr_attr (repr, "opacity");
+		val = sp_repr_get_attr (repr, "opacity");
 		if (val) {
 			sp_style_read_iscale24 (&style->opacity, val);
 		}
 	}
+	/* color */
+	if (!style->color.set) {
+		val = sp_repr_get_attr (repr, "color");
+		if (val) {
+			/* fixme: Disallow parsing paintservers */
+			sp_style_read_icolor (&style->color, val, style, (object) ? SP_OBJECT_DOCUMENT (object) : NULL);
+		}
+	}
 	/* fill */
 	if (!style->fill.set) {
-		val = sp_repr_attr (repr, "fill");
+		val = sp_repr_get_attr (repr, "fill");
 		if (val) {
 			sp_style_read_ipaint (&style->fill, val, style, (object) ? SP_OBJECT_DOCUMENT (object) : NULL);
 		}
 	}
 	/* fill-opacity */
 	if (!style->fill_opacity.set) {
-		val = sp_repr_attr (repr, "fill-opacity");
+		val = sp_repr_get_attr (repr, "fill-opacity");
 		if (val) {
 			sp_style_read_iscale24 (&style->fill_opacity, val);
 		}
@@ -345,7 +354,7 @@ sp_style_read (SPStyle *style, SPObject *object, SPRepr *repr)
 	SPS_READ_PENUM_IF_UNSET (&style->fill_rule, repr, "fill-rule", enum_fill_rule, TRUE);
 	/* stroke */
 	if (!style->stroke.set) {
-		val = sp_repr_attr (repr, "stroke");
+		val = sp_repr_get_attr (repr, "stroke");
 		if (val) {
 			sp_style_read_ipaint (&style->stroke, val, style, (object) ? SP_OBJECT_DOCUMENT (object) : NULL);
 		}
@@ -355,19 +364,19 @@ sp_style_read (SPStyle *style, SPObject *object, SPRepr *repr)
 	SPS_READ_PENUM_IF_UNSET (&style->stroke_linejoin, repr, "stroke-linejoin", enum_stroke_linejoin, TRUE);
 	/* stroke-opacity */
 	if (!style->stroke_opacity.set) {
-		val = sp_repr_attr (repr, "stroke-opacity");
+		val = sp_repr_get_attr (repr, "stroke-opacity");
 		if (val) {
 			sp_style_read_iscale24 (&style->stroke_opacity, val);
 		}
 	}
 	if (!style->stroke_dasharray_set) {
-		val = sp_repr_attr (repr, "stroke-dasharray");
+		val = sp_repr_get_attr (repr, "stroke-dasharray");
 		sp_style_read_dash (&style->stroke_dash, val);
 		style->stroke_dasharray_set = TRUE;
 	}
 	if (!style->stroke_dashoffset_set) {
 		/* fixme */
-		val = sp_repr_attr (repr, "stroke-dashoffset");
+		val = sp_repr_get_attr (repr, "stroke-dashoffset");
 		if (sp_svg_number_read_d (val, &style->stroke_dash.offset)) {
 			style->stroke_dashoffset_set = TRUE;
 		} else {
@@ -377,7 +386,7 @@ sp_style_read (SPStyle *style, SPObject *object, SPRepr *repr)
 
 	/* font-family */
 	if (!style->text_private || !style->text->font_family.set) {
-		val = sp_repr_attr (repr, "font-family");
+		val = sp_repr_get_attr (repr, "font-family");
 		if (val) {
 			if (!style->text_private) sp_style_privatize_text (style);
 			sp_style_read_istring (&style->text->font_family, val);
@@ -393,11 +402,11 @@ sp_style_read (SPStyle *style, SPObject *object, SPRepr *repr)
 			sp_style_merge_from_parent (style, SP_OBJECT_STYLE (object->parent));
 		}
 	} else {
-		if (sp_repr_parent (repr)) {
+		if (sp_repr_get_parent (repr)) {
 			SPStyle *parent;
 			/* fixme: This is not the prettiest thing (Lauris) */
 			parent = sp_style_new ();
-			sp_style_read (parent, NULL, sp_repr_parent (repr));
+			sp_style_read (parent, NULL, sp_repr_get_parent (repr));
 			sp_style_merge_from_parent (style, parent);
 			sp_style_unref (parent);
 		}
@@ -482,6 +491,10 @@ sp_style_merge_property (SPStyle *style, gint id, const guchar *val)
 	/* Misc */
 	case SP_PROP_CLIP:
 	case SP_PROP_COLOR:
+		if (!style->color.set) {
+			sp_style_read_icolor (&style->color, val, style, (style->object) ? SP_OBJECT_DOCUMENT (style->object) : NULL);
+		}
+		break;
 	case SP_PROP_CURSOR:
 		g_warning ("Unimplemented style property id: %d value: %s", id, val);
 		break;
@@ -621,6 +634,29 @@ sp_style_merge_property (SPStyle *style, gint id, const guchar *val)
 	}
 }
 
+static void
+trim_space (const unsigned char *str, unsigned int length,
+			unsigned int *left, unsigned int *right)
+{
+	const unsigned char *p;
+	unsigned int i;
+	
+	if (left) {
+		for (i = 0, p = str;
+			 p && isspace (*p) && i < length;
+			 i++, p++)
+			;
+		*left = i;
+	}
+	if (right) {
+		for (i = length - 1, p = str + i;
+			 p && isspace (*p) && i >= 0;
+			 i--, p--)
+			;
+		*right = i + 1;
+	}
+}
+
 /*
  * Parses style="fill:red;fill-rule:evenodd;" type string
  */
@@ -632,7 +668,7 @@ sp_style_merge_from_style_string (SPStyle *style, const guchar *p)
 
 	while (*p) {
 		const guchar *s, *e;
-		gint len, idx;
+		gint len, idx, left, right;
 		while (!isalpha (*p)) {
 			if (!*p) return;
 			p += 1;
@@ -652,14 +688,20 @@ sp_style_merge_from_style_string (SPStyle *style, const guchar *p)
 			g_warning ("Zero length style property at: %s", p);
 			return;
 		}
-		memcpy (c, p, len);
-		c[len] = '\0';
+		trim_space (p, len, &left, &right);
+		memcpy (c, &p[left], right);
+		c[right] = '\0';
 		idx = sp_attribute_lookup (c);
 		if (idx > 0) {
 			len = MIN (e - s - 1, 4095);
-			if (len > 0) memcpy (c, s + 1, len);
-			c[len] = '\0';
-			sp_style_merge_property (style, idx, c);
+			if (len > 0) {
+				trim_space (s + 1, len, &left, &right);
+				memcpy (c, s + 1 + left, right);
+				c[right] = '\0';
+				sp_style_merge_property (style, idx, c);
+			} else {
+				g_warning ("Strange style property value at: %s", p);
+			}
 		} else {
 			g_warning ("Unknown style property at: %s", p);
 		}
@@ -748,7 +790,14 @@ sp_style_merge_from_parent (SPStyle *style, SPStyle *parent)
 		style->visibility = parent->visibility;
 		style->visibility_set = TRUE;
 	}
-	if (!style->fill.set || style->fill.inherit) {
+
+	/* Color */
+	if (!style->color.set || style->color.inherit) {
+		/* We can use paint merge here because parent is guaranteed to be color */
+		sp_style_merge_ipaint (style, &style->color, &parent->color);
+	}
+
+	if (!style->fill.set || style->fill.inherit || style->fill.currentcolor) {
 		sp_style_merge_ipaint (style, &style->fill, &parent->fill);
 	}
 	if (!style->fill_opacity.set || style->fill_opacity.inherit) {
@@ -758,7 +807,7 @@ sp_style_merge_from_parent (SPStyle *style, SPStyle *parent)
 		style->fill_rule.computed = parent->fill_rule.computed;
 	}
 	/* Stroke */
-	if (!style->stroke.set || style->stroke.inherit) {
+	if (!style->stroke.set || style->stroke.inherit || style->stroke.currentcolor) {
 		sp_style_merge_ipaint (style, &style->stroke, &parent->stroke);
 	}
 	if (!style->stroke_width.set || style->stroke_width.inherit) {
@@ -848,6 +897,12 @@ static void
 sp_style_merge_ipaint (SPStyle *style, SPIPaint *paint, SPIPaint *parent)
 {
 	sp_style_paint_clear (style, paint, TRUE, FALSE);
+	if ((paint->set && paint->currentcolor) || parent->currentcolor) {
+		paint->currentcolor = TRUE;
+		paint->type = SP_PAINT_TYPE_COLOR;
+		sp_color_copy (&paint->value.color, &style->color.value.color);
+		return;
+	}
 	paint->type = parent->type;
 	switch (paint->type) {
 	case SP_PAINT_TYPE_COLOR:
@@ -893,6 +948,7 @@ sp_style_write_string (SPStyle *style)
 	if (style->opacity.set && style->opacity.value != SP_SCALE24_MAX) {
 		p += sp_style_write_iscale24 (p, c + BMAX - p, "opacity", &style->opacity, NULL, SP_STYLE_FLAG_IFSET);
 	}
+	p += sp_style_write_ipaint (p, c + BMAX - p, "color", &style->color, NULL, SP_STYLE_FLAG_IFSET);
 	p += sp_style_write_ipaint (p, c + BMAX - p, "fill", &style->fill, NULL, SP_STYLE_FLAG_IFSET);
 	p += sp_style_write_iscale24 (p, c + BMAX - p, "fill-opacity", &style->fill_opacity, NULL, SP_STYLE_FLAG_IFSET);
 	p += sp_style_write_ienum (p, c + BMAX - p, "fill-rule", enum_fill_rule, &style->fill_rule, NULL, SP_STYLE_FLAG_IFSET);
@@ -954,6 +1010,7 @@ sp_style_write_difference (SPStyle *from, SPStyle *to)
 	if (from->opacity.set && from->opacity.value != SP_SCALE24_MAX) {
 		p += sp_style_write_iscale24 (p, c + BMAX - p, "opacity", &from->opacity, &to->opacity, SP_STYLE_FLAG_IFDIFF);
 	}
+	p += sp_style_write_ipaint (p, c + BMAX - p, "color", &from->color, &to->color, SP_STYLE_FLAG_IFSET);
 	p += sp_style_write_ipaint (p, c + BMAX - p, "fill", &from->fill, &to->fill, SP_STYLE_FLAG_IFDIFF);
 	p += sp_style_write_iscale24 (p, c + BMAX - p, "fill-opacity", &from->fill_opacity, &to->fill_opacity, SP_STYLE_FLAG_IFDIFF);
 	p += sp_style_write_ienum (p, c + BMAX - p, "fill-rule", enum_fill_rule, &from->fill_rule, &to->fill_rule, SP_STYLE_FLAG_IFDIFF);
@@ -1043,6 +1100,9 @@ sp_style_clear (SPStyle *style)
 	style->display = TRUE;
 	style->visibility = TRUE;
 
+	style->color.type = SP_PAINT_TYPE_COLOR;
+	sp_color_set_rgb_float (&style->color.value.color, 0.0, 0.0, 0.0);
+
 	style->fill.type = SP_PAINT_TYPE_COLOR;
 	sp_color_set_rgb_float (&style->fill.value.color, 0.0, 0.0, 0.0);
 	style->fill_opacity.value = SP_SCALE24_MAX;
@@ -1395,12 +1455,37 @@ sp_style_read_ilength (SPILength *val, const guchar *str)
 	}
 }
 
+static void
+sp_style_read_icolor (SPIPaint *paint, const guchar *str, SPStyle *style, SPDocument *document)
+{
+	if (!strcmp (str, "inherit")) {
+		paint->set = TRUE;
+		paint->inherit = TRUE;
+		paint->currentcolor = FALSE;
+	} else {
+		guint32 color;
+
+		paint->type = SP_PAINT_TYPE_COLOR;
+		color = sp_color_get_rgba32_ualpha (&paint->value.color, 0);
+		color = sp_svg_read_color (str, color);
+		sp_color_set_rgb_rgba32 (&paint->value.color, color);
+		paint->set = TRUE;
+		paint->inherit = FALSE;
+		paint->currentcolor = FALSE;
+	}
+}
+
 static void
 sp_style_read_ipaint (SPIPaint *paint, const guchar *str, SPStyle *style, SPDocument *document)
 {
 	if (!strcmp (str, "inherit")) {
 		paint->set = TRUE;
 		paint->inherit = TRUE;
+		paint->currentcolor = FALSE;
+	} else if (!strcmp (str, "currentColor")) {
+		paint->set = TRUE;
+		paint->inherit = FALSE;
+		paint->currentcolor = TRUE;
 	} else {
 		guint32 color;
 		if (!strncmp (str, "url", 3)) {
@@ -1420,6 +1505,7 @@ sp_style_read_ipaint (SPIPaint *paint, const guchar *str, SPStyle *style, SPDocu
 						  G_CALLBACK (sp_style_paint_server_modified), style);
 				paint->set = TRUE;
 				paint->inherit = FALSE;
+				paint->currentcolor = FALSE;
 				return;
 			} else {
 				paint->set = FALSE;
@@ -1429,6 +1515,7 @@ sp_style_read_ipaint (SPIPaint *paint, const guchar *str, SPStyle *style, SPDocu
 			paint->type = SP_PAINT_TYPE_NONE;
 			paint->set = TRUE;
 			paint->inherit = FALSE;
+			paint->currentcolor = FALSE;
 			return;
 		}
 
@@ -1438,6 +1525,7 @@ sp_style_read_ipaint (SPIPaint *paint, const guchar *str, SPStyle *style, SPDocu
 		sp_color_set_rgb_rgba32 (&paint->value.color, color);
 		paint->set = TRUE;
 		paint->inherit = FALSE;
+		paint->currentcolor = FALSE;
 	}
 }
 
@@ -1511,7 +1599,7 @@ static void
 sp_style_read_pfloat (SPIFloat *val, SPRepr *repr, const guchar *key)
 {
 	const guchar *str;
-	str = sp_repr_attr (repr, key);
+	str = sp_repr_get_attr (repr, key);
 	if (str) {
 		sp_style_read_ifloat (val, str);
 	}
@@ -1522,7 +1610,7 @@ static void
 sp_style_read_penum (SPIEnum *val, SPRepr *repr, const guchar *key, const SPStyleEnum *dict, unsigned int inherit)
 {
 	const guchar *str;
-	str = sp_repr_attr (repr, key);
+	str = sp_repr_get_attr (repr, key);
 	if (str) {
 		sp_style_read_ienum (val, str, dict, inherit);
 	}
@@ -1532,7 +1620,7 @@ static void
 sp_style_read_plength (SPILength *val, SPRepr *repr, const guchar *key)
 {
 	const guchar *str;
-	str = sp_repr_attr (repr, key);
+	str = sp_repr_get_attr (repr, key);
 	if (str) {
 		sp_style_read_ilength (val, str);
 	}
@@ -1542,7 +1630,7 @@ static void
 sp_style_read_pfontsize (SPIFontSize *val, SPRepr *repr, const guchar *key)
 {
 	const guchar *str;
-	str = sp_repr_attr (repr, key);
+	str = sp_repr_get_attr (repr, key);
 	if (str) {
 		sp_style_read_ifontsize (val, str);
 	}
@@ -1706,6 +1794,8 @@ sp_style_write_ipaint (guchar *b, gint len, const guchar *key, SPIPaint *paint,
 	    ((flags & SP_STYLE_FLAG_IFDIFF) && sp_paint_differ (paint, base))) {
 		if (paint->inherit) {
 			return g_snprintf (b, len, "%s:inherit;", key);
+		} else if (paint->currentcolor) {
+			return g_snprintf (b, len, "%s:currentColor;", key);
 		} else {
 			switch (paint->type) {
 			case SP_PAINT_TYPE_COLOR:
