@@ -110,6 +110,7 @@ enum {
 struct _SPIPaint {
 	unsigned int set : 1;
 	unsigned int inherit : 1;
+	unsigned int currentcolor : 1;
 	unsigned int type : 2;
 	union {
 		SPColor color;
@@ -175,6 +176,9 @@ struct _SPStyle {
 	/* visibility */
 	unsigned int visibility : 1;
 
+	/* color */
+	SPIPaint color;
+
 	/* fill */
 	SPIPaint fill;
 	/* fill-opacity */
