@@ -48,11 +48,6 @@
 #include "sodipodi-private.h"
 #include "document.h"
 #include "sodipodi.h"
-#if 0
-#include "event-broker.h"
-#include "zoom-context.h"
-#endif
-#include "extension.h"
 #include "sp-item-transform.h"
 #include "desktop-handles.h"
 #include "interface.h"
@@ -66,11 +61,6 @@ static GtkWidget *sp_toolbox_edit_create (void);
 static GtkWidget *sp_toolbox_draw_create (void);
 static GtkWidget *sp_toolbox_object_create (void);
 static GtkWidget *sp_toolbox_selection_create (void);
-#ifndef WITH_MODULES
-#ifdef WITH_EXTENSIONS_TOOLBOX
-static GtkWidget *sp_toolbox_extension_create (void);
-#endif
-#endif
 static GtkWidget *sp_toolbox_zoom_create (void);
 static GtkWidget *sp_toolbox_node_create (void);
 
@@ -83,13 +73,6 @@ static void sp_maintoolbox_drag_data_received (GtkWidget * widget,
 					       guint event_time,
 					       gpointer user_data);
 
-/* fixme: Move this to some sensible place (Lauris) */
-static void sp_update_draw_toolbox (Sodipodi * sodipodi, SPEventContext * eventcontext, GObject *toolbox);
-
-#if 0
-static void sp_toolbox_object_flip_clicked (SPButton *button, gpointer data);
-#endif
-
 /* Drag and Drop */
 typedef enum {
 	URI_LIST
@@ -105,11 +88,12 @@ static guint ntoolbox_drop_target_entries = ENTRIES_SIZE(toolbox_drop_target_ent
 static void sp_maintoolbox_open_files(gchar * buffer);
 static void sp_maintoolbox_open_one_file_with_check(gpointer filename, gpointer unused);
 
+static GSList *toolboxes = NULL;
+
 static void
-sp_maintoolbox_destroy (GtkObject *object, gpointer data)
+sp_maintoolbox_window_destroy (void)
 {
-	sp_signal_disconnect_by_data (sodipodi, object);
-	sodipodi_unref ();
+	toolboxes = g_slist_remove (toolboxes, toolboxes->data);
 }
 
 void
@@ -118,15 +102,27 @@ sp_maintoolbox_create_toplevel (void)
 	GtkWidget *window, *toolbox;
 
 	/* Create window */
-	window = sp_window_new (_("Sodipodi"), FALSE);
+	window = sp_window_new (_("Sodipodi"), FALSE, FALSE);
+	g_signal_connect (G_OBJECT (window), "destroy", G_CALLBACK (sp_maintoolbox_window_destroy), NULL);
 
 	toolbox = sp_maintoolbox_new ();
 	gtk_widget_show (toolbox);
 	gtk_container_add (GTK_CONTAINER (window), toolbox);
 
 	gtk_widget_show (window);
+
+	/* Reference our sodipodi engine */
+	toolboxes = g_slist_prepend (toolboxes, window);
 };
 
+static void
+sp_maintoolbox_destroy (GtkObject *object, gpointer data)
+{
+	/* Disconnect tool tracking signals */
+	sp_signal_disconnect_by_data (sodipodi, object);
+	sodipodi_unref ();
+}
+
 GtkWidget *
 sp_maintoolbox_new (void)
 {
@@ -179,16 +175,6 @@ sp_maintoolbox_new (void)
 		}
 	}
 
-	g_signal_connect (G_OBJECT (SODIPODI), "set_eventcontext", G_CALLBACK (sp_update_draw_toolbox), toolbox);
-
-#ifndef WITH_MODULES
-#ifdef WITH_EXTENSIONS_TOOLBOX
-	/* Extension */
-	t = sp_toolbox_extension_create ();
-	gtk_widget_show (t);
-	gtk_box_pack_start (GTK_BOX (vbox), t, FALSE, FALSE, 0);
-#endif
-#endif
 	/* Zoom */
 	t = sp_toolbox_zoom_create ();
 	gtk_widget_show (t);
@@ -209,76 +195,20 @@ sp_maintoolbox_new (void)
 	/* g_signal_connect (G_OBJECT (toolbox), "delete_event", G_CALLBACK (sp_maintoolbox_delete_event), NULL); */
 	g_signal_connect (G_OBJECT (toolbox), "drag_data_received", G_CALLBACK (sp_maintoolbox_drag_data_received), NULL);
 
-	/* Reference our sodipodi engine */
+	/* Keep sodipodi alive */
 	sodipodi_ref ();
 
 	return toolbox;
 }
 
-enum {
-	SP_TOOLBOX_DRAW_RECT,
-	SP_TOOLBOX_DRAW_ARC,
-	SP_TOOLBOX_DRAW_STAR,
-	SP_TOOLBOX_DRAW_SPIRAL
-};
-
-enum {
-	SP_TOOLBOX_DRAW_FREEHAND,
-	SP_TOOLBOX_DRAW_PEN,
-	SP_TOOLBOX_DRAW_DYNAHAND
-};
-
-#if 0
-static void
-sp_toolbox_draw_set_object (SPButton *button, gpointer data)
+void
+sp_maintoolbox_close_all (void)
 {
-	unsigned int mode;
-
-	mode = sp_button_get_option (button);
-
-	switch (mode) {
-	case SP_TOOLBOX_DRAW_RECT:
-		sp_event_context_set_rect (NULL);
-		break;
-	case SP_TOOLBOX_DRAW_ARC:
-		sp_event_context_set_arc (NULL);
-		break;
-	case SP_TOOLBOX_DRAW_STAR:
-		sp_event_context_set_star (NULL);
-		break;
-	case SP_TOOLBOX_DRAW_SPIRAL:
-		sp_event_context_set_spiral (NULL);
-		break;
-	default:
-		g_warning ("Illegal draw code %d", mode);
-		break;
+	while (toolboxes) {
+		gtk_widget_destroy ((GtkWidget *) toolboxes->data);
 	}
 }
 
-static void
-sp_toolbox_draw_set_freehand (SPButton *button, gpointer data)
-{
-	unsigned int mode;
-
-	mode = sp_button_get_option (button);
-
-	switch (mode) {
-	case SP_TOOLBOX_DRAW_FREEHAND:
-		sp_event_context_set_freehand (NULL);
-		break;
-	case SP_TOOLBOX_DRAW_PEN:
-		sp_event_context_set_pen (NULL);
-		break;
-	case SP_TOOLBOX_DRAW_DYNAHAND:
-		sp_event_context_set_dynahand (NULL);
-		break;
-	default:
-		g_warning ("Illegal draw code %d", mode);
-		break;
-	}
-}
-#endif
-
 static GtkWidget *
 sp_toolbox_button_new (GtkWidget *t, int pos, const unsigned char *pxname, GtkSignalFunc handler, GtkTooltips *tt, unsigned char *tip)
 {
@@ -295,14 +225,6 @@ sp_toolbox_button_new (GtkWidget *t, int pos, const unsigned char *pxname, GtkSi
 	return b;
 }
 
-#if 0
-static void
-sp_toolbox_button_clicked (void *object, SPAction *action)
-{
-	sp_action_perform (action);
-}
-#endif
-
 static GtkWidget *
 sp_toolbox_button_new_from_verb (GtkWidget *t, int pos, unsigned int type, unsigned int verb, GtkTooltips *tt)
 {
@@ -322,19 +244,6 @@ sp_toolbox_button_new_from_verb (GtkWidget *t, int pos, unsigned int type, unsig
 	return b;
 }
 
-#if 0
-static GtkWidget *
-sp_toolbox_toggle_button_new (const unsigned char *pxname, GtkTooltips *tt, const unsigned char *tip)
-{
-	GtkWidget *b;
-
-	b = sp_button_new_from_data (24, SP_BUTTON_TYPE_TOGGLE, pxname, tip, tt);
-	gtk_widget_show (b);
-
-	return b;
-}
-#endif
-
 static GtkWidget *
 sp_toolbox_file_create (void)
 {
@@ -385,13 +294,16 @@ sp_toolbox_file_create (void)
 
 	repr = sodipodi_get_repr (SODIPODI, "toolboxes.file");
 	if (repr) {
-		gint state;
-		state = sp_repr_get_int_attribute (repr, "state", 0);
+		int state;
+		state = 0;
+		sp_repr_get_int (repr, "state", &state);
 		sp_toolbox_set_state (SP_TOOLBOX (tb), state);
 	}
 
 	gtk_signal_connect (GTK_OBJECT (tb), "set_state", GTK_SIGNAL_FUNC (sp_toolbox_set_state_handler), repr);
 
+        gtk_object_sink(GTK_OBJECT(tt));
+
 	return tb;
 }
 
@@ -418,13 +330,16 @@ sp_toolbox_edit_create (void)
 
 	repr = sodipodi_get_repr (SODIPODI, "toolboxes.edit");
 	if (repr) {
-		gint state;
-		state = sp_repr_get_int_attribute (repr, "state", 0);
+		int state;
+		state = 0;
+		sp_repr_get_int (repr, "state", &state);
 		sp_toolbox_set_state (SP_TOOLBOX (tb), state);
 	}
 
 	gtk_signal_connect (GTK_OBJECT (tb), "set_state", GTK_SIGNAL_FUNC (sp_toolbox_set_state_handler), repr);
 
+        gtk_object_sink(GTK_OBJECT(tt));
+
 	return tb;
 }
 
@@ -465,13 +380,16 @@ sp_toolbox_object_create (void)
 
 	repr = sodipodi_get_repr (SODIPODI, "toolboxes.object");
 	if (repr) {
-		gint state;
-		state = sp_repr_get_int_attribute (repr, "state", 0);
+		int state;
+		state = 0;
+		sp_repr_get_int (repr, "state", &state);
 		sp_toolbox_set_state (SP_TOOLBOX (tb), state);
 	}
 
 	gtk_signal_connect (GTK_OBJECT (tb), "set_state", GTK_SIGNAL_FUNC (sp_toolbox_set_state_handler), repr);
 
+        gtk_object_sink(GTK_OBJECT(tt));
+
 	return tb;
 }
 
@@ -499,13 +417,16 @@ sp_toolbox_selection_create (void)
 
 	repr = sodipodi_get_repr (SODIPODI, "toolboxes.selection");
 	if (repr) {
-		gint state;
-		state = sp_repr_get_int_attribute (repr, "state", 0);
+		int state;
+		state = 0;
+		sp_repr_get_int (repr, "state", &state);
 		sp_toolbox_set_state (SP_TOOLBOX (tb), state);
 	}
 
 	gtk_signal_connect (GTK_OBJECT (tb), "set_state", GTK_SIGNAL_FUNC (sp_toolbox_set_state_handler), repr);
 
+        gtk_object_sink(GTK_OBJECT(tt));
+
 	return tb;
 }
 
@@ -543,18 +464,20 @@ sp_toolbox_draw_create (void)
 	
 	repr = sodipodi_get_repr (SODIPODI, "toolboxes.draw");
 	if (repr) {
-		gint state;
-		state = sp_repr_get_int_attribute (repr, "state", 0);
+		int state;
+		state = 0;
+		sp_repr_get_int (repr, "state", &state);
 		sp_toolbox_set_state (SP_TOOLBOX (tb), state);
 	}
 
 	gtk_signal_connect (GTK_OBJECT (tb), "set_state", GTK_SIGNAL_FUNC (sp_toolbox_set_state_handler), repr);
 
+        gtk_object_sink(GTK_OBJECT(tt));
+
 	return tb;
 }
 
-#ifndef WITH_MODULES
-#ifdef WITH_EXTENSIONS_TOOLBOX
+#if 0
 
 static GtkWidget *
 sp_toolbox_extension_create (void)
@@ -595,17 +518,19 @@ sp_toolbox_extension_create (void)
 	repr = sodipodi_get_repr (SODIPODI, "toolboxes.extension");
 
 	if (repr) {
-		gint state;
-		state = sp_repr_get_int_attribute (repr, "state", 0);
+		int state;
+		state = 0;
+		sp_repr_get_int (repr, "state", &state);
 		sp_toolbox_set_state (SP_TOOLBOX (tb), state);
 
 		gtk_signal_connect (GTK_OBJECT (tb), "set_state", GTK_SIGNAL_FUNC (sp_toolbox_set_state_handler), repr);
 	}
 
+        gtk_object_sink(GTK_OBJECT(tt));
+
 	return tb;
 }
 #endif
-#endif /* !WITH_MODULES */
 
 static GtkWidget *
 sp_toolbox_zoom_create (void)
@@ -631,13 +556,16 @@ sp_toolbox_zoom_create (void)
 
 	repr = sodipodi_get_repr (SODIPODI, "toolboxes.zoom");
 	if (repr) {
-		gint state;
-		state = sp_repr_get_int_attribute (repr, "state", 0);
+		int state;
+		state = 0;
+		sp_repr_get_int (repr, "state", &state);
 		sp_toolbox_set_state (SP_TOOLBOX (tb), state);
 	}
 
 	gtk_signal_connect (GTK_OBJECT (tb), "set_state", GTK_SIGNAL_FUNC (sp_toolbox_set_state_handler), repr);
 
+        gtk_object_sink(GTK_OBJECT(tt));
+
 	return tb;
 }
 
@@ -676,24 +604,27 @@ sp_toolbox_node_create (void)
 
 	repr = sodipodi_get_repr (SODIPODI, "toolboxes.node");
 	if (repr) {
-		gint state;
-		state = sp_repr_get_int_attribute (repr, "state", 0);
+		int state;
+		state = 0;
+		sp_repr_get_int (repr, "state", &state);
 		sp_toolbox_set_state (SP_TOOLBOX (tb), state);
 	}
 
 	gtk_signal_connect (GTK_OBJECT (tb), "set_state", GTK_SIGNAL_FUNC (sp_toolbox_set_state_handler), repr);
 
+        gtk_object_sink(GTK_OBJECT(tt));
+
 	return tb;
 }
 
 static gint
 sp_toolbox_set_state_handler (SPToolBox * t, guint state, gpointer data)
 {
-	SPRepr * repr;
+	SPRepr *repr;
 
 	repr = (SPRepr *) data;
 
-	sp_repr_set_int_attribute (repr, "state", state);
+	sp_repr_set_int (repr, "state", state);
 
 	return FALSE;
 }
@@ -742,37 +673,3 @@ sp_maintoolbox_open_one_file_with_check (gpointer filename, gpointer unused)
 		}
 	}
 }
-
-static void
-sp_toolbox_verb_activate (unsigned int verb, const unsigned char *tname, const unsigned char *name)
-{
-	SPAction *action;
-	action = sp_verb_get_action (verb);
-	sp_action_set_active (action, tname && !strcmp (tname, name));
-}
-
-static void 
-sp_update_draw_toolbox (Sodipodi *sodipodi, SPEventContext *eventcontext, GObject *toolbox)
-{
-	const unsigned char *tname;
-
-	if (eventcontext != NULL) {
-		tname = gtk_type_name (GTK_OBJECT_TYPE (eventcontext));
-	} else {
-		tname = NULL;
-	}
-
-	sp_toolbox_verb_activate (SP_VERB_CONTEXT_SELECT, tname, "SPSelectContext");
-	sp_toolbox_verb_activate (SP_VERB_CONTEXT_NODE, tname, "SPNodeContext");
-	sp_toolbox_verb_activate (SP_VERB_CONTEXT_RECT, tname, "SPRectContext");
-	sp_toolbox_verb_activate (SP_VERB_CONTEXT_ARC, tname, "SPArcContext");
-	sp_toolbox_verb_activate (SP_VERB_CONTEXT_STAR, tname, "SPStarContext");
-	sp_toolbox_verb_activate (SP_VERB_CONTEXT_SPIRAL, tname, "SPSpiralContext");
-	sp_toolbox_verb_activate (SP_VERB_CONTEXT_PENCIL, tname, "SPPencilContext");
-	sp_toolbox_verb_activate (SP_VERB_CONTEXT_PEN, tname, "SPPenContext");
-	sp_toolbox_verb_activate (SP_VERB_CONTEXT_CALLIGRAPHIC, tname, "SPDynaDrawContext");
-	sp_toolbox_verb_activate (SP_VERB_CONTEXT_TEXT, tname, "SPTextContext");
-	sp_toolbox_verb_activate (SP_VERB_CONTEXT_ZOOM, tname, "SPZoomContext");
-	sp_toolbox_verb_activate (SP_VERB_CONTEXT_DROPPER, tname, "SPDropperContext");
-}
-
