@@ -19,6 +19,7 @@
 #include <libnr/nr-pixblock.h>
 #include <libart_lgpl/art_affine.h>
 #include <gtk/gtksignal.h>
+#include "../helper/sp-intl.h"
 #include "../helper/nr-plain-stuff.h"
 #include "../helper/nr-plain-stuff-gdk.h"
 #include "../macros.h"
@@ -42,14 +43,13 @@ static void sp_gradient_position_realize (GtkWidget *widget);
 static void sp_gradient_position_unrealize (GtkWidget *widget);
 static void sp_gradient_position_size_request (GtkWidget *widget, GtkRequisition *requisition);
 static void sp_gradient_position_size_allocate (GtkWidget *widget, GtkAllocation *allocation);
-static void sp_gradient_position_draw (GtkWidget *widget, GdkRectangle *area);
 static gint sp_gradient_position_expose (GtkWidget *widget, GdkEventExpose *event);
 
 static gint sp_gradient_position_button_press (GtkWidget *widget, GdkEventButton *event);
 static gint sp_gradient_position_button_release (GtkWidget *widget, GdkEventButton *event);
 static gint sp_gradient_position_motion_notify (GtkWidget *widget, GdkEventMotion *event);
 
-static void sp_gradient_position_gradient_destroy (SPGradient *gr, SPGradientPosition *im);
+static void sp_gradient_position_gradient_release (SPGradient *gr, SPGradientPosition *im);
 static void sp_gradient_position_gradient_modified (SPGradient *gr, guint flags, SPGradientPosition *im);
 static void sp_gradient_position_update (SPGradientPosition *img);
 
@@ -94,29 +94,28 @@ sp_gradient_position_class_init (SPGradientPositionClass *klass)
 
 	position_signals[GRABBED] = gtk_signal_new ("grabbed",
 						    GTK_RUN_FIRST | GTK_RUN_NO_RECURSE,
-						    object_class->type,
+						    GTK_CLASS_TYPE(object_class),
 						    GTK_SIGNAL_OFFSET (SPGradientPositionClass, grabbed),
 						    gtk_marshal_NONE__NONE,
 						    GTK_TYPE_NONE, 0);
 	position_signals[DRAGGED] = gtk_signal_new ("dragged",
 						    GTK_RUN_FIRST | GTK_RUN_NO_RECURSE,
-						    object_class->type,
+						    GTK_CLASS_TYPE(object_class),
 						    GTK_SIGNAL_OFFSET (SPGradientPositionClass, dragged),
 						    gtk_marshal_NONE__NONE,
 						    GTK_TYPE_NONE, 0);
 	position_signals[RELEASED] = gtk_signal_new ("released",
 						     GTK_RUN_FIRST | GTK_RUN_NO_RECURSE,
-						     object_class->type,
+						     GTK_CLASS_TYPE(object_class),
 						     GTK_SIGNAL_OFFSET (SPGradientPositionClass, released),
 						     gtk_marshal_NONE__NONE,
 						     GTK_TYPE_NONE, 0);
 	position_signals[CHANGED] = gtk_signal_new ("changed",
 						    GTK_RUN_FIRST | GTK_RUN_NO_RECURSE,
-						    object_class->type,
+						    GTK_CLASS_TYPE(object_class),
 						    GTK_SIGNAL_OFFSET (SPGradientPositionClass, changed),
 						    gtk_marshal_NONE__NONE,
 						    GTK_TYPE_NONE, 0);
-	gtk_object_class_add_signals (object_class, position_signals, LAST_SIGNAL);
 
 	object_class->destroy = sp_gradient_position_destroy;
 
@@ -124,7 +123,7 @@ sp_gradient_position_class_init (SPGradientPositionClass *klass)
 	widget_class->unrealize = sp_gradient_position_unrealize;
 	widget_class->size_request = sp_gradient_position_size_request;
 	widget_class->size_allocate = sp_gradient_position_size_allocate;
-	widget_class->draw = sp_gradient_position_draw;
+/*  	widget_class->draw = sp_gradient_position_draw; */
 	widget_class->expose_event = sp_gradient_position_expose;
 	widget_class->button_press_event = sp_gradient_position_button_press;
 	widget_class->button_release_event = sp_gradient_position_button_release;
@@ -162,7 +161,7 @@ sp_gradient_position_destroy (GtkObject *object)
 	sp_gradient_position_free_image_data (pos);
 
 	if (pos->gradient) {
-		gtk_signal_disconnect_by_data (GTK_OBJECT (pos->gradient), pos);
+		sp_signal_disconnect_by_data (pos->gradient, pos);
 		pos->gradient = NULL;
 	}
 
@@ -248,6 +247,7 @@ sp_gradient_position_size_allocate (GtkWidget *widget, GtkAllocation *allocation
 	}
 }
 
+#if 0
 static void
 sp_gradient_position_draw (GtkWidget *widget, GdkRectangle *area)
 {
@@ -259,6 +259,7 @@ sp_gradient_position_draw (GtkWidget *widget, GdkRectangle *area)
 		sp_gradient_position_paint (widget, area);
 	}
 }
+#endif
 
 static gint
 sp_gradient_position_expose (GtkWidget *widget, GdkEventExpose *event)
@@ -289,24 +290,28 @@ sp_gradient_position_button_press (GtkWidget *widget, GdkEventButton *event)
 			if (event->state & GDK_CONTROL_MASK) {
 				pos->dragging = TRUE;
 				pos->changed = FALSE;
-				gdk_pointer_grab (widget->window, FALSE, GDK_POINTER_MOTION_MASK | GDK_BUTTON_RELEASE_MASK, NULL, NULL, event->time);
+				gdk_pointer_grab (widget->window, FALSE,
+						  GDK_POINTER_MOTION_MASK | GDK_BUTTON_RELEASE_MASK,
+						  NULL, NULL, event->time);
 			} else {
 				float x1, y1;
 				pos->dragging = TRUE;
 				pos->changed = FALSE;
-				gtk_signal_emit (GTK_OBJECT (pos), position_signals[GRABBED]);
+				g_signal_emit (G_OBJECT (pos), position_signals[GRABBED], 0);
 				x1 = NR_MATRIX_DF_TRANSFORM_X (&pos->w2gs, event->x, event->y);
 				y1 = NR_MATRIX_DF_TRANSFORM_Y (&pos->w2gs, event->x, event->y);
 				if (!NR_DF_TEST_CLOSE (x1, pos->gdata.linear.x1, NR_EPSILON_F) ||
 				    !NR_DF_TEST_CLOSE (y1, pos->gdata.linear.y1, NR_EPSILON_F)) {
 					pos->gdata.linear.x1 = x1;
 					pos->gdata.linear.y1 = y1;
-					gtk_signal_emit (GTK_OBJECT (pos), position_signals[DRAGGED]);
+					g_signal_emit (G_OBJECT (pos), position_signals[DRAGGED], 0);
 					pos->changed = TRUE;
 					pos->need_update = TRUE;
 					if (GTK_WIDGET_DRAWABLE (pos)) gtk_widget_queue_draw (GTK_WIDGET (pos));
 				}
-				gdk_pointer_grab (widget->window, FALSE, GDK_POINTER_MOTION_MASK | GDK_BUTTON_RELEASE_MASK, NULL, NULL, event->time);
+				gdk_pointer_grab (widget->window, FALSE,
+						  GDK_POINTER_MOTION_MASK | GDK_BUTTON_RELEASE_MASK,
+						  NULL, NULL, event->time);
 			}
 		}
 	} else {
@@ -315,29 +320,33 @@ sp_gradient_position_button_press (GtkWidget *widget, GdkEventButton *event)
 			if (event->state & GDK_CONTROL_MASK) {
 				pos->dragging = TRUE;
 				pos->changed = FALSE;
-				gdk_pointer_grab (widget->window, FALSE, GDK_POINTER_MOTION_MASK | GDK_BUTTON_RELEASE_MASK, NULL, NULL, event->time);
+				gdk_pointer_grab (widget->window, FALSE,
+						  GDK_POINTER_MOTION_MASK | GDK_BUTTON_RELEASE_MASK,
+						  NULL, NULL, event->time);
 			} else if (event->state & GDK_SHIFT_MASK) {
 				float fx, fy;
 				pos->dragging = TRUE;
 				pos->changed = FALSE;
-				gtk_signal_emit (GTK_OBJECT (pos), position_signals[GRABBED]);
+				g_signal_emit (G_OBJECT (pos), position_signals[GRABBED], 0);
 				fx = NR_MATRIX_DF_TRANSFORM_X (&pos->w2gs, event->x, event->y);
 				fy = NR_MATRIX_DF_TRANSFORM_Y (&pos->w2gs, event->x, event->y);
 				if (!NR_DF_TEST_CLOSE (fx, pos->gdata.radial.fx, NR_EPSILON_F) ||
 				    !NR_DF_TEST_CLOSE (fy, pos->gdata.radial.fy, NR_EPSILON_F)) {
 					pos->gdata.radial.fx = fx;
 					pos->gdata.radial.fy = fy;
-					gtk_signal_emit (GTK_OBJECT (pos), position_signals[DRAGGED]);
+					g_signal_emit (G_OBJECT (pos), position_signals[DRAGGED], 0);
 					pos->changed = TRUE;
 					pos->need_update = TRUE;
 					if (GTK_WIDGET_DRAWABLE (pos)) gtk_widget_queue_draw (GTK_WIDGET (pos));
 				}
-				gdk_pointer_grab (widget->window, FALSE, GDK_POINTER_MOTION_MASK | GDK_BUTTON_RELEASE_MASK, NULL, NULL, event->time);
+				gdk_pointer_grab (widget->window, FALSE,
+						  GDK_POINTER_MOTION_MASK | GDK_BUTTON_RELEASE_MASK,
+						  NULL, NULL, event->time);
 			} else {
 				float cx, cy;
 				pos->dragging = TRUE;
 				pos->changed = FALSE;
-				gtk_signal_emit (GTK_OBJECT (pos), position_signals[GRABBED]);
+				g_signal_emit (G_OBJECT (pos), position_signals[GRABBED], 0);
 				cx = NR_MATRIX_DF_TRANSFORM_X (&pos->w2gs, event->x, event->y);
 				cy = NR_MATRIX_DF_TRANSFORM_Y (&pos->w2gs, event->x, event->y);
 				if (!NR_DF_TEST_CLOSE (cx, pos->gdata.radial.cx, NR_EPSILON_F) ||
@@ -346,12 +355,14 @@ sp_gradient_position_button_press (GtkWidget *widget, GdkEventButton *event)
 					pos->gdata.radial.cy = cy;
 					pos->gdata.radial.fx = cx;
 					pos->gdata.radial.fy = cy;
-					gtk_signal_emit (GTK_OBJECT (pos), position_signals[DRAGGED]);
+					g_signal_emit (G_OBJECT (pos), position_signals[DRAGGED], 0);
 					pos->changed = TRUE;
 					pos->need_update = TRUE;
 					if (GTK_WIDGET_DRAWABLE (pos)) gtk_widget_queue_draw (GTK_WIDGET (pos));
 				}
-				gdk_pointer_grab (widget->window, FALSE, GDK_POINTER_MOTION_MASK | GDK_BUTTON_RELEASE_MASK, NULL, NULL, event->time);
+				gdk_pointer_grab (widget->window, FALSE,
+						  GDK_POINTER_MOTION_MASK | GDK_BUTTON_RELEASE_MASK,
+						  NULL, NULL, event->time);
 			}
 		}
 	}
@@ -367,9 +378,9 @@ sp_gradient_position_button_release (GtkWidget *widget, GdkEventButton *event)
 	pos = SP_GRADIENT_POSITION (widget);
 
 	if ((event->button == 1) && pos->dragging) {
-		gtk_signal_emit (GTK_OBJECT (pos), position_signals[RELEASED]);
+		g_signal_emit (G_OBJECT (pos), position_signals[RELEASED], 0);
 		if (pos->changed) {
-			gtk_signal_emit (GTK_OBJECT (pos), position_signals[CHANGED]);
+			g_signal_emit (G_OBJECT (pos), position_signals[CHANGED], 0);
 			pos->changed = FALSE;
 		}
 		gdk_pointer_ungrab (event->time);
@@ -437,7 +448,7 @@ sp_gradient_position_motion_notify (GtkWidget *widget, GdkEventMotion *event)
 				pos->gdata.linear.x2 = NR_MATRIX_DF_TRANSFORM_X (&pos->w2gs, event->x, event->y);
 				pos->gdata.linear.y2 = NR_MATRIX_DF_TRANSFORM_Y (&pos->w2gs, event->x, event->y);
 			}
-			gtk_signal_emit (GTK_OBJECT (pos), position_signals[DRAGGED]);
+			g_signal_emit (G_OBJECT (pos), position_signals[DRAGGED], 0);
 			pos->changed = TRUE;
 			pos->need_update = TRUE;
 			if (GTK_WIDGET_DRAWABLE (pos)) gtk_widget_queue_draw (GTK_WIDGET (pos));
@@ -495,7 +506,7 @@ sp_gradient_position_motion_notify (GtkWidget *widget, GdkEventMotion *event)
 
 				pos->gdata.radial.r = hypot (x - cx, y - cy);
 			}
-			gtk_signal_emit (GTK_OBJECT (pos), position_signals[DRAGGED]);
+			g_signal_emit (G_OBJECT (pos), position_signals[DRAGGED], 0);
 			pos->changed = TRUE;
 			pos->need_update = TRUE;
 			if (GTK_WIDGET_DRAWABLE (pos)) gtk_widget_queue_draw (GTK_WIDGET (pos));
@@ -521,16 +532,14 @@ void
 sp_gradient_position_set_gradient (SPGradientPosition *pos, SPGradient *gradient)
 {
 	if (pos->gradient) {
-		gtk_signal_disconnect_by_data (GTK_OBJECT (pos->gradient), pos);
+		sp_signal_disconnect_by_data (pos->gradient, pos);
 	}
 
 	pos->gradient = gradient;
 
 	if (gradient) {
-		gtk_signal_connect (GTK_OBJECT (gradient), "destroy",
-				    GTK_SIGNAL_FUNC (sp_gradient_position_gradient_destroy), pos);
-		gtk_signal_connect (GTK_OBJECT (gradient), "modified",
-				    GTK_SIGNAL_FUNC (sp_gradient_position_gradient_modified), pos);
+		g_signal_connect (G_OBJECT (gradient), "release", G_CALLBACK (sp_gradient_position_gradient_release), pos);
+		g_signal_connect (G_OBJECT (gradient), "modified", G_CALLBACK (sp_gradient_position_gradient_modified), pos);
 	}
 
 	pos->need_update = TRUE;
@@ -540,7 +549,7 @@ sp_gradient_position_set_gradient (SPGradientPosition *pos, SPGradient *gradient
 }
 
 static void
-sp_gradient_position_gradient_destroy (SPGradient *gradient, SPGradientPosition *pos)
+sp_gradient_position_gradient_release (SPGradient *gradient, SPGradientPosition *pos)
 {
 	sp_gradient_position_set_gradient (pos, NULL);
 }
