@@ -8,12 +8,15 @@
  *
  * Copyright (C) 2001-2002 Lauris Kaplinski
  *
- * Released under GNU GPL, read the file 'COPYING' for more information
+ * This code is in public domain
  */
 
 #define noSPCS_PREVIEW
 
+#ifdef HAVE_CONFIG_H
 #include <config.h>
+#endif
+
 #include <math.h>
 #include <string.h>
 #include <stdlib.h>
@@ -22,9 +25,8 @@
 #include <gtk/gtklabel.h>
 #include <gtk/gtktable.h>
 #include <gtk/gtkspinbutton.h>
-#include <libgnome/gnome-defs.h>
-#include <libgnome/gnome-i18n.h>
 #include "../color.h"
+#include "../helper/sp-intl.h"
 #include "sp-color-preview.h"
 #include "sp-color-selector.h"
 
@@ -55,6 +57,9 @@ static void sp_color_selector_class_init (SPColorSelectorClass *klass);
 static void sp_color_selector_init (SPColorSelector *slider);
 static void sp_color_selector_destroy (GtkObject *object);
 
+static void sp_color_selector_show_all (GtkWidget *widget);
+static void sp_color_selector_hide_all (GtkWidget *widget);
+
 static void sp_color_selector_adjustment_any_changed (GtkAdjustment *adjustment, SPColorSelector *csel);
 static void sp_color_selector_slider_any_grabbed (SPColorSlider *slider, SPColorSelector *csel);
 static void sp_color_selector_slider_any_released (SPColorSlider *slider, SPColorSelector *csel);
@@ -102,31 +107,33 @@ sp_color_selector_class_init (SPColorSelectorClass *klass)
 
 	csel_signals[GRABBED] =  gtk_signal_new ("grabbed",
 						 GTK_RUN_FIRST | GTK_RUN_NO_RECURSE,
-						 object_class->type,
+						 GTK_CLASS_TYPE(object_class),
 						 GTK_SIGNAL_OFFSET (SPColorSelectorClass, grabbed),
 						 gtk_marshal_NONE__NONE,
 						 GTK_TYPE_NONE, 0);
 	csel_signals[DRAGGED] =  gtk_signal_new ("dragged",
 						 GTK_RUN_FIRST | GTK_RUN_NO_RECURSE,
-						 object_class->type,
+						 GTK_CLASS_TYPE(object_class),
 						 GTK_SIGNAL_OFFSET (SPColorSelectorClass, dragged),
 						 gtk_marshal_NONE__NONE,
 						 GTK_TYPE_NONE, 0);
 	csel_signals[RELEASED] = gtk_signal_new ("released",
 						 GTK_RUN_FIRST | GTK_RUN_NO_RECURSE,
-						 object_class->type,
+						 GTK_CLASS_TYPE(object_class),
 						 GTK_SIGNAL_OFFSET (SPColorSelectorClass, released),
 						 gtk_marshal_NONE__NONE,
 						 GTK_TYPE_NONE, 0);
 	csel_signals[CHANGED] =  gtk_signal_new ("changed",
 						 GTK_RUN_FIRST | GTK_RUN_NO_RECURSE,
-						 object_class->type,
+						 GTK_CLASS_TYPE(object_class),
 						 GTK_SIGNAL_OFFSET (SPColorSelectorClass, changed),
 						 gtk_marshal_NONE__NONE,
 						 GTK_TYPE_NONE, 0);
-	gtk_object_class_add_signals (object_class, csel_signals, LAST_SIGNAL);
 
 	object_class->destroy = sp_color_selector_destroy;
+
+	widget_class->show_all = sp_color_selector_show_all;
+	widget_class->hide_all = sp_color_selector_hide_all;
 }
 
 #define XPAD 4
@@ -135,7 +142,7 @@ sp_color_selector_class_init (SPColorSelectorClass *klass)
 static void
 sp_color_selector_init (SPColorSelector *csel)
 {
-	GtkWidget *t, *hb;
+	GtkWidget *t;
 	gint i;
 
 	csel->updating = FALSE;
@@ -182,17 +189,15 @@ sp_color_selector_init (SPColorSelector *csel)
 	gtk_misc_set_alignment (GTK_MISC (csel->l[i]), 1.0, 0.5);
 	gtk_widget_show (csel->rgbal);
 	gtk_table_attach (GTK_TABLE (t), csel->rgbal, 0, 1, i, i + 1, GTK_FILL, GTK_FILL, XPAD, YPAD);
-	hb = gtk_hbox_new (FALSE, XPAD);
-	gtk_widget_show (hb);
-	gtk_table_attach (GTK_TABLE (t), hb, 1, 3, i, i + 1, GTK_FILL, GTK_FILL, XPAD, YPAD);
-	csel->rgbae = gtk_entry_new_with_max_length (8);
-	gtk_object_set (GTK_OBJECT (csel->rgbae), "width", 64, NULL);
+	csel->rgbae = gtk_entry_new ();
+	gtk_entry_set_max_length (GTK_ENTRY (csel->rgbae), 16);
+	gtk_entry_set_width_chars (GTK_ENTRY (csel->rgbae), 10);
 	gtk_widget_show (csel->rgbae);
-	gtk_box_pack_start (GTK_BOX (hb), csel->rgbae, FALSE, FALSE, 0);
+	gtk_table_attach (GTK_TABLE (t), csel->rgbae, 1, 2, i, i + 1, GTK_FILL, GTK_FILL, XPAD, YPAD);
 #ifdef SPCS_PREVIEW
 	csel->p = sp_color_preview_new (0xffffffff);
 	gtk_widget_show (csel->p);
-	gtk_box_pack_start (GTK_BOX (hb), csel->p, TRUE, TRUE, 0);
+	gtk_table_attach (GTK_TABLE (t), csel->p, 2, 3, i, i + 1, GTK_FILL, GTK_FILL, XPAD, YPAD);
 #endif
 
 	gtk_signal_connect (GTK_OBJECT (csel->rgbae), "changed", GTK_SIGNAL_FUNC (sp_color_selector_rgba_entry_changed), csel);
@@ -220,6 +225,18 @@ sp_color_selector_destroy (GtkObject *object)
 		(* ((GtkObjectClass *) (parent_class))->destroy) (object);
 }
 
+static void
+sp_color_selector_show_all (GtkWidget *widget)
+{
+	gtk_widget_show (widget);
+}
+
+static void
+sp_color_selector_hide_all (GtkWidget *widget)
+{
+	gtk_widget_hide (widget);
+}
+
 GtkWidget *
 sp_color_selector_new (void)
 {
@@ -736,7 +753,8 @@ sp_color_selector_adjustment_changed (SPColorSelector *csel, guint channel)
 static void
 sp_color_selector_rgba_entry_changed (GtkEntry *entry, SPColorSelector *csel)
 {
-	gchar *t, *e;
+	const gchar *t;
+	gchar *e;
 	guint rgba;
 
 	if (csel->updating) return;
