@@ -19,8 +19,8 @@
 #include <gtk/gtkadjustment.h>
 
 #include "macros.h"
-#include "xml/repr.h"
-#include "xml/repr-private.h"
+/* #include "xml/repr-private.h" */
+
 #include "sp-xmlview-content.h"
 
 static void sp_xmlview_content_class_init (SPXMLViewContentClass * klass);
@@ -82,7 +82,8 @@ sp_xmlview_content_set_repr (SPXMLViewContent * text, SPRepr * repr)
 	if (repr) {
 		sp_repr_ref (repr);
 		sp_repr_add_listener (repr, &repr_events, text);
-		sp_repr_synthesize_events (repr, &repr_events, text);
+		/* Emulate content_changed event */
+		event_content_changed (repr, NULL, sp_repr_get_content (repr), text);
 	} else {
 		gtk_text_buffer_set_text (gtk_text_view_get_buffer (GTK_TEXT_VIEW (text)), "", 0);
 		gtk_text_view_set_editable (GTK_TEXT_VIEW (text), FALSE);
