@@ -20,8 +20,8 @@
 #include <gtk/gtkclist.h>
 #include <gtk/gtkctree.h>
 #include <gtk/gtkcontainer.h>
-#include "../xml/repr.h"
-#include "../xml/repr-private.h"
+/*#include "xml/repr-private.h" */
+
 #include "sp-xmlview-tree.h"
 
 typedef struct _NodeData {
@@ -189,7 +189,8 @@ add_node (SPXMLViewTree * tree, GtkCTreeNode * parent, GtkCTreeNode * before, SP
 	g_assert (tree != NULL);
 	g_assert (repr != NULL);
 
-	node = gtk_ctree_insert_node (GTK_CTREE (tree), parent, before, (gchar **)default_text, 2, NULL, NULL, NULL, NULL, ( SP_REPR_TYPE (repr) != SP_XML_ELEMENT_NODE ), FALSE);
+	node = gtk_ctree_insert_node (GTK_CTREE (tree), parent, before, (gchar **)default_text, 2,
+						NULL, NULL, NULL, NULL, !sp_repr_is_element (repr), FALSE);
 	g_assert (node != NULL);
 
 	data = node_data_new (tree, node, repr);
@@ -197,24 +198,36 @@ add_node (SPXMLViewTree * tree, GtkCTreeNode * parent, GtkCTreeNode * before, SP
 
 	gtk_ctree_node_set_row_data_full (GTK_CTREE (tree), data->node, data, node_data_free);
 
-	if ( SP_REPR_TYPE (repr) == SP_XML_TEXT_NODE ) {
-		vec = &text_repr_events;
-	} else if ( SP_REPR_TYPE (repr) == SP_XML_ELEMENT_NODE ) {
-		vec = &element_repr_events;
+	if (sp_repr_is_text (repr) || sp_repr_is_cdata (repr)) {
+		gtk_clist_freeze (GTK_CLIST (tree));
+		sp_repr_add_listener (repr, &text_repr_events, data);
+		/* Emulate content_changed event */
+		text_content_changed (repr, NULL, sp_repr_get_content (repr), data);
+		gtk_clist_thaw (GTK_CLIST (tree));
+	} else if (sp_repr_is_element (repr)) {
+		SPRepr *child, *ref;
+		SPReprAttr *attr;
+		gtk_clist_freeze (GTK_CLIST (tree));
+		element_attr_changed (repr, "id", NULL, NULL, data);
+		sp_repr_add_listener (repr, &element_repr_events, data);
+		/* Emulate events */
+		attr = sp_repr_attr_get_first (repr);
+		while (attr) {
+			element_attr_changed (repr, sp_repr_attr_get_key (repr, attr), NULL, sp_repr_attr_get_value (repr, attr), data);
+			attr = sp_repr_attr_get_next (repr, attr);
+		}
+		ref = NULL;
+		child = sp_repr_get_children (repr);
+		while (child) {
+			element_child_added (repr, child, ref, data);
+			ref = child;
+			child = sp_repr_get_next (child);
+		}
+		gtk_clist_thaw (GTK_CLIST (tree));
 	} else {
 		vec = NULL;
 	}
 
-	if (vec) {
-		gtk_clist_freeze (GTK_CLIST (tree));
-		if (SP_REPR_TYPE (repr) == SP_XML_ELEMENT_NODE) {
-			element_attr_changed (repr, "id", NULL, NULL, data);
-		}
-		sp_repr_add_listener (repr, vec, data);
-		sp_repr_synthesize_events (repr, vec, data);
-		gtk_clist_thaw (GTK_CLIST (tree));
-	}
-
 	return node;
 }
 
@@ -256,9 +269,9 @@ element_child_added (SPRepr * repr, SPRepr * child, SPRepr * ref, gpointer ptr)
 }
 
 void
-element_attr_changed (SPRepr * repr, const guchar * key, const guchar * old_value, const guchar * new_value, gpointer ptr)
+element_attr_changed (SPRepr * repr, const guchar * key, const guchar * old_value, const guchar *new_value, gpointer ptr)
 {
-	NodeData * data;
+	NodeData *data;
 	gchar *label;
 
 	data = (NodeData *) ptr;
@@ -268,9 +281,9 @@ element_attr_changed (SPRepr * repr, const guchar * key, const guchar * old_valu
 	if (strcmp (key, "id")) return;
 	
 	if (new_value) {
-		label = g_strdup_printf ("<%s id=\"%s\">", SP_REPR_NAME (repr), new_value);
+		label = g_strdup_printf ("<%s id=\"%s\">", sp_repr_get_name (repr), new_value);
 	} else {
-		label = g_strdup_printf ("<%s>", SP_REPR_NAME (repr));
+		label = g_strdup_printf ("<%s>", sp_repr_get_name (repr));
 	}
 	gtk_ctree_node_set_text (GTK_CTREE (data->tree), data->node, 0, label);
 	g_free (label);
@@ -307,16 +320,33 @@ element_order_changed (SPRepr * repr, SPRepr * child, SPRepr * oldref, SPRepr *
 }
 
 void
-text_content_changed (SPRepr * repr, const guchar * old_content, const guchar * new_content, gpointer ptr)
+text_content_changed (SPRepr *repr, const guchar * old_content, const guchar * new_content, gpointer ptr)
 {
 	NodeData *data;
-	gchar *label;
+	unsigned char c[64];
+	unsigned char *label;
+	int i;
 
 	data = (NodeData *) ptr;
 
 	if (data->tree->blocked) return;
 
-	label = g_strdup_printf ("\"%s\"", new_content);
+	if (!new_content) new_content = "";
+	if (strlen (new_content) < 64) {
+		strcpy (c, new_content);
+	} else {
+		strncpy (c, new_content, 32);
+		strncpy (c + 32, " ... ", 5);
+		strncpy (c + 37, new_content + strlen (new_content) - 26, 26);
+		c[63] = 0;
+	}
+	for (i = 0; c[i]; i++) if (c[i] < ' ') c[i] = ' ';
+
+	if (sp_repr_is_text (repr)) {
+		label = g_strdup_printf ("\"%s\"", c);
+	} else {
+		label = g_strdup_printf ("<![CDATA[%s]]>", c);
+	}
 	gtk_ctree_node_set_text (GTK_CTREE (data->tree), data->node, 0, label);
 	g_free (label);
 }
@@ -396,7 +426,7 @@ check_drag (GtkCTree * tree, GtkCTreeNode * node, GtkCTreeNode * new_parent, Gtk
 	old_parent = GTK_CTREE_ROW (node)->parent;
 
 	if (!old_parent || !new_parent) return FALSE;
-	if (SP_REPR_TYPE (NODE_DATA (new_parent)->repr) != SP_XML_ELEMENT_NODE) return FALSE;
+	if (!sp_repr_is_element (NODE_DATA (new_parent)->repr)) return FALSE;
 
 	/* fixme: we need add_child/remove_child/etc repr events without side-effects, so we can check here and give better visual feedback */
 
