@@ -13,6 +13,8 @@
  * Released under GNU GPL, read the file 'COPYING' for more information
  */
 
+
+
 /* The following are helper routines for making Sodipodi dialog widgets.
    All are prefixed with spw_, short for sodipodi_widget.  This is not to
    be confused with SPWidget, an existing datatype associated with SPRepr/
@@ -20,9 +22,11 @@
 */
 
 void
-spw_checkbutton(GtkWidget * dialog, GtkWidget * t,
-		const guchar * label, guchar * key, int col, int row,
-		int sensitive, GCallback cb);
+spw_checkbutton (GtkWidget *dialog, GtkWidget *table,
+		 const unsigned char *label,
+		 const unsigned char *key,
+		 int col, int row,
+		 int sensitive, GCallback cb);
 
 void
 spw_dropdown(GtkWidget * dialog, GtkWidget * t,
