@@ -44,7 +44,7 @@ static char * sp_wmf_image_name (void * context);
 #endif /* HAVE_LIBWMF */
 
 SPReprDoc *
-sp_repr_read_file (const gchar * filename, const gchar *default_ns)
+sp_repr_doc_new_from_file (const unsigned char *filename, const unsigned char *default_ns)
 {
 	const unsigned char *cdata;
 	int size;
@@ -92,7 +92,7 @@ sp_repr_read_file (const gchar * filename, const gchar *default_ns)
 
 
 SPReprDoc *
-sp_repr_read_mem (const gchar * buffer, gint length, const gchar *default_ns)
+sp_repr_doc_new_from_mem (const unsigned char *buffer, unsigned int length, const unsigned char *default_ns)
 {
 	xmlDocPtr doc;
 	SPReprDoc * rdoc;
@@ -121,7 +121,7 @@ sp_repr_do_read (xmlDocPtr doc, const gchar *default_ns)
 	if (doc == NULL) return NULL;
 	node = xmlDocGetRootElement (doc);
 	if (node == NULL) return NULL;
-	rdoc = sp_repr_document_new ("void");
+	rdoc = sp_repr_doc_new ("void");
 
 
 	prefix_map = g_hash_table_new (g_str_hash, g_str_equal);
@@ -138,11 +138,12 @@ sp_repr_do_read (xmlDocPtr doc, const gchar *default_ns)
 		if (default_ns) {
 			sp_repr_set_attr (repr, "xmlns", default_ns);
 		}
-		g_hash_table_foreach (prefix_map, (GHFunc)sp_repr_set_xmlns_attr, repr);
+		/* Not sure whether collating everything here makes sense (Lauris) */
+		/* g_hash_table_foreach (prefix_map, (GHFunc) sp_repr_set_xmlns_attr, repr); */
 		/* always include Sodipodi namespace */
 		sp_repr_set_xmlns_attr (sp_xml_ns_uri_prefix (SP_SODIPODI_NS_URI, "sodipodi"), SP_SODIPODI_NS_URI, repr);
 
-		if (!strcmp (sp_repr_name (repr), "svg") && default_ns && !strcmp (default_ns, SP_SVG_NS_URI)) {
+		if (!strcmp (sp_repr_get_name (repr), "svg") && default_ns && !strcmp (default_ns, SP_SVG_NS_URI)) {
 			
 			sp_repr_set_attr ((SPRepr *) rdoc, "doctype", sp_svg_doctype_str);
 			sp_repr_set_attr ((SPRepr *) rdoc, "comment", sp_comment_str);
@@ -195,22 +196,24 @@ static SPRepr *
 sp_repr_svg_read_node (SPXMLDocument *doc, xmlNodePtr node, const gchar *default_ns, GHashTable *prefix_map)
 {
 	SPRepr *repr, *crepr;
+	xmlNsPtr ns;
 	xmlAttrPtr prop;
 	xmlNodePtr child;
-	gchar c[256];
+	char c[256];
 
 #ifdef SP_REPR_IO_VERBOSE
+	int preserve;
+	preserve = xmlNodeGetSpacePreserve (node);
+	g_print ("node %s preserve %d\n", node->name, preserve);
 	g_print ("Node %d %s contains %s\n", node->type, node->name, node->content);
 #endif
 
-	if (node->type == XML_TEXT_NODE || node->type == XML_CDATA_SECTION_NODE)
-	{
+	if (node->type == XML_TEXT_NODE) {
 		xmlChar *p;
 		gboolean preserve;
-
 		preserve = (xmlNodeGetSpacePreserve (node) == 1);
-
 		for (p = node->content; p && *p; p++) {
+#if 0
 			if (!isspace (*p) || preserve) {
 				xmlChar *e;
 				unsigned char *s;
@@ -225,16 +228,36 @@ sp_repr_svg_read_node (SPXMLDocument *doc, xmlNodePtr node, const gchar *default
 				g_free (s);
 				return rdoc;
 			}
+#else
+			if (!isspace (*p)) {
+				return sp_xml_document_createTextNode (doc, node->content);
+			}
+#endif
 		}
 		return NULL;
 	}
 
+	if (node->type == XML_CDATA_SECTION_NODE) {
+		SPRepr *rdoc;
+		rdoc = sp_repr_new_cdata (node->content);
+		return rdoc;
+	}
+
 	if (node->type == XML_COMMENT_NODE) return NULL;
 	if (node->type == XML_ENTITY_DECL) return NULL;
 
 	sp_repr_qualified_name (c, 256, node->ns, node->name, default_ns, prefix_map);
 	repr = sp_repr_new (c);
 
+	for (ns = node->nsDef; ns != NULL; ns = ns->next) {
+		if (ns->prefix) {
+			g_snprintf (c, 256, "xmlns:%s", ns->prefix);
+			sp_repr_set_attr (repr, c, ns->href);
+		} else {
+			sp_repr_set_attr (repr, "xmlns", ns->href);
+		}
+	}
+
 	for (prop = node->properties; prop != NULL; prop = prop->next) {
 		if (prop->children) {
 			sp_repr_qualified_name (c, 256, prop->ns, prop->name, default_ns, prefix_map);
@@ -242,8 +265,9 @@ sp_repr_svg_read_node (SPXMLDocument *doc, xmlNodePtr node, const gchar *default
 		}
 	}
 
-	if (node->content)
+	if (node->content) {
 		sp_repr_set_content (repr, node->content);
+	}
 
 	child = node->xmlChildrenNode;
 	for (child = node->xmlChildrenNode; child != NULL; child = child->next) {
@@ -257,8 +281,8 @@ sp_repr_svg_read_node (SPXMLDocument *doc, xmlNodePtr node, const gchar *default
 	return repr;
 }
 
-void
-sp_repr_save_stream (SPReprDoc *doc, FILE *fp)
+unsigned int
+sp_repr_doc_write_stream (SPReprDoc *doc, FILE *fp)
 {
 	SPRepr *repr;
 	const unsigned char *str;
@@ -272,9 +296,12 @@ sp_repr_save_stream (SPReprDoc *doc, FILE *fp)
 	str = sp_repr_attr ((SPRepr *) doc, "comment");
 	if (str) fputs (str, fp);
 
-	repr = sp_repr_document_root (doc);
+	repr = sp_repr_doc_get_root (doc);
 
 	sp_repr_write_stream (repr, fp, 0);
+	fputs ("\n", fp);
+
+	return 1;
 }
 
 #ifdef WIN32
@@ -282,8 +309,8 @@ sp_repr_save_stream (SPReprDoc *doc, FILE *fp)
 #include <tchar.h>
 #endif
 
-void
-sp_repr_save_file (SPReprDoc *doc, const char *filename)
+unsigned int
+sp_repr_doc_write_file (SPReprDoc *doc, const unsigned char *filename)
 {
 #ifdef WIN32
 	TCHAR *tfilename;
@@ -302,19 +329,20 @@ sp_repr_save_file (SPReprDoc *doc, const char *filename)
 	file = fopen (filename, "w");
 #endif
 
-	g_return_if_fail (file != NULL);
+	g_return_val_if_fail (file != NULL, 0);
 
-	sp_repr_save_stream (doc, file);
+	sp_repr_doc_write_stream (doc, file);
 
 	fclose (file);
 
-	return;
+	return 1;
 }
 
 void
 sp_repr_print (SPRepr * repr)
 {
 	sp_repr_write_stream (repr, stdout, 0);
+	fputs ("\n", stdout);
 
 	return;
 }
@@ -323,6 +351,7 @@ sp_repr_print (SPRepr * repr)
 static void
 repr_quote_write (FILE * file, const gchar * val)
 {
+	if (!val) return;
 	for (; *val != '\0'; val++) {
 		switch (*val) {
 		case '"': fputs ("&quot;", file); break;
@@ -334,8 +363,17 @@ repr_quote_write (FILE * file, const gchar * val)
 	}
 }
 
-void
-sp_repr_write_stream (SPRepr * repr, FILE * file, gint level)
+static void
+repr_cdata_write (FILE *ofs, const unsigned char *val)
+{
+	if (!val) return;
+	fputs ("<![CDATA[", ofs);
+	fputs (val, ofs);
+	fputs ("]]>", ofs);
+}
+
+unsigned int
+sp_repr_write_stream (SPRepr *repr, FILE * file, unsigned int level)
 {
 	SPReprAttr *attr;
 	SPRepr *child;
@@ -343,8 +381,8 @@ sp_repr_write_stream (SPRepr * repr, FILE * file, gint level)
 	gboolean loose;
 	gint i;
 
-	g_return_if_fail (repr != NULL);
-	g_return_if_fail (file != NULL);
+	g_return_val_if_fail (repr != NULL, 0);
+	g_return_val_if_fail (file != NULL, 0);
 
 	if (level > 16) level = 16;
 	for (i = 0; i < level; i++) fputs ("  ", file);
@@ -359,7 +397,7 @@ sp_repr_write_stream (SPRepr * repr, FILE * file, gint level)
 		repr_quote_write (file, val);
 		putc ('"', file);
 	}
-	loose = TRUE;
+	loose = (sp_repr_get_xml_space (repr) != SP_REPR_XML_SPACE_PRESERVE);
 	for (child = repr->children; child != NULL; child = child->next) {
 		if (child->type == SP_XML_TEXT_NODE) {
 			loose = FALSE;
@@ -380,19 +418,28 @@ sp_repr_write_stream (SPRepr * repr, FILE * file, gint level)
 #endif
 		for (child = repr->children; child != NULL; child = child->next) {
 			if (child->type == SP_XML_TEXT_NODE) {
-				repr_quote_write (file, sp_repr_content (child));
+				repr_quote_write (file, child->content);
+			} else if (child->type == SP_XML_CDATA_NODE) {
+				if (child->content && !strstr (child->content, "]]>")) {
+					repr_cdata_write (file, child->content);
+				} else {
+					repr_quote_write (file, child->content);
+				}
 			} else {
 				sp_repr_write_stream (child, file, (loose) ? (level + 1) : 0);
+				if (loose) fputs ("\n", file);
 			}
 		}
 		
 		if (loose) {
 			for (i = 0; i < level; i++) fputs ("  ", file);
 		}
-		fprintf (file, "</%s>\n", sp_repr_name (repr));
+		fprintf (file, "</%s>", sp_repr_name (repr));
 	} else {
-		fputs (" />\n", file);
+		fputs (" />", file);
 	}
+
+	return 1;
 }
 
 #ifdef HAVE_LIBWMF
