@@ -34,6 +34,7 @@
 #endif
 
 #include "repr-private.h"
+#include "repr.h"
 
 static void sp_xml_ns_register_defaults ();
 static char *sp_xml_ns_auto_prefix (const char *uri);
@@ -203,7 +204,7 @@ sp_xml_element_setAttributeNS (SPXMLElement *element, const unsigned char *nr, c
 }
 
 SPRepr *
-sp_repr_children (SPRepr *repr)
+sp_repr_get_children (SPRepr *repr)
 {
 	g_return_val_if_fail (repr != NULL, NULL);
 
@@ -211,7 +212,7 @@ sp_repr_children (SPRepr *repr)
 }
 
 SPRepr *
-sp_repr_next (SPRepr *repr)
+sp_repr_get_next (SPRepr *repr)
 {
 	g_return_val_if_fail (repr != NULL, NULL);
 
@@ -222,11 +223,79 @@ int sp_repr_attr_is_set (SPRepr * repr, const char * key)
 {
 	char * result;
 
-	result = (char *) sp_repr_attr (repr, key);
+	result = (char *) sp_repr_get_attr (repr, key);
 
 	return (result != NULL);
 }
 
+SPReprAttr *
+sp_repr_attr_get_first (SPRepr *repr)
+{
+	g_return_val_if_fail (repr != NULL, NULL);
+
+	return repr->attributes;
+}
+
+SPReprAttr *
+sp_repr_attr_get_next (SPRepr *repr, SPReprAttr *ref)
+{
+	g_return_val_if_fail (repr != NULL, NULL);
+
+	return (ref) ? ref->next : repr->attributes;
+}
+
+const unsigned char *
+sp_repr_attr_get_key (SPRepr *repr, SPReprAttr *attr)
+{
+	g_return_val_if_fail (repr != NULL, NULL);
+	g_return_val_if_fail (attr != NULL, NULL);
+
+	return SP_REPR_ATTRIBUTE_KEY (attr);
+}
+
+const unsigned char *
+sp_repr_attr_get_value (SPRepr *repr, SPReprAttr *attr)
+{
+	g_return_val_if_fail (repr != NULL, NULL);
+	g_return_val_if_fail (attr != NULL, NULL);
+
+	return SP_REPR_ATTRIBUTE_VALUE (attr);
+}
+
+SPReprXmlSpaceType
+sp_repr_get_xml_space (SPRepr *repr)
+{
+	const unsigned char *space;
+
+	space = sp_repr_attr_inherited (repr, "xml:space");
+
+	if (space && !strcmp(space, "preserve"))
+		return SP_REPR_XML_SPACE_PRESERVE;
+
+	return SP_REPR_XML_SPACE_DEFAULT;
+}
+
+void
+sp_repr_set_xml_space (SPRepr *repr, SPReprXmlSpaceType space)
+{
+	const char *space_str;
+	switch (space) {
+	case SP_REPR_XML_SPACE_PRESERVE:
+		space_str = "preserve";
+		break;
+	case SP_REPR_XML_SPACE_DEFAULT:
+		space_str = "default";
+		break;
+	case SP_REPR_XML_SPACE_NONE:
+	default:
+		space_str = NULL;
+		break;
+	}
+
+	sp_repr_set_attr (repr, "xml:space", space_str);
+}
+
+#if 0
 double
 sp_repr_get_double_attribute (SPRepr * repr, const char * key, double def)
 {
@@ -284,20 +353,21 @@ sp_repr_set_int_attribute (SPRepr * repr, const char * key, int value)
 
 	return sp_repr_set_attr (repr, key, c);
 }
+#endif
 
 const char *
 sp_repr_doc_attr (SPRepr * repr, const char * key)
 {
 	SPRepr * p;
 
-	p = sp_repr_parent (repr);
+	p = sp_repr_get_parent (repr);
 
 	while (p != NULL) {
 		repr = p;
-		p = sp_repr_parent (p);
+		p = sp_repr_get_parent (p);
 	}
 
-	return sp_repr_attr (repr, key);
+	return sp_repr_get_attr (repr, key);
 }
 
 int
@@ -306,8 +376,8 @@ sp_repr_compare_position (SPRepr * first, SPRepr * second)
 	SPRepr * parent;
 	int p1, p2;
 
-	parent = sp_repr_parent (first);
-	g_assert (parent == sp_repr_parent (second));
+	parent = sp_repr_get_parent (first);
+	g_assert (parent == sp_repr_get_parent (second));
 
 	p1 = sp_repr_position (first);
 	p2 = sp_repr_position (second);
@@ -325,7 +395,7 @@ sp_repr_position (SPRepr * repr)
 	int pos;
 
 	g_assert (repr != NULL);
-	parent = sp_repr_parent (repr);
+	parent = sp_repr_get_parent (repr);
 	g_assert (parent != NULL);
 
 	pos = 0;
@@ -364,13 +434,13 @@ sp_repr_n_children (SPRepr * repr)
 	return n;
 }
 
-void
+unsigned int
 sp_repr_append_child (SPRepr * repr, SPRepr * child)
 {
 	SPRepr * ref;
 
-	g_assert (repr != NULL);
-	g_assert (child != NULL);
+	g_return_val_if_fail (repr != NULL, 0);
+	g_return_val_if_fail (child != NULL, 0);
 
 	ref = NULL;
 	if (repr->children) {
@@ -378,19 +448,16 @@ sp_repr_append_child (SPRepr * repr, SPRepr * child)
 		while (ref->next) ref = ref->next;
 	}
 
-	sp_repr_add_child (repr, child, ref);
+	return sp_repr_add_child (repr, child, ref);
 }
 
-void sp_repr_unparent (SPRepr * repr)
+unsigned int
+sp_repr_unparent (SPRepr *repr)
 {
-	SPRepr * parent;
+	g_return_val_if_fail (repr != NULL, 0);
+	g_return_val_if_fail (repr->parent != NULL, 0);
 
-	g_assert (repr != NULL);
-
-	parent = sp_repr_parent (repr);
-	g_assert (parent != NULL);
-
-	sp_repr_remove_child (parent, repr);
+	return sp_repr_remove_child (repr->parent, repr);
 }
 
 SPRepr * sp_repr_duplicate_and_parent (SPRepr * repr)
@@ -399,7 +466,7 @@ SPRepr * sp_repr_duplicate_and_parent (SPRepr * repr)
 
 	g_assert (repr != NULL);
 
-	parent = sp_repr_parent (repr);
+	parent = sp_repr_get_parent (repr);
 	g_assert (parent != NULL);
 
 	new = sp_repr_duplicate (repr);
@@ -436,8 +503,8 @@ sp_repr_attr_inherited (SPRepr *repr, const unsigned char *key)
 	g_assert (repr != NULL);
 	g_assert (key != NULL);
 
-	for (current = repr; current != NULL; current = sp_repr_parent (current)) {
-		val = sp_repr_attr (current, key);
+	for (current = repr; current != NULL; current = sp_repr_get_parent (current)) {
+		val = sp_repr_get_attr (current, key);
 		if (val != NULL)
 			return val;
 	}
@@ -467,9 +534,7 @@ sp_repr_set_attr_recursive (SPRepr *repr, const unsigned char *key, const unsign
  * lookup child by (@key, @value)
  */
 SPRepr *
-sp_repr_lookup_child (SPRepr       *repr,
-                      const unsigned char *key,
-                      const unsigned char *value)
+sp_repr_lookup_child (SPRepr *repr, const unsigned char *key, const unsigned char *value)
 {
 	SPRepr *child;
 	SPReprAttr *attr;
@@ -492,6 +557,24 @@ sp_repr_lookup_child (SPRepr       *repr,
 	return NULL;
 }
 
+SPRepr *
+sp_repr_lookup_child_by_name (SPRepr *repr, const unsigned char *name)
+{
+	SPRepr *child;
+	unsigned int quark;
+
+	g_return_val_if_fail (repr != NULL, NULL);
+	g_return_val_if_fail (name != NULL, NULL);
+
+	quark = g_quark_from_string (name);
+
+	for (child = repr->children; child != NULL; child = child->next) {
+		if (child->name == quark) return child;
+	}
+
+	return NULL;
+}
+
 /* Convenience */
 unsigned int
 sp_repr_get_boolean (SPRepr *repr, const unsigned char *key, unsigned int *val)
@@ -502,7 +585,7 @@ sp_repr_get_boolean (SPRepr *repr, const unsigned char *key, unsigned int *val)
 	g_return_val_if_fail (key != NULL, FALSE);
 	g_return_val_if_fail (val != NULL, FALSE);
 
-	v = sp_repr_attr (repr, key);
+	v = sp_repr_get_attr (repr, key);
 
 	if (v != NULL) {
 		if (!g_strcasecmp (v, "true") ||
@@ -528,7 +611,7 @@ sp_repr_get_int (SPRepr *repr, const unsigned char *key, int *val)
 	g_return_val_if_fail (key != NULL, FALSE);
 	g_return_val_if_fail (val != NULL, FALSE);
 
-	v = sp_repr_attr (repr, key);
+	v = sp_repr_get_attr (repr, key);
 
 	if (v != NULL) {
 		*val = atoi (v);
@@ -547,7 +630,7 @@ sp_repr_get_double (SPRepr *repr, const unsigned char *key, double *val)
 	g_return_val_if_fail (key != NULL, FALSE);
 	g_return_val_if_fail (val != NULL, FALSE);
 
-	v = sp_repr_attr (repr, key);
+	v = sp_repr_get_attr (repr, key);
 
 	if (v != NULL) {
 		return arikkei_strtod_exp (v, 256, val);
