@@ -17,6 +17,7 @@
 
 #include <string.h>
 #include <stddef.h>
+#include <assert.h>
 
 #include <glib.h>
 
@@ -59,6 +60,22 @@ SPReprClass _sp_repr_xml_text_class = {
 	repr_finalize
 };
 
+SPReprClass _sp_repr_xml_cdata_class = {
+	sizeof (SPRepr),
+	NULL,
+	repr_init,
+	repr_copy,
+	repr_finalize
+};
+
+SPReprClass _sp_repr_xml_comment_class = {
+	sizeof (SPRepr),
+	NULL,
+	repr_init,
+	repr_copy,
+	repr_finalize
+};
+
 static SPRepr *sp_repr_new_from_code (SPReprClass *type, int code);
 static void sp_repr_remove_attribute (SPRepr *repr, SPReprAttr *attr);
 static void sp_repr_remove_listener (SPRepr *repr, SPListener *listener);
@@ -97,14 +114,33 @@ sp_repr_new (const unsigned char *name)
 SPRepr *
 sp_repr_new_text (const unsigned char *content)
 {
-	SPRepr * repr;
-	g_return_val_if_fail (content != NULL, NULL);
+	SPRepr *repr;
 	repr = sp_repr_new_from_code (SP_XML_TEXT_NODE, g_quark_from_static_string ("text"));
 	repr->content = g_strdup (content);
 	repr->type = SP_XML_TEXT_NODE;
 	return repr;
 }
 
+SPRepr *
+sp_repr_new_cdata (const unsigned char *content)
+{
+	SPRepr *repr;
+	repr = sp_repr_new_from_code (SP_XML_CDATA_NODE, g_quark_from_static_string ("cdata"));
+	repr->content = g_strdup (content);
+	repr->type = SP_XML_CDATA_NODE;
+	return repr;
+}
+
+SPRepr *
+sp_repr_new_comment (const unsigned char *content)
+{
+	SPRepr *repr;
+	repr = sp_repr_new_from_code (SP_XML_COMMENT_NODE, g_quark_from_static_string ("comment"));
+	repr->content = g_strdup (content);
+	repr->type = SP_XML_COMMENT_NODE;
+	return repr;
+}
+
 static void
 repr_init (SPRepr *repr)
 {
@@ -119,11 +155,11 @@ repr_init (SPRepr *repr)
 static void
 repr_doc_init (SPRepr *repr)
 {
-	SPReprDoc *doc=(SPReprDoc *)repr;
+	SPReprDoc *doc= (SPReprDoc *) repr;
 
 	repr_init (repr);
-
 	repr->doc = doc;
+	doc->root = NULL;
 	doc->log = NULL;
 	doc->is_logging = 0;
 }
@@ -160,8 +196,8 @@ repr_finalize (SPRepr *repr)
 {
 	SPReprListener *rl;
 	/* Parents reference children */
-	g_assert (repr->parent == NULL);
-	g_assert (repr->next == NULL);
+	assert (repr->parent == NULL);
+	assert (repr->next == NULL);
 	repr->doc = NULL;
 
 	for (rl = repr->listeners; rl; rl = rl->next) {
@@ -187,10 +223,10 @@ repr_doc_finalize (SPRepr *repr)
 static SPRepr *
 sp_repr_attach (SPRepr *parent, SPRepr *child)
 {
-	g_assert (parent != NULL);
-	g_assert (child != NULL);
-	g_assert (child->parent == NULL);
-	g_assert (child->doc == NULL && parent->doc == NULL);
+	assert (parent != NULL);
+	assert (child != NULL);
+	assert (child->parent == NULL);
+	assert (child->doc == NULL && parent->doc == NULL);
 
 	child->parent = parent;
 
@@ -198,7 +234,7 @@ sp_repr_attach (SPRepr *parent, SPRepr *child)
 }
 
 SPRepr *
-sp_repr_duplicate (const SPRepr *repr)
+sp_repr_duplicate (SPRepr *repr)
 {
 	SPRepr *new;
 
@@ -241,14 +277,14 @@ repr_copy (SPRepr *to, const SPRepr *from)
 void
 repr_doc_copy (SPRepr *to, const SPRepr *from)
 {
-	SPReprDoc *to_doc=(SPReprDoc *)to;
+	SPReprDoc *to_doc= (SPReprDoc *) to;
 	repr_copy (to, from);
 	to_doc->log = NULL;
 	to_doc->is_logging = 0;
 }
 
 const unsigned char *
-sp_repr_name (const SPRepr *repr)
+sp_repr_get_name (SPRepr *repr)
 {
 	g_return_val_if_fail (repr != NULL, NULL);
 
@@ -256,15 +292,15 @@ sp_repr_name (const SPRepr *repr)
 }
 
 const unsigned char *
-sp_repr_content (const SPRepr *repr)
+sp_repr_get_content (SPRepr *repr)
 {
-	g_assert (repr != NULL);
+	g_return_val_if_fail (repr != NULL, NULL);
 
 	return SP_REPR_CONTENT (repr);
 }
 
 const unsigned char *
-sp_repr_attr (const SPRepr *repr, const unsigned char *key)
+sp_repr_get_attr (SPRepr *repr, const unsigned char *key)
 {
 	SPReprAttr *ra;
 	unsigned int q;
@@ -437,8 +473,8 @@ sp_repr_set_attr (SPRepr *repr, const unsigned char *key, const unsigned char *v
 static void
 sp_repr_remove_attribute (SPRepr *repr, SPReprAttr *attr)
 {
-	g_assert (repr != NULL);
-	g_assert (attr != NULL);
+	assert (repr != NULL);
+	assert (attr != NULL);
 
 	if (attr == repr->attributes) {
 		repr->attributes = attr->next;
@@ -447,7 +483,7 @@ sp_repr_remove_attribute (SPRepr *repr, SPReprAttr *attr)
 		prev = repr->attributes;
 		while (prev->next != attr) {
 			prev = prev->next;
-			g_assert (prev != NULL);
+			assert (prev != NULL);
 		}
 		prev->next = attr->next;
 	}
@@ -457,9 +493,9 @@ sp_repr_remove_attribute (SPRepr *repr, SPReprAttr *attr)
 }
 
 SPRepr *
-sp_repr_parent (SPRepr * repr)
+sp_repr_get_parent (SPRepr * repr)
 {
-	g_assert (repr != NULL);
+	assert (repr != NULL);
 
 	return repr->parent;
 }
@@ -470,11 +506,11 @@ sp_repr_add_child (SPRepr *repr, SPRepr *child, SPRepr *ref)
 	SPReprListener * rl;
 	unsigned int allowed;
 
-	g_assert (repr != NULL);
-	g_assert (child != NULL);
-	g_assert (!ref || ref->parent == repr);
-	g_assert (child->parent == NULL);
-	g_assert (child->doc == NULL || child->doc == repr->doc);
+	assert (repr != NULL);
+	assert (child != NULL);
+	assert (!ref || ref->parent == repr);
+	assert (child->parent == NULL);
+	assert (child->doc == NULL || child->doc == repr->doc);
 
 	allowed = TRUE;
 	for (rl = repr->listeners; rl != NULL; rl = rl->next) {
@@ -512,11 +548,11 @@ bind_document (SPReprDoc *doc, SPRepr *repr)
 {
 	SPRepr *child;
 
-	g_assert (repr->doc == NULL);
+	assert (repr->doc == NULL);
 
 	repr->doc = doc;
 
-	for ( child = repr->children ; child != NULL ; child = child->next ) {
+	for (child = repr->children ;child != NULL ;child = child->next) {
 		bind_document (doc, child);
 	}
 }
@@ -528,9 +564,9 @@ sp_repr_remove_child (SPRepr *repr, SPRepr *child)
 	SPRepr *ref;
 	unsigned int allowed;
 
-	g_assert (repr != NULL);
-	g_assert (child != NULL);
-	g_assert (child->parent == repr);
+	assert (repr != NULL);
+	assert (child != NULL);
+	assert (child->parent == repr);
 
 	ref = NULL;
 	if (child != repr->children) {
@@ -643,36 +679,13 @@ sp_repr_set_position_absolute (SPRepr * repr, int pos)
 	sp_repr_change_order (parent, repr, ref);
 }
 
-void
-sp_repr_synthesize_events (SPRepr *repr, const SPReprEventVector *vector, void * data)
-{
-	
-	if (vector->attr_changed) {
-		SPReprAttr *attr;
-		attr = repr->attributes;
-		for ( ; attr ; attr = attr->next ) {
-			vector->attr_changed (repr, g_quark_to_string (attr->key), NULL, attr->value, data);
-		}
-	}
-	if (vector->child_added) {
-		SPRepr *child, *ref;
-		ref = NULL; child = repr->children;
-		for ( ; child ; ref = child, child = child->next ) {
-			vector->child_added (repr, child, ref, data);
-		}
-	}
-	if (vector->content_changed) {
-		vector->content_changed (repr, NULL, repr->content, data);
-	}
-}
-
 void
 sp_repr_add_listener (SPRepr *repr, const SPReprEventVector *vector, void * data)
 {
 	SPReprListener *rl, *last;
 
-	g_assert (repr != NULL);
-	g_assert (vector != NULL);
+	assert (repr != NULL);
+	assert (vector != NULL);
 
 	last = NULL;
 	if (repr->listeners) {
@@ -746,7 +759,7 @@ sp_repr_nth_child (const SPRepr * repr, int n)
 /* fixme: Do this somewhere, somehow The Right Way (TM) */
 
 SPReprDoc *
-sp_repr_document_new (const char *rootname)
+sp_repr_doc_new (const unsigned char *rootname)
 {
 	SPReprDoc * doc;
 	SPRepr * root;
@@ -765,19 +778,20 @@ sp_repr_document_new (const char *rootname)
 	root = sp_repr_new (rootname);
 
 	sp_repr_add_child (&doc->repr, root, 0);
+	doc->root = root;
 	sp_repr_unref (root);
 
 	return doc;
 }
 
 void
-sp_repr_document_ref (SPReprDoc * doc)
+sp_repr_doc_ref (SPReprDoc * doc)
 {
 	sp_repr_ref ((SPRepr *) doc);
 }
 
 void
-sp_repr_document_unref (SPReprDoc * doc)
+sp_repr_doc_unref (SPReprDoc * doc)
 {
 	sp_repr_unref ((SPRepr *) doc);
 }
@@ -785,26 +799,27 @@ sp_repr_document_unref (SPReprDoc * doc)
 void
 sp_repr_document_set_root (SPReprDoc *doc, SPRepr *repr)
 {
-	g_assert (doc != NULL);
-	g_assert (repr != NULL);
-	g_assert (doc->repr.children != NULL);
-	g_assert (repr->doc == NULL || repr->doc == doc);
+	assert (doc != NULL);
+	assert (repr != NULL);
+	assert (doc->repr.children != NULL);
+	assert (repr->doc == NULL || repr->doc == doc);
 
 	sp_repr_remove_child (&doc->repr, doc->repr.children);
 	sp_repr_add_child (&doc->repr, repr, NULL);
+	doc->root = repr;
 }
 
 SPReprDoc *
-sp_repr_document (const SPRepr *repr)
+sp_repr_get_doc (SPRepr *repr)
 {
 	return repr->doc;
 }
 
 SPRepr *
-sp_repr_document_root (const SPReprDoc *doc)
+sp_repr_doc_get_root (SPReprDoc *doc)
 {
-	g_assert (doc != NULL);
-	return doc->repr.children;
+	assert (doc != NULL);
+	return doc->root;
 }
 
 /*
@@ -813,17 +828,17 @@ sp_repr_document_root (const SPReprDoc *doc)
  */
 
 unsigned int
-sp_repr_document_merge (SPReprDoc *doc, const SPReprDoc *src, const unsigned char *key)
+sp_repr_doc_merge (SPReprDoc *doc, SPReprDoc *src, const unsigned char *key)
 {
 	SPRepr *rdoc;
-	const SPRepr *rsrc;
+	SPRepr *rsrc;
 
 	g_return_val_if_fail (doc != NULL, FALSE);
 	g_return_val_if_fail (src != NULL, FALSE);
 	g_return_val_if_fail (key != NULL, FALSE);
 
-	rdoc = sp_repr_document_root (doc);
-	rsrc = sp_repr_document_root (src);
+	rdoc = sp_repr_doc_get_root (doc);
+	rsrc = sp_repr_doc_get_root (src);
 	
 	return sp_repr_merge (rdoc, rsrc, key);
 }
@@ -834,7 +849,7 @@ sp_repr_document_merge (SPReprDoc *doc, const SPReprDoc *src, const unsigned cha
  */
 
 unsigned int
-sp_repr_merge (SPRepr *repr, const SPRepr *src, const unsigned char *key)
+sp_repr_merge (SPRepr *repr, SPRepr *src, const unsigned char *key)
 {
 	SPRepr *child;
 	SPReprAttr *attr;
@@ -854,7 +869,7 @@ sp_repr_merge (SPRepr *repr, const SPRepr *src, const unsigned char *key)
 	for (child = src->children; child != NULL; child = child->next) {
 		SPRepr *rch;
 		const unsigned char *id;
-		id = sp_repr_attr (child, key);
+		id = sp_repr_get_attr (child, key);
 		if (id) {
 			rch = sp_repr_lookup_child (repr, key, id);
 			if (rch) {
@@ -1004,3 +1019,21 @@ sp_listener_free (SPListener *listener)
 	free_listener = listener;
 }
 
+unsigned int
+sp_repr_is_element (SPRepr *repr)
+{
+	return SP_REPR_TYPE (repr) == SP_XML_ELEMENT_NODE;
+}
+
+unsigned int
+sp_repr_is_text (SPRepr *repr)
+{
+	return SP_REPR_TYPE (repr) == SP_XML_TEXT_NODE;
+}
+
+unsigned int
+sp_repr_is_cdata (SPRepr *repr)
+{
+	return SP_REPR_TYPE (repr) == SP_XML_CDATA_NODE;
+}
+
